/*****************************************************************************
Copyright 2006-2008, 2011-2016 Teradata Corporation.
All Rights Reserved.
Teradata Corporation CONFIDENTIAL AND TRADE SECRET.

This copyrighted material is the Confidential, Unpublished
Property of the Teradata Corporation. This copyright notice
and any other copyright notices included in machine readable
copies must be reproduced on all authorized copies.

Module:      tdsql.h

Author:      Teradata Corporation
             Teradata Client Product Engineering
             ODBC

Created:     10/04/2006

Description: This files defines Teradata specific attributes for ODBC
             connection, statement and descriptor objects. These are
             extensions to the standard. 

             This file was made since the ODBC 12.0 release introduces a
             new set of Teradata extenstions to the ODBC attributes.

  Revision History: 

   Revision     Date        DR    Userid   Comments
   -----------  --------   ------ -------- -----------------------------------
   12.00.00.00  10/04/2006  98321 nh518691 Initial version
   12.00.00.01  10/30/2006  98318 jb518661 Added Teradata specific descriptors
                                           with prefix SQL_DESC_TD
   13.00.00.00  10/11/2007 115761 jb518661 Renamed file from TDAttributes.h as part of
                                           splitting files for data conversion refactoring.
   13.00.00.00  10/25/2007 115298 jb518661 Added Period data types.

   12.00.00.02  10/15/2008 127044 nh518691 Added defintions for column character
                                           set and export width.
   14.00.00.00  03/29/2011 141363 nh518691 Definitions for NUMBER data type. 
   14.00.00.00  04/01/2011 141363 nh518691 Define driver specific descriptor values
                                           in the range defined for ODBC 3.8 (0x4000-0x7FFF)
                           JI-ODBC
   ------------------------------------------------------------------------------
   14.10.00.00  01/06/2012 6300 nh518691 Added XML data type
   15.00.00.00  09/02/2013   4604 nh518691 Added rowcount statement attribute 
   15.00.00.00  09/19/2013   11067   nh518691 Aded JSON data type
   15.00.00.00  11/13/2013   11075   rk185015 Added Teradata specific UDT attributes.
  15.10c.00.00  09/23/2014 ODBC-7802 st255032 Added support for Trusted SQL
   16.00.00.00  03/07/16     16550   kh255043 Support for DATASET avro

*****************************************************************************/

#ifndef _TDODBC_TDSQL_H_
#define _TDODBC_TDSQL_H_

/*
 * Teradata Driver Defined Connection Attributes
 */

#ifndef SQL_CONNECT_OPT_DRVR_START
#define SQL_CONNECT_OPT_DRVR_START 1000
#endif

#define SQL_CONNECT_TCPVENDOR   (SQL_CONNECT_OPT_DRVR_START)
#define SQL_CONNECT_RUNSTRING   (SQL_CONNECT_OPT_DRVR_START + 300)

/*
 * Teradata Driver Defined Statement attributes
 */

#define SQL_ATTR_TDATA_HOST_ID           13001      
#define SQL_ATTR_TDATA_SESSION_NUMBER    13002     
#define SQL_ATTR_TDATA_SESSION_CHARSET   13003    

/* Auto-generated key retrieval (DR 98321) */
#define SQL_ATTR_AGKR                    13004             

/* Attribute to enable/disable data encryption */
#define SQL_ATTR_DATA_ENCRYPTION         13008      /* DR69335-Vittal */

/* Attribute to get the activity row count (unsigned 64-bit value) */
#define SQL_ATTR_TDATA_ROWCOUNT          13009      /* ODBC-4604 */

/* Attribute for Trusted Sessions, Trusted SQL */
#define SQL_ATTR_TRUSTED_SQL             13010      /* DOCB-7802 */


/* 
 * Teradata Driver Defined statement attributes 
 */

/*
 * Values for SQL_ATTR_DATA_ENCRYPTION statement attribute
 */
#define SQL_DATA_ENCRYPTION_ON      1
#define SQL_DATA_ENCRYPTION_OFF     0

/*
 * Values for SQL_ATTR_AGKR statement attribute
 */
#define SQL_AGKR_NO                 0
#define SQL_AGKR_IDENTITY_COLUMN    1
#define SQL_AGKR_WHOLE_ROW          2

/*
 * Teradata Driver Defined Column/Descriptor attributes
 */

/* ODBC 3.8 defines SQL_DRIVER_DESCRIPTOR_BASE */
#ifndef SQL_DRIVER_DESCRIPTOR_BASE
#define SQL_DRIVER_DESCRIPTOR_BASE 0x4000
#endif

#ifndef SQL_COLUMN_TD_DRIVER_START
#define SQL_COLUMN_TD_DRIVER_START 1300
#endif

#ifndef SQL_COLUMN_ACTIVITY_TYPE
#define SQL_COLUMN_ACTIVITY_TYPE      (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START)
#define SQL_COLUMN_COST_ESTIMATE      (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 1)
#define SQL_COLUMN_FORMAT             (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 2)
#define SQL_COLUMN_TITLE              (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 3)
#define SQL_COLUMN_ACTUAL_NAME        (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 4)
#define SQL_COLUMN_CHARACTER_SET      (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 5)
#define SQL_COLUMN_EXPORT_WIDTH       (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 6)
#define SQL_COLUMN_EXPORT_WIDTH_ADJ   (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 7)
#define SQL_COLUMN_EXPORT_BYTES       (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 8)
#define SQL_COLUMN_TDODBC_TYPE        (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 9)
#define SQL_COLUMN_TD_UDT_NAME        (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 10)
#define SQL_COLUMN_TD_UDT_INDICATOR   (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 11)
#endif

#define SQL_COLUMN_TD_DRIVER_END      (SQL_DRIVER_DESCRIPTOR_BASE + SQL_COLUMN_TD_DRIVER_START + 11)
#define SQL_COLUMN_TD_DRIVER_MAX      1349 /* End of old Teradata reserved range. Replaced by 3.8 spec. */


/* 
 * The following are the corresponding Teradata specific descriptors 
 */

#define SQL_DESC_TD_ACTIVITY_TYPE      SQL_COLUMN_ACTIVITY_TYPE
#define SQL_DESC_TD_COST_ESTIMATE      SQL_COLUMN_COST_ESTIMATE
#define SQL_DESC_TD_FORMAT             SQL_COLUMN_FORMAT
#define SQL_DESC_TD_TITLE              SQL_COLUMN_TITLE
#define SQL_DESC_TD_ACTUAL_NAME        SQL_COLUMN_ACTUAL_NAME
#define SQL_DESC_TD_CHARACTER_SET      SQL_COLUMN_CHARACTER_SET 
#define SQL_DESC_TD_EXPORT_WIDTH       SQL_COLUMN_EXPORT_WIDTH
#define SQL_DESC_TD_EXPORT_WIDTH_ADJ   SQL_COLUMN_EXPORT_WIDTH_ADJ
#define SQL_DESC_TD_EXPORT_BYTES       SQL_COLUMN_EXPORT_BYTES
#define SQL_DESC_TD_ODBC_TYPE          SQL_COLUMN_TDODBC_TYPE
#define SQL_DESC_TD_UDT_NAME           SQL_COLUMN_TD_UDT_NAME
#define SQL_DESC_TD_UDT_INDICATOR      SQL_COLUMN_TD_UDT_INDICATOR

#define SQL_DRIVER_DESCRIPTOR_END      SQL_COLUMN_TD_DRIVER_END

/*
 * DR141363 Map old descriptor value to new ODBC 3.8 values. Used to maintain binary compatibilty
 */
#define MAP_DRIVER_DESCRIPTOR_VALUE(Value) (((Value) >= SQL_COLUMN_TD_DRIVER_START && \
                                             (Value) <= SQL_COLUMN_TD_DRIVER_MAX) ?   \
                                             (Value) + SQL_DRIVER_DESCRIPTOR_BASE : (Value));

/* 
 * Teradata column character set codes 
 */
#define SQL_TD_CS_UNDEFINED           0
#define SQL_TD_CS_LATIN               1
#define SQL_TD_CS_UNICODE             2
#define SQL_TD_CS_KANJISJIS           3
#define SQL_TD_CS_GRAPHIC             4
#define SQL_TD_CS_KANJI1              5


/* DR 115298 */
/*
 * Teradata Driver Defined Data Types
 *
 * DR 141363: The ODBC 3.8 defined  range for driver specific SQL types is from 0x4000 - 0x7FFF.
 * To maintain compatability the existing published PERIOD types are retained but will be depricated.
 */

/*
 * ODBC 3.8 driver defined SQL type codes. Range 16384 - 32767
 */

#ifndef SQL_DRIVER_SQL_TYPE_BASE
#define SQL_DRIVER_SQL_TYPE_BASE 0x4000
#endif


/* Internal type codes for NUMBER data types */
#define SQL_TD_FIXED_NUMBER                        18001 
#define SQL_TD_FLOATING_NUMBER                     18002

/* XML data type */
#define SQL_TD_XML                                 18003

/* JSON data type */
#define SQL_TD_JSON                                18004
#define SQL_TD_WJSON                               18005

#define SQL_TD_DATASET_AVRO                        18006 // ODBC-16550

/* UDT indicator */
#define SQL_TD_UDT                                 21000

/*
 * UDT Types
 *    Value - Type
 *    1  Structured UDT.
 *    2  Distinct UDT.
 *    3  Internal UDT.
 *    0 - For all other data types.
 */

#define SQL_TD_UDT_BASE                           0
#define SQL_TD_UDT_STRUCTURED                     1
#define SQL_TD_UDT_DISTINCT                       2
#define SQL_TD_UDT_INTERNAL                       3

#define SQL_TD_TYPE_MIN                            -1049
#define SQL_TD_TYPE_MAX                            -1045

/* Period data types */

#define SQL_PERIOD_DATE                            -1049
#define SQL_PERIOD_TIME                            -1048
#define SQL_PERIOD_TIME_WITH_TIME_ZONE             -1047
#define SQL_PERIOD_TIMESTAMP                       -1046
#define SQL_PERIOD_TIMESTAMP_WITH_TIME_ZONE        -1045


/* Maximum precision for Period time and timestamp types */

#define SQL_PERIOD_TIME_MAX_PRECISION                  6
#define SQL_PERIOD_TIMESTAMP_MAX_PRECISION             6

/* Lengths in characters of Period data type character representations */

#define SQL_PERIOD_DATE_LEN                           28
#define SQL_PERIOD_TIME_LEN                           24 /* Add 2*(p+1), if p>0 */
#define SQL_PERIOD_TIME_WITH_TIME_ZONE_LEN            36 /* Add 2*(p+1), if p>0 */
#define SQL_PERIOD_TIMESTAMP_LEN                      46 /* Add 2*(p+1), if p>0 */
#define SQL_PERIOD_TIMESTAMP_WITH_TIME_ZONE_LEN       58 /* Add 2*(p+1), if p>0 */

/* Lengths in bytes of Period data type binary representations */

#define SQL_PERIOD_DATE_OCTET_LEN                     8
#define SQL_PERIOD_TIME_OCTET_LEN                     12
#define SQL_PERIOD_TIME_WITH_TIME_ZONE_OCTET_LEN      16
#define SQL_PERIOD_TIMESTAMP_OCTET_LEN                20
#define SQL_PERIOD_TIMESTAMP_WITH_TIME_ZONE_OCTET_LEN 24

/* The scale factor for seconds fields in Period time and timestamp types. The
 * seconds fields are scaled integers. The seconds with fractional part are:
 *     Seconds = SecondsField / SQL_PERIOD_SECONDS_SCALE
 */

#define SQL_PERIOD_SECONDS_SCALE                 1000000

/* The zero base for time zone fields in Period time and timestamp types with time zone:
 *     TimeZoneField <  SQL_PERIOD_TZ_BASE: TimeZone =  - TimeZoneField.
 *     TimeZoneField >= SQL_PERIOD_TZ_BASE: TimeZone =   TimeZoneField - SQL_PERIOD_TZ_BASE.
 */

#define SQL_PERIOD_TZ_BASE                            16


#endif /* _TDODBC_TDSQL_H_ */

