/*****************************************************************************

Copyright 2008 - 2013 by Teradata Corporation.
All Rights Reserved.

Module:      adhoc.cpp

Author:      
             Teradata Corporation
             Teradata Warehouse Engineering
             ODBC

Created:     

Description: This is a sample program called adhoc.  It is an interactive
             tool for submitting SQL statements to Teradata.

Functions: 
 
Revision History: 
 
   Revision    Date       DR    Userid   Comments
   ----------- --------   ----- -------- -----------------------------------
   03.04.00.00 09/04/01   52740 NDFSODBC Ported to 3.51
   03.04.00.00 07/09/04   90788 cc151010 Remove default odbcadm pass/uid
   13.00.00.00 01/23/08         jk121225 Updates with copyright and header
   13.00.00.03 11/19/09  137636 sm185109 Use <iostream> instead of <iostream.h> on LINUX
  14.10d.00.00 10/02/12 ODBC-8138  tm255015 Add MacOS X support.
   14.10.00.xx 06/11/13 ODBC-11303 st255032 Fix compiler issues on Mac OS X and warnings.
*****************************************************************************/

#ifdef HP_UX
#define signed
#endif

#include <stdio.h>
#if defined(LINUX) || defined(MACOSX)
#include <iostream>
using namespace std;
#else
#include <iostream.h>
#endif

#if defined(INTERSOLV)
#if defined(NCR_MPRAS) || defined(LINUX)|| defined(HP_UX) || defined(AIXVER42) || defined(SOLARIS)
#include "sqlunx.h"
#else
#include "sqlunix.h"
#endif
#endif

#ifdef WIN32
#include <windows.h>
#endif

#include <sql.h>
#include <sqlext.h>
#include <string.h>
#include <stdlib.h>

#if !defined(WIN32) && !defined(LINUX) && !defined(MACOSX)
#include <sys/stream.h>
#endif

#define TRUE		1
#define FALSE		0

#define MAX_STR		255
#define MAXLINE		1000000
#define MAX_NAME_LEN	50
#define MAXCOLS		2048

#define MAXCONNECTIONS	100   //dr52740

extern int print_err(HENV,HDBC,HSTMT);
extern int getline(char*,const int);
extern SQLUINTEGER display_size(SQLSMALLINT, SQLULEN, SQLCHAR *);

//DR52182 - NDFSODBC
void printstring(char *);

int main(int argc, char **argv)
{

HENV	henv;
HDBC	hdbc;
HSTMT	hstmt;

int conns=0;		//dr52740

int	len;			/* length of returned line         */
char	line[MAXLINE];		/* returned line                   */
UCHAR	dsn[MAX_STR];		/* Data Source Name                */
UCHAR	pwd[MAX_STR];		/* password of user from odbc.ini  */
UCHAR	uid[MAX_STR];		/* name of user from odbc.ini      */
UCHAR	sqlstr[MAXLINE];	/* SQL String used for testing     */

RETCODE	rc;
int	i;
UCHAR	errmsg[256];
UCHAR	colname[32];
SWORD	coltype;
SWORD	colnamelen;
SWORD	nullable;
SQLULEN collen[MAXCOLS];
SWORD	scale;
SQLLEN  outlen[MAXCOLS];
UCHAR	*data[MAXCOLS];
SWORD	nresultcols;
SQLLEN  rowcount;
UCHAR	db[MAX_NAME_LEN + 1];
SWORD	dblen;

/*******************************************************************
 *  Get ODBC data source name; prompt for it if necessary          *
 *  Program assumes that all necessary data source information     *
 *  is in .odbc.ini file, and will prompt user for password.       *
 *  (Not that Informix driver does not require password, and       *
 *  prompt can be ingored.)                                        *
 *******************************************************************/
fflush(stdout);

if (argc == 1)                 /* if no Data Source Name entered  */
	{
	cout << "Enter Data Source Name[coop_0]: " << flush;
	len = getline(line, MAXLINE);

	if (!strcmp(line, "exit") || !strcmp(line, "quit"))
		return( 0 );
	
	if (len)
		strcpy((char *) dsn, line);
	else
		strcpy((char *) dsn, "coop_0");
	}
else
	{
	strcpy((char *) dsn, argv[1]);
	}

//DR90788 remove odbcadm prompt
cout <<"Enter userId: " << flush;
len = getline(line, MAXLINE);
	if (!strcmp(line, "exit") || !strcmp(line, "quit"))
		return( 0 );

if (len)
{
    strcpy((char *) uid, line);
}
else
{
    //DR90788 copy NULL if non provided
    strcpy((char *) uid, "");
}

//DR90788 remove odbcadm prompt
cout <<"Enter password: " << flush;
len = getline(line, MAXLINE);
	if (!strcmp(line, "exit") || !strcmp(line, "quit"))
		return( 0 );
	
if (len)
{
    strcpy((char *) pwd, line);
}
else
{
    //DR90788 copy NULL if non provided
    strcpy((char *) pwd, "");
}

/*******************************************************************
 *  Allocate environment and connection handles                    *
 *  Connect to the data source                                     *
 *  Allocate a statement handle                                    *
 *******************************************************************/

rc = SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&henv);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(SQL_NULL_HENV, SQL_NULL_HDBC, SQL_NULL_HSTMT));

rc = SQLSetEnvAttr(henv,SQL_ATTR_ODBC_VERSION,(void *)SQL_OV_ODBC3,4);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, SQL_NULL_HDBC, SQL_NULL_HSTMT));

rc = SQLAllocHandle(SQL_HANDLE_DBC,henv, &hdbc);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, SQL_NULL_HDBC, SQL_NULL_HSTMT));

rc = SQLConnect(hdbc, 
				dsn, strlen((const char *)dsn), 
				uid, strlen((const char *)uid), 
				pwd, strlen((const char *)pwd));

if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout << "\n...ODBC connection successful.\n" << endl;

rc = SQLGetInfo(hdbc, SQL_ODBC_VER, &db, (SWORD) sizeof(db), &dblen);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout <<"ODBC version        = -" << (char *)db << "-" << endl;

rc = SQLGetInfo(hdbc, SQL_DBMS_NAME, &db, (SWORD) sizeof(db), &dblen);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout <<"DBMS name           = -" << (char *)db << "-" << endl;

rc = SQLGetInfo(hdbc, SQL_DBMS_VER, &db, (SWORD) sizeof(db), &dblen);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout <<"DBMS version        = -" << (char *)db << "-" << endl;

rc = SQLGetInfo(hdbc, SQL_DRIVER_NAME, &db, (SWORD) sizeof(db), &dblen);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout <<"Driver name         = -" << (char *)db << "-" << endl;

rc = SQLGetInfo(hdbc, SQL_DRIVER_VER, &db, (SWORD) sizeof(db), &dblen);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout <<"Driver version      = -" << (char *)db << "-" << endl;

rc = SQLGetInfo(hdbc, SQL_DRIVER_ODBC_VER, &db, (SWORD) sizeof(db), &dblen);
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

cout <<"Driver ODBC version = -" << (char *)db << "-\n" << endl;

rc = SQLAllocHandle(SQL_HANDLE_STMT,hdbc, &hstmt);    
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));

while(1)
{


/*******************************************************************
 *  Get AD-HOC SQL query from user.                                *
 *******************************************************************/

cout <<"Enter SQL string: " << flush;
len = getline(line, MAXLINE);

if (!strcmp(line, "exit") || !strcmp(line, "quit"))
	break;

strcpy((char *) sqlstr, line);

/*******************************************************************
 *  Execute the SQL statement                                      *
 *******************************************************************/

rc = SQLExecDirect(hstmt, sqlstr, SQL_NTS);

if (rc != SQL_SUCCESS)
{
	print_err(henv, hdbc, hstmt);
	continue;
}

/*******************************************************************
 *  See what kind of statement it was.  If there are no result     *
 *  columns, the statement is not a SELECT statement. If the numbe * 
 *  of affected rows is greater than 0, the statement was probably *
 *  an UPDATE, INSERT or DELETE statement, so print thhe number of *
 *  affected rows.  If the number of affected rows is 0, the       *
 *  statement is probably a DDL statement, so print that the       *
 *  operation was successful and commit it.                        *
 *******************************************************************/
//DR52740
do 
{

rc = SQLNumResultCols(hstmt, &nresultcols);

if (nresultcols == 0)
{
	SQLRowCount(hstmt, &rowcount);

	if (rowcount > 0)
		cout << (unsigned long)rowcount << " rows affected." << endl;
	else
		cout <<"Operation was successful." << endl;

	SQLTransact(henv, hdbc, SQL_COMMIT);
}

/*******************************************************************
 *  Otherwise, display the column names of the result set and use  *
 *  the display_size() function to compute the length needed by    *
 *  each data type.  Next, bind the columns and specify all        *
 *  data will be converted to char.  Finally, fetch and print each *
 *  row, printing truncation messages as necessary.                *
 *******************************************************************/

else
	{
	for (i = 0; i < nresultcols; i++)
	{
		SQLDescribeCol(hstmt, i+1, colname, (SWORD) sizeof(colname),
			&colnamelen, &coltype, &collen[i], &scale, &nullable);
		collen[i] = display_size(coltype, collen[i], colname);
        switch(coltype)
        {
                case SQL_INTERVAL_YEAR:
                case SQL_INTERVAL_MONTH:
                case SQL_INTERVAL_DAY:
                case SQL_INTERVAL_HOUR:
                case SQL_INTERVAL_MINUTE:
                case SQL_INTERVAL_SECOND:
                case SQL_INTERVAL_YEAR_TO_MONTH:
                case SQL_INTERVAL_DAY_TO_HOUR:
                case SQL_INTERVAL_DAY_TO_MINUTE:
                case SQL_INTERVAL_DAY_TO_SECOND:
                case SQL_INTERVAL_HOUR_TO_MINUTE:
                case SQL_INTERVAL_HOUR_TO_SECOND:
                case SQL_INTERVAL_MINUTE_TO_SECOND:
	                data[i] = (UCHAR *) malloc((size_t)(sizeof(SQL_INTERVAL_STRUCT) + 1));
                        break;
                default:
		data[i] = (UCHAR *) malloc((size_t)(collen[i] + 1));
	}
                //DR52182 - NDFSODBC ->>
                //printf("%*.*s ",collen[i], collen[i], colname);
                printstring((char *)colname);
                //<<- DR52182 - NDFSODBC

	//	cout << (char *)collen[i] << " ";
	//	cout << (char *)collen[i] << " ";
	//	cout << (char *)colname << flush;

	switch(coltype)
	{
		case SQL_INTERVAL_YEAR:
		case SQL_INTERVAL_MONTH:				
		case SQL_INTERVAL_DAY:				
		case SQL_INTERVAL_HOUR:				
		case SQL_INTERVAL_MINUTE:			
		case SQL_INTERVAL_SECOND:               
		case SQL_INTERVAL_YEAR_TO_MONTH:		
		case SQL_INTERVAL_DAY_TO_HOUR:			 
		case SQL_INTERVAL_DAY_TO_MINUTE:		
		case SQL_INTERVAL_DAY_TO_SECOND:			
		case SQL_INTERVAL_HOUR_TO_MINUTE:			
		case SQL_INTERVAL_HOUR_TO_SECOND:			
		case SQL_INTERVAL_MINUTE_TO_SECOND:	
                        rc = SQLBindCol(hstmt, i+1, SQL_C_CHAR, data[i], sizeof(SQL_INTERVAL_STRUCT)+1,
                        &outlen[i]);
			break;
		default:
			rc = SQLBindCol(hstmt, i+1, SQL_C_CHAR, data[i], collen[i]+1, 
			&outlen[i]);
	}
	}
	cout << endl;

	while (TRUE)
	{
		rc = SQLFetch(hstmt);
		if (rc == SQL_NO_DATA_FOUND) break;
		if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO)
		{
			errmsg[0] = '\0';
			for (i = 0; i < nresultcols; i++)
			{
				if (outlen[i] == SQL_NULL_DATA)
					strcpy((char *)data[i], "NULL");
				else
				if (outlen[i] > collen[i])
				{
					sprintf((char *) &errmsg[strlen((char *) errmsg)],
							"%d chars truncated, col %d\n",
							(int) (outlen[i] - collen[i] + 1),
							i + 1);
				}                  /* end if outlen ...               */
                        //DR52182 - NDFSODBC ->>
                  //printf("%*.*s ", collen[i], collen[i], data[i]i);
                                printstring((char *)data[i]);
                                cout << " ";
                                //<<- DR52182 - NDFSODBC

			fflush(stdout);
			//	cout << (char *)collen[i] << " ";
			//	cout << (char *)collen[i] << " ";
			//	cout << (char *)data[i] << flush;
			}                      /* end for loop                    */
			cout << "\n" << (char *)&errmsg << endl;
		}                        /* end if rc == SQL_SUCCESS ...    */
		else
			/* return(print_err(henv,hdbc,hstmt)); */
			print_err(henv,hdbc,hstmt);
	}                          /* end while TRUE                  */
	}                            /* end else nresultcols != 0       */
 
/*******************************************************************
 *  Free the data buffers                                          *
 *******************************************************************/

for (i = 0; i < nresultcols; i++)
	{
	free(data[i]);
	}

cout << endl;

}while((rc = SQLMoreResults(hstmt))==0);

}   /* end while(1) */

rc = SQLFreeHandle(SQL_HANDLE_STMT,hstmt);  /* free the statement handle  */
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, hstmt));

rc = SQLDisconnect(hdbc);      /* Disconnect from the data source */
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));
 
SQLFreeHandle(SQL_HANDLE_DBC,hdbc);          /* Free the connection handle      */
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, hdbc, SQL_NULL_HSTMT));
 
SQLFreeHandle(SQL_HANDLE_ENV,henv);              /* Free the environment handle     */
if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
	return(print_err(henv, SQL_NULL_HDBC, SQL_NULL_HSTMT));
cout << "\nODBC connection closed." << endl;

return 0;
}

/**
 * Prints string to stdout
 *
 *
 * Fixes Documented : 
 *   Date         DR/RFC  Owner      Description
 *   -----------------------------------------------------------
 *   11/23/2000   52182   NDFSODBC   Created
 *  
 *  NOTES:
 */
void printstring(char *str)
{
 int i,len=strlen(str);

 for(i=0;i<len;i++)
 {
        if(*(str+i)=='\r')
                cout << "\n";
        else
            cout << *(str+i);
  }
}

