/*****************************************************************************

Copyright 2010-2015 by Teradata Corporation.
All Rights Reserved.

Module:      adhoc.c

Author:
             Teradata Corporation
             Teradata Warehouse Engineering
             ODBC

Created:

Description: This is a sample program called adhoc.  It is an interactive
             tool for submitting SQL statements to Teradata.

Functions:

Revision History:

   Revision    Date       DR/RFC  Userid   Comments
   ----------- --------   ------- -------- -----------------------------------
   03.00.00.00 06/11/01     52182 NDFSODBC Fixes for UNIX-Show-Table to output
                                            in the correct format
   13.00.00.00 01/23/08           jk121225 Updates with copyright and header
   13.00.00.00 07/27/09    134217 ff185017 Updates for Windows compatibility
   13.10f.00.00 11/11/09   137269 jk121225 Fixed regression for newline input and truncated column names
   13.10f.00.00 11/11/09    86872 jk121225 moved the display_size to adhoc.c, a more logic place.
   13.10f.00.00 11/11/09    86872 jk121225 fixed logic to handle all data type where target is SQL_C_CHAR.
   13.10f.00.00 11/11/09   114663 jk121225 Added SQL_WCHAR, SQL_WVARCHAR, and SQL_WLONGVARCHAR.
   13.10f.00.00 11/11/09   109900 jk121225 Adding a space between column names.
   13.10f.00.00 11/17/09    86872 jk121225 Applying code review changes for DR 86872, DR 109900, DR 114663, and DR 137269.
   13.10f.00.00 11/24/09   114663 jk121225 Applying fix where TIME and TIMESTAMP LITERAL are not at max
   13.10g.00.00 01/07/10   138429 kk250025 Changing dblen type to SQLSMALLINT in DisplayInfo()
   13.10.00.01  01/29/10   138569 kk250025 changes to display_size() to avoid errors for PERIOD datatypes.
   13.10.00.06  05/04/11   150430 jk121225 renamed getline function to kr_getline due to conflict.
                                           Should use getline version from O/S library. RFC candidate 
  14e.00.00.00  06/17/11   151802 jk121225 Don't display password to console when entered by user.
  14e.00.00.00  06/17/11   151802 jk121225 Don't do ECHO functions when not a terminal.
   14.10.00.00  10/22/12          jb518661 Adding display of more types for convenience.
                         ODBC-JIs
                         ---------
  14.10d.00.00  10/02/12 ODBC-8138  tm255015 Switch to using termios.h instead of termio.h
   14.10.00.xx  06/11/13 ODBC-11303 st255032 Fix compilation issues on Mac OS X and warnings.
   14.10.00.xx  05/01/14 12346    pk186005 Fixed issue on Mac OS X with xcode 5.0
   14.10.00.xx  05/01/14 11743    pk186005 Increased MAX_NAME_LEN value to accommodate EON.
   15.00.00.xx  08/01/14 ODBC-11790 CD185012 Fix - The help for tdxodbc is showing adhoc.
   15.10.00.00  10/07/14 ODBC-12443 js186097 Implemented diagnostic tool for adhoc and tdxodbc.
   15.10.00.00  12/30/14 ODBC-15153 js186097 Fixed re-compilation issue on MacOSX by changing
                                             #ifdef IODBC to #ifdef MACOSX before method
                                             GetPrivateProfileString_iODBC. Also fixed segmentation
                                             fault issue on Solaris-Sparc by changing value for
                                             "in_default" to an empty string instead of NULL. 

*****************************************************************************/



/*---------------------------------------------------------------------*/
/*--- SECTION: Required Headers ---------------------------------------*/


/* =========================== */
/*  - Standard Header Files -  */
/* =========================== */

#if defined(WIN32) || defined(_WIN64)
/*
 * windows.h has to be included before using any of
 * the ODBC related header files in WINDOWS
 */
#    include <windows.h>
#    include <stdio.h>
#    include <io.h>
#    include <strsafe.h>

/*
 * getopt.h provides the same functionality as
 * the UNIX getopt, for WINDOWS builds
 */
#    include "getopt.h"

    DWORD     fdwOldMode;
	
#else
/*
 * the function getopt is provided by the header
 * file unistd.h in UNIX builds
 */
#    include <unistd.h>
/*
 * stdlib.h has to be included before using any of
 * the ODBC related header files in UNIX
 */
#    include <stdlib.h>
#    include <termios.h>
#    include <errno.h>
#    include <dlfcn.h>
#endif

/*
 * sqlext.h defines SQL extensions for ODBC.  It in turn includes
 * sql.h, which includes sqltypes.h.    Hence, only the inclusion
 * of sqlext.h is necessary for compiling ODBC based programs.
 */
#include <sqlext.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <odbcinst.h>

#include <tdsql.h>  /* TTU 13.0*/



/* ============================================== */
/*  - Required Functions Without a Header File -  */
/* ============================================== */

extern int print_err(HENV henv,HDBC hdbc,HSTMT hstmt);

extern int kr_getline (char s[], int lim);


UDWORD display_size (
               SWORD coltype,
               UDWORD collen,
               UCHAR *colname);

void replace_password_with_asterisk(
             unsigned char *connstr_for_display);

/*
** DR 151802
*/
int SetTerminalEchoOFF();
int SetTerminalEchoON();


/*--- END: Required Headers -------------------------------------------*/
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*--- SECTION: Defines, consts, and static declarations ---------------*/

/* Minimum and maximum macros */
#if (!defined max)
#   define max(a,b)    (((a) > (b)) ? (a) : (b))
#endif
#if (!defined min)
#   define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif



/* ================================== */
/*  - Cross-Compiler compatibility -  */
/* ================================== */

/* definition of bool type for ANSI C compilers */
#ifndef __cplusplus
#ifndef bool
#   define bool    UCHAR
#endif
#ifndef false
#   define false   0
#endif
#ifndef true
#   define true    1
#endif
#endif



/* ============================== */
/*  - Defined Limits and Sizes -  */
/* ============================== */

#define MAX_STR         255
#define MAXLINE         1000000
#define MAX_NAME_LEN    512       //ODBC-11743
#define MAXCOLS         2048

#define SPACE_DELIMITER " "

/* used to calculate same-casts */
const SQLLEN MAX_SQLLEN_SIZE = (SQLLEN) (((SQLULEN) -1) >> 1);


#define MAX_NUM_PRECISION   15      /* Maximum mantissa's length        */

/*********************************************************************/
/*  Define max length of char string representation of number as:    */
/*    =  max(precision) + leading sign + E + exp sign + max exp leng */
/*    =  15             + 1            + 1 + 1        + 2            */
/*    =  15 + 5                                                      */
/*********************************************************************/
#define MAX_NUM_STRING_SIZE (MAX_NUM_PRECISION + 5)



/* ============================================================== */
/*  - Different NULL-casts to avoid warnings of some compilers -  */
/* ============================================================== */
#define CHAR_WIDTH  1
#define SIZE_OF_NULL_TERMINATOR     CHAR_WIDTH

#define NULL_PPUCHAR    (UCHAR * *) 0
#define NULL_PUCHAR     (UCHAR *) 0
#define NULL_PCHAR      (char *) 0
#define NULL_PSQLULEN   (SQLULEN *) 0
#define NULL_PSQLLEN    (SQLLEN *) 0

/* ODBC Connection String keywords */
#define ODBC_KEYWORD_DRIVER     "DRIVER="
#define ODBC_KEYWORD_SERVER     "DBCNAME="
#define ODBC_KEYWORD_USER       "UID="
#define ODBC_KEYWORD_PASSWORD   "PWD="
#define ODBC_KEYWORD_SEPARATOR  "; "
#ifdef WIN32
const char ODBC_INI[] = "ODBC.INI";
#else
const char ODBC_INI[] = ".odbc.ini"; // ODBC initialization file
#endif

/* Adhoc error strings */
#define ERROR_EOF "EOF"



/* =================================================== */
/*  - Command line adhoc options [used by getopt()] -  */
/* =================================================== */
static char const ADHOC_CMD_OPTION_LIST[]  =   "c:d:u:p:C:T:S:n:ht";



/* =====================================*/
/*  - Simple help/usage instructions -  */
/* =====================================*/
#define ADHOC_HELP_USAGE(help)       "\n\
USAGE:  %s\n\
    %s -c SQLConnect [-d <dsn>] [-u <user name>] [-p <password>]\n\
    %s -c SQLDriverConnect [ -C <connection string> | [[-S <server name>] [-u <user name>] [-p <password>] [-T <driver name>]] ] [-t [-n <num of connections>]] \n\
OPTIONS:\n\
    -c <connect type>       [SQLConnect | SQLDriverConnect]     default:SQLConnect\n\
    -d <dsn>\n\
    -u <user name>\n\
    -p <password>\n\
    -C <connection string>  example: -C \"DRIVER=Teradata; DBCNAME=MYCOP1; UID=dbc; PWD=dbc;\"\n\
    -T <driver name>        default: -T Teradata\n\
    -S <server name>        example: -S MYCOP1\n\
    -h                      Display usage\n\
    -t                      connect test only\n\
    -n <num of connections> applicable only with -t\n\
\n", help, help, help

#define TDXODBC	        "tdxodbc"
#define TDXODBC_WIN     "tdxodbc.exe"
#define ADHOC	        "adhoc"

/* ========================================= */
/*  - Selection of API to use for Connect -  */
/*   -      (see: adhoc_conntype_t)      -   */
/* ========================================= */
/* List of Adhoc's valid strings for the option -c */
/* NOTE:
    The index of this constant array of strings
    is named by the enum adhoc_conntype_t for
    the appropriate option */
static char const * const ADHOC_CONNTYPE_KEYWORDS[] =  {\
    "SQLConnect",           /* (adhoc -c SQLConnect) */\
    "SQLDriverConnect",     /* (adhoc -c SQLDriverConnect) */\
    NULL_PCHAR};



/* ========================================== */
/*  - Default DRIVER for connection string -  */
/* ========================================== */
static char const DEFAULT_DRIVER[]  =   "Teradata";



/* ====================================================== */
/*  - Adhoc MACRO for allocation/reallocation attempts -  */
/* ====================================================== */

/***************************************************************************\
|
|   Macro:      ALLOC_BUFFER(buffname, buffsize, bufftype, returnerror)
|   Purpose:    Attempt to realloc(buffname, ((buffsize)+1)*sizeof(bufftype))
|               It realloc() fails, prints an error message including
|               the line where the error occurred and returns 'returnerror'
|
|   USE: [returning something]
|           int foo()
|           {
|               char * buffer;
|               size_t len = 124;
|               ALLOC_BUFFER(buffer, len, char, -1);
|               free(buffer);
|               return 0;
|           }
|        [not returning anything]
|           void foo()
|           {
|               char * buffer;
|               size_t len = 124;
|               ALLOC_BUFFER(buffer, len, char, );
|               free(buffer);
|           }
|
\***************************************************************************/
#define ALLOC_BUFFER(buffname, buffsize, bufftype, returnerror) \
    if (NULL == (buffname = (bufftype *) realloc(buffname, ((buffsize) + SIZE_OF_NULL_TERMINATOR) * sizeof(bufftype)))) \
    { \
        printf("\nadhoc: %s=realloc(%ud) failed in line %d\n", #buffname, (unsigned int)(buffsize), __LINE__); \
        return returnerror; \
    } else memset(buffname, 0, ((buffsize) + 1) * sizeof(bufftype))


/*--- END: Defines, consts, and static declarations -------------------*/
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*--- SECTION: Types, Structs, Enums, and global variables ------------*/

/* ========================================= */
/*  - Selection of API to use for Connect -  */
/*   -   (see ADHOC_CONNTYPE_KEYWORDS)   -   */
/* ========================================= */
/* NOTE:
    The number represented by each of the following
    numeric IDENTIFIERS corresponds to the index of
    the array of strings ADHOC_CONNTYPE_KEYWORDS */
typedef enum
{
    ADHOC_CONNTYPE_SQLCONNECT,     /* i.e. use SQLConnect() */
    ADHOC_CONNTYPE_SQLDRIVERCONNECT       /* i.e. use SQLDriverConnect() */
} adhoc_conntype_t;



/* ================== */
/*  - Input/Output -  */
/* ================== */
bool interactive;               /* (default:true) user-input loop   */
char *  line;                   /* returned line buffer             */
size_t  len;                    /* length of returned line buffer   */


/* ===================== */
/*  - ODBC Parameters -  */
/* ===================== */
int odbc_connected;            /* count : successful connects */
/*
NOTE: It would be nice if we could check that we are connected first.
    in ODBC 3.5 SQL_ATTR_CONNECTION_DEAD could be a nice attribute to consider
    the current implementation with the flag 'odbc_connected', could still
    become outdated if a connection is lost
*/


adhoc_conntype_t useConnAPI;    /* used in the cmd option -c        */

/* NOTE: in UNIX, dsn mechanism happens through the "odbc.ini" file */
UCHAR * dsn;                    /* Data Source Name string          */
UCHAR * uid;                    /* User name string                 */
UCHAR * pwd;                    /* Password string                  */
UCHAR * connstr;                /* Connection string                */
UCHAR * connstr_without_pwd;    /* Connection string for display purposes */
UCHAR * driver;                 /* Driver string                    */
UCHAR * server;                 /* Server string (DBCNAME)          */

UCHAR * sqlstr;                 /* SQL String used for testing      */

UCHAR * name;                    /* Application Name string          	*/
UCHAR * token;                   /* Intermediate string                 */
UCHAR * token_last; 		 /* Last most string			*/
int  length;			 /* Length of string			*/

BOOL bConnectOnly;              /* Connect only test                */
int  iNumConnections;           /* num. of connections, valid only when connect only test flag set */



/* ===================================== */
/*  - Arguments in ODBC API Functions -  */
/* ===================================== */

UCHAR *     colname;
SWORD       coltype;
SWORD       colnamelen;
SWORD       nullable;
SQLULEN *   ColumnSizePtrArray;
SQLULEN *   ColumnBufferLengthArray;
SWORD       scale;
SQLLEN *    outlen;
UCHAR * *   data;
SWORD       nresultcols;


/*--- END: Types, Structs, Enums, and global variables ----------------*/
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*--- SECTION: Prototypes / Declarations ------------------------------*/

bool InitializeGlobalAdhocResources();

void FreeGlobalAdhocResources();

bool ProcessCmdLineOptions(int argc, char *argv[]);

bool CheckGetoptParameters();

bool GetMissingParameter(const char * promptmsg,
                         char * buffer,
                         size_t buffersize,
                         size_t * lengthreturned);
						 
bool ReadPrivateProfileString(char * tdata_path);

void GetPrivateProfileString_iODBC(LPCSTR in_sectionKey, 
                         LPCSTR in_entryKey, 
                         LPCSTR in_default, 
                         LPSTR io_buffer, 
                         unsigned int in_bufferSize, 
                         LPCSTR in_filename);

bool OpenLibrary(char * tdata_path, char * error_buffer);

bool RunDiagnosticTool();

int SetEnv(HENV * henv);

int AllocateConnectionHandles(HDBC **hdbc);

int SetDbc(HENV * henv, HDBC * hdbc);

int GetODBCInfo(HENV * henv, HDBC * hdbc);

void FreeODBCResources(HENV * henv, HDBC * hdbc, HSTMT * hstmt);

int DisplayInfo(HENV henv, HDBC hdbc, UWORD infoTag, const char * caption);

int ProcessSqlStmt(HENV * henv, HDBC hdbc, HSTMT * hstmt);

void printstring(char * str);


/*--- END: Prototypes / Declarations ----------------------------------*/
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*--- SECTION: ENTRY POINT main() -------------------------------------*/

/***************************************************************************\
|
|   Method:     main()
|
|   @param [in] argc – number of arguments from the command line call
|
|   @param [in] argv – array of null-terminated strings with each
|                      of the arguments from command line call
|
|       NOTE:   the argument "char **envp" is absent for better portability.
|
|   @return:    0 (SQL_SUCCESS) - AdHoc exits without error
|
|       NOTE:   non-zero value  - Not possible with current implementation,
|                                   but it should be addressed in a later
|                                   code review (find comments with #ERROR#)
|
\***************************************************************************/
int main(int argc, char *argv[])
{
    /*
    **  declaration of main()'s variables
    */
    HENV    henv;               /* sql environment handle           */
    HDBC*   hdbc;               /* sql connection handle            */
    HSTMT   hstmt;              /* sql statement handle             */

    RETCODE rc;                 /* to collect sql return codes      */



    /*
    **  initialization of main()'s variables
    */
    henv =  SQL_NULL_HENV;      /* not allocated                    */
    hstmt = SQL_NULL_HSTMT;     /* not allocated                    */

	

    /*
    **  initialization of adhoc's global variables
    */
    if (!InitializeGlobalAdhocResources())
    { /* error allocating memory */
        FreeGlobalAdhocResources();
        return -1;   /* #ERROR# adhoc error codes need to be revised */
    }



    /*
    **  Get all cmd line options provided using getopt()
    */
    if (!ProcessCmdLineOptions(argc, argv))
    { /* main() should quit adhoc, due to incorrect cmd line options
         or to errors allocating memory */
    length = strlen(argv[0]);
    ALLOC_BUFFER(name, length, UCHAR, false);
    ALLOC_BUFFER(token, length, UCHAR, false);
    ALLOC_BUFFER(token_last, length, UCHAR, false);
    strcpy((char *)name, argv[0]);
    token = (unsigned char *)strtok((char *)name,"/\\");
    while (token != NULL)
    {
        strcpy((char *)token_last, (char *)token);
        token = (unsigned char *)strtok (NULL, "/\\");
    }

    if (!strcmp((char *)token_last, TDXODBC) || !strcmp((char *)token_last, TDXODBC_WIN))
    {
	printf (ADHOC_HELP_USAGE(TDXODBC));
    }
    else
    {
	printf (ADHOC_HELP_USAGE(ADHOC));
    }

        FreeGlobalAdhocResources();
        return -1;   /* #ERROR# adhoc error codes need to be revised */
    }

    /*
    **  Get necessary ODBC parameters for adhoc to connect
    */
    if (!CheckGetoptParameters())
    { /* main() should quit adhoc, either due to errors
         or to a "quit adhoc" user request */
        FreeGlobalAdhocResources();
        return -1;   /* #ERROR# adhoc error codes need to be revised */
    }


    /*
    ** Initialize connection-handles
    */
    if(SQL_SUCCESS != AllocateConnectionHandles(&hdbc))
    {
        FreeGlobalAdhocResources();
        return -1;  /* #ERROR# adhoc error codes need to be revised */
    }


    /****************************************************************
     *  STAGE 1: Adhoc tries to connect                             *
     ****************************************************************/
    /*
    **  ODBC: Allocate environment and set environment attributes
    */
    rc = SetEnv(&henv);


    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    {
        FreeODBCResources(&henv, hdbc, &hstmt);
        FreeGlobalAdhocResources();
        return rc;  /* Return first API ODBC error */
    }

    /*
    **  ODBC: Allocate connection and connect
    */
    rc = SetDbc(&henv, hdbc);

    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    {
        FreeODBCResources(&henv, hdbc, &hstmt);
        FreeGlobalAdhocResources();
        return rc;  /* Return first API ODBC error */
    }

    /*
    **  ODBC: Display connection info
    */
    rc = GetODBCInfo(&henv, hdbc);

    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    {
        FreeODBCResources(&henv, hdbc, &hstmt);
        FreeGlobalAdhocResources();
        return rc;  /* Return first API ODBC error */
    }

    if(false == bConnectOnly)
    {
        /*
        **  ODBC: Allocate statement
        */
        rc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc[0], &hstmt);

        if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        {
            /* Display diagnostics */
            print_err(henv, hdbc[0], SQL_NULL_HSTMT);

            /* clean up and exit */
            FreeODBCResources(&henv, hdbc, &hstmt);
            FreeGlobalAdhocResources();
            return rc;  /* Return first API ODBC error */
        }

		
        /****************************************************************
         *  STAGE 2: Adhoc's main loop  (interactive mode)              *
         ****************************************************************/

        printf("\n(type quit to terminate adhoc) ");

        while(interactive)
        {

            /*
            **  Get AD-HOC SQL query from user
            */
            printf("\nEnter SQL string : ");
            len = kr_getline(line, MAXLINE);


            if (-1 == len)
            { /* EOF of a text file redirected into stdin */
                printf("\nEOF detected\n");
                break;
            } /* IF: EOF*/

            if (0 == len)
                continue;   /* do not process empty as SQL-Statement */

            if (!strcmp(line, "exit") || !strcmp(line, "quit"))
            { /* terminate adhoc command */
                printf("\n'%s' command detected\n", line);
                break;
            } /* IF: terminate adhoc command */


            /*
            **  Process the SQL statement
            */
            strcpy((char *) sqlstr, line);

            rc = ProcessSqlStmt(&henv, hdbc[0], &hstmt);

        } /* WHILE: (interactive) */
    }

    /****************************************************************
     *  STAGE 3: Disconnect, Clean up, and Exit                     *
     ****************************************************************/

    /* Disconnects if necessary, and frees all ODBC handles that are not NULL   */
    FreeODBCResources(&henv, hdbc, &hstmt);

    /* clean up and exit */
    FreeGlobalAdhocResources();

    return rc;  /* #ERROR# return adhoc error code */

} /* main() */

/*--- END: ENTRY POINT main() -----------------------------------------*/
/*---------------------------------------------------------------------*/





/*---------------------------------------------------------------------*/
/*--- SECTION: Implementation of function prototypes ------------------*/

/***************************************************************************\
|
|   Method:     InitializeGlobalAdhocResources()
|   Purpose:    Initialize global variables and allocate dynamic memory
|
|   @return     true if all malloc() were successful
|               false at least one malloc() failed
|       POST:   All global variables will be initially set to NULL, to
|               guarantee NULL pointers for any unallocated location,
|               and allow FreeGlobalAdhocResources() to free ONLY the
|               memory successfully allocated
|
\***************************************************************************/
bool InitializeGlobalAdhocResources()
{
    interactive = true;         /* (default:true) user-input loop   */
    line =  NULL_PCHAR;
    len =   0;


    odbc_connected = 0;

    useConnAPI = ADHOC_CONNTYPE_SQLCONNECT; /* ONLY an initial value.   The
                                                default value is a function
                                                of adhoc's cmd line options,
                                                and it is determined in
                                                ProcessCmdLineOptions() */


    /* Clear all values and NULL all pointer */
    dsn =   NULL_PUCHAR;
    pwd =   NULL_PUCHAR;
    uid =   NULL_PUCHAR;
    connstr =   NULL_PUCHAR;
    connstr_without_pwd =   NULL_PUCHAR;
    driver =    NULL_PUCHAR;
    server =    NULL_PUCHAR;

    sqlstr =    NULL_PUCHAR;
    
    name =    NULL_PUCHAR;
    token =    NULL_PUCHAR;
    token_last =    NULL_PUCHAR;
    length =    0;

    colname =       NULL_PUCHAR;
    coltype =       0;
    colnamelen =    0;
    nullable =      0;
    ColumnSizePtrArray =    NULL_PSQLULEN;
    ColumnBufferLengthArray =    NULL_PSQLULEN;
    scale =         0;
    outlen =    NULL_PSQLLEN;
    data =      NULL_PPUCHAR;
    nresultcols =   0;


    /* Try to allocate memory for all global buffers */
    ALLOC_BUFFER (line, MAXLINE, char, false);

    /* NOTE: functions like ProcessCmdLineOptions() or
            CheckGetoptParameters() will allocate (as needed) the
            strings dsn, uid, pwd, connstr, connstr_without_pwd, driver, server,
            and file */

    ALLOC_BUFFER(sqlstr, MAXLINE, UCHAR, false);

    ALLOC_BUFFER(colname, MAX_NAME_LEN, UCHAR, false);
    ALLOC_BUFFER(ColumnSizePtrArray, MAXCOLS, SQLULEN, false);
    ALLOC_BUFFER(ColumnBufferLengthArray, MAXCOLS, SQLULEN, false);
    ALLOC_BUFFER(outlen, MAXCOLS, SQLLEN, false);
    ALLOC_BUFFER(data, MAXCOLS, UCHAR *, false);


    /* return with no errors */
    return true;

} /* InitializeGlobalAdhocResources() */



/***************************************************************************\
|
|   Method:     FreeGlobalAdhocResources()
|   Purpose:    Free all global variables successfully allocated
|               by InitializeGlobalAdhocResources()
|       PRE:    Any realloc() or free() performed on these global
|               variables must ensure they point to NULL when memory
|               is no longer allocated.
|               Otherwise FreeGlobalAdhocResources() will crash.
|
\***************************************************************************/
void FreeGlobalAdhocResources()
{
    int i;                      /* index used in for-loops          */


    if (NULL_PCHAR != line)         { free(line);       line = NULL_PCHAR;      }

    if (NULL_PUCHAR != dsn)         { free(dsn);        dsn = NULL_PUCHAR;      }
    if (NULL_PUCHAR != uid)         { free(uid);        uid = NULL_PUCHAR;      }
    if (NULL_PUCHAR != pwd)         { free(pwd);        pwd = NULL_PUCHAR;      }
    if (NULL_PUCHAR != connstr)     { free(connstr);    connstr = NULL_PUCHAR;  }
    if (NULL_PUCHAR != connstr_without_pwd)     { free(connstr_without_pwd);    connstr_without_pwd = NULL_PUCHAR;  }
    if (NULL_PUCHAR != driver)      { free(driver);     driver = NULL_PUCHAR;   }
    if (NULL_PUCHAR != server)      { free(server);     server = NULL_PUCHAR;   }

    if (NULL_PUCHAR != sqlstr)      { free(sqlstr);     sqlstr = NULL_PUCHAR;   }

    if (NULL_PUCHAR != colname)     { free(colname);    colname = NULL_PUCHAR;  }
    if (NULL_PSQLULEN != ColumnSizePtrArray)    { free(ColumnSizePtrArray);     ColumnSizePtrArray = NULL_PSQLULEN; }
    if (NULL_PSQLULEN != ColumnBufferLengthArray)    { free(ColumnBufferLengthArray);     ColumnBufferLengthArray = NULL_PSQLULEN; }
    if (NULL_PSQLLEN != outlen)     { free(outlen);     outlen = NULL_PSQLLEN;  }
    if (NULL_PUCHAR != name)        { free(name);       name = NULL_PUCHAR;  }
    if (NULL_PUCHAR != token)       { free(token);      token = NULL_PUCHAR;  }
    if (NULL_PUCHAR != token_last)  { free(token_last); token_last = NULL_PUCHAR;  }

    if (NULL_PPUCHAR != data)
    {
        for (i = 0; i < MAXCOLS; i++)
        {
            if (NULL_PUCHAR != data[i]) { free(data[i]); data[i] = NULL_PUCHAR; }
        }
        free(data); data = NULL_PPUCHAR;
    }

} /* FreeGlobalAdhocResources() */


/***************************************************************************\
|
|   Method:     ProcessCmdLineOptions()
|   Purpose:    Use getopt() to get all the options provided
|               in adhoc's command line call.   This function
|               will also set general adhoc defaults.
|
|   @param [in] argc – from main(): number of arguments from the cmd line call
|
|   @param [in] argv – from main(): array of null-terminated strings with each
|                      of the arguments from command line call
|
|   @return:    true    - if all options were valid
|               false   - otherwise (main() should quit adhoc)
|
\***************************************************************************/

bool ProcessCmdLineOptions(int argc, char *argv[])
{
    int option;                 /* used for getopt() return codes   */
    size_t lenoptarg;           /* strlen(optarg)                   */
    int count;                  /* number of options processed      */
    bool found;                 /* used to search for keywords      */
    bool conntype_explicit;     /* true if option -c was found      */
    int i;                      /* used as index in loops           */

    conntype_explicit = false;
    found = false;
    count = 0;
    i = 0;

    /* get arguments with getopt(): (return -1 if no more arguments) */
    while (-1 != (option = getopt(argc, argv, ADHOC_CMD_OPTION_LIST)))
    { /* <while> all the arguments in the command line */

        count++;

        /* select which argument was detected */
        switch (option)
        {
        case 'c' : /* <connection type> */
            conntype_explicit = true;

            found = false;
            i = 0;
            while (NULL_PCHAR != ADHOC_CONNTYPE_KEYWORDS[i])
            { /* search for correct keyword as argument */
                if (!(found = (0 == strcmp(optarg, ADHOC_CONNTYPE_KEYWORDS[i]))))
                    i++;
                else
                    break;
            }

            if (!found)
            {
                printf("\nadhoc: command line option #%d (-c) had an invalid argument.   Valid arguments are {", count);
                i = 0;
                while (NULL_PCHAR != ADHOC_CONNTYPE_KEYWORDS[i])
                    printf("%s%s", ((0==i) ? "" : ", "), ADHOC_CONNTYPE_KEYWORDS[i++]);
                printf("}.   Failed in line %d\n", __LINE__);
                return false;
            }

            useConnAPI = (adhoc_conntype_t) i;

            break;

        case 'd' : /* <dsn> */
            lenoptarg = strlen(optarg);
            ALLOC_BUFFER(dsn, lenoptarg, UCHAR, false);

            strcpy((char *) dsn, optarg);
            break;

        case 'u' : /* <user> */
            lenoptarg = strlen(optarg);
            ALLOC_BUFFER(uid, lenoptarg, UCHAR, false);

            strcpy((char *) uid, optarg);
            break;

        case 'p' : /* <password> */
            lenoptarg = strlen(optarg);
            ALLOC_BUFFER(pwd, lenoptarg, UCHAR, false);

            strcpy((char *) pwd, optarg);
            break;

        case 'C' : /* <connect string>    (e.g. "DRIVER=) */
            lenoptarg = strlen(optarg);
            ALLOC_BUFFER(connstr, lenoptarg, UCHAR, false);
            ALLOC_BUFFER(connstr_without_pwd, lenoptarg, UCHAR, false);

            strcpy((char *) connstr, optarg);
            strcpy((char *) connstr_without_pwd, optarg);
            break;

        case 'T' : /* <driver> */
            lenoptarg = strlen(optarg);
            ALLOC_BUFFER(driver, lenoptarg, UCHAR, false);

            strcpy((char *) driver, optarg);
            break;

        case 'S' : /* <server> */
            lenoptarg = strlen(optarg);
            ALLOC_BUFFER(server, lenoptarg, UCHAR, false);

            strcpy((char *) server, optarg);
            break;

        case 't' : /* <connection-test only> */
            bConnectOnly = true;
            break;

        case 'n' : /* <number of connections> */
            iNumConnections = atoi(optarg);
            break;

        case 'h' : /* simply return false. main() will print the message help/usage */
            return false;

        default: /* not a valid option */
            /* Adhoc should terminate */
            printf("\nadhoc: command line option #%d was invalid.   Valid arguments are {", count);
            i = 0;
            while (0 != ADHOC_CMD_OPTION_LIST[i])
            {
                if (':' != ADHOC_CMD_OPTION_LIST[i])
                    printf("%s%c", ((0==i) ? "" : ", "), ADHOC_CMD_OPTION_LIST[i]);

                i++;
            }
            printf("}.   Failed in line %d\n", __LINE__);
            return false;

        }

    } /* <while> all the arguments in the command line */



    /* set default connection type */
    if (!conntype_explicit)
    {

        /* first index of ADHOC_CONNTYPE_KEYWORDS[] by default */
        useConnAPI = ADHOC_CONNTYPE_SQLCONNECT;

        /* if "-d <dsn>" was NOT provided, but
            either "-C <connection string>" or
            or "-S <server>" was provided,
            then change the default to ADHOC_CONNTYPE_SQLDRIVERCONNECT */
        if ( ((NULL_PUCHAR == dsn) || (((UCHAR) 0) == dsn[0])) &&
             ( ((NULL_PUCHAR != connstr) && (((UCHAR) 0) != connstr[0])) ||
               ((NULL_PUCHAR != server) && (((UCHAR) 0) != server[0]))
             )
           )
        {
           useConnAPI = ADHOC_CONNTYPE_SQLDRIVERCONNECT;
        }

    }



    /* Set default DRIVER */
    if ((NULL_PUCHAR == driver) || (((UCHAR) 0) == driver[0]))
    {
        len = strlen(DEFAULT_DRIVER);
        ALLOC_BUFFER(driver, len, UCHAR, false);

        strcpy((char *) driver, DEFAULT_DRIVER);
    }


    return true;

} /* ProcessCmdLineOptions() */



/***************************************************************************\
|
|   Method:     CheckGetoptParameters()
|   Purpose:    Get ODBC parameters that were not provided on
|               the cmd line, and could not be defaulted to a
|               reasonable value, by PROMPTING the user
|
|   List of parameters required:
|       (1) For adhoc's connection type = ADHOC_CONNTYPE_SQLCONNECT
|               -dsn
|               -user
|               -password
|       (2) For adhoc's connection type = ADHOC_CONNTYPE_SQLDRIVERCONNECT
|           Either:
|           2.1)-connection string
|           Or:
|           2.2)-driver
|               -server
|               -user
|               -password
|             NOTE: if a connection string is not provided, this
|               function will generate one based on driver, server,
|               user, and password, because ODBC requires a connection
|               string to connect using SQLDriverConnect()
|
|   @return:    true    - if all tasks successful and user input was
|                           not identified as a request for adhoc to quit
|               false   - otherwise (main() should quit adhoc)
|
|   Remarks:
|   Requirements:
|   See Also:
|  
|   Fixes Documented :
|     Date         DR/RFC  Owner      Description
|     -----------------------------------------------------------
|     06/22/2011   151802  jk121225   Add logic to disable password ECHO to console
|
\***************************************************************************/

bool CheckGetoptParameters()
{
	int rc;
    char * connstrFormat; /* format of connstr when it is built by adhoc */
    connstrFormat = NULL_PCHAR;



    printf("\n");

    /* select which argument was detected */
    switch (useConnAPI)
    {
    case ADHOC_CONNTYPE_SQLDRIVERCONNECT :

        /* Check connection string */
        if ((NULL_PUCHAR == connstr) || (((UCHAR) 0) == connstr[0]))
        { /* IF: connection string is not provided */


            /* # Step1: Get missing Keyword+Value pairs
                        to build the connection string # */

            /* Check server */
            if (((NULL_PUCHAR == connstr) || (((UCHAR) 0) == connstr[0])) &&
                ((NULL_PUCHAR == server) || (((UCHAR) 0) == server[0])))
            {
                if (!GetMissingParameter("Enter Server Name: ", line, MAXLINE, &len))
                {
                    printf("\n\nadhoc: %s ...couldn't get [Server Name] in line %d\n",
                        line, __LINE__);
                    return false;
                }

                if (!strcmp(line, "exit") || !strcmp(line, "quit"))
                    return false;

                ALLOC_BUFFER(server, len, UCHAR, false);
                strcpy((char *) server, line);
            }


            /* Check user */
            if ((NULL_PUCHAR == uid) || (((UCHAR) 0) == uid[0]))
            {
                if (!GetMissingParameter("Enter UserID: ", line, MAXLINE, &len))
                {
                    printf("\n\nadhoc: %s ...couldn't get [UserID] in line %d\n",
                        line, __LINE__);
                    return false;
                }

                if (!strcmp(line, "exit") || !strcmp(line, "quit"))
                    return false;

                ALLOC_BUFFER(uid, len, UCHAR, false);
                strcpy((char *) uid, line);
            }


            /* Check password */
            if ((0 == pwd) || (0 == pwd[0]))
            {
                /*
                ** Turn OFF terminal ECHO so users can't see the typed password
                */
                rc = SetTerminalEchoOFF();
                if ( rc > 1 )
                {
                    printf("\n\nadhoc: Unable to turn off ECHO to terminal [%d] in line %d\n",
                        rc, __LINE__);
                    return false;
                }

                if (!GetMissingParameter("Enter Password: ", line, MAXLINE, &len))
                {
                    printf("\n\nadhoc: ...couldn't get [Data Password] in line %d\n",
                        __LINE__);

                    rc = SetTerminalEchoON();
                    if ( rc > 1 )
                    {
                        printf("\n\nadhoc: Unable to turn ON ECHO to terminal [%d] in line %d\n",
                            rc, __LINE__);
                    }
                    return false;
                }

                /*
                ** Turn ON terminal ECHO so users can't see the typed password
                */
                rc = SetTerminalEchoON();
                if ( rc > 1 )
                {
                    printf("\n\nadhoc: Unable to turn ON ECHO to terminal [%d] in line %d\n",
                        rc, __LINE__);
                    return false;
                }

                /* newline was suppressed when ECHO OFF was in effect, so add one now. */
                printf("\n");

                if (!strcmp(line, "exit") || !strcmp(line, "quit"))
                    return false;

                ALLOC_BUFFER(pwd, len, UCHAR, false);
                strcpy((char *) pwd, line);
            }

            /* # Step2: Build the connection string
                        from Driver+Server+User+Password # */

            /* DRIVER=<driver>;     */
            len = strlen((char *) ODBC_KEYWORD_DRIVER) +
                strlen((char *) driver) +
                strlen((char *) ODBC_KEYWORD_SEPARATOR);

            /* DBCNAME=<server>;    */
            len += strlen((char *) ODBC_KEYWORD_SERVER) +
                strlen((char *) server) +
                strlen((char *) ODBC_KEYWORD_SEPARATOR);

            /* UID=<uid>;           */
            len += strlen((char *) ODBC_KEYWORD_USER) +
                strlen((char *) uid) +
                strlen((char *) ODBC_KEYWORD_SEPARATOR);

            /* PWD=<pwd>;           */
            len += strlen((char *) ODBC_KEYWORD_PASSWORD) +
                strlen((char *) pwd) +
                strlen((char *) ODBC_KEYWORD_SEPARATOR);

            len += SIZE_OF_NULL_TERMINATOR;

            ALLOC_BUFFER(connstrFormat, len, char, false);
            sprintf(connstrFormat, "%s%s%s%s%s%s%s%s%s%s%s%s",
                ODBC_KEYWORD_DRIVER,   "%s", ODBC_KEYWORD_SEPARATOR,
                ODBC_KEYWORD_SERVER,   "%s", ODBC_KEYWORD_SEPARATOR,
                ODBC_KEYWORD_USER,     "%s", ODBC_KEYWORD_SEPARATOR,
                ODBC_KEYWORD_PASSWORD, "%s", ODBC_KEYWORD_SEPARATOR);

            ALLOC_BUFFER(connstr, len, UCHAR, false);
            sprintf((char *) connstr, connstrFormat,
                (char *) driver, (char *) server, (char *) uid, (char *) pwd);

        } /* IF: connection string is not provided */


        break;

    default : /* ADHOC_CONNTYPE_SQLCONNECT */

        /* Check dsn */
        if ((NULL_PUCHAR == dsn) || (((UCHAR) 0) == dsn[0]))
        {
            if (!GetMissingParameter("Enter Data Source Name: ", line, MAXLINE, &len))
            {
                printf("\n\nadhoc: %s ...couldn't get [Data Source Name] in line %d\n",
                    line, __LINE__);
                return false;
            }

            if (!strcmp(line, "exit") || !strcmp(line, "quit"))
                return false;

            ALLOC_BUFFER(dsn, len, UCHAR, false);
            strcpy((char *) dsn, line);
        }


        /* Check user */
        if ((NULL_PUCHAR == uid) || (((UCHAR) 0) == uid[0]))
        {
            if (!GetMissingParameter("Enter UserID: ", line, MAXLINE, &len))
            {
                printf("\n\nadhoc: %s ...couldn't get [UserID] in line %d\n",
                    line, __LINE__);
                return false;
            }

            if (!strcmp(line, "exit") || !strcmp(line, "quit"))
                return false;

            ALLOC_BUFFER(uid, len, UCHAR, false);
            strcpy((char *) uid, line);
        }


        /* Check password */
        if ((0 == pwd) || (0 == pwd[0]))
        {
            /*
            ** Turn OFF terminal ECHO so users can't see the typed password
            */
            rc = SetTerminalEchoOFF();
            if ( rc > 1 )
            {
                printf("\n\nadhoc: Unable to turn off ECHO to terminal [%d] in line %d\n",
                    rc, __LINE__);
                return false;
            }

            if (!GetMissingParameter("Enter Password: ", line, MAXLINE, &len))
            {
                rc = SetTerminalEchoON();
                if ( rc > 1 )
                {
                    printf("\n\nadhoc: Unable to turn ON ECHO to terminal [%d] in line %d\n",
                        rc, __LINE__);
                }

                printf("\n\nadhoc: ...couldn't get [Data Password] in line %d\n",
                    __LINE__);
                return false;
            }

            /*
            ** Turn ON terminal ECHO so users can't see the typed password
            */
            rc = SetTerminalEchoON();
            if ( rc > 1 )
            {
                printf("\n\nadhoc: Unable to turn ON ECHO to terminal [%d] in line %d\n",
                    rc, __LINE__);
                return false;
            }

            /* newline was suppressed when ECHO OFF was in effect, so add one now */
            printf("\n");

            if (!strcmp(line, "exit") || !strcmp(line, "quit"))
                return false;

            ALLOC_BUFFER(pwd, len, UCHAR, false);
            strcpy((char *) pwd, line);
        }

        break;

    } /* SWITCH: (useConnAPI) */


    if (NULL_PCHAR != connstrFormat)    { free(connstrFormat);  connstrFormat = NULL_PCHAR; }

    if(false == bConnectOnly || iNumConnections <=0 || iNumConnections > 1000)
    {
        iNumConnections = 1;
    }

    return true;

} /* CheckGetoptParameters() */



/***************************************************************************\
|
|   Method:     GetMissingParameter()
|   Purpose:    Display 'promptmsg', and get user input.   empty lines are
|               considered acceptable input.
|               If successful, user input will be copied to 'buffer'
|               If error, a descriptive message will be copied to 'buffer'
|
|   @param [in] promptmsg       - pointer to a NULL terminated string with
|                                   a message to display before geting stdin
|
|   @param [out] buffer         - pointer to a previously allocated buffer to
|                                   get the returned string, which can be
|                                   either the input from stdin (if successful)
|                                   or a string describing the error (if error)
|                   IMPORTANT: buffer is not reallocated.  If 'buffersize' is
|                               not big enough to store a string it could
|                               crop the string or be empty altogether
|
|   @param [in] buffersize      - size of 'buffer'
|
|   @param [in] lengthreturned  - length of string stored in 'buffer'
|
|   @return:    true if successful ('buffer' will be non-empty user input)
|               false if error ('buffer' will be a descriptive error string,
|                               if buffersize is big enough for such string)
|
\***************************************************************************/
bool GetMissingParameter(const char * promptmsg,
                         char * buffer,
                         size_t buffersize,
                         size_t * lengthreturned)
{
    (*lengthreturned) = 0;

    memset(buffer, 0, buffersize);

    if (NULL_PCHAR != promptmsg)
        printf("%s", promptmsg);

    /* accept empty line for backward compatibility */
    *lengthreturned = (size_t) kr_getline(buffer, MAXLINE);

    if (-1 == (*lengthreturned))
    { /* EOF of a text file redirected into stdin */
        (*lengthreturned) = strlen(ERROR_EOF);
        if (buffersize > (*lengthreturned))
            strcpy(buffer, ERROR_EOF); /* Descriptive message for the failure */
        else
            (*lengthreturned) = 0;

        return false;
    } /* IF: EOF*/

    return true;

} /* GetMissingParameter() */

/***************************************************************************\
|
|   Method:     ReadPrivateProfileString()
|   Purpose:    Wrapper method for SQLGetPrivateProfileString() and
|               GetPrivateProfileString_iODBC(). This method attempts to 
|               retrieve the installation path of the TDODBC driver.
|
|   @param [out] tdata_path       - String containing the installation path
|                                   of the TDODBC driver. 
|
|   @return:    true if the directory path to the tdata.xx library has been successfully found
|               false if the directory path to tdata.xx has not been found
|
\***************************************************************************/
bool ReadPrivateProfileString(char * tdata_path)
{
    char in_default[8];
    in_default[0] = '\0'; 

    #if !defined(MACOSX)
        SQLGetPrivateProfileString(
            (const char*) dsn,
            "Driver",
            in_default,
            tdata_path,
            MAX_STR + SIZE_OF_NULL_TERMINATOR,
            ODBC_INI );
		
    #else
        GetPrivateProfileString_iODBC(
            (const char*) dsn,
            "Driver",
            in_default,
            tdata_path,
            MAX_STR + SIZE_OF_NULL_TERMINATOR,
            ODBC_INI );
    #endif
	
    if(strstr(tdata_path, "tdata") == NULL){ 
        printf("No driver entry found under the DSN : %s\n", dsn);
        return false; 
    }
  
    return true;
	
} /* ReadPrivateProfileString() */

/***************************************************************************\
|
|   Method:     GetPrivateProfileString_iODBC()
|   Purpose:    The iODBC driver manager has a bug in SQLGetPrivateProfileString in how it retrieves values
|               from the user and system odbc.ini.
|               When using iODBC, this function should replace any call to SQLGetPrivateProfileString, 
|               as it is a wrapper with additional code to try to retrieve the correct result.  See ODBC-11322.
|
|               The expected fallback behaviour for SQLGetPrivateProfileString when mode is ODBC_BOTH_DSN is:
|               1. If section and key are present in user odbc.ini, return key value
|               2. If either section and/or key are missing in user file, check and return value from system odbc.ini
|               3. If not present in system file, return default value.
|
|              iODBC has the following bugs:
|               1. If the value wasn't found in the user file but the default is a non-empty string, it will return the default without checking the system file.
|               2. If the value existed as an empty string in the user file, iODBC will fall back to checking and returning values from the system file.
|               3. Expanding on issue 2, if the value was present but empty in the user file and not present in the system file, the default will be returned 
|                  (empty or not) ignoring the empty value in the user file.

|   @param [in] LPCSTR in_sectionKey
|   @param [in] LPCSTR in_entryKey
|   @param [in] LLPCSTR in_default
|   @param [in] LPSTR io_buffer
|   @param [in] unsigned int in_bufferSize
|   @param [in] LPCSTR in_filename, default is ODBC_INI
|
|   @param [out] tdata_path       - String containing the installation path
|                                   of the TDODBC driver. 
|   See Also:
|     ODBC-11322 - iODBC SQLGetPrivateProfileString bug.
|
\***************************************************************************/
#ifdef MACOSX 
    void GetPrivateProfileString_iODBC(LPCSTR in_sectionKey, LPCSTR in_entryKey, LPCSTR in_default, LPSTR io_buffer, unsigned int in_bufferSize, LPCSTR in_filename)
    {
        int result = 0;

        /* iODBC has a broken fallback implementation between user and system
           dsn searches when the config mode is set to ODBC_BOTH_DSN. ODBC-11328 */
        UWORD configMode;
        SQLGetConfigMode(&configMode);
        if (ODBC_BOTH_DSN != configMode)
        {
            /* Use the function normally since there is no fallback needed. */
            result = SQLGetPrivateProfileString(
            in_sectionKey,
            in_entryKey,
            in_default,
            io_buffer,
            in_bufferSize,
            in_filename );
        }
        else
        {
            /* Implement our own fallback using a kludge default value to test
               the difference between missing and empty values.
               Use the values from ValidateDataSourceName. */
            int kludgeDefault = 0x00ffffff;
            SQLSetConfigMode(ODBC_USER_DSN);
            result = SQLGetPrivateProfileString(
                in_sectionKey,
                in_entryKey,
                (const char*)&kludgeDefault,
                io_buffer,
                in_bufferSize,
                in_filename );

            if ( 0 == strcmp(io_buffer, (const char*)&kludgeDefault) ){
                /* Fallback to checking system dsn. */
                SQLSetConfigMode(ODBC_SYSTEM_DSN);
                result = SQLGetPrivateProfileString(
                    in_sectionKey,
                    in_entryKey,
                    (const char*)&kludgeDefault,
                    io_buffer,
                    in_bufferSize,
                    in_filename );
                
                if ( 0 == strcmp(io_buffer, (const char*)&kludgeDefault) ){
                    /* Still got the default. Use the real default. */
                    strcpy(io_buffer, in_default);
                }
            }
            /* Restore the old config mode. */
            SQLSetConfigMode(ODBC_BOTH_DSN);
        }
		
    } /* GetPrivateProfileString_iODBC() */
#endif

/***************************************************************************\
|
|   Method:     OpenLibrary()
|   Purpose:    Wrapper method for dlopen() / LoadLibraryEx() logic. This method
|               is called by RunDiagnosticTool() to perform a platform independent 
|               load of the tdata.xx library.
|
|   @param [in] tdata_path        - Pointer to memory containing the installation
|                                   path of the TDODBC driver. 
|
|   @param [out] error_buffer     - String containing error message from dlopen()
|                                   or LoadLibraryEx() if there is one. 
|
|   @return:    true if the library was successfully loaded
|               false if the load was unsuccessful
|
\***************************************************************************/
bool OpenLibrary(char * tdata_path, char * error_buffer)
{
    #if defined(WIN32) || defined(_WIN64)
        HMODULE windll; /* HMODULE to use LoadLibraryEx() */      
	
    #else 
        void * handle; /* Handle to use dlopen() */
		
    #endif
	
    bool open_error; /* used to determine whether or not there was an error opening the library */
	
    /* Win32 and Win64 use LoadLibraryEx to attempt to open each library */
    #if defined(WIN32) || defined(_WIN64)  

        /* Set error mode to not display message box with error message */
        SetErrorMode(SEM_FAILCRITICALERRORS);
        windll = LoadLibraryEx(tdata_path, NULL, LOAD_WITH_ALTERED_SEARCH_PATH);
        if(windll == NULL){
            strcpy(error_buffer, "Dynamic load failed!\n");
            open_error = true;
        }

        /* UNIX-like systems use dlopen to attempt to open each library */
    #else
        handle = dlopen(tdata_path, RTLD_LAZY);
        if (!handle) {
            strcpy(error_buffer, dlerror());
            open_error = true;
        }
        else{
            dlerror();
            if ((error_buffer = dlerror()) != NULL) {
                strcpy(error_buffer, dlerror());
                open_error = true;
            }
        }			
    #endif
	
	    /* Free local resources */
    #if !defined(WIN32) && !defined(_WIN64)
        if (handle){dlclose(handle);}		
    #else
        FreeLibrary(windll);
    #endif
	
    if(open_error == true){
        return false;
    }

    return true;
	
} /* OpenLibrary() */

/***************************************************************************\
|
|   Method:     RunDiagnosticTool()
|   Purpose:    If adhoc/tdxodbc attempts to make a connection to a database 
|               but fails, RunDiagnosticTool executes dlopen() or LoadLibraryEx(),
|               depending on the OS, in order to attempt to load the TDODBC library
|               and report any errors resulting from the attempted load. This
|               provides diagnostic information such as which dependent libraries are 
|               missing or why the TDODBC library could not be loaded correctly.
|
|   @return:    Returns true if successful to prevent compiler warnings
|               for using ALLOC_BUFFER() in a function returning void.
|
\***************************************************************************/
bool RunDiagnosticTool()
{  
    /* Declaration of variables */
    char * tdata_path; /* String to contain directory path to tdata.xx library */
    char * error_buffer; /* Buffer to hold potential error message */
	
    tdata_path = NULL_PCHAR;
    error_buffer = NULL_PCHAR;
	
    /* Allocate memory for tdata_path and error_buffer */
    ALLOC_BUFFER(tdata_path, MAX_STR + SIZE_OF_NULL_TERMINATOR, char, false);
    ALLOC_BUFFER(error_buffer, MAX_STR + SIZE_OF_NULL_TERMINATOR, char, false);

    /* Call to ReadPrivateProfileString to retrieve directory path to tdata.xx library */
    if(ReadPrivateProfileString(tdata_path) == true){
        /* After receiving valid path to tdata.xx library, attempt to open and display error message if there is one */
        if(OpenLibrary(tdata_path, error_buffer) == false){
            printf("%s\n", error_buffer);
        }
    }
		
    /* Free local resources */
    if(NULL_PCHAR != tdata_path) { free(tdata_path); tdata_path = NULL_PCHAR;}
    if(NULL_PCHAR != error_buffer) { free(tdata_path); error_buffer = NULL_PCHAR;}
	
    return true;	
	
} /* RunDiagnosticTool() */

/***************************************************************************\
|
|   Method:     SetEnv()
|   Purpose:    Allocate ODBC environment and set environment attributes
|
|   @param [in/out] henv    - pointer to a NULL sql environment handle
|
|   @return:    SQL_SUCCESS or SQL_SUCCESS_WITH_INFO - if all tasks successful
|               other SQL RETURN CODE of the
|               FIRST ODBC API function that failed  - otherwise
|
\***************************************************************************/
int SetEnv(HENV * henv)
{
    RETCODE rc;                 /* to collect sql return codes      */

    /*
    **  ODBC: Allocate environment handle
    */
    rc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, henv);

    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    {
        print_err(SQL_NULL_HENV, SQL_NULL_HDBC, SQL_NULL_HSTMT);
        return rc;  /* Return first API ODBC error */
    }


    /*
    **  ODBC: Set Attributes for environment
    */
    rc = SQLSetEnvAttr(*henv,
        SQL_ATTR_ODBC_VERSION,
        (SQLPOINTER) SQL_OV_ODBC3,
        0);   /* last argument is ignored because SQL_OV_ODBC3 is an integer */

    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    {
        print_err(SQL_NULL_HENV, SQL_NULL_HDBC, SQL_NULL_HSTMT);
        return rc;  /* Return first API ODBC error */
    }

    return rc; /* Return last successful API return code */

} /* SetEnv() */

/***************************************************************************\
|
|   Method:     AllocateConnections()
|   Purpose:    Allocate DBC handles
|
|   @param [in/out] henv    - pointer to a HDBC
|
|   @return:    SQL_SUCCESS or SQL_SUCCESS_WITH_INFO - if all tasks successful
|               other SQL RETURN CODE of the
|               FIRST ODBC API function that failed  - otherwise
|
\***************************************************************************/
int AllocateConnectionHandles(HDBC **hdbc)
{
    int i;

    *hdbc = NULL;
    *hdbc = (HDBC *)malloc(sizeof(HDBC) * iNumConnections);
    if(NULL == *hdbc)
    {
        return SQL_ERROR;
    }

    for(i=0; i<iNumConnections; i++)
    {
        (*hdbc)[i] = SQL_NULL_HDBC;      /* not allocated                    */
    }

    return SQL_SUCCESS;
}


/**
 * Description: replace_password_with_asterisk
 *   The replace_password_with_asterisk() is responsible for replacing the 
 *   password value with an asterisk in the connect string before displaying 
 *   it to the console.  It is important not to reveal the password when the 
 *   adhoc program is being used by the users, especially when there are on 
 *   lookers.
 *
 *   Search Tokens:
 *       1) Locate "PWD=" position
 *       2) Advance postion right after equal sign
 *       3) Locate ";" position
 *       4) Repeat 1-3 if multiple PWD TOKENS had been used.
 *   Deviating from "PWD=" with "PWD =" would cause a problem and
 *   a missing semicolon.
 *
 * @ end description
 * Parameters:
 *   Name:  Purpose
 *   ---------------------------------------------------------
 *   @param [in/out] connstr_buffer   - connection string buffer for displaying
 *   @ end params
 * Return Values:
 *   @return none 
 *   @ end return values
 *
 * Remarks:
 * 1) If the connection string is not of the proper syntax, then the PWD may or
 *    not be replaced.  And possibly more than just the password value would be
 *    replaced with an asterisk.  See Search Tokens above.
 *
 * Requirements:
 * See Also:
 *
 * Fixes Documented :
 *   Date         DR/RFC  Owner      Description
 *   -----------------------------------------------------------
 *   06/22/2011   151802  jk121225   Initial version
 */

void replace_password_with_asterisk(
             unsigned char *connstr_for_display)
{   
    char *pwdstartvalptr;
    char *pwdendvalptr;
    char *pwdtmpptr;

    pwdstartvalptr = (char *) connstr_for_display;
    while ( NULL != (pwdstartvalptr = strstr(pwdstartvalptr, "PWD=" )) )
    {
        /* look for equal sign */
        pwdtmpptr = strstr(pwdstartvalptr, "=");
        if ( NULL == pwdtmpptr )
        {
            /*
            ** syntax error.  did not find an equal sign
            ** right after the PWD token. Abandon the replacement
            ** of the password value with an *
            */
            break;
        }

        /* look for semi colon */
        pwdstartvalptr  = pwdtmpptr + 1;
        pwdtmpptr = strstr(pwdstartvalptr, ";");
        if ( NULL == pwdtmpptr )
        {
            /*
            ** syntax error.  did not find a semi colon
            ** right after the PWD value. Abandon the replacement
            ** of the password value with an *
            */
            break;
        }

        /*
        ** replace password value with an * and shift the connect string
        ** to fill the gap left by the password value replacement.
        */
        pwdendvalptr = pwdtmpptr + 1;
        strcpy(pwdstartvalptr, "*;");
        pwdstartvalptr = pwdstartvalptr + 2;
        memmove(pwdstartvalptr, pwdendvalptr, (strlen(pwdendvalptr) + 1)); //ODBC-12346
    }
}


/***************************************************************************\
|
|   Method:     SetDbc()
|   Purpose:    Allocate ODBC connection and connect
|
|   @param [in/out] henv    - pointer to a NULL sql environment handle
|
|   @param [in/out] hdbc    - pointer to a NULL sql connection handle
|
|   @return:    SQL_SUCCESS or SQL_SUCCESS_WITH_INFO - if all tasks successful
|               other SQL RETURN CODE of the
|               FIRST ODBC API function that failed  - otherwise
|
\***************************************************************************/

int SetDbc(HENV * henv, HDBC * hdbc)
{
    RETCODE rc;                 /* to collect sql return codes              */
    SWORD outlength;            /* used in SQLDriverConnect                 */
    SWORD maxsize;              /* used in SQLDriverConnect                 */
    char * sqlstate;            /* used to retrieve SQLState from print_err */
    int i;

    outlength = 0;
    sqlstate = NULL_PCHAR;

    /* safe cast of MAXLINE to SWORD */
    maxsize = (SWORD) (((UWORD) -1) >> 2);  /* SQLDriverConnect()'s
                                               maximum size allowed */
    if (((size_t) maxsize) > MAXLINE)
        maxsize = (SWORD) MAXLINE;
    else
        maxsize--;  /* to allow NULL-terminator     */



    for(i=0; i<iNumConnections; i++)
    {
        /*
        **  ODBC: Allocate connection
        */
        rc = SQLAllocHandle(SQL_HANDLE_DBC, *henv, &hdbc[i]);

        if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        {
            print_err(*henv, SQL_NULL_HDBC, SQL_NULL_HSTMT);
            return rc;  /* Return first API ODBC error */
        }


        /*
        **  ODBC: Connect
        */
        switch (useConnAPI)
        {
        case ADHOC_CONNTYPE_SQLDRIVERCONNECT:

            /* DO NOT display Password to the console, replace it with an asterisk */
            replace_password_with_asterisk(connstr_without_pwd);

            printf("\nConnecting with SQLDriverConnect(\"%s\")...\n", connstr_without_pwd);

            rc = SQLDriverConnect(hdbc[i], (SQLHWND) 0, connstr, SQL_NTS,
                (SQLCHAR *) line, maxsize, &outlength, SQL_DRIVER_NOPROMPT);
            len = (size_t) outlength;
            break;

        default: /* ADHOC_CONNTYPE_SQLCONNECT */
            printf("\nConnecting with SQLConnect(DSN=%s,UID=%s,PWD=*)...\n", dsn, uid);

            rc = SQLConnect(hdbc[i], dsn, SQL_NTS, uid, SQL_NTS, pwd, SQL_NTS);
            break;

        }

        if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        {
            /* Allocate memory for sqlstate */
            ALLOC_BUFFER(sqlstate, MAX_STR + SIZE_OF_NULL_TERMINATOR, char, false);

            /* Redirect stdout to string to contain sqlstate */
            setbuf(stdout, sqlstate);
            print_err(SQL_NULL_HENV, hdbc[i], SQL_NULL_HSTMT); 

            /* Print adhoc/tdxodbc error message */
            fflush(stdout);

            /* Reset stdout to output to console */
            setbuf(stdout, NULL);
            strcat(sqlstate, "\0");

            /* If the print_err error message contains SQLState "IM003" call RunDiagnosticTool() */
            #if !defined(SOLARIS) && !defined(HP_UX)
            if(strstr(sqlstate, "IM003") != NULL){
                RunDiagnosticTool(); /*Diagnostic tool method is called since there was a connection error*/
            }
            #else
                RunDiagnosticTool();
            #endif

            /* Free sqlstate */
			if(NULL_PCHAR != sqlstate){ free(sqlstate); sqlstate = NULL_PCHAR;}

            return rc;  /* Return first API ODBC error */
        }

        odbc_connected++;
    }

    printf("\n.....ODBC connection successful.\n\n");

    return rc; /* Return last successful API return code */

} /* SetDbc() */



/***************************************************************************\
|
|   Method:     GetODBCInfo()
|   Purpose:    Display connection info
|
|   @param [in/out] henv    - pointer to a NULL sql environment handle
|
|   @param [in/out] hdbc    - pointer to a NULL sql connection handle
|
|   @return:    SQL_SUCCESS or SQL_SUCCESS_WITH_INFO - if all tasks successful
|               other SQL RETURN CODE of the
|               FIRST ODBC API function that failed  - otherwise
|
\***************************************************************************/
int GetODBCInfo(HENV * henv, HDBC * hdbc)
{
    RETCODE rc;                 /* to collect sql return codes      */


    /*
    **  ODBC: Display Connection Information
    */

    /* Get ODBC version */
    rc = DisplayInfo(*henv, hdbc[0], SQL_ODBC_VER, "ODBC version        = ");
    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        return rc;

    /* Get DBMS name */
    rc = DisplayInfo(*henv, hdbc[0], SQL_DBMS_NAME, "DBMS name           = ");
    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        return rc;

    /* Get DBMS version */
    rc = DisplayInfo(*henv, hdbc[0], SQL_DBMS_VER, "DBMS version        = ");
    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        return rc;

    /* Get Driver name */
    rc = DisplayInfo(*henv, hdbc[0], SQL_DRIVER_NAME, "Driver name         = ");
    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        return rc;

    /* Get Driver version */
    rc = DisplayInfo(*henv, hdbc[0], SQL_DRIVER_VER, "Driver version      = ");
    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        return rc;

    /* Get Driver ODBC version */
    rc = DisplayInfo(*henv, hdbc[0], SQL_DRIVER_ODBC_VER, "Driver ODBC version = ");
    if ((SQL_SUCCESS  != rc) && (SQL_SUCCESS_WITH_INFO != rc))
        return rc;


    return rc; /* Return last successful API return code */

} /* GetODBCInfo() */



/***************************************************************************\
|
|   Method:     FreeODBCResources()
|   Purpose:    Disconnects if necessary, and frees
|               all ODBC handles that are not NULL
|
|   @param [in/out] henv    - pointer to a sql environment handle
|
|   @param [in/out] hdbc    - pointer to a sql connection handle
|
|   @param [in/out] hstmt   - pointer to a sql statement handle
|
\***************************************************************************/
void FreeODBCResources(HENV * henv, HDBC * hdbc, HSTMT * hstmt)
{
    RETCODE rc;                 /* to collect sql return codes      */


    if (SQL_NULL_HSTMT != *hstmt)
    {
        rc = SQLFreeHandle(SQL_HANDLE_STMT, *hstmt);  /* free the statement handle    */
        if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
            print_err(*henv, hdbc[0], *hstmt);

        *hstmt = SQL_NULL_HSTMT;
    }


    if (NULL != hdbc)
    {
        int i;
        for(i=0; i<iNumConnections; i++)
        {
            if (odbc_connected)
            {
                rc = SQLDisconnect(hdbc[i]);            /* Disconnect from the data source  */
                if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
                    print_err(*henv, *hdbc, SQL_NULL_HSTMT);

                odbc_connected--;
            }

            if(SQL_NULL_HDBC != hdbc[i])
            {
                rc = SQLFreeHandle(SQL_HANDLE_DBC, hdbc[i]);/* Free the connection handle       */
                if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
                    print_err(*henv, hdbc[i], SQL_NULL_HSTMT);

                hdbc[i] = SQL_NULL_HDBC;
            }
        }
    }


    if (SQL_NULL_HENV != *henv)
    {

        rc = SQLFreeHandle(SQL_HANDLE_ENV, *henv);   /* Free the environment handle      */
        if (rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
            print_err(*henv, SQL_NULL_HDBC, SQL_NULL_HSTMT);

        *henv = SQL_NULL_HENV;
    }


    printf("\nODBC connection closed.\n");

} /* FreeODBCResources() */



/***************************************************************************\
|
|   Method:     DisplayInfo()
|   Purpose:    Get ODBC info specified by infoTag using
|               ODBC API SQLGetInfo(), and display it in
|               one line, preceded by 'caption'
|
|   @param [in] henv    - sql environment handle
|
|   @param [in] hdbc    - sql connection handle
|
|   @param [in] infoTag - valid InfoType value (refer to SQLGetInfo() specs)
|                             E.g.: SQL_ODBC_VER, SQL_DBMS_NAME...
|
|   @param [in] caption - NULL or a pointer to a NULL-Terminated string.
|
|   @return:    the SQL RETURN CODE of the ODBC API call to SQLGetInfo()
|
|	Revision History:
|   Revision     Date      DR/RFC  Userid   Comments
|   -----------  --------  ------- -------- -----------------------------------
|  	13.10g.00.00 01/07/10  138429  kk250025 Changing dblen type to SQLSMALLINT in DisplayInfo().
|
\***************************************************************************/
int DisplayInfo(HENV henv, HDBC hdbc, UWORD infoTag, const char * caption)
{
    RETCODE rc;                 /* to collect sql return codes      */
    char *  db;                 /* returned string 'db'             */
    SQLSMALLINT  dblen;              /* length (and size) of 'db'        */

    db      = NULL_PCHAR;
    dblen   = 0;


    /*
    **  Get ODBC info specified by infoTag
    */
    /* Get the length necessary for the buffer to hold the returned string */
    rc = SQLGetInfo(hdbc, infoTag, db, (SQLSMALLINT) 0, &dblen);

    if ((SQL_SUCCESS == rc) || (SQL_SUCCESS_WITH_INFO == rc))
    { /* data was truncated, but SQLGetInfo succeeded */

        /******/
        /*** Workaround known UNIX bug (dblen is not updated) ***/
        if (0 == dblen)
        { /* bug: SQLGetInfo() should always have some bytes to return */
            /* allocate a default fixed large number for the buffer db */
            dblen = MAX_STR; /* a default fixed large number */
        }
        /******/

        /* allocate to a size that fits the information that SQLGetInfo() returns */
        ALLOC_BUFFER(db, dblen + SIZE_OF_NULL_TERMINATOR, char, -1);  /* #ERROR# adhoc error codes need to be revised */

        /* try SQLGetInfo() again; now without truncation */
        rc = SQLGetInfo(hdbc, infoTag, db, dblen + SIZE_OF_NULL_TERMINATOR, &dblen);
    }

    if ((SQL_SUCCESS != rc) && (SQL_SUCCESS_WITH_INFO != rc))
    {
        print_err(henv, hdbc, SQL_NULL_HSTMT);
        return rc;  /* Return first API ODBC error */
    }


    /* Display the information retrieved into 'db' */
    if ((NULL_PCHAR) != caption)
        printf("%s", caption);

    printf("-%s-\n", db);


    free(db);
    db = NULL_PCHAR;

    return rc;  /* Return the SQL RETURN CODE of SQLGetInfo() */

} /* DisplayInfo() */



/***************************************************************************\
|
|   Method:     ProcessSqlStmt()
|   Purpose:    Execute a statement stored in 'sqlstr' and
|               display the results made available afterwards
|
|   @param [in/out] henv    - pointer to a sql environment handle
|
|   @param [in/out] hdbc    - pointer to a sql connection handle
|
|   @param [in/out] hstmt   - pointer to a sql statement handle
|
|   @return:    SQL_SUCCESS or SQL_SUCCESS_WITH_INFO - if all tasks successful
|               other SQL RETURN CODE of the
|               FIRST ODBC API function that failed  - otherwise
|
|   Revision History:
|   Revision    Date       DR/RFC  Userid   Comments
|   ----------- --------   ------- -------- -----------------------------------
|  13.10f.00.00 11/11/09    86872 jk121225 fixed logic to handle all data type where target is SQL_C_CHAR.
|
|
\***************************************************************************/
int ProcessSqlStmt(HENV * henv, HDBC hdbc, HSTMT * hstmt)
{
    RETCODE rc;                 /* to collect sql return codes      */
    int     ColumnNumber;       /* index used in for-loops          */
    int     sqlstmtcnt = 0;     /* sql statement counter; identification for multi sql statement requests */
    SQLLEN  rowcount;



    printf("\nExecuting SQLExecDirect(\"%s\")...\n\n", sqlstr);

    /*
    **  Execute the SQL statement
    */
    rc = SQLExecDirect(*hstmt, sqlstr, SQL_NTS);
    if (rc != SQL_SUCCESS)
    {
        print_err(*henv, hdbc, *hstmt);
        return rc;
    }




    /*******************************************************************/
    /*  See what kind of statement it was.  If there are no result     */
    /*  columns, the statement is not a SELECT statement. If the numbe */
    /*  of affected rows is greater than 0, the statement was probably */
    /*  an UPDATE, INSERT or DELETE statement, so print thhe number of */
    /*  affected rows.  If the number of affected rows is 0, the       */
    /*  statement is probably a DDL statement, so print that the       */
    /*  operation was successful and commit it.                        */
    /*******************************************************************/

    do
    { /* DO: display results */

        rowcount =      (SQLLEN) 0;

        rc = SQLNumResultCols(*hstmt, &nresultcols);
        if (rc != SQL_SUCCESS)
        {
            print_err(*henv, hdbc, *hstmt);
            if (rc != SQL_SUCCESS_WITH_INFO)
            {
                return(rc);
            }
        }

        sqlstmtcnt++;

        if (0 == nresultcols)
        { /* IF: any results? NO */

            rc = SQLRowCount(*hstmt, &rowcount);
            if (rc != SQL_SUCCESS)
            {
                print_err(*henv, hdbc, *hstmt);
                if (rc != SQL_SUCCESS_WITH_INFO)
                {
                    return(rc);
                }
            }

            if (rowcount > 0)
            {
                printf("SQL Statement [%d]: %ld rows affected.\n", sqlstmtcnt, rowcount);
            }
            else
            {
                printf("SQL Statement [%d]: Operation was successful.\n", sqlstmtcnt);
            }

            rc = SQLTransact(*henv, hdbc, SQL_COMMIT);
            if (rc != SQL_SUCCESS)
            {
                print_err(*henv, hdbc, *hstmt);
                if (rc != SQL_SUCCESS_WITH_INFO)
                {
                    return(rc);
                }
            }
        }
        else
        { /* Fetch Result Set */

            /*******************************************************************/
            /*  Otherwise, display the column names of the result set and use  */
            /*  the display_size() function to compute the length needed by    */
            /*  each data type.  Next, bind the columns and specify all        */
            /*  data will be converted to char.  Finally, fetch and print each */
            /*  row, printing truncation messages as necessary.                */
            /*******************************************************************/

            rc = SQLRowCount(*hstmt, &rowcount);
            if (rc != SQL_SUCCESS)
            {
                print_err(*henv, hdbc, *hstmt);
                if (rc != SQL_SUCCESS_WITH_INFO)
                {
                    return(rc);
                }
            }
            printf("SQL Statement [%d]: %ld rows affected.\n\n", sqlstmtcnt, rowcount);

            /*
            ** Fetched data will always be returned as SQL_C_CHAR.
            ** supporting other data types just complicates the program
            */
            for (ColumnNumber = 0; ColumnNumber < nresultcols; ColumnNumber++)
            {
                rc = SQLDescribeCol(
                    *hstmt, (UWORD) ColumnNumber + 1, colname, (SWORD) MAX_NAME_LEN,
                    &colnamelen, &coltype, &ColumnSizePtrArray[ColumnNumber], &scale, &nullable);
                if (rc != SQL_SUCCESS)
                {
                    print_err(*henv, hdbc, *hstmt);
                    if (rc != SQL_SUCCESS_WITH_INFO)
                    {
                        return(rc);
                    }
                }


                /*
                **  Initialize DATA buffers
                */
                /* Column size can fit in UDWORD, hence no loss */
                ColumnBufferLengthArray[ColumnNumber] = display_size(coltype,
                                                    (UDWORD) ColumnSizePtrArray[ColumnNumber],
                                                    colname);

                ALLOC_BUFFER(data[ColumnNumber], ColumnBufferLengthArray[ColumnNumber], UCHAR, -1);

                printstring((char *) colname);
                printstring(SPACE_DELIMITER);


                rc = SQLBindCol(
                        *hstmt,
                        (UWORD) ColumnNumber + 1,
                        SQL_C_CHAR,
                        data[ColumnNumber],
                        (SQLLEN) ColumnBufferLengthArray[ColumnNumber] + 1,
                        &outlen[ColumnNumber]);

                if (rc != SQL_SUCCESS)
                {
                    print_err(*henv, hdbc, *hstmt);
                    if (rc != SQL_SUCCESS_WITH_INFO)
                    {
                        return(rc);
                    }
                }
            }

            printf("\n");


            while (true)
            {
                rc = SQLFetch(*hstmt);

                if (rc == SQL_NO_DATA_FOUND) break;

                if (rc != SQL_SUCCESS)
                {
                    print_err(*henv, hdbc, *hstmt);
                    if (rc != SQL_SUCCESS_WITH_INFO)
                    {
                        break;
                    }
                }

                for (ColumnNumber = 0; ColumnNumber < nresultcols; ColumnNumber++)
                {
                    if (outlen[ColumnNumber] == SQL_NULL_DATA)
                    {
                        strcpy((char *)data[ColumnNumber], "NULL");
                    }
                    else if (outlen[ColumnNumber] == SQL_NO_TOTAL)
                    {
                        strcpy((char *)data[ColumnNumber], "NO_TOTAL");
                    }

                    printstring((char *)data[ColumnNumber]);
                    printf(" ");

                } /* FOR: (ColumnNumber < nresultcols) */

                printf("\n");

            } /* WHILE:(true) */

        } /* IF-ELSE: any results? (nresultcols == 0) */

        /*
        **  Free DATA buffers
        */
        for (ColumnNumber = 0; ColumnNumber < nresultcols; ColumnNumber++)
        {
            free(data[ColumnNumber]);
            data[ColumnNumber] = NULL_PUCHAR;
        }

        printf("\n");

        rc = SQLMoreResults(*hstmt);

        if (SQL_SUCCESS_WITH_INFO == rc)
        {
            print_err(*henv, hdbc, *hstmt);
        }

    } while (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO);

    if (SQL_NO_DATA != rc)
    {
        print_err(*henv, hdbc, *hstmt);
    }

    return rc;

} /* ProcessSqlStmt() */


/***************************************************************************\
|
|   Method:     display_size()
|   Purpose:    Obtain a size to assign to formatted displays of different
|               data types
|               Both the column size and column name Length are considered when
|               determining the display size.
|
|       NOTE:   The following function is included for completeness, but
|               is not relevant for understanding the function of ODBC.
|
|   @param [in] coltype first argument from main()
|
|   @param [in] collen  second argument from main()
|
|   @param [in] colname third argument from main()
|
|   @return  the size to assign to formatted displays of 'coltype' data type
|
|
|   Revision History:
|   Revision    Date       DR/RFC  Userid   Comments
|   ----------- --------   ------- -------- -----------------------------------
|   03.00.00.00 06/11/01   dr52740 NDFSODBC Added types: SQL_TYPE_DATE,
|                                           SQL_TYPE_TIME, SQL_TYPE_TIMESTAMP.
|   13.00.00.00 01/23/08                    Added types: SQL_PERIOD_DATE,
|                                           SQL_PERIOD_TIME,
|                                           SQL_PERIOD_TIME_WITH_TIME_ZONE,
|                                           SQL_PERIOD_TIMESTAMP,
|                                           SQL_PERIOD_TIMESTAMP_WITH_TIME_ZONE.
|  13.10f.00.00 11/11/09    86872 jk121225 fixed logic to handle all data type where target is SQL_C_CHAR.
|  13.10f.00.00 11/24/09   114663 jk121225 Applying fix where TIME and TIMESTAMP LITERAL are not at max
|  13.10.00.01  01/29/10   138569 kk250025 changes to display_size() to avoid errors for INTERVAL/PERIOD datatypes.
|
\***************************************************************************/
UDWORD display_size (
               SWORD coltype,
               UDWORD collen,
               UCHAR *colname)
{
    /*
    ** Refer to the ODBC Programmers Reference.
    ** There is a Display Size section which has the number of characters required for each type
    **
    ** NOTE:
    **      The literals defined below are used to determine the maximum length of the SQL Data Type
    **      and as a learning tool to show what format these different SQL Data Types are.
    */
    int  SQLDataType_BufferSize;
    char DECIMAL_OR_NUMERIC_LITERAL[] = "-1234567890.12345";
    char BIT_LITERAL[] = "1";
    char TINYINT_LITERAL[] = "-123";
    char SMALLINT_LITERAL[] = "-123456";
    char INTEGER_LITERAL[] = "-12345678901";
    char BIGINT_LITERAL[] = "-12345678901234567890";
    char REAL_LITERAL[] = "-1234567.E+12";
    char FLOAT_OR_DOUBLE_LITERAL[] = "-1.2345678901234567890123456789012345678E+123";  /* Also covers the case where the type is NUMBER */
    char DATE_LITERAL[] = "yyyy-mm-dd";
    char TIME_LITERAL[] = "hh:mm:ss.ffffff+hh:mi";  /* fractional portion would be 9 + fractional characters */
    char TIMESTAMP_LITERAL[] = "yyyy-mm-dd hh:mm:ss.ffffff+hh:mi"; /* fractional portion would be 20 + fractional characters */
    const char PERIOD_DATE_LITERAL[] = "('2005-02-03', '2006-02-04')";
    const char PERIOD_TIME_LITERAL[] = "('12:12:12.340000+05:30', '13:12:12.340000+05:30')";
    const char PERIOD_TIMESTAMP_WITH_TIME_ZONE[] = "('2005-02-03 12:12:12.000045+08:00', '2005-02-03 12:12:12.000048+08:00')";
    int  INTERVAL_DISPLAY_SIZE = 55; /* would like to define literals as a learning tool */
    const int LARGE_TYPE_MAX_DISPLAY_SIZE = 1000;


    /*
    ** Assume native size
    */
    SQLDataType_BufferSize = collen;

    switch (coltype)
    {
    case SQL_CHAR:
    case SQL_VARCHAR:
        break;
    case SQL_WCHAR:
    case SQL_WVARCHAR:
        break;
    case SQL_LONGVARBINARY:
    case SQL_LONGVARCHAR:
    case SQL_WLONGVARCHAR:
            SQLDataType_BufferSize = LARGE_TYPE_MAX_DISPLAY_SIZE;
        break;
    case SQL_DECIMAL:
    case SQL_NUMERIC:
            SQLDataType_BufferSize = strlen(DECIMAL_OR_NUMERIC_LITERAL);
        break;
    case SQL_BIT:
            SQLDataType_BufferSize = strlen(BIT_LITERAL);
        break;
    case SQL_TINYINT:
            SQLDataType_BufferSize = strlen(TINYINT_LITERAL);
        break;
    case SQL_SMALLINT:
            SQLDataType_BufferSize = strlen(SMALLINT_LITERAL);
        break;
    case SQL_INTEGER:
            SQLDataType_BufferSize = strlen(INTEGER_LITERAL);
        break;
    case SQL_BIGINT:
            SQLDataType_BufferSize = strlen(BIGINT_LITERAL);
        break;
    case SQL_REAL:
            SQLDataType_BufferSize = strlen(REAL_LITERAL);
        break;
    case SQL_FLOAT:
    case SQL_DOUBLE:
            SQLDataType_BufferSize = strlen(FLOAT_OR_DOUBLE_LITERAL);
        break;
    case SQL_BINARY:
    case SQL_VARBINARY:
        return(max((int) 2 * collen, (int) strlen((char *) colname)));

    case SQL_DATE:               /* 2x spec */
    case SQL_TYPE_DATE:          /* 3x spec */
            SQLDataType_BufferSize = strlen(DATE_LITERAL);;
        break;
    case SQL_TIME:              /* 2x spec */
    case SQL_TYPE_TIME:         /* 3x spec */
            SQLDataType_BufferSize = strlen(TIME_LITERAL);
        break;
    case SQL_TYPE_TIMESTAMP:
            SQLDataType_BufferSize = strlen(TIMESTAMP_LITERAL);
        break;
    case SQL_INTERVAL_YEAR:
    case SQL_INTERVAL_MONTH:
    case SQL_INTERVAL_DAY:
    case SQL_INTERVAL_HOUR:
    case SQL_INTERVAL_MINUTE:
    case SQL_INTERVAL_SECOND:
    case SQL_INTERVAL_YEAR_TO_MONTH:
    case SQL_INTERVAL_DAY_TO_HOUR:
    case SQL_INTERVAL_DAY_TO_MINUTE:
    case SQL_INTERVAL_DAY_TO_SECOND:
    case SQL_INTERVAL_HOUR_TO_MINUTE:
    case SQL_INTERVAL_HOUR_TO_SECOND:
    case SQL_INTERVAL_MINUTE_TO_SECOND:
            SQLDataType_BufferSize = INTERVAL_DISPLAY_SIZE;
        break;
    case SQL_PERIOD_DATE:
            SQLDataType_BufferSize = strlen(PERIOD_DATE_LITERAL);
        break;
    case SQL_PERIOD_TIME:
    case SQL_PERIOD_TIME_WITH_TIME_ZONE:
            SQLDataType_BufferSize = strlen(PERIOD_TIME_LITERAL);
        break;
    case SQL_PERIOD_TIMESTAMP:
    case SQL_PERIOD_TIMESTAMP_WITH_TIME_ZONE:
            SQLDataType_BufferSize = strlen(PERIOD_TIMESTAMP_WITH_TIME_ZONE);
        break;
    case SQL_TD_XML:
            SQLDataType_BufferSize = LARGE_TYPE_MAX_DISPLAY_SIZE;
        break;
    default:
        printf("Unknown datatype, %d\n", coltype);
        return (0);

    } /* SWITCH:(coltype) */

    return( max( SQLDataType_BufferSize, (int) strlen((char *) colname) ) + 1 );

} /* display_size() */



/***************************************************************************\
|
|   Method:     printstring()
|   Purpose:    solve DR52182 problems.     Prints out a string
|
|   @param [in] str – null-terminated string to print out
|
\***************************************************************************/
void printstring(char * str)
{ /* from DR52182 */
    size_t str_len;             /* length of argument str           */
    size_t i;                   /* index used in for-loops          */

    str_len = strlen(str);


    for (i = 0; i < str_len; i++)
    { /* FOR: output str char by char, after applying filter */
        if ('\r' == (*(str+i)))
            printf("\n");
        else
            printf("%c", (*(str+i)));

    } /* FOR: output str char by char, after applying filter */

} /* printstring() */


/*--- END: Implementation of function prototypes ----------------------*/
/*---------------------------------------------------------------------*/




#if defined(WIN32) || defined(_WIN64)

/**
 * Description: SetTerminalEchoON() - WINDOWS
 *   Use SetTerminalEchoON() to enable the ECHO to the terminal
 *
 * @ end description
 * Parameters:
 *   Name:  Purpose
 *   ---------------------------------------------------------
 *   @param [in/out] none
 *   @ end params
 * Return Values:
 *   @return int    0     - function successful
 *   @return int    1     - GetStdHandle() failed
 *   @return int    2     - SetConsoleMode() failed
 *   @ end return values
 *
 * Remarks:
 * Requirements:
 * 1) This function requires the SetTerminalEchoOFF to be called
 *    first so tha the fdwOldMode variable is set.
 * See Also:
 *
 * Fixes Documented :
 *   Date         DR/RFC  Owner      Description
 *   -----------------------------------------------------------
 *   06/22/2011   151802  jk121225   Initial version
 *   07/05/2011   151802  jk121225   Windows needs to check for tty console
 */

int SetTerminalEchoON()  /* WINDOWS */
{
    HANDLE    hStdin;
    BOOL      ret;
	int       IO_in;

    IO_in = 0;                     /* Want to check stdin (file descriptor 0). */
    ret = _isatty(IO_in);           /* stdin is a terminal. */
    if (ret == 0)                  /* stdin is a terminal. */
    {
        return(1);                /* Indicates stdin is not a terminal. */
    }

    hStdin = GetStdHandle(STD_INPUT_HANDLE);
    if (hStdin == INVALID_HANDLE_VALUE )
    {
      return(2);
    }

    ret = SetConsoleMode(hStdin, fdwOldMode);
    if ( ! ret )
    {
        return(3);
    }

    return(0);
}


#else


/**
 * Description: SetTerminalEchoON() - UNIX
 *   Use SetTerminalEchoON() to enable the ECHO to the terminal
 *
 * @ end description
 * Parameters:
 *   Name:  Purpose
 *   ---------------------------------------------------------
 *   @param [in/out] none
 *   @ end params
 * Return Values:
 *   @return int    0     - function successful
 *   @return int    1     - isatty() failed
 *   @return int    2     - ioctl(TCGETA) failed
 *   @return int    3     - ioctl(TCSETA) failed
 *   @ end return values
 *
 * Remarks:
 *   1) Additional error information can be reported if this appears to be a
 *      problem by referencing errno.
 *
 * Requirements:
 * See Also:
 *
 * Fixes Documented :
 *   Date         DR/RFC  Owner      Description
 *   -----------------------------------------------------------
 *   06/22/2011   151802  jk121225   Initial version
 */

int SetTerminalEchoON()   /*  UNIX  */
{
    struct  termios  term_info;
    int             ret;
    int             IO_in;

    IO_in = 0;                    /* Want to check stdin (file descriptor 0). */
    ret = isatty(IO_in);           /* stdin is a terminal. */
    if (ret != 1)                  /* stdin is a terminal. */
    {
        return(1);                /* Indicates stdin is not a terminal. */
    }

    /*
    **  Turn on terminals echo. 
    **  1) First, get the current terminal info. 
    **  2) Change parameter in ioctl to use stdin, it was stdout.
    */

    ret = tcgetattr(IO_in, &term_info);
    if (ret != 0)
    {
        return(2);
    }

    term_info.c_lflag |= (ECHO);              /* Turn on echo flag. */

    /*
    ** Finally, reset the terminal info.
    */
    ret = tcsetattr(IO_in, TCSANOW, &term_info);
    if (ret != 0)
    {
        return(3);
    }

    return(0);

}

#endif



#if defined(WIN32) || defined(_WIN64)

/**
 * Description: SetTerminalEchoOFF() - WINDOWS
 *   Use SetTerminalEchoOFF() to disable the ECHO to the terminal
 *
 * @ end description
 * Parameters:
 *   Name:  Purpose
 *   ---------------------------------------------------------
 *   @param [in/out] none
 *   @ end params
 * Return Values:
 *   @return int    0     - function successful
 *   @return int    1     - GetStdHandle() failed
 *   @return int    2     - FlushConsoleInputBuffer() failed
 *   @return int    3     - GetConsoleMode() failed
 *   @return int    4     - SetConsoleMode() failed
 *   @ end return values
 *
 * Remarks:
 *   1) Additional error information can be reported if this appears to be a
 *      problem by alling GetLastError()
 *
 * Requirements:
 * See Also:
 *
 * Fixes Documented :
 *   Date         DR/RFC  Owner      Description
 *   -----------------------------------------------------------
 *   06/22/2011   151802  jk121225   Initial version
 *   07/05/2011   151802  jk121225   Windows needs to check for tty console
 */

int SetTerminalEchoOFF()  /* WINDOWS */
{
    HANDLE    hStdin;
    DWORD     fdwMode;
    BOOL      ret;
	int       IO_in;

    IO_in = 0;                     /* Want to check stdin (file descriptor 0). */
    ret = _isatty(IO_in);           /* stdin is a terminal. */
    if (ret == 0)                  /* stdin is a terminal. */
    {
        return( 1 );                /* Indicates stdin is not a terminal. */
    }


    /*------------------------------------------------------------------*/
    /* Get handles to STDIN and STDOUT.                                 */
    /*------------------------------------------------------------------*/
    hStdin = GetStdHandle(STD_INPUT_HANDLE);
    if (hStdin == INVALID_HANDLE_VALUE )
    {
      return( 2 );
    }

    ret = FlushConsoleInputBuffer(hStdin);
    if ( ! ret )
    {
        return( 3 );
    }

    /*----------------------------------------------------------------*/
    /* Turn off the line input mode, and echo the input mode.         */
    /*----------------------------------------------------------------*/
    ret = GetConsoleMode(hStdin, &fdwOldMode);
    if ( ! ret )
    {
      return(4);
    }

    /* Turn off echo */
    fdwMode = fdwOldMode & ~(ENABLE_ECHO_INPUT);

    ret = SetConsoleMode(hStdin, fdwMode);
    if ( ! ret )
    {
      return(5);
    }

    return(0);
}


#else     /*   UNIX   */


/**
 * Description: SetTerminalEchoOFF() - UNIX
 *   Use SetTerminalEchoOFF() to disable the ECHO to the terminal
 *
 * @ end description
 * Parameters:
 *   Name:  Purpose
 *   ---------------------------------------------------------
 *   @param [in/out] none
 *   @ end params
 * Return Values:
 *   @return int    0     - function successful
 *   @return int    1     - isatty() failed
 *   @return int    2     - ioctl(TCGETA) failed
 *   @return int    3     - ioctl(TCSETA) failed
 *   @ end return values
 *
 * Remarks:
 *   1) Additional error information can be reported if this appears to be a
 *      problem by referencing errno.
 *
 * Requirements:
 * See Also:
 *
 * Fixes Documented :
 *   Date         DR/RFC  Owner      Description
 *   -----------------------------------------------------------
 *   06/22/2011   151802  jk121225   Initial version
 */

int SetTerminalEchoOFF()  /* UNIX */
{
    struct  termios  term_info;
    int             ret;
    int             IO_in;

    IO_in = 0;                     /* Want to check stdin (file descriptor 0). */
    ret = isatty(IO_in);           /* stdin is a terminal. */
    if ( ret != 1 )                /* stdin is a terminal. */
    {
        return(1);                 /* Indicates stdin is not a terminal. */
    }

    /*
    **  Turn OFF terminals echo. 
    **  1) First, get the current terminal info. 
    */
    ret = tcgetattr(IO_in, &term_info);
    if (ret != 0)
    {
        return(2);
    }

    term_info.c_lflag &= ~(ECHO);      /* Turn off echo flag. */

    /* Apply terminal info without ECHO flag. */
    ret = tcsetattr(IO_in, TCSANOW, &term_info);
    if (ret != 0)
    {
        return(3);
    }

    return(0);
}

#endif



/*===========================================================================*
    End of File:    adhoc.c
 *===========================================================================*/

