/*************************************************************************
*
* Title: Paral-X Defined Constants
*
* Copyright 1999-2020, by Teradata Corporation.
* All Rights Reserved.
* TERADATA PROPRIETARY AND CONFIDENTIAL--RESTRICTED
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description  Paral-X Defined Constants
*
* Purpose
*
*
* History Information
*
*  Revision     Date    D(C)R DID                Comments
* ----------- -------- -----  --- ---------------------------------------
* 01.00.00.01 11301999  00000 AY2 Initial Creation
* 01.00.00.02 02222000  49437 AY2 Extended PX_MaxMessageLength from 100 to 512
* 03.00.00.01 02072001  52185 NJ  Support OperatorType attribute
* 04.00.00.01 06182001  56304 MW  Merge of NT port
* 05.00.00.01 07102004  87714 CFL MVS Support                       @001
* 05.00.00.02 12072004  91327 NT  Externalize Library Names         @002
* 05.00.00.03 02012005  92618 MDV Added warning exit code
* 09.00.00.00 06092006 105252 CFL Dual Active Support               @003
* 13.00.00.00 20070913 115589 APY Support for LOB                   @004
* 13.00.00.01 10042007 115565 TSL Copyright changes
* 13.00.00.02 01022008 115588 CFL Buffer Mode support               @005
* 13.01.00.01 12022008 124601 APY Support for operational metadata  @006
* 14.00.00.00 04232011 141402 WJS Support NUMBER data type          @007
* 14.10.00.00 12042012 TPT13241  CFL Turn off interval checkpointing @008
* 15.00.00.00 09152013 TPT19374  APY Turn off all checkpoints       @009
* 16.00.00.01 01082016 TPT-27075 MDV Add PX_SYSATTR_LogDirectory    @010
* 16.00.00.02 01192016 TPT-27273 MDV Add PX_SYSATTR_InstallRootDir  @011
* 16.00.00.03 01222016 TPT-19360 MDV Support operator sending max row size @012
* 16.00.00.04 02102016 TPT-25646 MDV Add PX_SYSATTR_MessageSize
* 16.00.00.05 03062017 TPT-29416 MDV Add PX_SYSATTR_LatencyIsUsed
* 17.00.00.01 11142019 TPT-33903 MDV Support dynamic LOB streaming 
*************************************************************************/
#ifndef PXDEF__H
#define PXDEF__H

#include "pxos.h"

#ifdef _OSNT_
#define PX_TEST
#define STATIC   static
#else
#define STATIC
#endif

#define PX_MaxNodeCoordCount    10
#define PX_MaxExecutorCount     10
#define PX_MaxTableCount        10
#define PX_MaxModuleNameLength  16
#define PX_MaxColumnNameLength  32
#define PX_MaxTableNameLength   32
#define PX_MaxTableSchemas      1
#define PX_MaxRowLength         64260
#define PX_MaxColumnLength      64000
#define PX_MaxRowColumns        2048     /* need to revisit  */
#define PX_True                 1
#define PX_False                0
#define PX_DSNameLength         16
#define PX_MaxBlockSize         1048576    //@012 increase blocksize from 65000 
#define PX_MaxSQLSize           1048576
#define PX_MaxTasks             25
#define PX_MaxTaskNameLen       32
#define PX_MaxMessageLength     512
#define PX_MaxCharSize          64000
#define PX_MaxPrecSize          18

#define PX_MaxRowSize_64K       64        //TPT-33903
#define PX_MaxRowSize_1M        1024      //TPT-33903
#define PX_RowMustFitMaxSize    0         //TPT-33903
#define PX_RowExceedMaxSize     1         //TPT-33903
#define PX_RowInfoBufferLen     12        //TPT-33903

#ifdef WIN32
#define PX_CoordinatorName      "coordinator.exe"         /*@002*/
#define PX_ExecutorName         "executor.exe"            /*@002*/
#elif defined(__MVS__)                                             /*@001*/
#define PX_CoordinatorName      "COORD"                   /*@002*/ /*@001*/
#define PX_ExecutorName         "EXECUTOR"                         /*@001*/
#else
#define PX_CoordinatorName      "coordinator"             /*@002*/
#define PX_ExecutorName         "executor"                /*@002*/
#endif
#define PX_InputStreamName      "Input"
#define PX_OutputStreamName     "Output"
#define PX_VarGroupName         "PARAL-X" //"DS_Startup"
#define PX_ConsumerGroupName    "_TWB_Consumer_Group_"             /*@005*/
#define PX_ExecutorGroupName    "_TWB_Executor_Group_"             /*@005*/
#define PX_MasterGroupName      "_TWB_Master_Group_"               /*@005*/
#define PX_BufferLayoutVarName  "_TWB_BufferLayout_"               /*@005*/
#define PX_BufferModeInfoVarName "_TWB_BufferModeInfo_"            /*@005*/
#define PX_StatusLogName        "TWB_STATUS"
#define PX_SrcTgtLogName        "TWB_SRCTGT"
#define PX_MetadataLogName      "TWB_EVENTS"                       /*@005*/
#define PX_MaxAttrNameLength    256
#define PX_NULL_SID             0xff
#define PX_NULL_ROWID           -1
#define PX_RETRY_LIMIT          5
#define PX_MAX_DML_GROUPS       1024
#define PX_LEN_BYTES            10
#define PX_LEN_BYTES_LOB_INLINE 8                         /*@004*/
#define PX_LEN_BYTES_NUMBER     1                         /*@007*/
#define PX_VersionStringSize    16
#define PX_MaxAnsiDateSize      10

#define PX_ExitCode_Success     0
#define PX_ExitCode_Warning     4                                 /*DR92618*/
#define PX_ExitCode_UserError   8
#define PX_ExitCode_FatalError  12

#define PX_SYSATTR_InstanceNumber     "SYSTEM_INSTANCENUMBER"
#define PX_SYSATTR_PhaseNumber        "SYSTEM_PHASENUMBER"
#define PX_SYSATTR_MaxInstances       "SYSTEM_MAXINSTANCES"
#define PX_SYSATTR_SystemCode         "SYSTEM_SYSTEMCODE"
#define PX_SYSATTR_Checkpoint         "SYSTEM_CHECKPOINT"
#define PX_SYSATTR_CheckpointObject   "SYSTEM_CHECKPOINTOBJECT"
#define PX_SYSATTR_Restart            "SYSTEM_RESTARTMODE"
#define PX_SYSATTR_OperatorType       "SYSTEM_OPERATORTYPE" /*DR52185*/
#define PX_SYSATTR_DMLOptions         "SYSTEM_DML_OPTIONS"
#define PX_SYSATTR_DMLGroups          "SYSTEM_DML_GROUPS"
#define PX_SYSATTR_TableSet           "SYSTEM_TABLE_SET"
#define PX_SYSATTR_RetryInformation   "SYSTEM_RETRY_INFORMATION"   /*@003*/
#define PX_SYSATTR_UserRestart        "SYSTEM_USER_RESTART"        /*@003*/
#define PX_SYSATTR_BufferMode         "SYSTEM_BUFFERMODE"          /*@005*/
#define PX_SYSATTR_CheckpointOff      "SYSTEM_CHECKPOINTOFF"       /*@008*/
#define PX_SYSATTR_NoCheckpoint       "SYSTEM_NOCHECKPOINT"        /*@009*/
#define PX_SYSATTR_LogDirectory       "SYSTEM_LOGDIRECTORY"        /*@010*/
#define PX_SYSATTR_InstallRootDir     "SYSTEM_INSTALLROOTDIR"      /*@011*/
#define PX_SYSATTR_LatencyOn          "SYSTEM_LATENCYON"        //TPT-29416

                                                                   /*@005*/
#define PX_BufferLengthSize           "BUFFERLENGTHSIZE"           /*@005*/
#define PX_BufferHeaderSize           "BUFFERHEADERSIZE"           /*@005*/
#define PX_BufferTrailerSize          "BUFFERTRAILERSIZE"          /*@005*/
#define PX_BufferMaxSize              "BUFFERMAXSIZE"              /*@005*/
#define PX_ProducerGroupName          "_TWB_Producer_Group_"         //@012 
#define PX_MaxRowSizeVarName          "_TWB_MaxRowSize_"             //@012 
#define PX_MaxRowSizeInfoVarName      "_TWB_MaxRowSizeInfo_"         //@012
#define PX_SYSATTR_JobMaxRowSizeAllow "SYSTEM_JOBMAXROWSIZE_ALLOW"   //@012
 
#define PX_SYSATTR_MessageSize        "SYSTEM_MESSAGESIZE"         //TPT-25646

#define PX_SYSATTR_JobMaxUsingBytes   "SYSTEM_JOBMAXUSINGBYTES"    //TPT-33903
#define PX_SYSATTR_ExceedMaxRowSize   "SYSTEM_EXCEEDMAXROWSIZE"    //TPT-33903

#endif
