/***********************************************************************/
/*                                                                     */
/* File:     pxdtypes.h                                                */
/*                                                                     */  
/* Purpose:  Hold the identifiers of all of the supported data types   */
/*           of the Teradata Parallel Transporter product, for use by  */
/*           by all TPT components that need to reference specific     */
/*           data types.                                               */
/*                                                                     */
/*           ***********************************************           */   
/*           * Copyright 1999-2011 by Teradata Corporation *           */
/*           *                                             *           */
/*           * All Rights Reserved.                        *           */
/*           *                                             *           */
/*           * TERADATA CONFIDENTIAL AND TRADE SECRET      *           */
/*           ***********************************************           */
/*                                                                     */
/*  Revision     Date   D(C)R  DID             Comments                */
/* ----------- -------- ------ --- ----------------------------------- */
/* 01.00.00.01 1999????        APY File created.                       */
/* 01.00.00.02 20060223  98401 WJS Added PX_DataType_BigInt.           */
/* 13.00.00.00 20070913 115589 APY Support for LOB                     */
/* 13.00.00.01 20071112 115896 WJS Added temporal data types; dropped  */
/*                                 obsolete BLOB & CLOB Locator types; */
/*                                 and created character string        */  
/*                                 print names for all data types      */
/* 14.00.00.00 20110330 141402 WJS Added the NUMBER data type          */ 
/* 14.00.00.01 20110708 150323 APY Added the HEX data type             */
/* 14.10.00.00 20120319 TPT142 RB  Added the XML data type             */
/* 15.00.00.00 20131014 TPT19233 RB  Added the JSON data type          */
/*                                                                     */
/***********************************************************************/

#ifndef PXDTYPES__H 
#define PXDTYPES__H

#include <limits.h>

typedef enum PX_DataType
{
  PX_DataType_None = 0,
  PX_DataType_Integer,
  PX_DataType_SmallInt,
  PX_DataType_Float,
  PX_DataType_Decimal,
  PX_DataType_Char,
  PX_DataType_ByteInt,
  PX_DataType_VarChar,
  PX_DataType_LongVarChar,
  PX_DataType_Byte,
  PX_DataType_VarByte,
  PX_DataType_Date,
  PX_DataType_Graphic,
  PX_DataType_VarGraphic,
  PX_DataType_LongVarGraphic,
  PX_DataType_Date_ANSI,
  PX_DataType_Boolean,
  PX_DataType_LongVarByte,
  PX_DataType_NullableVarChar,
  PX_DataType_NullableFloat,
  PX_DataType_NullableDecimal,
  PX_DataType_NullableChar,
  PX_DataType_NullableLongVarChar,
  PX_DataType_NullableInteger,
  PX_DataType_NullableSmallInt,
  PX_DataType_NullableByte,
  PX_DataType_NullableVarByte,
  PX_DataType_NullableLongVarByte,
  PX_DataType_NullableByteInt,
  PX_DataType_NullableDate,
  PX_DataType_NullableGraphic,
  PX_DataType_NullableVarGraphic,
  PX_DataType_NullableLongVarGraphic,
  PX_DataType_NullableDate_ANSI,
  PX_DataType_NullableBoolean,
  PX_DataType_Time,
  PX_DataType_Time_WithTimeZone,
  PX_DataType_BigInt,
  PX_DataType_NullableBigInt,
  PX_DataType_BLOB,
  PX_DataType_CLOB,
  PX_DataType_BLOB_ByName,
  PX_DataType_CLOB_ByName,
  PX_DataType_TimeStamp,
  PX_DataType_TimeStamp_WithTimeZone,
  PX_DataType_Interval_Year,
  PX_DataType_Interval_YearToMonth,
  PX_DataType_Interval_Month,
  PX_DataType_Interval_Day,
  PX_DataType_Interval_DayToHour,
  PX_DataType_Interval_DayToMinute,
  PX_DataType_Interval_DayToSecond,
  PX_DataType_Interval_Hour,
  PX_DataType_Interval_HourToMinute,
  PX_DataType_Interval_HourToSecond,
  PX_DataType_Interval_Minute,
  PX_DataType_Interval_MinuteToSecond,
  PX_DataType_Interval_Second,
  PX_DataType_Period_Date,
  PX_DataType_Period_Time,
  PX_DataType_Period_Time_WithTimeZone,
  PX_DataType_Period_TimeStamp,
  PX_DataType_Period_TimeStamp_WithTimeZone,
  PX_DataType_Number,
  PX_DataType_VarCharHex,
  PX_DataType_XML,
  PX_DataType_XML_ByName,
  PX_DataType_JSON,
  PX_DataType_JSON_ByName,

  /* Please add new data type codes before this line */

#ifdef __MVS__
  PX_DataType_MAX_ = 2147483647
#else
  PX_DataType_MAX_
#endif

} PX_DataType;
 
#if defined(PXDTYPES_STRDEF)
 
static char PX_DataType_Name[][35] =
{
  "UNTYPED",
  "INTEGER",
  "SMALLINT",
  "FLOAT",
  "DECIMAL",
  "CHAR",
  "BYTEINT",
  "VARCHAR",
  "LONG VARCHAR",
  "BYTE",
  "VARBYTE",
  "INTDATE",
  "GRAPHIC",
  "VARGRAPHIC",
  "LONG VARGRAPHIC",
  "ANSIDATE",
  "BOOLEAN",
  "LONG VARBYTE",
  "NULLABLE VARCHAR",
  "NULLABLE FLOAT",
  "NULLABLE DECIMAL",
  "NULLABLE CHAR",
  "NULLABLE LONG VARCHAR",
  "NULLABLE INTEGER",
  "NULLABLE SMALLINT",
  "NULLABLE BYTE",
  "NULLABLE VARBYTE",
  "NULLABLE LONG VARBYTE",
  "NULLABLE BYTEINT",
  "NULLABLE INTDATE",
  "NULLABLE GRAPHIC",
  "NULLABLE VARGRAPHIC",
  "NULLABLE LONG VARGRAPHIC",
  "NULLABLE ANSIDATE",
  "NULLABLE BOOLEAN", 
  "TIME",
  "TIME WITH TIME ZONE",
  "BIGINT",
  "NULLABLE BIGINT",
  "BLOB",
  "CLOB",
  "BLOB BY NAME",
  "CLOB BY NAME",
  "TIMESTAMP",
  "TIMESTAMP WITH TIME ZONE",      
  "INTERVAL YEAR",
  "INTERVAL YEAR TO MONTH",
  "INTERVAL MONTH",
  "INTERVAL DAY",
  "INTERVAL DAY TO HOUR",
  "INTERVAL DAY TO MINUTE",
  "INTERVAL DAY TO SECOND",
  "INTERVAL HOUR",
  "INTERVAL HOUR TO MINUTE",
  "INTERVAL HOUR TO SECOND",
  "INTERVAL MINUTE",
  "INTERVAL MINUTE TO SECOND",
  "INTERVAL SECOND",
  "PERIOD(INTDATE)",
  "PERIOD(TIME)",
  "PERIOD(TIME WITH TIME ZONE)",
  "PERIOD(TIMESTAMP)",
  "PERIOD(TIMESTAMP WITH TIME ZONE)",
  "NUMBER",
  "VARCHARHEX",
  "XML",
  "XML BY NAME",
  "JSON",
  "JSON BY NAME"
}; 
 
#endif
#endif
 
