/*************************************************************************
*
* Title: Paral-X Logger Definitions Header
* Copyright 2001, by Teradata Corporation.  All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:  Paral-X Logger Facility Definitions
*
* Purpose:      Logger definitions shared between users of the Operator API
*               and the Paral-X infrastructure's logging functions
*
* History Information:  Extracted from pipc_logapi.h
*
*  Revision     Date    DCR  DID               Comments
* ----------- -------- ----- ---  ---------------------------------------
* 03.02.00.01 02052002 54475 WJS  Isolate pxoper.h from PIPC
* 03.02.00.02 04242002 54475 APY  Move PX_Error type to pxoper.h
* 09.00.00.00 07102006 67814 CFL  Fix "LOG_" defines                 @001
* 13.00.00.01 10042007 115565 TSL Copyright changes
*
***************************************************************************/

#ifndef PX_LOG_DEF
#define PX_LOG_DEF

/* Log Entry Data Field Data Types */

#define EOPTYPE      0

#define INT16TYPE    1
#define INT32TYPE    2
#define INT64TYPE    3
#define CHARTYPE     4
#define STRINGTYPE   5
#define FLOAT32TYPE  6
#define FLOAT64TYPE  7

/* Log Entry Categories */

#ifndef PXLOG_DEFS                                              /*@001==>*/
#  if !defined(WIN32)
#    include <syslog.h>
#    ifdef __MVS__
#      define SYSLOG_EMERG    0
#      define SYSLOG_ALERT    1
#      define SYSLOG_CRIT     2
#      define SYSLOG_ERR      3
#      define SYSLOG_WARNING  4
#      define SYSLOG_NOTICE   5
#      define SYSLOG_INFO     6
#      define SYSLOG_DEBUG    7
#    else
#    if LOG_EMERG == 0
#      define SYSLOG_EMERG 0
#    else
#      error Check LOG_EMERG value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_ALERT == 1
#      define SYSLOG_ALERT 1
#    else
#      error Check LOG_ALERT value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_CRIT == 2
#      define SYSLOG_CRIT 2
#    else
#      error Check LOG_CRIT value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_ERR == 3
#      define SYSLOG_ERR 3
#    else
#      error Check LOG_ERR value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_WARNING == 4
#      define SYSLOG_WARNING  4
#    else
#      error Check LOG_WARNING value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_NOTICE == 5
#      define SYSLOG_NOTICE 5
#    else
#      error Check LOG_NOTICE value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_INFO == 6
#      define SYSLOG_INFO 6
#    else
#      error Check LOG_INFO value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    if LOG_DEBUG == 7
#      define SYSLOG_DEBUG 7
#    else
#      error Check LOG_DEBUG value in syslog.h and modify pxlogdef.h accordingly
#    endif
#    endif
#    undef  LOG_WARNING
#    undef  LOG_INFO
#    undef  LOG_DEBUG
#  endif
#  define PXLOG_DEFS       0
#  define PXLOG_DEBUG      1
#  define PXLOG_TRACE      2
#  define PXLOG_INFO       3
#  define PXLOG_WARNING    4
#  define PXLOG_USERERROR  5
#  define PXLOG_FATALERROR 6
#  define LOG_DEBUG        PXLOG_DEBUG
#  define LOG_TRACE        PXLOG_TRACE
#  define LOG_INFO         PXLOG_INFO
#  define LOG_WARNING      PXLOG_WARNING
#  define LOG_USERERROR    PXLOG_USERERROR
#  define LOG_FATALERROR   PXLOG_FATALERROR
#else
#  error PXLOG_DEFS already defined                             /*<==@001*/
#endif

/* Destinations. Multiple destinations are supported */

#define LOG_DEST_CONS  0x00000001
#define LOG_DEST_LOG   0x00000002

/***************************************************/
/* Macros to compose call to PX_WriteLog           */
/***************************************************/

#define LOG_INT16(val)   (int16)INT16TYPE,   (val)
#define LOG_INT32(val)   (int16)INT32TYPE,   (val)
#define LOG_INT64(val)   (int16)INT64TYPE,   (val)
#define LOG_CHAR(val)    (int16)CHARTYPE,    (val)
#define LOG_STRING(val)  (int16)STRINGTYPE,  (val)
#define LOG_FLOAT(val)   (int16)FLOAT32TYPE, (val)
#define LOG_DOUBLE(val)  (int16)FLOAT64TYPE, (val)
#define LOG_EOP          (int16)EOPTYPE
#define PXEOP            (int16)EOPTYPE

#endif
