/*************************************************************************
*
* Title: Paral-X Common Macros
*
* Copyright 1999-2012, by Teradata Corporation.  All Rights Reserved.
* TERADATA PROPRIETARY AND CONFIDENTIAL--RESTRICTED
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description: Paral-X Common Macros
*
* History Information (using JIRA issue numbers)
*
* Revision    Date     Issue      DID      Comments
* ----------- -------- ---------- -------- ----------------------------------
* 15.00.00.00 10302013 TPT-19233  RWB      Support for JSON
* 14.10.00.01 03262012 TPT-142    MDV      Support XML data types
*
*
* History Information (using DART DR number)
*
*  Revision     Date   D(C)R  DID               Comments
* ----------- -------- ------ --- ---------------------------------------
* 14.00.00.00 04032011 141402 WJS Support NUMBER data type
* 13.00.00.00 09132007 115589 APY Support for LOB
* 05.00.00.01 12033004  68229 CJS Added PX_DebugBreak() macros
* 03.00.00.01 02072001  49538 NJ  C pxoper.h support
* 01.00.00.01 11301999  00000 AY2 Initial Creation
*    
*************************************************************************/

#ifndef PXMACROS__H                                                                  
#define PXMACROS__H                                                                  

#include <string.h>  /*DR49538*/
#include <stdlib.h>  /*DR49538*/
                                                                                
#ifndef MIN                                                                     
#define MIN(A, B) (((A) < (B)) ? (A) : (B))                                     
#endif                                                                          
                                                                                
#ifndef MAX                                                                     
#define MAX(A, B) (((A) > (B)) ? (A) : (B))                                     
#endif                                                                          
                                                                                
#ifndef ROUND                                                                   
#define ROUND(X,Y) ((X) % (Y) == 0 ? (X) : ((Y) * (1 + ((X)/(Y)))))             
#endif                                                                          
                                                                                
#ifndef NULL                                                                    
#define NULL   0                                                                
#endif                                                                          

#ifndef FALSE 
#define FALSE   0
#endif

#ifndef TRUE 
#define TRUE    1  
#endif
                                                                        
#ifndef SELECT                                                                  
#define SELECT if (0)                                                           
#define WHEN(EXPR) } else if (EXPR) {                                               
#define OTHERWISE } else {                                                      
#endif 

/* TPT-142: add XML and XML_ByName to the definition */
/* TPT-19233: add JSON and JSON_ByName to the def    */
#define PX_VarDataType(DT) \
((DT == PX_DataType_VarChar                       || \
  DT == PX_DataType_NullableVarChar               || \
  DT == PX_DataType_LongVarChar                   || \
  DT == PX_DataType_NullableLongVarChar           || \
  DT == PX_DataType_VarByte                       || \
  DT == PX_DataType_BLOB                          || \
  DT == PX_DataType_CLOB                          || \
  DT == PX_DataType_BLOB_ByName                   || \
  DT == PX_DataType_CLOB_ByName                   || \
  DT == PX_DataType_XML                           || \
  DT == PX_DataType_XML_ByName                    || \
  DT == PX_DataType_JSON                          || \
  DT == PX_DataType_JSON_ByName                   || \
  DT == PX_DataType_Period_TimeStamp              || \
  DT == PX_DataType_Period_TimeStamp_WithTimeZone || \
  DT == PX_DataType_Number                        || \
  DT == PX_DataType_LongVarGraphic) ? PX_True : PX_False) 

#define PX_WriteMessage(SPEC, DATA) \
{ \
	printf("%s: ", trc_func); \
	printf(SPEC, DATA); \
}

#define PX_Malloc(LEN) malloc(LEN)

#define PX_Free(ADDR) free(ADDR)


/****************************************************/
/*                                                  */
/* Debugging Breakpoint Macros                      */
/*                                                  */
/* Example:  PX_DebugBreak("TWB_BREAK_PUTROW");     */
/*           $ export TWB_BREAK_PUTROW=Y (break)    */
/*           $ export TWB_BREAK_PUTROW=N (no break) */
/*                                                  */
/****************************************************/

#if defined(__DEBUG__)

static volatile long EnableBreak = 1;

#if defined(WIN32)

#define PX_DebugBreak(ENV) \
  if (getenv(ENV)) if (0 == strcmp(getenv(ENV), "Y")) if (EnableBreak) DebugBreak();

#else

#define PX_DebugBreak(ENV) \
  if (getenv(ENV)) if (0 == strcmp(getenv(ENV), "Y")) { while (EnableBreak > 0) sleep(15); }

#endif /* if defined(WIN32) */

#else

#define PX_DebugBreak(ENV)

#endif /* if defined(__DEBUG__) */

#endif

