/*****************************************************************************
*
* Title: Paral-X Operator Header
* Copyright 1999-2020 by Teradata Corporation.  All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description  Paral-X Operator Header
*
* Purpose
*
* History Information (using JIRA issue numbers)
*
* Revision    Date       Issue    DID      Comments
* ----------- -------- ---------- -------- ----------------------------------
* 17.00.00.04 05072020 TPT-32128  MDV      Support projection in streaming LOB
* 17.00.00.03 04142020 TPT-33656  RC185029 Solaris toolset upgrade.
* 17.00.00.02 10072019 TPT-33903  MDV      Support dynamic LOB Stream 
*
*
* History Information
*
*  Revision     Date    D(C)R   DID                  Comments
* ----------- -------- ------ -------  --------------------------------------
* 01.00.00.01 11301999  00000 AY2      Initial Creation
* 02.00.01.01 07312000  48823 ASG      OS/390 USS
* 03.00.00.01 02072001  49538 NJ       C pxoper.h support
* 03.01.00.00 06182001  56304 MW       Merge of NT port
* 03.01.00.01 10042001  56448 CSG      Merge of USS port
* 03.01.00.02 11022001  55674 AY       Support for Continuous Load Operator
* 03.02.00.01 01312002  54475 WJS      Isolate oper.h from PIPC log functions
* 03.02.00.02 04152002  58779 CSG      Merge of Solaris port
* 03.02.00.03 04182002  58780 TSL      Port to HP-UX
* 03.02.00.04 04242002  54475 APY      Move PX_Error type in pxlogdef.h to
*                                      pxoper.h
* 04.00.00.01 08092002  58786 CSG      AIX Port
* 05.00.00.00 07012004  86292 WJS      Enforce version compatibility of TWB
*                                      components
* 05.01.00.01 06092005  96132 APY      Support code re-entrancy
* 05.01.00.02 07052005  69036 SS185031 Linux Porting
* 05.01.00.03 07062005  96133 APY      Event method support
* 09.00.00.00 06292006 105254 CFL      Operator Command Interface        @001
* 13.00.00.01 10042007 115565 TSL      Copyright changes
* 13.00.00.02 10062007 115588 CFL      Buffer Mode support               @002
* 13.00.00.02 12062007 118578 APY      Support for LOB                   @003
* 13.01.00.01 12022008 124601 APY      Support for operational metadata  @004
* 13.10.00.01 11012009 134889 APY      Support for metatdata column      @005
* 13.10.00.02 05202010 137957 WJS      Support for MACROCHARSET col attr @006
* 14.00.00.00 02102011 147718 vm185012 Give full operator version to TPTAPI
* 14.00.00.01 02282011 147718 vm185012 Give full operator version to TPTAPI
* 14.10.00.00 09112012 TPT651     CFL  Mac OS X support                  @007
* 14.10.00.00 12112012 TPT13241  CFL  Turn off checkpointing             @008
* 14.10.00.01 12202011 TPT12920  CFL  Add shared memory functions        @009
* 15.00.00.00 09152013 TPT19374  APY  Turn off all checkpoints           @010
* 15.00.00.01 09232013 TPT17998  RWB  Support column attr getters        @011
* 16.00.00.01 09072016 TPT25869 rp186055 Use CNFP build macro instead of TPT_Version string
* 16.00.00.02 11082016 TPT-28922 MDV  Remove pxmdtypes.h from pxtypes.h
* 16.00.00.03 02012017 TPT29702  RJP  Add PX_GetUserName
* 17.00.00.01 03112019 TPT-33314 RC185029 Static Windows pxcommon lib.
*******************************************************************************/

#ifndef PXOPER__H
#define PXOPER__H

/* static char PXOP_Version[] = "03.01.00.00";    DR86292 */

#define PX_EXPORT

#include <stdio.h>     /* DR54475 */
#include <stdarg.h>    /* DR54475 */

#include "pxdef.h"
#include "pxstc.h"
#include "pxtypes.h"
#include "pxmacros.h"  /* DR49538 */

/* DR54475  :begin

#ifndef LOGGERFACILITY
#define LOGGERFACILITY
#endif
#include "pipcif.h"
#include "pxlogapi.h"

   DR54475  :end */

#include "pxlogdef.h"   /* DR54475 */
#include "pxversion.h"  /* Contains TWB_Component_Version value - DR86292 */

typedef void *                  PX_LogHandle;           /* DR54475 */
typedef void *                  PX_OperatorHandle;      /* DR49538 */
typedef void *                  PX_TableSchemaHandle;   /* DR49538 */
typedef void *                  PX_ColumnSchemaHandle;  /* DR49538 */
typedef PX_TableSchemaHandle    PX_SchemaHandle;
typedef PX_TableSchemaHandle   *PX_TableSchemaList;     /* DR49538 */
typedef PX_TableSchemaList      PX_SchemaList;
typedef PX_ColumnSchemaHandle  *PX_ColumnSchemaList;    /* DR49538 */

/* DR49538 :begin */

typedef enum PX_ObjectTypeCode
{
  PX_ObjectType_None        = 0,
  PX_ObjectType_Operator,
  PX_ObjectType_TableSchema,
  PX_ObjectType_Log,
  PX_ObjectType_ColumnSchema

} PX_ObjectTypeCode;

/* DR49538 :end */

/* DR49538 :begin */

#ifdef __cplusplus
extern "C" {
#endif

/* Moved the following lines from, above extern  - DR56448 */

  PXSTC_Code PX_Initiate(PX_OperatorHandle  operHandle, PX_LongInt phaseNumber);
  PXSTC_Code PX_Terminate(PX_OperatorHandle operHhandle, PX_LongInt phaseNumber);
  PXSTC_Code PX_Execute(PX_OperatorHandle operHandle, PX_LongInt phaseNumber);
  PXSTC_Code PX_Checkpoint(PX_OperatorHandle operHandle, PX_LongInt phaseNumber);
  PXSTC_Code PX_Restart(PX_OperatorHandle operHandle, PX_LongInt phaseNumber);
  PXSTC_Code PX_Event(PX_OperatorHandle operHandle, PX_LongInt phaseNumber); /* DR96133 Event method support */

/* typedef PXSTC_Code (*PXOP_FuncPtr)(void *); */

typedef PXSTC_Code (*PXOP_FuncPtr)(PX_OperatorHandle, PX_LongInt);

typedef struct
{
  char *             modName;
  char *             version;
  PX_MultiPhaseType  multiPhaseType;
  PXOP_FuncPtr       initiate;
  PXOP_FuncPtr       execute;
  PXOP_FuncPtr       terminate;
  PXOP_FuncPtr       checkpoint;
  PXOP_FuncPtr       restart;
  PXOP_FuncPtr       event;  /* DR96133 Event method support */

} PXOP_OperFuncVector;

/* DR58780 */
#if defined(__MVS__) || defined (__sun) || defined(HPUX) || defined(AIX)
PXSTC_Code PX_GetOperatorMethods(PXOP_OperFuncVector  **fv);
#endif

/*************************************************************************/
/* Typedefs for Operator API functions                                   */
/*************************************************************************/

typedef PXSTC_Code  (* PXOP_CGV)       /* PXOP_CreateGlobalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName,
   PX_Length          varLength,
   PX_Count           varCount);

typedef PXSTC_Code  (* PXOP_DGV)       /* PXOP_DeleteGlobalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName);

typedef PXSTC_Code  (* PXOP_RGV)       /* PXOP_ReadGlobalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName,
   PX_Count           index,
   PX_Addr           *addr,
   PX_Length         *varLength);

typedef PXSTC_Code  (* PXOP_UGV)       /* PXOP_UpdateGlobalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName,
   PX_Addr            addr,
   PX_Length          varLength,
   PX_Count           index);

typedef PXSTC_Code  (* PXOP_CUV)       /* PXOP_CreateUniversalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName,
   PX_Length          varLength,
   PX_Count           varCount);

typedef PXSTC_Code  (* PXOP_DUV)       /* PXOP_DeleteUniversalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName);

typedef PXSTC_Code  (* PXOP_RUV)       /* PXOP_ReadUniversalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName,
   PX_Count           index,
   PX_Addr           *addr,
   PX_Length         *varLength);

typedef PXSTC_Code  (* PXOP_UUV)       /* PXOP_UpdateUniversalVar */
  (PX_OperatorHandle  operHandle,
   PX_Name            varName,
   PX_Addr            addr,
   PX_Length          varLength,
   PX_Count           index);

typedef PXSTC_Code  (* PXOP_RCR)       /* PXOP_ReadCheckpointRecord */
  (PX_OperatorHandle  operHandle,
   PX_Addr           *addr,
   PX_Length         *length);

typedef PXSTC_Code  (* PXOP_WCR)       /* PXOP_WriteCheckpointRecord */
  (PX_OperatorHandle  operHandle,
   PX_Addr            addr,
   PX_Length          length);

typedef PX_SchemaType  (* PXOP_GITST)  /* PXOP_GetInputTableSchemaType  -  05-31-2000 */
  (PX_OperatorHandle  operHandle);

typedef PX_SchemaType  (* PXOP_GOTST)  /* PXOP_GetOutputTableSchemaType -  05-31-2000 */
  (PX_OperatorHandle  operHandle);

typedef PXSTC_Code  (* PXOP_AS)        /* PXOP_AllocateStorage */
  (PX_OperatorHandle  operHandle,
   PX_Length          length,
   PX_Addr           *addr);

typedef PXSTC_Code  (* PXOP_FS)        /* PXOP_FreeStorage */
  (PX_OperatorHandle  operHandle,
   PX_Addr            addr);

typedef PXSTC_Code  (* PXOP_SEM)       /* PXOP_SetErrorMessage */
  (PX_OperatorHandle  operHandle,
   PX_ErrorText       msg,
   PX_Length          length);

typedef PXSTC_Code  (* PXOP_BTSD)      /* PXOP_BeginTableSchemaDefinition */
  (PX_OperatorHandle      operHandle,
   PX_SchemaType          schemaType,
   PX_TableSchemaHandle  *tableSchemaHandle);

typedef PXSTC_Code  (* PXOP_GTSL)      /* PXOP_GetTableSchemaList */
  (PX_OperatorHandle    operHandle,
   PX_SchemaType        schemaType,
   PX_TableSchemaList  *tableSchemaList,
   PX_Count            *tableSchemaCount);

typedef PXSTC_Code  (* PXOP_PTSL)      /* PXOP_PutTableSchemaList */
  (PX_OperatorHandle   operHandle,
   PX_SchemaType       schemaType,
   PX_TableSchemaList  tableSchemaList,
   PX_Count            tableSchemaCount);

typedef PXSTC_Code  (* PXOP_GA)        /* PXOP_GetAttribute */
  (PX_OperatorHandle   operHandle,
   PX_Name             attrName,
   PX_AttributeValue  *value,
   PX_Length          *length);

typedef PXSTC_Code  (* PXOP_PA)        /* PXOP_PutAttribute */
  (PX_OperatorHandle  operHandle,
   PX_Name            attrName,
   PX_AttributeValue  value,
   PX_Length          length);

typedef PXSTC_Code  (* PXOP_GAA)       /* PXOP_GetArrayAttribute */
  (PX_OperatorHandle  operHandle,
   PX_Name            attrName,
   PX_Array          *value,
   PX_Count          *elementCount,
   PX_DataType       *datatype);

typedef PXSTC_Code  (* PXOP_PAA)       /* PXOP_PutArrayAttribute */
  (PX_OperatorHandle  operHandle,
   PX_Name            attrName,
   PX_Array           value,
   PX_Count           elementCount,
   PX_DataType        dataType);

typedef PXSTC_Code  (* PXOP_GDO)       /* PXOP_GetDMLOptions */
  (PX_OperatorHandle  operHandle,
   PX_UByte          *options);

typedef PX_Count  (* PXOP_GFS)        /* PXOP_GetFilterSize */
  (PX_OperatorHandle  operHandle);

typedef PX_Count  (* PXOP_GOES)        /* PXOP_GetOutputExpressionSize */
  (PX_OperatorHandle  operHandle);

typedef PXSTC_Code  (*PXOP_GFCN)       /* PXOP_GetFilterColumnName */
  (PX_OperatorHandle  operHandle,
   PX_Int             index,
   PX_ColumnName     *name,
   PX_Length         *length);

typedef PXSTC_Code  (*PXOP_EF)        /* PXOP_EvaluateFilter */
  (PX_OperatorHandle  operHandle,
   PX_Count           count,
   PX_Addr           *addrs,
   PX_Length         *lengths,
   PX_DataType       *types);

typedef PXSTC_Code  (*PXOP_GFOV)       /* PXOP_GetFilterOutputValues */
  (PX_OperatorHandle  operHandle,
   PX_Count           count,
   PX_Addr           *addrs,
   PX_Length         *lengths,
   PX_DataType       *types);

typedef PX_OperatorType  (* PXOP_GOT)  /* PXOP_GetOperatorType */
  (PX_OperatorHandle  operHandle);

typedef PX_Boolean  (* PXOP_LO)        /* PXOP_LatencyOn */
  (PX_OperatorHandle  operHandle);

typedef PXSTC_Code  (* PXOP_OCI)  /* PXOP_OpenCommandInterface */  /*@001*/
  (PX_OperatorHandle  operHandle,                                  /*@001*/
   PX_Name            cmdType);                                    /*@001*/
                                                                   /*@001*/
typedef PXSTC_Code  (* PXOP_CCI)  /* PXOP_CloseCommandInterface */ /*@001*/
  (PX_OperatorHandle  operHandle);                                 /*@001*/
                                                                   /*@001*/
typedef PXSTC_Code  (* PXOP_SO)   /* PXOP_ShmObtain */             /*@009*/
  (PX_OperatorHandle  operHandle,                                  /*@009*/
   PX_Length          varLength,                                   /*@009*/
   void **            tokenAddr);                                  /*@009*/
                                                                   /*@009*/
typedef PXSTC_Code  (* PXOP_SR)   /* PXOP_ShmRelease */            /*@009*/
  (PX_OperatorHandle  operHandle,                                  /*@009*/
   void *             token);                                      /*@009*/
                                                                   /*@009*/
typedef PXSTC_Code  (* PXOP_SA)   /* PXOP_ShmAttach */             /*@009*/
  (PX_OperatorHandle  operHandle,                                  /*@009*/
   void *             token,                                       /*@009*/
   PX_Addr            shmAddr);                                    /*@009*/

typedef PXSTC_Code  (* PXOP_GUN)  /* PXOP_GetUserName */           /*TPT-29702*/
  (PX_OperatorHandle  operHandle,
   PX_Name           *username );

/*************************************************************************/
/* Typedefs for Logger API functions - DR 54475                          */
/*************************************************************************/

typedef struct PX_err_code
{
  signed    e_class;
  signed    e_reason;
  uint32    e_syst;

} PX_Error;

typedef void (*PX_ErrorProc) (PX_Error *);

typedef PXSTC_Code  (*PXOP_LI)         /* PX_LogInit */
  (PX_OperatorHandle  operHandle,
   char              *componentName,
   PXInt32            instance,
   char              *outputLogReport,
   PX_ErrorProc       errorProc,
   PX_LogHandle      *logHandle);

typedef PXSTC_Code  (*PXOP_LW)         /* PX_WriteLog */
  (PX_LogHandle       logHandle,
   PXInt32            messageNumber,
   PXInt32            destination,
   PXInt32            category,
   va_list            argListPtr);

typedef PXSTC_Code  (*PXOP_LT)         /* PX_LogTerminate */
  (PX_LogHandle       logHandle);

typedef PXSTC_Code  (*PXOP_SLE)        /* PX_SetLogError */
  (PX_LogHandle       logHandle,
   PX_ErrorProc       errorProc);

#if defined(TPT_OMD_BUILD)              /*TPT-28922*/
typedef PXSTC_Code  (*PXOP_WML)         /* PX_WriteMetadataLog */
  (PX_OperatorHandle  operHandle,       /*@004*/
   PXOMD_Code         eventCode,        /*@004*/
   PX_DataType        dataType,         /*@004*/
   PX_Addr            dataAddr,         /*@004*/
   PX_Length          dataLength,       /*@004*/
   PXInt32            relationshipID,   /*@004*/
   PXInt32            itemIndex);       /*@004*/ 
#endif                                  /*TPT-28922*/

/*************************************************************************/
/* Typedefs for Table Schema API functions                               */
/*************************************************************************/

typedef PXSTC_Code  (* PXTBS_FCS)      /* FinalizeTableSchemaDefinition */
  (PX_TableSchemaHandle  tableSchemaHandle);

typedef PXSTC_Code  (* PXTBS_GRS)      /* GetRowSlow */
  (PX_TableSchemaHandle  tableSchemaHandle);

typedef PXSTC_Code  (* PXTBS_GR)       /* GetRow */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_Count             *itemCount,
   PX_Indicator        **nullIndicators,
   PX_ByteOffset       **offsets,
   PX_Length           **lengths,
   PX_Addr              *rowAddr,
   PX_Length            *rowLength);

typedef PXSTC_Code  (* PXTBS_GRA)      /* GetRowAttributes */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_SourceID          *sourceID,
   PX_RowID             *rowID,
   PX_Addr              *dmlGroupIndicators);

typedef PXSTC_Code  (* PXTBS_PRA)      /* PutRowAttributes */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_SourceID           sourceID,
   PX_RowID              rowID,
   PX_Addr               dmlGroupIndicators);

typedef PXSTC_Code  (* PXTBS_PR)       /* PutRow */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_Count              itemCount,
   PX_Indicator         *nullIndicators,
   PX_ByteOffset        *offsets,
   PX_Length            *lengths,
   PX_Addr               rowAddr,
   PX_Length             rowLength);

typedef PXSTC_Code  (* PXTBS_CCS)      /* CreateColumnSchema */
  (PX_TableSchemaHandle    tableSchemaHandle,
   PX_ColumnName           columnName,
   PX_DataType             dataType,
   PX_Length               dataLength,
   PX_ByteOffset           offset,
   PX_Precision            precision,
   PX_Scale                scale,
   PX_Indicator            nullIndicator,
   PX_ColumnSchemaHandle  *outColumnSchema);

typedef PXSTC_Code  (* PXTBS_CMCS)      /* CreateMetadataColumnSchema */
  (PX_TableSchemaHandle    tableSchemaHandle,
   PX_ColumnName           columnName,
   PX_DataType             dataType,
   PX_Length               dataLength,
   PX_ByteOffset           offset,
   PX_Precision            precision,
   PX_Scale                scale,
   PX_Indicator            nullIndicator,
   PX_MetadataType         metadataType,
   PX_ColumnSchemaHandle  *outColumnSchema);

typedef PXSTC_Code  (* PXTBS_CCSCS)     /* CreateMacroCharSetColumnSchema @006 begin */
  (PX_TableSchemaHandle    tableSchemaHandle,
   PX_ColumnName           columnName,
   PX_DataType             dataType,
   PX_Length               dataLength,
   PX_ByteOffset           offset,
   PX_Precision            precision,
   PX_Scale                scale,
   PX_Indicator            nullIndicator,
   PX_MacroCharSetName     macroCharSetName,
   PX_ColumnSchemaHandle  *outColumnSchema);                           /* @006 end */ 

typedef PXSTC_Code  (* PXTBS_GCL)     /* GetColumnSchemaList */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_ColumnSchemaList  *columnSchemaList,
   PX_Count             *columnSchemaCount);

typedef PXSTC_Code  (* PXTBS_GB)       /* GetBuffer */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_Indicator         *nullIndicator,
   PX_Addr              *bufferAddr,
   PX_Length            *bufferLength);

typedef PXSTC_Code  (* PXTBS_PB)       /* PutBuffer */
  (PX_TableSchemaHandle  tableSchemaHandle,
   PX_Indicator          nullIndicator,
   PX_Addr               bufferAddr,
   PX_Length             bufferLength);

typedef PX_Addr     (* PXTBS_AB)       /* AllocateBuffer */        /*@002*/
  (PX_TableSchemaHandle  tableSchemaHandle,                        /*@002*/
   PX_Length             bufferLength);                            /*@002*/
                                                                   /*@002*/
/*************************************************************************/
/* Typedefs for Column Schema API functions                              */
/*************************************************************************/

typedef PXSTC_Code  (* PXCOS_GCS)     /* GetColumnSchema */
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_ColumnName         *columnName,
   PX_DataType           *dataType,
   PX_Length             *dataLength,
   PX_ByteOffset         *offset,
   PX_Precision          *precision,
   PX_Scale              *scale,
   PX_Indicator          *nullAllowed);

typedef PXSTC_Code  (* PXCOS_GMCS)     /* GetMetadataColumnSchema  @005 */
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_ColumnName         *columnName,
   PX_DataType           *dataType,
   PX_Length             *dataLength,
   PX_ByteOffset         *offset,
   PX_Precision          *precision,
   PX_Scale              *scale,
   PX_Indicator          *nullAllowed,
   PX_MetadataType       *metadataType);

typedef PXSTC_Code  (* PXCOS_GMC)     /* GetMacroCharSet  @006 */
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_MacroCharSetName   *charsetNameAddr,
   PX_Length             *charsetNameLength);

typedef PXSTC_Code  (* PXCOS_GCV)     /* GetColumnValue */
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_Indicator          *indicator,
   PX_Addr               *dataAddr,
   PX_Length             *dataLength);

typedef PXSTC_Code  (* PXCOS_PCV)     /* PutColumnValue */
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_Indicator           indicator,
   PX_Addr                dataAddr,
   PX_Length              dataLength);

typedef PXSTC_Code  (* PXCOS_GLL)     /* GetLOBLength @003*/
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_LongLength         *lobLength);

typedef PXSTC_Code  (* PXCOS_PLL)     /* PutLOBLength @003*/
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_LongLength          lobLength);

typedef PXSTC_Code (*PXCOS_GDFS)    /* GetDateFormatStrings @007 */
  (PX_ColumnSchemaHandle  columnSchemaHandle,
   PX_FormatString* format_in,
   PX_FormatString* format_out);

typedef PXSTC_Code (*PXCOS_GATR)   /* GetAttribue @011 */
  (PX_ColumnSchemaHandle columnSchemaHandle,
   PX_Name             attrName,
   PX_AttributeValue  *value,
   PX_Length          *length);

/*************************************************************************/
/* Typedefs for the operator, table schema, and column schema objects    */
/*************************************************************************/

typedef struct PXOP_APIFV
{
  PXOP_CGV     CreateGlobalVar;
  PXOP_DGV     DeleteGlobalVar;
  PXOP_RGV     ReadGlobalVar;
  PXOP_UGV     UpdateGlobalVar;
  PXOP_CUV     CreateUniversalVar;
  PXOP_DUV     DeleteUniversalVar;
  PXOP_RUV     ReadUniversalVar;
  PXOP_UUV     UpdateUniversalVar;
  PXOP_RCR     ReadCheckpointRecord;
  PXOP_WCR     WriteCheckpointRecord;
  PXOP_GITST   GetInputTableSchemaType;     /* 05-31-2000 */
  PXOP_GOTST   GetOutputTableSchemaType;    /* 05-31-2000 */
  PXOP_AS      AllocateStorage;
  PXOP_FS      FreeStorage;
  PXOP_SEM     SetErrorMessage;
  PXOP_BTSD    BeginTableSchemaDefinition;
  PXOP_GTSL    GetTableSchemaList;
  PXOP_PTSL    PutTableSchemaList;
  PXOP_GA      GetAttribute;
  PXOP_PA      PutAttribute;
  PXOP_GAA     GetArrayAttribute;
  PXOP_PAA     PutArrayAttribute;
  PXOP_GDO     GetDMLOptions;
  PXOP_GOT     GetOperatorType;
  PXOP_GFS     GetFilterSize;
  PXOP_GOES    GetOutputExpressionSize;
  PXOP_GFCN    GetFilterColumnName;
  PXOP_EF      EvaluateFilter;
  PXOP_GFOV    GetFilterOutputValues;
  PXOP_LO      LatencyOn;
  PXOP_LI      LogInit;                     /* DR 54475 */
  PXOP_LW      WriteLog;                    /* DR 54475 */
  PXOP_LT      LogTerminate;                /* DR 54475 */
  PXOP_SLE     SetLogError;                 /* DR 54475 */
  PXOP_OCI     OpenCommandInterface;        /* @001 */
  PXOP_CCI     CloseCommandInterface;       /* @001 */
#if defined(TPT_OMD_BUILD)                  /*TPT-28922*/
  PXOP_WML     WriteMetadataLog;            /* @004 */
#endif                                      /*TPT-28922*/
  PXOP_SO      ShmObtain;                                              /*@009*/
  PXOP_SR      ShmRelease;                                             /*@009*/
  PXOP_SA      ShmAttach;                                              /*@009*/
  PXOP_GUN     GetUserName;                                            /*TPT-29702*/

} PXOP_APIFV;

typedef struct PXTBS_APIFV
{
  PXTBS_FCS    FinalizeTableSchemaDefinition;
  PXTBS_GRS    GetRowSlow;
  PXTBS_GR     GetRow;
  PXTBS_GRA    GetRowAttributes;
  PXTBS_PRA    PutRowAttributes;
  PXTBS_PR     PutRow;
  PXTBS_CCS    CreateColumnSchema;
  PXTBS_CMCS   CreateMetadataColumnSchema;      /* @005 */
  PXTBS_GCL    GetColumnSchemaList;
  PXTBS_GB     GetBuffer;
  PXTBS_PB     PutBuffer;
  PXTBS_AB     AllocateBuffer;                  /* @002 */
  PXTBS_CCSCS  CreateMacroCharSetColumnSchema;  /* @006 */

} PXTBS_APIFV;

typedef struct PXCOS_APIFV
{
  PXCOS_GCS    GetColumnSchema;
  PXCOS_GCV    GetColumnValue;
  PXCOS_PCV    PutColumnValue;
  PXCOS_GLL    GetLOBLength;            /* @003 */
  PXCOS_PLL    PutLOBLength;            /* @003 */
  PXCOS_GMCS   GetMetadataColumnSchema; /* @005 */
  PXCOS_GMC    GetMacroCharSet;         /* @006 */
  PXCOS_GDFS   GetDateFormatStrings;    /* @007 */
  PXCOS_GATR   GetAttribute;            /* @011 */
 
} PXCOS_APIFV;

typedef struct PXOP_OBJ
{
  char         eyeCatcher[4];
  long         objectType;
  void        *objectHandle;
  void        *logHandle;
  void        *storageAddr;
  void        *reserved1;
  void        *reserved2;
  void        *reserved3;
  void        *reserved4;
  PXOP_APIFV   apiFuncVector;

} PXOP_OBJ;

/* DR 54475 */
typedef struct PXLOG_OBJ
{
  char         eyeCatcher[4];
  long         objectType;
  void        *objectHandle;
  PXOP_OBJ    *operHandle;

} PXLOG_OBJ;

typedef struct PXTBS_OBJ
{
  char                   eyeCatcher[4];
  long                   objectType;
  void                  *objectHandle;
  PXTBS_APIFV            apiFuncVector;
  PX_TableSchemaHandle  *cHandles[PX_MaxTableSchemas];  /* 04-19-2000 */

} PXTBS_OBJ;

typedef struct PXCOS_OBJ
{
  char                    eyeCatcher[4];
  long                    objectType;
  void                   *objectHandle;
  PXCOS_APIFV             apiFuncVector;
  PX_ColumnSchemaHandle  *cHandles[PX_MaxRowColumns];   /* 04-19-2000 need to verify */

} PXCOS_OBJ;

/*************************************************************************/
/* #defines for Operator's macros                                        */
/*************************************************************************/

#define PX_BeginTableSchemaDefinition(OPER, TYPE, TSCHEMA) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.BeginTableSchemaDefinition(OPER, TYPE, TSCHEMA) \
 : PXSTC_BadHandle)

#define PX_ReadCheckpointRecord(OPER, ADDR, LEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.ReadCheckpointRecord(OPER, ADDR, LEN) \
 : PXSTC_BadHandle)

#define PX_WriteCheckpointRecord(OPER, ADDR, LEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.WriteCheckpointRecord(OPER, ADDR, LEN) \
 : PXSTC_BadHandle)

#define PX_CreateGlobalVar(OPER, NAME, LEN, CNT) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.CreateGlobalVar(OPER, NAME, LEN, CNT) \
 : PXSTC_BadHandle)

#define PX_DeleteGlobalVar(OPER, NAME) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.DeleteGlobalVar(OPER, NAME) \
 : PXSTC_BadHandle)

#define PX_ReadGlobalVar(OPER, NAME, IDX, ADDR, LEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.ReadGlobalVar(OPER, NAME, IDX, ADDR, LEN) \
 : PXSTC_BadHandle)

#define PX_UpdateGlobalVar(OPER, NAME, ADDR, LEN, IDX) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.UpdateGlobalVar(OPER, NAME, ADDR, LEN, IDX) \
 : PXSTC_BadHandle)

#define PX_CreateUniversalVar(OPER, NAME, LEN, CNT) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.CreateUniversalVar(OPER, NAME, LEN, CNT) \
 : PXSTC_BadHandle)

#define PX_DeleteUniversalVar(OPER, NAME) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.DeleteUniversalVar(OPER, NAME) \
 : PXSTC_BadHandle)

#define PX_ReadUniversalVar(OPER, NAME, IDX, ADDR, LEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.ReadUniversalVar(OPER, NAME, IDX, ADDR, LEN) \
 : PXSTC_BadHandle)

#define PX_UpdateUniversalVar(OPER, NAME, ADDR, LEN, IDX) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.UpdateUniversalVar(OPER, NAME, ADDR, LEN, IDX) \
 : PXSTC_BadHandle)

/*004*/
#if defined(TPT_OMD_BUILD)              /*TPT-28922*/

#define PX_WriteMetadataLog(OPER, CODE, TYPE, LEN, ADDR, RI, IDX) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.WriteMetadataLog(OPER, CODE, TYPE, LEN, ADDR, RI, IDX) \
 : PXSTC_BadHandle)

#endif                                  /*TPT-28922*/

/* 05-31-2000 */

#define PX_GetInputTableSchemaType(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetInputTableSchemaType(OPER) \
 : PX_NoSchema)

#define PX_GetOutputTableSchemaType(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetOutputTableSchemaType(OPER) \
 : PX_NoSchema)

/* End */

#define PX_AllocateStorage(OPER, LEN, ADDR) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.AllocateStorage(OPER, LEN, ADDR) \
 : PXSTC_BadHandle)

#define PX_FreeStorage(OPER, ADDR) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.FreeStorage(OPER, ADDR) \
 : PXSTC_BadHandle)

#define PX_SetErrorMessage(OPER, MSG, LEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.SetErrorMessage(OPER, MSG, LEN) \
 : PXSTC_BadHandle)

#define PX_GetTableSchemaList(OPER, TYPE, TSCHEMALIST, TSCHEMACOUNT) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetTableSchemaList(OPER, TYPE, TSCHEMALIST, TSCHEMACOUNT) \
 : PXSTC_BadHandle)

#define PX_GetAttribute(OPER, ATTR, ATTRADDR, ATTRLEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetAttribute(OPER, ATTR, ATTRADDR, ATTRLEN) \
 : PXSTC_BadHandle)

#define PX_PutAttribute(OPER, ATTR, ATTRADDR, ATTRLEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.PutAttribute(OPER, ATTR, ATTRADDR, ATTRLEN) \
 : PXSTC_BadHandle)

#define PX_GetArrayAttribute(OPER, ATTR, ATTRADDR, ATTRCNT, DTYPE) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetArrayAttribute(OPER, ATTR, ATTRADDR, ATTRCNT, DTYPE) \
 : PXSTC_BadHandle)

#define PX_PutArrayAttribute(OPER, ATTR, ATTRADDR, ATTRCNT, DTYPE) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.PutArrayAttribute(OPER, ATTR, ATTRADDR, ATTRCNT, DTYPE) \
 : PXSTC_BadHandle)

#define PX_GetDMLOptions(OPER, OPTIONS) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetDMLOptions(OPER, OPTIONS) \
 : PXSTC_BadHandle)

#define PX_GetOperatorType(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetOperatorType(OPER) \
 : PX_OperatorType_None)

#define PX_LatencyOn(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.LatencyOn(OPER) \
 : PX_False)

#define PX_GetFilterSize(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetFilterSize(OPER) \
 : PXSTC_BadHandle)

#define PX_GetOutputExpressionSize(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetOutputExpressionSize(OPER) \
 : PXSTC_BadHandle)

#define PX_GetFilterColumnName(OPER, NDX, COLNAME, LEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetFilterColumnName(OPER, NDX, COLNAME, LEN) \
 : PXSTC_BadHandle)

#define PX_EvaluateFilter(OPER, CNT, VALS, LENS, TYPES) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.EvaluateFilter(OPER, CNT, VALS, LENS, TYPES) \
 : PXSTC_BadHandle)

#define PX_GetFilterOutputValues(OPER, CNT, VALS, LENS, TYPES) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetFilterOutputValues(OPER, CNT, VALS, LENS, TYPES) \
 : PXSTC_BadHandle)

#define PX_GetStorageAddr(OPER, SADDR) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 (((*SADDR = ((PXOP_OBJ *)(OPER))->storageAddr) == NULL) ? PXSTC_NullPointer : PXSTC_Success) \
 : PXSTC_BadHandle)

#define PX_PutStorageAddr(OPER, SADDR) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 (((((PXOP_OBJ *)(OPER))->storageAddr = SADDR) == NULL) ? PXSTC_NullPointer : PXSTC_Success) \
 : PXSTC_BadHandle)

#define PX_FinalizeTableSchemaDefinition(TSCHEMA) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.FinalizeTableSchemaDefinition(TSCHEMA) \
 : PXSTC_BadHandle)

#define PX_CreateColumnSchema(TSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, CSCHEMA) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.CreateColumnSchema(TSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, CSCHEMA) \
 : PXSTC_BadHandle)

#define PX_CreateMetadataColumnSchema(TSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, MTYPE, CSCHEMA) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.CreateMetadataColumnSchema(TSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, MTYPE, CSCHEMA) \
 : PXSTC_BadHandle)
                                                              /*@006 ===>*/
#define PX_CreateMacroCharSetColumnSchema(TSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, CSET, CSCHEMA) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.CreateMacroCharSetColumnSchema(TSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, CSET, CSCHEMA) \
 : PXSTC_BadHandle)
                                                              /*===> @006*/
#define PX_GetColumnSchemaList(TSCHEMA, COLLIST, COLCNT) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.GetColumnSchemaList(TSCHEMA, COLLIST, COLCNT) \
 : PXSTC_BadHandle)

#define PX_GetRowSlow(TSCHEMA) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.GetRowSlow(TSCHEMA) \
 : PXSTC_BadHandle)

#define PX_GetRow(TSCHEMA, CNT, INDL, OFFL, LENL, RADDR, RLEN) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.GetRow(TSCHEMA, CNT, INDL, OFFL, LENL, RADDR, RLEN) \
 : PXSTC_BadHandle)

#define PX_GetRowAttributes(TSCHEMA, SID, RID, DMLMASK) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.GetRowAttributes(TSCHEMA, SID, RID, DMLMASK) \
 : PXSTC_BadHandle)

#define PX_PutRowAttributes(TSCHEMA, SID, RID, DMLMASK) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.PutRowAttributes(TSCHEMA, SID, RID, DMLMASK) \
 : PXSTC_BadHandle)

#define PX_PutRow(TSCHEMA, CNT, INDL, OFFL, LENL, RADDR, RLEN) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.PutRow(TSCHEMA, CNT, INDL, OFFL, LENL, RADDR, RLEN) \
 : PXSTC_BadHandle)

#define PX_GetBuffer(TSCHEMA, IND, BADDR, BLEN) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.GetBuffer(TSCHEMA, IND, BADDR, BLEN) \
 : PXSTC_BadHandle)

#define PX_PutBuffer(TSCHEMA, IND, BADDR, BLEN) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.PutBuffer(TSCHEMA, IND, BADDR, BLEN) \
 : PXSTC_BadHandle)
                                                              /*@002 ===>*/
#define PX_AllocateBuffer(TSCHEMA, BLEN) \
(((PXTBS_OBJ *)(TSCHEMA))->objectType == PX_ObjectType_TableSchema ? \
 ((PXTBS_OBJ *)(TSCHEMA))->apiFuncVector.AllocateBuffer(TSCHEMA, BLEN) \
 : NULL)
                                                              /*<=== @002*/
#define PX_GetColumnSchema(CSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetColumnSchema(CSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND) \
 : PXSTC_BadHandle)
                                                              /*@005 ===>*/

#define PX_GetMetadataColumnSchema(CSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, MTYPE) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetMetadataColumnSchema(CSCHEMA, COLNAME, DTYPE, DLEN, OFF, PR, SCALE, IND, MTYPE) \
 : PXSTC_BadHandle)
                                                              /*<=== @005  @006 ===>*/
#define PX_GetMacroCharSet(CSCHEMA, CSNAME, CSNAMELEN) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetMacroCharSet(CSCHEMA, CSNAME, CSNAMELEN) \
 : PXSTC_BadHandle)
                                                              /*<=== @006*/
#define PX_GetColumnValue(CSCHEMA, IND, VAL, LEN) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetColumnValue(CSCHEMA, IND, VAL, LEN) \
 : PXSTC_BadHandle)

#define PX_PutColumnValue(CSCHEMA, IND, VAL, LEN) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.PutColumnValue(CSCHEMA, IND, VAL, LEN) \
 : PXSTC_BadHandle)

#define PX_GetLOBLength(CSCHEMA, LOBLEN) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetLOBLength(CSCHEMA, LOBLEN) \
 : PXSTC_BadHandle)

#define PX_PutLOBLength(CSCHEMA, LOBLEN) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.PutLOBLength(CSCHEMA, LOBLEN) \
 : PXSTC_BadHandle)

/* @007 */
#define PX_GetDateFormatStrings(CSCHEMA, FIN, FOUT) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetDateFormatStrings(CSCHEMA, FIN, FOUT) \
 : PXSTC_BadHandle)
/* @007 */

/* @011 */
#define PX_GetColumnAttribute(CSCHEMA, NAME, ADDR, LEN) \
(((PXCOS_OBJ *)(CSCHEMA))->objectType == PX_ObjectType_ColumnSchema ? \
 ((PXCOS_OBJ *)(CSCHEMA))->apiFuncVector.GetAttribute(CSCHEMA, NAME, ADDR, LEN) \
 : PXSTC_BadHandle)
/* @011 */

/* DR 54475: begin */

#define PX_LogInit(OPER, COMP, INST, RPT, EPROC, LOGHDL) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.LogInit(OPER, COMP, INST, RPT, EPROC, LOGHDL) \
 : PXSTC_BadHandle)

#define PX_LogTerminate(LOG) \
(((PXLOG_OBJ *)(LOG))->objectType == PX_ObjectType_Log ? \
 ((PXLOG_OBJ *)(LOG))->operHandle->apiFuncVector.LogTerminate(LOG) \
 : PXSTC_BadHandle)

/* DR58780 - Solution to fix compile problem on HP-UX */

#define PX_SetLogError(LOG, EPROC) \
 ((PXLOG_OBJ *)(LOG))->operHandle->apiFuncVector.SetLogError(LOG, EPROC)

/* DR 54475: end */
                                                               /*@001 ==>*/
#define PX_OpenCommandInterface(OPER, CMDTYPE) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.OpenCommandInterface(OPER, CMDTYPE) \
 : PXSTC_BadHandle)

#define PX_CloseCommandInterface(OPER) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.CloseCommandInterface(OPER) \
 : PXSTC_BadHandle)
                                                               /*<== @001*/
                                                               /*@009 ==>*/
#define PX_ShmObtain(OPER, LEN, TOKENADDR) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.ShmObtain(OPER, LEN, TOKENADDR) \
 : PXSTC_BadHandle)
                                                               
#define PX_ShmRelease(OPER, TOKEN) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.ShmRelease(OPER, TOKEN) \
 : PXSTC_BadHandle)
                                                               
#define PX_ShmAttach(OPER, TOKEN, SHMADDR) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.ShmAttach(OPER, TOKEN, SHMADDR) \
 : PXSTC_BadHandle)

/*TPT-29702*/
#define PX_GetUserName(OPER, UNAME) \
(((PXOP_OBJ *)(OPER))->objectType == PX_ObjectType_Operator ? \
 ((PXOP_OBJ *)(OPER))->apiFuncVector.GetUserName(OPER, UNAME) \
 : PXSTC_BadHandle)

                                                               /*<== @009*/
                                                               /*@008 ==>*/
#define PX_CHECKPOINT_OFF(OPER, RC) ((RC) = PXSTC_Duplicate == ((RC) = PX_CreateUniversalVar(OPER, PX_SYSATTR_CheckpointOff, 2, 0)) ? PXSTC_Success : RC)
                                                                                 
#define PX_CHECKPOINT_ON(OPER, RC)  ((RC) = PXSTC_BadHandle == ((RC) = PX_DeleteUniversalVar(OPER, PX_SYSATTR_CheckpointOff)) ? RC : PXSTC_Success)   
                                                               /*<== @008*/
                                                               /*@010 ==>*/
#define PX_NoCheckPoint(OPER, RC) ((RC) = PXSTC_Duplicate == ((RC) = PX_CreateUniversalVar(OPER, PX_SYSATTR_NoCheckpoint, 2, 0)) ? PXSTC_Success : RC)
                                                               /*<== @010*/
#ifdef __cplusplus
} /* for extern "C" */
#endif

/* DR49538 :end */

#ifdef WIN32

#ifdef __cplusplus
extern "C" {
#endif

__declspec(dllexport)
PXSTC_Code PX_GetOperatorMethods(PXOP_OperFuncVector **fv);

#ifdef __cplusplus
}
#endif

#endif /* WIN32 */

/* DR58780 */
#if defined(__MVS__) || defined (__sun) || defined(HPUX) || defined(AIX) || defined(__linux__) || defined(__APPLE__) /*@007*/

/* DR147718 : BEGIN */
/* DR86292 - Get operator version from PX_Component_Version instead of PXOP_Version (Begin) */

#define PX_DefineOperator(OPER_NAME, MPTYPE) \
static PXOP_OperFuncVector PX_FV = \
{(char *)TWB_VER(OPER_NAME), (char *)TWB_VER(CNFP_SYNC_VERSION), MPTYPE, \
 PX_Initiate, PX_Execute, PX_Terminate, PX_Checkpoint, PX_Restart, PX_Event}; \
PXSTC_Code  PX_GetOperatorMethods(PXOP_OperFuncVector **fv) {*fv = &PX_FV; return(PXSTC_Success);}

#elif defined(_OSUNIX_)

#define PX_DefineOperator(OPER_NAME, MPTYPE) \
extern "C" PXSTC_Code PX_GetOperatorMethods(PXOP_OperFuncVector **fv); \
static PXOP_OperFuncVector PX_FV = \
{(char *)TWB_VER(OPER_NAME), (char *)TWB_VER(CNFP_SYNC_VERSION), MPTYPE, \
 PX_Initiate, PX_Execute, PX_Terminate, PX_Checkpoint, PX_Restart, PX_Event}; \
PXSTC_Code  PX_GetOperatorMethods(PXOP_OperFuncVector **fv) {*fv = &PX_FV; return(PXSTC_Success);}

#elif defined(WIN32)

#define PX_DefineOperator(OPER_NAME, MPTYPE) \
static PXOP_OperFuncVector PX_FV = \
{(char *)TWB_VER(OPER_NAME), (char *)TWB_VER(CNFP_SYNC_VERSION), MPTYPE, \
 PX_Initiate, PX_Execute, PX_Terminate, PX_Checkpoint, PX_Restart, PX_Event}; \
PXSTC_Code  PX_GetOperatorMethods(PXOP_OperFuncVector **fv) {*fv = &PX_FV; return(PXSTC_Success);}

/* DR86292 - Get operator version from PX_Component_Version instead of PXOP_Version (End) */
/* DR147718 : END */

#endif

/********************************************************************/
/* PX_LogWrite: log-writing function for users of the Operator API. */
/*              it replaces the set of actual calling arguments     */
/*              beyond 'category' with the address of the list of   */
/*              of these agruments and calls a vectored function.   */
/*              to write the log (DR 54475).                        */
/********************************************************************/

/* DR 96132 Comment out the following for the support of multi-threaded application
#ifndef _OPERATOR_MAIN_

extern PXSTC_Code
PX_LogWrite(PX_LogHandle       logHandle,
            PXInt32            messageNumber,
            PXInt32            destination,
            PXInt32            category,
            ... );

#else
*/
static PXSTC_Code
PX_LogWrite(PX_LogHandle  logHandle,
            PXInt32       messageNumber,
            PXInt32       destination,
            PXInt32       category,
            ... )
{
  va_list   argListPtr;

  va_start(argListPtr, category);

  return( (((PXLOG_OBJ *)logHandle)->operHandle)->apiFuncVector.WriteLog(logHandle,
                                                                         messageNumber,
                                                                         destination,
                                                                         category,
                                                                         argListPtr) );
}

/*
#endif
*/

#endif

