/*************************************************************************
*
* Title: Paral-X Common Data Types
*
* Copyright 1999-2020, by Teradata Corporation.  All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description  Paral-X Common Data Types
*
* Purpose
*
*
* History Information
*
*  Revision     Date   D(C)R     DID      Comments
* ----------- -------- --------- -------- ---------------------------------
* 17.00.00.02 04082020 TPT-34182 MDV      Support projection in Streaming LOB
* 17.00.00.01 03122019 TPT-18835 dl185011 METADATA(FileRecordNumber) is 
*                                         supported in schema.
* 16.00.00.01 11082016 TPT-28922 MDV Remove inclusion of pxmdtypes.h
* 14.10.00.02 10012012 TPT-580   RB  Added PX_FormatString type      @007
* 14.10.00.01 12062011 TPT-3635  CFL 64 bit support
* 14.00.00.00 04132011 141402 WJS Support NUMBER data type           @006
* 13.10.00.05 05202010 137957 WJS Support MACROCHARSET col obj attr  @005
* 13.10.00.04 11052009 134899 APY Support for metatdata columns      @004
* 13.10.00.03 10192009 123828 MDV Support TMSM
* 13.10.00.02 08302009 134189 CFL Buffer Mode enhancement            @003
* 13.10.00.01 20070826 135241 MDV Support Operational metadata for DC
* 13.00.00.03 20070913 115589 APY Support for LOB                    @002
* 13.00.00.02 10052007 115588 CFL Buffer Mode support                @001
* 13.00.00.01 10042007 115565 TSL Copyright changes
* 05.02.00.01 03202006  89715 APY Support for "pxutypes64.h" & "datatype.h"
* 05.00.00.02 12032004  68229 CJS Added PX_ExpressionType macros
* 05.00.00.01 12032004  68229 CJS Added PX_Record structure
* 03.01.00.02 06182001  56304 MW  Merge of NT port
* 03.02.00.01 03252002  54475 WJS Isolate pxoper.h from PIPC
* 03.01.00.01 03082001  52641 AY2 Support for Multiload operator
* 03.00.00.01 02072001  49538 NJ  C pxoper.h support
* 01.00.00.01 11301999  00000 AY2 Initial Creation
**************************************************************************/

#ifndef PXTYPES__H
#define PXTYPES__H

/* Make it unnecessary to comment this out under UNIX - DR54475 **

#if defined(_MSC_VER) && _MSC_VER >= 1200

   #include <list>

#else

   #include <stl.h>

#endif */

#include "pxdtypes.h"

#if defined(TPT_OMD_BUILD)                            /*TPT-28922*/
#include "pxmdtypes.h"                                 /*DR123828*/
#endif                                                /*TPT-28922*/

#include "pxutypes.h"                                                  /*TPT-3635*/

typedef void *         PX_PathHandle;

#if defined(WIN32)

   #include <windows.h>
   typedef HMODULE     PX_LibraryHandle;

#else

   typedef void *      PX_LibraryHandle;

#endif

typedef PXInt32        PX_Count;            /* DR54475 */
typedef PXInt8         PX_Boolean;          /* DR54475 */
typedef PX_Boolean     PX_TruthValue;
typedef PXUInt32       PX_Length;           /* DR54475 */
typedef PXUInt64       PX_LongLength;       /* DR115589*/
typedef PXInt16        PX_Int;              /* DR54475 */
typedef PXInt32        PX_LongInt;          /* DR54475 */
typedef PX_Boolean     PX_Indicator;
typedef PXInt32        PX_LSN;              /* DR54475 */
typedef PXInt16        PX_Precision;        /* DR54475; changed from PXUInt16 @006 */ 
typedef PXInt16        PX_Scale;            /* DR54475; changed from PXUInt16 @006 */
typedef PXUInt32       PX_ByteOffset;       /* DR54475 */
typedef const char *   PX_ColumnName;
typedef const char *   PX_TableName;        /* DR49538 */
typedef char *         PX_Name;             /* DR49538 */
typedef char *         PX_ErrorText;
typedef char *         PX_Owner;            /* DR49538 */
typedef PXUInt32       PX_RowID;            /* DR54475 */
typedef PXUInt8        PX_SourceID;         /* DR54475 */
typedef PXUInt8        PX_UByte;            /* DR54475 */
typedef char           PX_SByte;
typedef PX_SByte       PX_Byte;
typedef PX_Byte *      PX_Addr;
typedef PX_Byte **     PX_Array;
typedef PX_Byte *      PX_AttributeValue;
typedef PX_Byte *      PX_CheckpointObject;
typedef PXInt32        PX_EventCode;
typedef const char *   PX_MacroCharSetName;  /* @005 */
typedef PX_Byte *      PX_FormatString;      /* @007 */

typedef enum PX_AttributeCode
 {
   PX_AttributeCode_None = 0,
   PX_AttributeCode_InstanceNumber,
   PX_AttributeCode_Instances,
   PX_AttributeCode_PhaseNumber,
   PX_AttributeCode_SystemCode,
   PX_AttributeCode_SQL,
   PX_AttributeCode_CheckpointObject,
   PX_AttributeCode_UserAreaAddr

 } PX_AttributeCode;

typedef enum PX_SchemaType
 {
   PX_NoSchema = 0,
   PX_InputSchema,
   PX_InputSchemaAny,
   PX_OutputSchema,
   PX_OutputSchemaAny,
   PX_InputOutputSchema,
   PX_InputOutputSchemaAny,
   PX_ProjectedSchema            //TPT-34182

 } PX_SchemaType;

typedef enum PX_MultiPhaseType
 {
   PX_NoMultiPhase = 0,
   PX_MultiPhase

 } PX_MultiPhaseType;

typedef enum PX_OperatorType
 {
   PX_OperatorType_None = 0,
   PX_OperatorType_Producer,
   PX_OperatorType_Consumer,
   PX_OperatorType_Filter,
   PX_OperatorType_Projector,
   PX_OperatorType_Standalone,
   /** Please add new state code before this line **/
   PX_OperatorType_MAX_ = 0x7FFFFFFFL

 } PX_OperatorType;

typedef struct
 {
   PX_Length   length;
   PX_Addr     data;

 } PX_VarChar;

typedef struct
 {
   PX_RowID        rowID;
   unsigned char   dmlGroupIndicators[1];

 } PX_RowAttr;

typedef struct
 {
   union
   {
     PX_Length     intLength;
     char          charLength[sizeof(PX_Length)];
   } length;
   PX_Indicator  deleteByte;
   char          data[1];

 } PX_Row;

#define PX_RowStructSize (sizeof(PX_Length) + sizeof(PX_Indicator))

typedef struct
 {
   PX_Length     blockSize;
   PX_Length     maxRowLength;
   PX_Length     dataLength;
   PX_Count      rowCount;
   PX_SourceID   sourceID;
   char          data[1];

 } PX_DataBlock;

#define PX_DataBlockStructSize (sizeof(PX_Length) * 3 + sizeof(PX_Count) + sizeof(char))

typedef struct                                                     /*@001*/
  {                                                                /*@001*/
  PX_Length     UsedSize;                                          /*@001*/
  PX_Boolean    IndicatorMode;                                     /*@001*/
  } PX_DataBufferHeader;                                           /*@001*/
                                                                   /*@001*/
typedef struct                                                     /*@003*/
  {                                                                /*@003*/
  PX_Count      BuffCnt;                                           /*@003*/
  PX_Length     BuffSize;                                          /*@003*/
  } PX_MetaBufferHeader;                                           /*@003*/
                                                                   /*@003*/
#if defined(PX_RECORD_STATIC_BUFFERS)

typedef struct PX_Record
{
 /******************************/
 /* Row Attribute Members      */
 /******************************/
 PX_RowID       rowID;
 PX_SourceID    sourceID;
 PX_Byte        dmlMask      [100];

 PX_Boolean     colNulls    [2048];
 PX_ByteOffset  colOffsets  [2048];
 PX_ByteOffset  colLengths  [2048];

  PX_Count       nColumns;
  PX_Count       nNIBs;

 PX_Byte        data       [65536];
 PX_Length      length;
}
PX_Record;

#else

typedef struct PX_Record
{
 /******************************/
 /* Row Attribute Members      */
 /******************************/
 PX_RowID       rowID;
 PX_SourceID    sourceID;
 PX_Byte       *dmlMask;

 PX_Boolean    *colNulls;
 PX_ByteOffset *colOffsets;
 PX_ByteOffset *colLengths;

  PX_Count       nColumns;
  PX_Count       nNIBs;

 PX_Byte       *data;
 PX_Length      length;
}
PX_Record;

#endif    /* #if defined(PX_RECORD_STATIC_BUFFERS) */

/* DR68229 */
#define PX_ExpressionType_Filter      0
#define PX_ExpressionType_Projection  1
#define PX_ExpressionType_DML         2

/* DR123828: Move typedef enum PXOMD_Code and PXOMD_ATR[][32] */
/*           to the header file pxmdtypes.h                   */

/* @004 */
typedef enum PX_MetadataType
{
  PX_MetadataType_None             = 0,
  PX_MetadataType_FileName,
  PX_MetadataType_FileRecordNumber                                 /*TPT18835*/
} PX_MetadataType;

#if defined(PXMTYPES_STRDEF)
static char PX_MetadataType_Name[][17] =                           /*TPT18835*/
{
  "None",
  "FileName",
  "FileRecordNumber"                                               /*TPT18835*/
};

#endif

#endif
