/************************************************************************
*
* Title: pxutypes.h
*
* Copyright 2002-2007, by Teradata Corporation.  All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Purpose:     To define a set of length-invariant "universal" standard
*              numeric data types for general use throughout Teradata
*              Warehouse Builder code.
*
* Description:
*
* History Information:
*
*  Revision     Date    DCR  DID             Comments
* ----------- -------- ----- ---  ------------------------------------
* 03.02.00.01 03092002 54475 WJS  File created from pipc_os.h
* 03.02.00.02 04192002 58780 TSL  Port to HP-UX
* 04.00.00.01 08112002 58786 CSG  AIX Port
* 13.00.00.01 10042007 115565 TSL Copyright changes
* 14.10.00.00 12062011 TPT-3635  CFL 64 bit support                                 
* 14.10.00.01 08212012 TPT-651   CFL Mac OS X support                    @001       
*
***********************************************************************/

#ifndef PXUTYPES_H
#define PXUTYPES_H

/**********************************************************************/
/* MP-RAS                                                             */ 
/**********************************************************************/

#if defined(__HIGHC__)  /* The MPRAS platform indicator */

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    typedef long long              PXInt64;
    typedef unsigned long long     PXUInt64;
    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif  /* __HIGHC__ */

/**********************************************************************/
/* Windows NT                                                         */ 
/**********************************************************************/

#if defined(_MSC_VER) || defined(WIN32) || defined(__CYGWIN__) || defined(__OPENNT)

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif

#if defined(_MSC_VER) || defined(WIN32) && !defined(__CYGWIN__) && !defined(__OPENNT)

    typedef __int64                PXInt64;
    typedef unsigned __int64       PXUInt64; 
    
#else

#if defined(__CYGWIN__) || defined(__OPENNT)

    typedef long long              PXInt64;
    typedef unsigned long long     PXUInt64;

#endif  /* CYGWIN */

#endif  /* Windows NT */

/**********************************************************************/
/* OS390/USS                                                          */ 
/**********************************************************************/

#if defined(__MVS__)  /* The OS390/USS platform indicator */

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    
  #if (__IBMCPP__ >= 22060)  /*  OS/390 C/C++ V2R6 and up */

    typedef long long              PXInt64; 
    typedef unsigned long long     PXUInt64;

  #else

    typedef long int               PXInt64;
    typedef unsigned long int      PXUInt64;

  #endif  /* __IBMCPP__ */

    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif  /* __MVS__ */

/**********************************************************************/
/* Solaris                                                            */ 
/**********************************************************************/

#if defined(__SUNPRO_CC) || defined(__SUNPRO_C)  /* The Sun platform indicators */

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    typedef long long              PXInt64;
    typedef unsigned long long     PXUInt64;
    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif  /* __SUNPRO_CC */

/**********************************************************************/
/* Linux                                                              */ 
/**********************************************************************/

#if defined(__linux__)

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    typedef long long              PXInt64;
    typedef unsigned long long     PXUInt64;
    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif  /* __linux__ */

#if defined(__APPLE__)                                                 /*@001*/
                                                                       /*@001*/
    typedef char                   PXInt8;                             /*@001*/
    typedef unsigned char          PXUInt8;                            /*@001*/
    typedef short                  PXInt16;                            /*@001*/
    typedef unsigned short         PXUInt16;                           /*@001*/
    typedef int                    PXInt32;                            /*@001*/
    typedef unsigned int           PXUInt32;                           /*@001*/
    typedef long long              PXInt64;                            /*@001*/
    typedef unsigned long long     PXUInt64;                           /*@001*/
    typedef float                  PXFloat32;                          /*@001*/
    typedef double                 PXFloat64;                          /*@001*/
                                                                       /*@001*/
#endif                                                                 /*@001*/

/**********************************************************************/
/* HP-UX - DR58780                                                    */ 
/**********************************************************************/

#if defined(HPUX)

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    typedef long long              PXInt64;
    typedef unsigned long long     PXUInt64;
    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif  /* HPUX */

#if defined(AIX)

    typedef char                   PXInt8;
    typedef unsigned char          PXUInt8;
    typedef short                  PXInt16;
    typedef unsigned short         PXUInt16;
    typedef int                    PXInt32;                            /*TPT-3635*/
    typedef unsigned int           PXUInt32;                           /*TPT-3635*/
    typedef long long              PXInt64;
    typedef unsigned long long     PXUInt64;
    typedef float                  PXFloat32;
    typedef double                 PXFloat64;

#endif  /* AIX */

/**********************************************************************/
/* Provide backward compatibility                                     */ 
/**********************************************************************/

#ifndef AIX           /*AIX Port these data types are already defined in AIX*/
typedef PXInt8        int8;
typedef PXUInt8       uint8;
typedef PXInt16       int16;
typedef PXUInt16      uint16;
typedef PXInt32       int32;
typedef PXUInt32      uint32;
typedef PXInt64       int64;
typedef PXUInt64      uint64;
typedef PXFloat32     float32;
typedef PXFloat64     float64;
#endif

#ifdef  AIX
typedef PXUInt8       uint8;
typedef PXUInt16      uint16;
typedef PXUInt32      uint32;
typedef PXUInt64      uint64;
typedef PXFloat32     float32;
typedef PXFloat64     float64;
#endif
#endif

