/***************************************************************************
*
* TITLE: dcrnfyxt.c .... an example notify exit...
*
* COPYRIGHT 2020-2020. Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET.
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Purpose       As a test and an example for users.
*
* Description   stuff for notify.
*
* Notes         1) the procedure must be called "_dynamn"
*               2) should return 0 unless there is a problem.
*
* History Information
*
* Revision    Date       DR        DID       Comments
* ----------- ---------- ------    ------    -----------------------------------
* 17.00.00.01 04/14/2020 TPT33656  RC185029  Solaris toolset upgrade.
* 16.00.00.01 04/12/2017 TPT30102  rp186055  Reformat DC code base
* 15.10.00.00 08/29/2014 TPT22633  rl186024  Fix compilation issue on 64bit AIX
* 13.10.00.00 02/24/2010 136549    CFL       Initial code.
*
***************************************************************************/

#include <stdlib.h>
#include <string.h>

#ifdef WIN32
   #include <malloc.h>
#elif defined (__MVS__)
   #include <dynit.h>
#endif

#include <stdio.h>

typedef int                    int32;
typedef unsigned int           uint32;

#ifdef WIN32
   typedef unsigned __int64       uint64;
#else
   typedef unsigned long long     uint64;
#endif

#define TASKIDSIZE  32

typedef enum
{
   NDEventInitialize              = 0 ,
   NDEventDirectoryScanComplete   = 1 ,
   NDEventFileReadComplete        = 2 ,
   NDEventCheckpoint              = 3 ,
   NDEventTerminate               = 4
} NfyEvent;

/***************************************/
/* Structures for User Exit Interface  */
/***************************************/

struct DCN_Initialize
{
   int32  Event;

   uint32 Length;             /* Total length of event record, including this */
                              /* header structure.                            */
   int32  OperatorNumber;
   char   TaskID[TASKIDSIZE]; /* NULL terminated task id string               */
   uint32 OperatorHandle;
   uint32 UtilityId;
   int32  OperatorCount;

   /***************************************************************************/
   /* The following offset fields contain the byte offsets to data items in   */
   /* the event record that are null terminated strings. Given the address of */
   /* this structure, the address of such a string may be obtained from the   */
   /* offset as follows:                                                      */
   /*                                                                         */
   /* char *PtrString = (char *)PtrThisStruct + PtrThisStruct->StringOffset;  */

   uint32 JobIDOffset;        /* Byte offset of job id (has sequence number)  */
   uint32 VersionIdOffset;    /* Byte offset of version id                    */
   uint32 UtilityNameOffset;  /* Byte offset of utility name                  */
   uint32 UserStringOffset;   /* Byte offset of user string                   */

   /* End of offset fields                                                    */
   /***************************************************************************/

   char   Timestamp[26];      /* Timestamp with no terminating NULL.          */
                              /* Format is YYYY-MM-DDbHH:MM:SS.SSSSSS         */
                              /*           00000000011111111112222222         */
                              /*           12345678901234567890123456         */
};

struct DCN_DirectoryScanComplete
{
   int32  Event;

   uint32 Length;             /* Total length of event record, including this */
                              /* header structure.                            */
   int32  OperatorNumber;
   char   TaskID[TASKIDSIZE]; /* NULL terminated task id string               */
   uint32 FileCount;          /* Number of files in file list                 */

   /***************************************************************************/
   /* The following offset fields contain the byte offsets to data items in   */
   /* the event record that are null terminated strings. Given the address of */
   /* this structure, the address of such a string may be obtained from the   */
   /* offset as follows:                                                      */
   /*                                                                         */
   /* char *PtrString = (char *)PtrThisStruct + PtrThisStruct->StringOffset;  */

   uint32 ScriptNameOffset;   /* Byte offset of script name                   */
   uint32 JobNameOffset;      /* Byte offset of job name (no sequence number) */
   uint32 JobIDOffset;        /* Byte offset of job id (has sequence number)  */
   uint32 FileListOffset;     /* Byte offset of file name strings             */
                              /* Note: there are FileCount NULL terminated    */
                              /* file name strings arranged end-to-end in the */
                              /* file list                                    */

   /* End of offset fields                                                    */
   /***************************************************************************/

   char   Timestamp[26];      /* Timestamp with no terminating NULL.          */
                              /* Format is YYYY-MM-DDbHH:MM:SS.SSSSSS         */
                              /*           00000000011111111112222222         */
                              /*           12345678901234567890123456         */
};

struct DCN_FileReadComplete
{
   int32  Event;

   uint32 Length;             /* Total length of event record, including this */
                              /* header structure.                            */
   int32  OperatorNumber;
   char   TaskID[TASKIDSIZE]; /* NULL terminated task id string               */
   uint32 FileListIndex;      /* Position in the directory scan file list     */
   uint64 RecordCount;
   uint64 FileSize;

   /***************************************************************************/
   /* The following offset fields contain the byte offsets to data items in   */
   /* the event record that are null terminated strings. Given the address of */
   /* this structure, the address of such a string may be obtained from the   */
   /* offset as follows:                                                      */
   /*                                                                         */
   /* char *PtrString = (char *)PtrThisStruct + PtrThisStruct->StringOffset;  */

   uint32 ScriptNameOffset;   /* Byte offset of script name                   */
   uint32 JobNameOffset;      /* Byte offset of job name (no sequence number) */
   uint32 JobIDOffset;        /* Byte offset of job id (has sequence number)  */
   uint32 FileNameOffset;     /* Byte offset of file name                     */

   /* End of offset fields                                                    */
   /***************************************************************************/

   char   Timestamp[26];      /* Timestamp with no terminating NULL.          */
                              /* Format is YYYY-MM-DDbHH:MM:SS.SSSSSS         */
                              /*           00000000011111111112222222         */
                              /*           12345678901234567890123456         */
};

struct DCN_Checkpoint
{
   int32  Event;

   uint32 Length;             /* Total length of event record, including this */
   /* header structure.                            */
   int32  OperatorNumber;
   char   TaskID[TASKIDSIZE]; /* NULL terminated task id string               */

   /***************************************************************************/
   /* The following offset fields contain the byte offsets to data items in   */
   /* the event record that are null terminated strings. Given the address of */
   /* this structure, the address of such a string may be obtained from the   */
   /* offset as follows:                                                      */
   /*                                                                         */
   /* char *PtrString = (char *)PtrThisStruct + PtrThisStruct->StringOffset;  */

   uint32 ScriptNameOffset;   /* Byte offset of script name                   */
   uint32 JobNameOffset;      /* Byte offset of job name (no sequence number) */
   uint32 JobIDOffset;        /* Byte offset of job id (has sequence number)  */

   /* End of offset fields                                                    */
   /***************************************************************************/

   char   Timestamp[26];      /* Timestamp with no terminating NULL.          */
                              /* Format is YYYY-MM-DDbHH:MM:SS.SSSSSS         */
                              /*           00000000011111111112222222         */
                              /*           12345678901234567890123456         */
};

struct DCN_Terminate
{
   int32  Event;

   int32  OperatorNumber;
   char   TaskID[TASKIDSIZE]; /* NULL terminated task id string               */
   uint32 ReturnCode;
   char   Timestamp[26];      /* Timestamp with no terminating NULL.          */
                              /* Format is YYYY-MM-DDbHH:MM:SS.SSSSSS         */
                              /*           00000000011111111112222222         */
                              /*           12345678901234567890123456         */
};

typedef union
{
   int32 Event;
   struct DCN_Initialize            Initialize;
   struct DCN_DirectoryScanComplete DirectoryScanComplete;
   struct DCN_FileReadComplete      FileReadComplete;
   struct DCN_Checkpoint            Checkpoint;
   struct DCN_Terminate             Terminate;
} DCNotifyExitParm;

/*************************************************************************
*
*   CODE STARTS HERE
*
*************************************************************************/

#ifdef WIN32
   __declspec(dllexport) int32 _dynamn(DCNotifyExitParm *P)
#else
   int32 _dynamn(DCNotifyExitParm *P)
#endif
{
   static FILE *fp;

   if(!fp)
   {
      if(NDEventInitialize == P->Event)
      {

#ifdef __MVS__
         __dyn_t  ip;
         char oname[16] = "DD:NFYEXIT";

      /* dyninit(&ip);
         ip.__ddname = "????????";
         ip.__sysout = 'A';

         if(dynalloc(&ip))
         return 4;

         sprintf(oname, "dd:%s", ip.__ddname); */
#else
         char *jid;
         char *oname;

         jid = (char *)P + P->Initialize.JobIDOffset;

         oname = (char *)malloc(strlen(jid) + 32);

         if(!oname)
            return 4;

         sprintf(oname, "%s.dcr%d.notify.out", jid, 
                                               P->Initialize.OperatorNumber);
#endif

         fp = fopen(oname, "w");

#ifndef __MVS__
         /* free(oname); */
#endif

         if(!fp)
            return 8;
      }
      else
         return 8;
   }  /* if(!fp) */

   switch(P->Event)
   {
      case NDEventInitialize:
         fprintf(fp,
                "%.26s exit called @ Data Connector Operator initialization.\n",
                P->Initialize.Timestamp);

         fprintf(fp,
                 "              Task id:          %s\n",
                 P->Initialize.TaskID);

         fprintf(fp,
                 "              Operator number:  %d\n",
                 P->Initialize.OperatorNumber);

         fprintf(fp,
                 "              Operator count:   %d\n",
                 P->Initialize.OperatorCount);

         fprintf(fp,
                 "              Job Id:           %s\n",
                 (char *)P + P->Initialize.JobIDOffset);

         fprintf(fp,
                 "              Version Id:       %s\n",
                 (char *)P + P->Initialize.VersionIdOffset);

         fprintf(fp,
                 "              Utility Id:       %d\n",
                 P->Initialize.UtilityId);

         fprintf(fp,
                 "              Utility Name:     %s\n",
                 (char *)P + P->Initialize.UtilityNameOffset);

         fprintf(fp,
                 "              UserString:       '%s'\n",
                 (char *)P + P->Initialize.UserStringOffset);
         fprintf(fp,
                 "              Oper Handle:      0x%x\n",
                 P->Initialize.OperatorHandle);
         break;

      case NDEventDirectoryScanComplete:
      {
         uint32 i;
         char  *ptr;

         fprintf(fp,
                 "%.26s exit called @ directory scan complete.\n",
                 P->DirectoryScanComplete.Timestamp);

         fprintf(fp,
                 "              Script name:      %s\n",
                 (char *)P + P->DirectoryScanComplete.ScriptNameOffset);

         fprintf(fp,
                 "              Job name:         %s\n",
                 (char *)P + P->DirectoryScanComplete.JobNameOffset);

         fprintf(fp,
                 "              Job id:           %s\n",
                 (char *)P + P->DirectoryScanComplete.JobIDOffset);

         fprintf(fp,
                 "              File count:       %d\n",
                 P->DirectoryScanComplete.FileCount);

         for(i = 0, ptr = (char *)P + P->DirectoryScanComplete.FileListOffset;
               i < P->DirectoryScanComplete.FileCount;
               i++, ptr += strlen(ptr) + 1)
            fprintf(fp,
                    "              File %04.4d:        %s\n",
                    i + 1, ptr);

         break;
      }

      case NDEventFileReadComplete:
         fprintf(fp,
                 "%.26s exit called @ file read complete.\n",
                 P->FileReadComplete.Timestamp);

         fprintf(fp,
                 "              Script name:      %s\n",
                 (char *)P + P->FileReadComplete.ScriptNameOffset);

         fprintf(fp,
                 "              Job name:         %s\n",
                 (char *)P + P->FileReadComplete.JobNameOffset);

         fprintf(fp,
                 "              Job id:           %s\n",
                 (char *)P + P->FileReadComplete.JobIDOffset);

         fprintf(fp,
                 "              File name:        %s\n",
                 (char *)P + P->FileReadComplete.FileNameOffset);

         fprintf(fp,
#if defined (__sun) || defined (LINUX) || defined (__MVS__)
                 "              Record count:     %lld\n",
#elif WIN32
                 "              Record count:     %I64d\n",
#else
                 "              Record count:     %d\n",
#endif
                 P->FileReadComplete.RecordCount);

         fprintf(fp,
                 "              File list index:  %d\n",
                 P->FileReadComplete.FileListIndex);

         fprintf(fp,
#if defined (__sun) || defined (LINUX) || defined (__MVS__)
                 "              File size:        %lld\n",
#elif WIN32
                 "              File size:        %I64d\n",
#else
                 "              File size:        %d\n",
#endif
                 P->FileReadComplete.FileSize);

         break;

      case NDEventCheckpoint:
         fprintf(fp,
                 "%.26s exit called @ checkpoint.\n",
                 P->Checkpoint.Timestamp);

         fprintf(fp,
                 "              Script name:      %s\n",
                 (char *)P + P->Checkpoint.ScriptNameOffset);

         fprintf(fp,
                 "              Job name:         %s\n",
                 (char *)P + P->Checkpoint.JobNameOffset);

         fprintf(fp,
                 "              Job id:           %s\n",
                 (char *)P + P->Checkpoint.JobIDOffset);

         break;

      case NDEventTerminate:
         fprintf(fp,
                 "%.26s exit called @ Data Connector Operator termination: "
                 "return code %d.\n",
                 P->Terminate.Timestamp,
                 P->Terminate.ReturnCode);

         fclose(fp);

         break;
   }  /* end switch */

   return 0;
}
