/***************************************************************************
*
* TITLE: expnfyxt.c .... an example notify exit...                  
*
* COPYRIGHT 1999-2012. Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET.
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Purpose       As a test and an example for users.
*
* Description   stuff for notify.
*
* Notes         1) should probably include "expnfyxt.h"
*               2) the procedure must be called "_dynamn"
*               3) should return 0 unless there is a problem.
*
* History Information (using JIRA issue numbers)
*
* Revision    Date       Issue      DID      Comments
* ----------- ---------- ---------- -------- ----------------------------------
* 14.10.00.02 10/04/2012 TPT-14472  TSL      Support ExitEON option
* 14.10.00.01 06/28/2012 TPT-2302   TSL      Support ESS
* 14.00.00.01 11/02/2011 TPT-4978   TSL      Support 64-bit row count variables
*
* History Information (using DR numbers)
*                                                                       
* Revision    Date       DR     DID    Comments                           
* ----------- ---------- ------ ------ -----------------------------------
* 13.00.00.01 10/04/2007 115565 TSL    Copyright changes
* 05.02.00.01 01/26/2006 60328 SF3     Native 64-bit port
* 05.00.00.01 01/08/2005 91955 SF3     Fix UserString display
*                                      Display Operator Handle
* 02.04.00.01 04/02/2003 67849 SF3     Initial implementation
*
***************************************************************************/
#include <stdio.h>
#include <string.h>

typedef int            Int32;                                     /* DR60328 */
typedef unsigned int  UInt32;                                     /* DR60328 */
#ifdef WIN32
typedef unsigned __int64   UInt64;                                /* TPT-4978 */
#else
typedef unsigned long long UInt64;                                /* TPT-4978 */
#endif

#define NOTIFYID_LOAD          1
#define NOTIFYID_UPDATE        2
#define NOTIFYID_EXPORT        3
#define NOTIFYID_INSERT        4
#define NOTIFYID_STREAM        5

#define MAXVERSIONIDLEN       32
#define MAXUTILITYNAMELEN     32
#define MAXUSERNAMELEN        64
#define MAXUSERSTRLEN         80
#define MAXFILENAMELEN       256
#define MAXREQUESTLEN      32000

typedef enum {
    NXEventInitialize     = 0,
    NXEventFileInmodOpen  = 1,
    NXEventDBSRestart     = 9,
    NXEventCLIError       = 10,
    NXEventDBSError       = 11,
    NXEventexit           = 12,
    NXEventExportBegin    = 31,
    NXEventReqSubmitBegin = 32,
    NXEventReqSubmitEnd   = 33,
    NXEventReqFetchBegin  = 34,
    NXEventFileOutmodOpen = 35,
    NXEventStmtFetchBegin = 36,
    NXEventStmtFetchEnd   = 37,
    NXEventReqFetchEnd    = 38,
    NXEventExportEnd      = 39,
    NXEventBlockCount     = 40,
    NXEventStmtFetchEnd64 = 41,
    NXEventReqFetchEnd64  = 42,
    NXEventExportEnd64    = 43,
    NXEventReqSubmitEnd64 = 44,
    NXEventStmtFetchBegin64 = 45,
    NXEventBlockCount64   = 46, 
    NXEventInitializeEON  = 47
} NfyExpEvent;
 
typedef struct _EXNotifyexitParm {
   UInt32 Event;       
   union {
      struct {
         UInt32 VersionLen;
         char   VersionId[MAXVERSIONIDLEN];
         UInt32 UtilityId;
         UInt32 UtilityNameLen;
         char   UtilityName[MAXUTILITYNAMELEN];
         UInt32 UserNameLen;
         char   UserName[MAXUSERNAMELEN];
         UInt32 UserStringLen;
         char   UserString[MAXUSERSTRLEN];
         UInt32 OperatorHandle;
      }Initialize;
      struct {
         UInt32  VersionLen;
         char   *VersionId;
         UInt32  UtilityId;
         UInt32  UtilityNameLen;
         char   *UtilityName;
         UInt32  UserNameLen;
         char   *UserName;
         UInt32  UserStringLen;
         char   *UserString;
         UInt32  OperatorHandle;
      }InitializeEON;
      struct {
         UInt32 FileNameLen;
         char   FileOrInmodName[MAXFILENAMELEN];
         UInt32 dummy;
      } FileInmodOpen ;
      struct {
         UInt32 dummy;
      } DBSRestart;
      struct {
         UInt32 ErrorCode;
      } CLIError;
      struct {
         UInt32 ErrorCode;
      } DBSError;
      struct {
         UInt32 ReturnCode;
      } exit;
      struct {
         UInt32 dummy;
      } ExportBegin;
      struct {
         UInt32 RequestLen;
         char Request[MAXREQUESTLEN];
      } ReqSubmitBegin;
      struct {
         UInt32 StatementCnt;
         UInt32 BlockCnt;
      } ReqSubmitEnd;
      struct {
         UInt32 StatementCnt;
         UInt64 BlockCnt;
      } ReqSubmitEnd64;
      struct {
         UInt32 dummy;
      } ReqFetchBegin;
      struct {
         UInt32 FileNameLen;
         char   FileOrOutmodName[MAXFILENAMELEN];
      } FileOutmodOpen;
      struct {
         UInt32 StatementNo;
         UInt32 BlockCnt;
      } StmtFetchBegin;
      struct {
         UInt32 StatementNo;
         UInt64 BlockCnt;
      } StmtFetchBegin64;
      struct {
         UInt32 Records;
      } StmtFetchEnd;
      struct {
         UInt64 Records;
      } StmtFetchEnd64;
      struct {
         UInt32 RecsExported;
         UInt32 RecsRejected;
      } ReqFetchEnd;
      struct {
         UInt64 RecsExported;
         UInt64 RecsRejected;
      } ReqFetchEnd64;
      struct {
         UInt32 RecsExported;
         UInt32 RecsRejected;
      } ExportEnd;
      struct {
         UInt64 RecsExported;
         UInt64 RecsRejected;
      } ExportEnd64;
   } Vals;
} EXNotifyexitParm;
 
/*************************************************************************
*
*   CODE STARTS HERE
*
*************************************************************************/

#ifdef WIN32
__declspec(dllexport) Int32 _dynamn(EXNotifyexitParm *P)
#else
Int32 _dynamn(EXNotifyexitParm *P)
#endif
{
   char   tempstr[MAXUSERSTRLEN+1];
   static FILE *fp;

   if (!fp) {
#ifdef __MVS__
      if (!(fp = fopen("DD:NFYEXIT", "w")))
         return(1);
#else
      if (!(fp = fopen("NFYEXIT.OUT", "w")))
         return(1);
#endif
   }

    switch(P->Event) {

    case NXEventInitialize :   /* Nothing */
        fprintf(fp, "exit called @ Export Operator Initialization.\n");
        fprintf(fp,
                "              Version Id:   %s.\n",
                P->Vals.Initialize.VersionId);
        fprintf(fp,
                "              Utility Id:   %d.\n",
                P->Vals.Initialize.UtilityId);
        fprintf(fp,
                "              Utility Name: %s.\n",
                P->Vals.Initialize.UtilityName);
        fprintf(fp,
                "              User Name:    %s.\n",
                P->Vals.Initialize.UserName);
        if (P->Vals.Initialize.UserStringLen) {
           memset(tempstr, 0x0, sizeof(tempstr));
           memcpy(tempstr, P->Vals.Initialize.UserString, 
                  P->Vals.Initialize.UserStringLen);
           fprintf(fp, "              UserString:   '%s'\n", tempstr);
        }
        fprintf(fp,
                "              Oper Handle:  0x%x\n",
                P->Vals.Initialize.OperatorHandle);
        break;

    case NXEventInitializeEON :
        fprintf(fp, "exit called @ Export Operator Initialization EON.\n");
        fprintf(fp,
                "              Version Id:   %s.\n",
                P->Vals.InitializeEON.VersionId);
        fprintf(fp,
                "              Utility Id:   %d.\n",
                P->Vals.InitializeEON.UtilityId);
        fprintf(fp,
                "              Utility Name: %s.\n",
                P->Vals.InitializeEON.UtilityName);
        fprintf(fp,
                "              User Name:    %s.\n",
                P->Vals.InitializeEON.UserName);
        if (P->Vals.InitializeEON.UserStringLen)
           fprintf(fp, "              UserString:   '%s'\n",
                   P->Vals.InitializeEON.UserString);
        fprintf(fp,
                "              Oper Handle:  0x%x\n",
                P->Vals.InitializeEON.OperatorHandle);
        break;

    case NXEventFileInmodOpen:
        fprintf(fp, "exit called @ input file open: %s\n",
                P->Vals.FileInmodOpen.FileOrInmodName);
        break;

    case NXEventDBSRestart :
        fprintf(fp, "exit called @ RDBMS restart detected\n");
        break;

    case NXEventCLIError :
        fprintf(fp, "exit called @ CLI error %d\n",
                P->Vals.CLIError.ErrorCode);
        break;

    case NXEventDBSError :
        fprintf(fp, "exit called @ DBS error %d\n",
                P->Vals.DBSError.ErrorCode);
        break;

    case NXEventexit :
        fprintf(fp,   
           "exit called @ Export Operator exiting: return code %d.\n",
            P->Vals.exit.ReturnCode);
        fclose(fp);
        break;

    case NXEventExportBegin :
        fprintf(fp, "exit called @ Export beginning.\n");
        break;

    case NXEventReqSubmitBegin :
        fprintf(fp, "exit called @ request submitted: '%s'.\n",
            P->Vals.ReqSubmitBegin.Request);
        break;

    case NXEventReqSubmitEnd :
        fprintf(fp, "exit called @ request done: %d statement(s), \
%d blocks.\n",
            P->Vals.ReqSubmitEnd.StatementCnt,
            P->Vals.ReqSubmitEnd.BlockCnt);
        break;

    case NXEventReqSubmitEnd64 :
        fprintf(fp, "exit called @ request done: %d statement(s), \
%llu blocks.\n",
            P->Vals.ReqSubmitEnd64.StatementCnt,
            (UInt64)P->Vals.ReqSubmitEnd64.BlockCnt);
        break;

    case NXEventReqFetchBegin :
        fprintf(fp, "exit called @ request fetch beginning.\n");
        break;

    case NXEventFileOutmodOpen:
        fprintf(fp, "exit called @ output file open: %s\n",
                P->Vals.FileOutmodOpen.FileOrOutmodName);
        break;

    case NXEventStmtFetchBegin :
        fprintf(fp, "exit called @ statement fetch beginning: stmt #%d, \
%d blocks.\n",
            P->Vals.StmtFetchBegin.StatementNo,
            P->Vals.StmtFetchBegin.BlockCnt);
        break;

    case NXEventStmtFetchBegin64 :
        fprintf(fp, "exit called @ statement fetch beginning: stmt #%d, \
%llu blocks.\n",
            P->Vals.StmtFetchBegin64.StatementNo,
            (UInt64)P->Vals.StmtFetchBegin64.BlockCnt);
        break;

    case NXEventStmtFetchEnd :
        fprintf(fp, "exit called @ statement fetch end: %d records.\n",
            P->Vals.StmtFetchEnd.Records);
        break;

    case NXEventStmtFetchEnd64 :
        fprintf(fp, "exit called @ statement fetch end: %llu records.\n",
            (UInt64)P->Vals.StmtFetchEnd64.Records);
        break;

    case NXEventReqFetchEnd :
        fprintf(fp, "exit called @ request fetch ends: \
Records exported: %d, Records rejected: %d\n",
            P->Vals.ReqFetchEnd.RecsExported,
            P->Vals.ReqFetchEnd.RecsRejected);
        break;

    case NXEventReqFetchEnd64 :
        fprintf(fp, "exit called @ request fetch ends: \
Records exported: %llu, Records rejected: %llu\n",
            (UInt64)P->Vals.ReqFetchEnd64.RecsExported,
            (UInt64)P->Vals.ReqFetchEnd64.RecsRejected);
        break;

    case NXEventExportEnd :
        fprintf(fp, "exit called @ Export ends: \
Records exported: %d, Records rejected: %d\n",
            P->Vals.ExportEnd.RecsExported,
            P->Vals.ExportEnd.RecsRejected);
        break;

    case NXEventExportEnd64 :
        fprintf(fp, "exit called @ Export ends: \
Records exported: %llu, Records rejected: %llu\n",
            (UInt64)P->Vals.ExportEnd64.RecsExported,
            (UInt64)P->Vals.ExportEnd64.RecsRejected);
        break;
    }

    return(0);
}
