/***************************************************************************
*
* TITLE: ldnfyext.c .... an example notify exit...
*
* Copyright 1999-2012 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description  This file is a sample user exit routine for
*              processing NOTIFY events
*
* History Information (using JIRA issue numbers)
*
* Revision    Date       Issue      DID      Comments
* ----------- ---------- ---------- -------- ----------------------------------
* 14.10.00.01 10/04/2012 TPT-14472  TSL      Support ExitEON option
* 14.00.00.01 11/02/2011 TPT-3789   TSL      Support 64-bit row count variables
*
* History Information (using DR numbers)
*
* Revision    Date       DCR    DID    Comments
* ----------- ---------- ------ ------ ---------------------------------------
* 13.00.00.01 10/04/2007 115565 TSL    Copyright changes
* 05.02.00.01 01/26/2006 60325 SF3     Native 64-bit port
* 05.00.00.01 01/08/2005 91955 SF3     Fix Display UserString
* 03.04.00.02 08/01/2003 68935 TSL     Display UserString
* 03.04.00.01 04/01/2003 67847 SF3     Initial Version
*
* Notes         The entry point to this User Exit must
*               be called "_dynamn"
*
***************************************************************************/
#include <stdio.h>
#include <string.h>

typedef int             Int32;                                    /* DR60325 */
typedef unsigned int    UInt32;                                   /* DR60325 */
#ifdef WIN32
typedef unsigned __int64   UInt64;                                /* TPT-3789 */
#else
typedef unsigned long long UInt64;                                /* TPT-3789 */
#endif

typedef enum { NFEventInitialize      =  0 ,
               NFEventInmodOpen       =  1 ,
               NFEventPhaseIBegin     =  2 ,
               NFEventCheckPoint      =  3 ,
               NFEventPhaseIEnd       =  4 ,
               NFEventPhaseIIBegin    =  5 ,
               NFEventPhaseIIEnd      =  6 ,
               NFEventDropErrTableI   =  7 ,
               NFEventDropErrTableII  =  8 ,
               NFEventDBSRestart      =  9 ,
               NFEventCLIError        = 10 ,
               NFEventDBSError        = 11 ,
               NFEventExit            = 12 ,
               NFEventCheckPoint64    = 13 ,
               NFEventPhaseIEnd64     = 14 ,
               NFEventPhaseIIEnd64    = 15 ,
               NFEventDropErrTableI64 = 16 ,
               NFEventDropErrTableII64 = 17 ,                   /* TPT-14472 */
               NFEventInitializeEON    = 18 ,                   /* TPT-14472 */
               NFEventPhaseIBeginEON   = 19                     /* TPT-14472 */
} NfyLDEvent;

#define NOTIFYID_LOAD          1
#define NOTIFYID_UPDATE        2
#define NOTIFYID_EXPORT        3
#define NOTIFYID_INSERT        4
#define NOTIFYID_STREAM        5

#define MAXVERSIONIDLEN       32
#define MAXUTILITYNAMELEN     32
#define MAXUSERNAMELEN        64
#define MAXUSERSTRLEN         80
#define MAXTABLENAMELEN      128
#define MAXFILENAMELEN       256

typedef struct _LDNotifyExitParm {
   Int32 Event;
   union {
      struct {
         UInt32 VersionLen;
         char   VersionId[MAXVERSIONIDLEN];
         UInt32 UtilityId;
         UInt32 UtilityNameLen;
         char   UtilityName[MAXUTILITYNAMELEN];
         UInt32 UserNameLen;
         char   UserName[MAXUSERNAMELEN];
         UInt32 UserStringLen;
         char   UserString[MAXUSERSTRLEN];
         UInt32 OperatorHandle;
      } Initialize;
      struct {
         UInt32  VersionLen;
         char   *VersionId;
         UInt32  UtilityId;
         UInt32  UtilityNameLen;
         char   *UtilityName;
         UInt32  UserNameLen;
         char   *UserName;
         UInt32  UserStringLen;
         char   *UserString;
         UInt32  OperatorHandle;
      } InitializeEON;
      struct {
         UInt32 InmodNameLen;
         char   InmodName[MAXFILENAMELEN];
      } InmodOpen ;
      struct {
         UInt32 TableNameLen;
         char   TableName[MAXTABLENAMELEN];
         UInt32 dummy;
      } PhaseIBegin ;
      struct {
         UInt32  TableNameLen;
         char   *TableName;
         UInt32  dummy;
      } PhaseIBeginEON ;
      struct {
         UInt32 RecordCount;
      } CheckPoint;
      struct {
         UInt64 RecordCount;
      } CheckPoint64;
      struct {
         UInt32 RecsRead;
         UInt32 RecsSkipped;
         UInt32 RecsRejected;
         UInt32 RecsSent;
      } PhaseIEnd ;
      struct {
         UInt64 RecsRead;
         UInt64 RecsSkipped;
         UInt64 RecsRejected;
         UInt64 RecsSent;
      } PhaseIEnd64 ;
      struct {
         UInt32 dummy;
      } PhaseIIBegin;
      struct {
         UInt32 Inserts;
         UInt32 dummy1;
         UInt32 dummy2;
         UInt32 dummy3;
      } PhaseIIEnd;
      struct {
         UInt64 Inserts;
         UInt64 dummy1;
         UInt64 dummy2;
         UInt32 dummy3;
      } PhaseIIEnd64;
      struct {
         UInt32 Rows;
         UInt32 dummy;
      } DropErrTableI;
      struct {
         UInt64 Rows;
         UInt32 dummy;
      } DropErrTableI64;
      struct {
         UInt32 Rows;
         UInt32 dummy;
      } DropErrTableII ;
      struct {
         UInt64 Rows;
         UInt32 dummy;
      } DropErrTableII64 ;
      struct {
         UInt32 dummy;
      } DBSRestart;
      struct {
         UInt32 ErrorCode;
      } CLIError;
      struct {
         UInt32 ErrorCode;
      } DBSError;
      struct {
         UInt32 ReturnCode;
      } Exit;
   } Vals;
} LDNotifyExitParm;

/*************************************************************************
*
*   CODE STARTS HERE
*
*************************************************************************/
#ifdef WIN32
__declspec(dllexport) Int32 _dynamn(LDNotifyExitParm *P)
#else
Int32 _dynamn(LDNotifyExitParm *P)
#endif
{
   char   tempstr[MAXUSERSTRLEN+1];
   static FILE *fp;

   if (!fp) {
#ifdef __MVS__
      if (!(fp = fopen("DD:NFYEXIT", "w")))
         return(1);
#else
      if (!(fp = fopen("NFYEXIT.OUT", "w")))
         return(1);
#endif
   }

   switch(P->Event) {
     case NFEventInitialize :
        fprintf(fp,
                "exit called @ Load Operator Notify initialization.\n");
        fprintf(fp,
                "              Version Id:   %s.\n",
                P->Vals.Initialize.VersionId);
        fprintf(fp,
                "              Utility Id:   %d.\n",
                P->Vals.Initialize.UtilityId);
        fprintf(fp,
                "              Utility Name: %s.\n",
                P->Vals.Initialize.UtilityName);
        fprintf(fp,
                "              User Name:    %s.\n",
                P->Vals.Initialize.UserName);
        if (P->Vals.Initialize.UserStringLen) {
           memset(tempstr, 0x0, sizeof(tempstr));
           memcpy(tempstr, P->Vals.Initialize.UserString, 
                  P->Vals.Initialize.UserStringLen);
           fprintf(fp, "              UserString:   '%s'\n", tempstr);
        }
        fprintf(fp,
                "              Oper Handle:  0x%x\n",
                P->Vals.Initialize.OperatorHandle);
        break;

     case NFEventInitializeEON :
        fprintf(fp,
                "exit called @ Load Operator Notify initialization EON.\n");
        fprintf(fp,
                "              Version Id:   %s.\n",
                P->Vals.InitializeEON.VersionId);
        fprintf(fp,
                "              Utility Id:   %d.\n",
                P->Vals.InitializeEON.UtilityId);
        fprintf(fp,
                "              Utility Name: %s.\n",
                P->Vals.InitializeEON.UtilityName);
        fprintf(fp,
                "              User Name:    %s.\n",
                P->Vals.InitializeEON.UserName);
        if (P->Vals.InitializeEON.UserStringLen)
           fprintf(fp, "              UserString:   '%s'\n",
                   P->Vals.InitializeEON.UserString);
        fprintf(fp,
                "              Oper Handle:  0x%x\n",
                P->Vals.InitializeEON.OperatorHandle);
        break;

     case NFEventInmodOpen :
        fprintf(fp,
                "exit called @ Load Operator file/inmod open: %s.\n",
                P->Vals.InmodOpen.InmodName);
        break;

     case NFEventPhaseIBegin :
        fprintf(fp,
                "exit called @ Load Operator phase I (start) for table %s.\n",
                P->Vals.PhaseIBegin.TableName);
        break;

     case NFEventPhaseIBeginEON :
        fprintf(fp,
             "exit called @ Load Operator phase I (start EON) for table %s.\n",
             P->Vals.PhaseIBeginEON.TableName);
        break;

     case NFEventCheckPoint :
        fprintf(fp,
                "exit called @ Load Operator checkpoint : %d records loaded.\n",
                P->Vals.CheckPoint.RecordCount);
        break;

     case NFEventCheckPoint64 :
        fprintf(fp,
           "exit called @ Load Operator checkpoint : %llu records loaded.\n",
            (UInt64)P->Vals.CheckPoint64.RecordCount);
        break;

     case NFEventPhaseIEnd :
        fprintf(fp,
                "exit called @ Load Operator phase I (end).\n");
        fprintf(fp,
                "              Records read:     %d\n",
                P->Vals.PhaseIEnd.RecsRead);
        fprintf(fp,
                "              Records skipped:  %d\n",
                P->Vals.PhaseIEnd.RecsSkipped);
        fprintf(fp,
                "              Records rejected: %d\n",
                P->Vals.PhaseIEnd.RecsRejected);
        fprintf(fp,
                "              Records sent:     %d\n",
                P->Vals.PhaseIEnd.RecsSent);
        break;

     case NFEventPhaseIEnd64 :
        fprintf(fp,
                "exit called @ Load Operator phase I (end).\n");
        fprintf(fp,
                "              Records read:     %llu\n",
                (UInt64)P->Vals.PhaseIEnd64.RecsRead);
        fprintf(fp,
                "              Records skipped:  %llu\n",
                (UInt64)P->Vals.PhaseIEnd64.RecsSkipped);
        fprintf(fp,
                "              Records rejected: %llu\n",
                (UInt64)P->Vals.PhaseIEnd64.RecsRejected);
        fprintf(fp,
                "              Records sent:     %llu\n",
                (UInt64)P->Vals.PhaseIEnd64.RecsSent);
        break;

     case NFEventPhaseIIBegin :
        fprintf(fp,
                "exit called @ Load Operator phase II (start).\n");
        break;

     case NFEventPhaseIIEnd :
        fprintf(fp,
             "exit called @ Load Operator phase II (end): %d records loaded.\n",
             P->Vals.PhaseIIEnd.Inserts);
        break;

     case NFEventPhaseIIEnd64 :
        fprintf(fp,
           "exit called @ Load Operator phase II (end): %llu records loaded.\n",
           (UInt64)P->Vals.PhaseIIEnd64.Inserts);
        break;

     case NFEventDropErrTableI :
        fprintf(fp,
             "exit called @ Load Operator ET 1 Drop : %d records in table.\n",
             P->Vals.DropErrTableI.Rows);
        break;

     case NFEventDropErrTableI64 :
        fprintf(fp,
             "exit called @ Load Operator ET 1 Drop : %llu records in table.\n",
             (UInt64)P->Vals.DropErrTableI64.Rows);
        break;

     case NFEventDropErrTableII :
        fprintf(fp,
             "exit called @ Load Operator ET 2 Drop : %d records in table.\n",
             P->Vals.DropErrTableII.Rows);
        break;

     case NFEventDropErrTableII64 :
        fprintf(fp,
             "exit called @ Load Operator ET 2 Drop : %llu records in table.\n",
             (UInt64)P->Vals.DropErrTableII64.Rows);
        break;

     case NFEventDBSRestart :
        fprintf(fp,
                "exit called @ Load Operator detects DBS restart.\n");
        break;

     case NFEventCLIError :
        fprintf(fp,
                "exit called @ Load Operator detects CLI error: %d.\n",
                P->Vals.CLIError.ErrorCode);
        break;

     case NFEventDBSError :
        fprintf(fp,
                "exit called @ Load Operator detects DBS error: %d.\n",
                P->Vals.DBSError.ErrorCode);
        break;

     case NFEventExit :
	fprintf(fp,
                "exit called @ Load Operator exiting. Return code: %d.\n",
                P->Vals.Exit.ReturnCode);
        fclose(fp);
        break;
   }

   return(0);
}

/******************************************************************************
*
* End of FlNfyExt.c
*
******************************************************************************/
