/***************************************************************************
*
* TITLE: stmnfyxt.c .... an example notify exit...
*
* Purpose       As a test and an example for users.
*
* COPYRIGHT 1999-2012 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description   stuff for notify.
*
* History Information (using JIRA issue numbers)
*
* Revision    Date       Issue      DID      Comments
* ----------- ---------- ---------- -------- ----------------------------------
* 14.10.00.02 10/04/2012 TPT-14472  TSL      Support ExitEON option
* 14.10.00.01 06/28/2012 TPT-2302   TSL      Support ESS
* 14.00.00.01 11/02/2011 TPT-3893   TSL      Support 64-bit row count variables
*
* History Information (using DR numbers)
*
* Revision    Date       DR     DID    Comments
* ----------- ---------- ------ ------ -----------------------------------
* 13.00.00.01 10/03/2007 115563 TSL    Copyright changes
* 05.02.00.02 05/24/2006 60329 TSL     Move fclose to NMEventExit event
* 05.02.00.01 01/26/2006 60329 SF3     Native 64-bit port
* 05.00.00.01 01/08/2005 91955 SF3     Fix UserString display
*                                      Display Operator Handle
* 01.03.00.01 05/19/2003 67850 WTH     Notify Feature Implementation
*
* Notes       The procedure must be called "_dynamn"
*
***************************************************************************/
#include <stdio.h>
#include <string.h>

typedef int             Int32;                                    /* DR60329 */
typedef unsigned int    UInt32;                                   /* DR60329 */
#ifdef WIN32
typedef unsigned __int64   UInt64;                                /* TPT-3893 */
#else
typedef unsigned long long UInt64;                                /* TPT-3893 */
#endif

typedef enum {
    NMEventInitialize    = 0,
    NMEventFileInmodOpen = 1,
    NMEventCkptBeg       = 2,
    NMEventImportBegin   = 3,
    NMEventImportEnd     = 4,
    NMEventErrorTable    = 5,
    NMEventDBSRestart    = 6,
    NMEventCLIError      = 7,
    NMEventDBSError      = 8,
    NMEventExit          = 9,
    NMEventTableStats    = 10,
    NMEventCkptEnd       = 11,
    NMEventRunStats      = 12,
    NMEventDMLError      = 13,                                  /* TPT-3893 */
    NMEventCkptBeg64     = 14,                                  /* TPT-3893 */
    NMEventCkptEnd64     = 15,                                  /* TPT-3893 */
    NMEventErrorTable64  = 16,                                  /* TPT-3893 */
    NMEventRunStats64    = 17,                                  /* TPT-3893 */
    NMEventImportEnd64   = 18,                                  /* TPT-3893 */
    NMEventTableStats64  = 19,                        /* TPT-2302 TPT-14472 */
    NMEventInitializeEON = 20                                  /* TPT-14472 */
} NfySTMEvent;

/**************************************/
/* Structure for User Exit Interface  */
/**************************************/

#define NOTIFYID_LOAD          1
#define NOTIFYID_UPDATE        2
#define NOTIFYID_EXPORT        3
#define NOTIFYID_INSERT        4
#define NOTIFYID_STREAM        5

#define MAXVERSIONIDLEN       32
#define MAXUTILITYNAMELEN     32
#define MAXUSERNAMELEN        64
#define MAXUSERSTRLEN         80
#define MAXTABLENAMELEN      128
#define MAXFILENAMELEN       256

/**************************************/
/* Stream Operator Specific Constants */
/**************************************/

#define  STMDUPROW       9901
#define  STMMISROW       9903

typedef enum {
    DEFeedbackDefault   = 0,
    DEFeedbackNoLogging = 1
} DMLErrorFeedbackType;

typedef struct _STNotifyExitParm {
   UInt32 Event;                    /* should be NfySTMEvent values */
   union {
      struct {
         UInt32 VersionLen;
         char   VersionId[MAXVERSIONIDLEN];
         UInt32 UtilityId;
         UInt32 UtilityNameLen;
         char   UtilityName[MAXUTILITYNAMELEN];
         UInt32 UserNameLen;
         char   UserName[MAXUSERNAMELEN];
         UInt32 UserStringLen;
         char   UserString[MAXUSERSTRLEN];
         UInt32 OperatorHandle;
      } Initialize;
      struct {
         UInt32  VersionLen;
         char   *VersionId;
         UInt32  UtilityId;
         UInt32  UtilityNameLen;
         char   *UtilityName;
         UInt32  UserNameLen;
         char   *UserName;
         UInt32  UserStringLen;
         char   *UserString;
         UInt32  OperatorHandle;
      } InitializeEON;
      struct {
         UInt32 FileNameLen;
         char   FileOrInmodName[MAXFILENAMELEN];
         UInt32 ImportNo;
      } FileInmodOpen;
      struct {
         UInt32 RecordCount;
      } CheckPoint;
      struct {
         UInt64 RecordCount;
      } CheckPoint64;
      struct {
         char  *TableName;
         UInt32 Rows;
      } ErrorTable;
      struct {
         char  *TableName;
         UInt64 Rows;
      } ErrorTable64;
      struct {
         UInt32 dummy;
      } DBSRestart;
      struct {
         UInt32 ErrorCode;
      } CLIError;
      struct {
         UInt32 ErrorCode;
      } DBSError;
      struct {
         UInt32 ReturnCode;
      } Exit;
      struct {
         UInt32 ImportNo;
      } ImportBegin;
      struct {
         UInt32 ImportNo;
         UInt32 RecsRead;
         UInt32 RecsSkipped;
         UInt32 RecsRejected;
         UInt32 RecsSent;
         UInt32 RecsCausedDBSError;
      } ImportEnd;
      struct {
         UInt32 ImportNo;
         UInt64 RecsRead;
         UInt64 RecsSkipped;
         UInt64 RecsRejected;
         UInt64 RecsSent;
         UInt64 RecsCausedDBSError;
      } ImportEnd64;
      struct {
         char   StmtType;
         char  *DatabaseName;
         char  *TableOrMacroName;
         UInt32 ActivityCount;
      } TableStats;
      struct {                                                   /* TPT-2302 */
         char   StmtType;                                        /* TPT-2302 */
         char  *DatabaseName;                                    /* TPT-2302 */
         char  *TableOrMacroName;                                /* TPT-2302 */
         UInt64 ActivityCount;                                   /* TPT-2302 */
      } TableStats64;                                            /* TPT-2302 */
      struct {
         UInt32 ImportNo;
         UInt32 SQLStmtSent;
         UInt32 RequestSent;
         UInt32 RecsRead;
         UInt32 RecsSkipped;
         UInt32 RecsRejected;
         UInt32 RecsSent;
         UInt32 RecsCausedDBSError;
      } RunStats;
      struct {
         UInt32 ImportNo;
         UInt64 SQLStmtSent;
         UInt64 RequestSent;
         UInt64 RecsRead;
         UInt64 RecsSkipped;
         UInt64 RecsRejected;
         UInt64 RecsSent;
         UInt64 RecsCausedDBSError;
      } RunStats64;
      struct {
         UInt32  ImportNo;
         UInt32  ErrorCode;
         char   *ErrorMsg;
         UInt32  RecordNo;
         unsigned char InputSeqNo;
         unsigned char DMLSeqNo;
         unsigned char SMTSeqNo;
         char   *ErrorData;
         UInt32  ErrorDataLen;
         UInt32 *feedback;
       } DMLError;
   } Vals;
} STNotifyExitParm ;

/*************************************************************************
*
*   CODE STARTS HERE
*
*************************************************************************/
#ifdef WIN32
__declspec(dllexport) Int32 _dynamn(STNotifyExitParm *P)
#else
Int32 _dynamn(STNotifyExitParm *P)
#endif
{
   Int32  i;
   char   tempstr[MAXUSERSTRLEN+1];
   static FILE *fp;

   if (!fp) {
#ifdef __MVS__
      if (!(fp = fopen("DD:NFYEXIT", "w")))
         return(1);
#else
      if (!(fp = fopen("NFYEXIT.OUT", "w")))
         return(1);
#endif
   }

   switch(P->Event) {

      case NMEventInitialize :
         fprintf(fp, "exit called @ Stream Operator Notify init.\n");
         fprintf(fp,
                 "              Version Id:   %s.\n",
                 P->Vals.Initialize.VersionId);
         fprintf(fp,
                 "              Utility Id:   %d.\n",
                 P->Vals.Initialize.UtilityId);
         fprintf(fp,
                 "              Utility Name: %s.\n",
                 P->Vals.Initialize.UtilityName);
         fprintf(fp,
                 "              User Name:    %s.\n",
                 P->Vals.Initialize.UserName);
         if (P->Vals.Initialize.UserStringLen) {
            memset(tempstr, 0x0, sizeof(tempstr));
            memcpy(tempstr, P->Vals.Initialize.UserString,
                   P->Vals.Initialize.UserStringLen);
            fprintf(fp, "              UserString:   '%s'\n", tempstr);
         }
         fprintf(fp,
                 "              Oper Handle:  0x%x\n",
                 P->Vals.Initialize.OperatorHandle);
         break;

      case NMEventInitializeEON :
         fprintf(fp, "exit called @ Stream Operator Notify init EON.\n");
         fprintf(fp,
                 "              Version Id:   %s.\n",
                 P->Vals.InitializeEON.VersionId);
         fprintf(fp,
                 "              Utility Id:   %d.\n",
                 P->Vals.InitializeEON.UtilityId);
         fprintf(fp,
                 "              Utility Name: %s.\n",
                 P->Vals.InitializeEON.UtilityName);
         fprintf(fp,
                 "              User Name:    %s.\n",
                 P->Vals.InitializeEON.UserName);
         if (P->Vals.InitializeEON.UserStringLen)
            fprintf(fp, "              UserString:   '%s'\n",
                    P->Vals.InitializeEON.UserString);
         fprintf(fp,
                 "              Oper Handle:  0x%x\n",
                 P->Vals.InitializeEON.OperatorHandle);
         break;

      case NMEventFileInmodOpen :
         fprintf(fp, "exit called @ File open: import[%u]: %s\n",
                 P->Vals.FileInmodOpen.ImportNo,
                 P->Vals.FileInmodOpen.FileOrInmodName);
         break;

      case NMEventCkptBeg :
         fprintf(fp, "exit called @ Checkpoint Begin: %u records loaded.\n",
                 P->Vals.CheckPoint.RecordCount);
         break;

      case NMEventCkptBeg64 :
         fprintf(fp, "exit called @ Checkpoint Begin: %llu records loaded.\n",
                 (UInt64)P->Vals.CheckPoint64.RecordCount);
         break;

      case NMEventCkptEnd :
         fprintf(fp, "exit called @ Checkpoint End: %u records loaded.\n",
                 P->Vals.CheckPoint.RecordCount);
         break;

      case NMEventCkptEnd64 :
         fprintf(fp, "exit called @ Checkpoint End: %llu records loaded.\n",
                 (UInt64)P->Vals.CheckPoint64.RecordCount);
         break;

      case NMEventErrorTable :
         fprintf(fp,
               "exit called @ Error Table : %s. %u record(s) in table.\n",
                P->Vals.ErrorTable.TableName, P->Vals.ErrorTable.Rows);
         break;

      case NMEventErrorTable64 :
         fprintf(fp,
               "exit called @ Error Table : %s. %llu record(s) in table.\n",
                P->Vals.ErrorTable64.TableName,
                (UInt64)P->Vals.ErrorTable64.Rows);
         break;

      case NMEventDBSRestart :
         fprintf(fp, "exit called @ RDBMS restarted\n");
         break;

      case NMEventCLIError :
         fprintf(fp, "exit called @ CLI error %u\n",
                 P->Vals.CLIError.ErrorCode);
         break;

      case NMEventDBSError :
         fprintf(fp, "exit called @ DBS error %u\n",
                 P->Vals.DBSError.ErrorCode);
         break;

      case NMEventExit :
         fprintf(fp, "exit called @ Notify out of scope: return code %u.\n",
                 P->Vals.Exit.ReturnCode);
         fclose(fp);                                /* DR60329 */
         break;

      case NMEventImportBegin :
         fprintf(fp, "exit called @ Import %u starting\n",
                 P->Vals.ImportBegin.ImportNo);
         break;

      case NMEventImportEnd :
         fprintf(fp, "exit called @ Import %u ending.\n",
                      P->Vals.ImportEnd.ImportNo);
         fprintf(fp, "Records Read:               %u\n",
                 P->Vals.ImportEnd.RecsRead);
         fprintf(fp, "Records Skipped:            %u\n",
                 P->Vals.ImportEnd.RecsSkipped);
         fprintf(fp, "Records Rejected:           %u\n",
                 P->Vals.ImportEnd.RecsRejected);
         fprintf(fp, "Records Sent:               %u\n",
                 P->Vals.ImportEnd.RecsSent);
         fprintf(fp, "Records resulting in RDBMS errors %u\n",
                 P->Vals.ImportEnd.RecsCausedDBSError);
         break;

      case NMEventImportEnd64 :
         fprintf(fp, "exit called @ Import %u ending.\n",
                      P->Vals.ImportEnd64.ImportNo);
         fprintf(fp, "Records Read:               %llu\n",
                 (UInt64)P->Vals.ImportEnd64.RecsRead);
         fprintf(fp, "Records Skipped:            %llu\n",
                 (UInt64)P->Vals.ImportEnd64.RecsSkipped);
         fprintf(fp, "Records Rejected:           %llu\n",
                 (UInt64)P->Vals.ImportEnd64.RecsRejected);
         fprintf(fp, "Records Sent:               %llu\n",
                 (UInt64)P->Vals.ImportEnd64.RecsSent);
         fprintf(fp, "Records resulting in RDBMS errors %llu\n",
                 (UInt64)P->Vals.ImportEnd64.RecsCausedDBSError);
         break;

      case NMEventTableStats :
         fprintf(fp, "exit called @ Table Stats: \n");
         if(P->Vals.TableStats.StmtType == 'I') {
               fprintf(fp,"Rows Inserted    :      "
                 "%u \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 P->Vals.TableStats.ActivityCount,
                 P->Vals.TableStats.TableOrMacroName,
                 P->Vals.TableStats.DatabaseName);
         }
         if(P->Vals.TableStats.StmtType == 'U') {
               fprintf(fp,"Rows Updated     :      "
                 "%u \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 P->Vals.TableStats.ActivityCount,
                 P->Vals.TableStats.TableOrMacroName,
                 P->Vals.TableStats.DatabaseName);
         }
         if(P->Vals.TableStats.StmtType == 'D') {
               fprintf(fp,"Rows Deleted     :      "
                 "%u \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 P->Vals.TableStats.ActivityCount,
                 P->Vals.TableStats.TableOrMacroName,
                 P->Vals.TableStats.DatabaseName);
         }
         if(P->Vals.TableStats.StmtType == 'M') {
               fprintf(fp,"Rows Merged      :      "
                 "%u \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 P->Vals.TableStats.ActivityCount,
                 P->Vals.TableStats.TableOrMacroName,
                 P->Vals.TableStats.DatabaseName);
         }
        break;

      case NMEventTableStats64 :
         fprintf(fp, "exit called @ Table Stats64: \n");
         if(P->Vals.TableStats64.StmtType == 'I') {
               fprintf(fp,"Rows Inserted    :      "
                 "%llu \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 (UInt64)P->Vals.TableStats64.ActivityCount,
                 P->Vals.TableStats64.TableOrMacroName,
                 P->Vals.TableStats64.DatabaseName);
         }
         if(P->Vals.TableStats64.StmtType == 'U') {
               fprintf(fp,"Rows Updated     :      "
                 "%llu \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 (UInt64)P->Vals.TableStats64.ActivityCount,
                 P->Vals.TableStats64.TableOrMacroName,
                 P->Vals.TableStats64.DatabaseName);
         }
         if(P->Vals.TableStats64.StmtType == 'D') {
               fprintf(fp,"Rows Deleted     :      "
                 "%llu \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 (UInt64)P->Vals.TableStats64.ActivityCount,
                 P->Vals.TableStats64.TableOrMacroName,
                 P->Vals.TableStats64.DatabaseName);
         }
         if(P->Vals.TableStats64.StmtType == 'M') {
               fprintf(fp,"Rows Merged      :      "
                 "%llu \nTable/Macro Name :  %s \nDatabase Name"
                 "    :    %s \n",
                 (UInt64)P->Vals.TableStats64.ActivityCount,
                 P->Vals.TableStats64.TableOrMacroName,
                 P->Vals.TableStats64.DatabaseName);
         }
        break;

     case NMEventRunStats :
        fprintf(fp, "exit called @ Run States \n");
        fprintf(fp, "Import %u \n", P->Vals.RunStats.ImportNo);
        fprintf(fp, "Statement Sent     : %u \n"
                    "Request Sent       : %u \n"
                    "Records Read       : %u \n"
                    "Records Skipped    : %u \n"
                    "Records Rejected   : %u \n"
                    "Records Sent       : %u \n"
                    "Records resulting in RDBMS errors  : %u \n",
                    P->Vals.RunStats.SQLStmtSent,
                    P->Vals.RunStats.RequestSent,
                    P->Vals.RunStats.RecsRead,
                    P->Vals.RunStats.RecsSkipped,
                    P->Vals.RunStats.RecsRejected,
                    P->Vals.RunStats.RecsSent,
                    P->Vals.RunStats.RecsCausedDBSError);
       break;

     case NMEventRunStats64 :
        fprintf(fp, "exit called @ Run States \n");
        fprintf(fp, "Import %u \n", P->Vals.RunStats64.ImportNo);
        fprintf(fp, "Statement Sent     : %llu \n"
                    "Request Sent       : %llu \n"
                    "Records Read       : %llu \n"
                    "Records Skipped    : %llu \n"
                    "Records Rejected   : %llu \n"
                    "Records Sent       : %llu \n"
                    "Records resulting in RDBMS errors  : %llu \n",
                    (UInt64)P->Vals.RunStats64.SQLStmtSent,
                    (UInt64)P->Vals.RunStats64.RequestSent,
                    (UInt64)P->Vals.RunStats64.RecsRead,
                    (UInt64)P->Vals.RunStats64.RecsSkipped,
                    (UInt64)P->Vals.RunStats64.RecsRejected,
                    (UInt64)P->Vals.RunStats64.RecsSent,
                    (UInt64)P->Vals.RunStats64.RecsCausedDBSError);
       break;

    case NMEventDMLError :
       fprintf(fp, "exit called @ DML Error \n");
       fprintf(fp, "Import %u \n", P->Vals.DMLError.ImportNo);
       fprintf(fp, "Error code: %u\n"
                   "Error text: %s\n"
                   "Record number: %u\n"
                   "Input Sequence number: %d\n"
                   "DML number: %d\n"
                   "Statement number: %d \n"
                   "Error data length : %u \n"
                   "Received feedback : %u \n",
                   P->Vals.DMLError.ErrorCode,
                   P->Vals.DMLError.ErrorMsg,
                   P->Vals.DMLError.RecordNo,
                   P->Vals.DMLError.InputSeqNo,
                   P->Vals.DMLError.DMLSeqNo,
                   P->Vals.DMLError.SMTSeqNo,
                   P->Vals.DMLError.ErrorDataLen,
                   *(P->Vals.DMLError.feedback));
       fprintf(fp, "Error data: ");
       for (i=0 ;i<P->Vals.DMLError.ErrorDataLen; i++) {
            fprintf(fp, "%c", P->Vals.DMLError.ErrorData[i]);
       }
       fprintf(fp, "\n");
       if (P->Vals.DMLError.ErrorCode == STMDUPROW) {
          *(P->Vals.DMLError.feedback) = DEFeedbackNoLogging;
          fprintf(fp, "STMDUPROW: feedback = %u\n", DEFeedbackNoLogging);
       }
       if (P->Vals.DMLError.ErrorCode == STMMISROW) {
          *(P->Vals.DMLError.feedback) = DEFeedbackNoLogging;
          fprintf(fp, "STMMISROW: feedback = %u\n", DEFeedbackNoLogging);
       }
       break;

     default :
       fprintf(fp,"\nAn Invalid Event Passed to the Exit Routine\n");
       break;
    }
    return(0);
}
/******************************************************************************
*
* End of stmnfyxt.c
*
******************************************************************************/
