/***************************************************************************
*
* TITLE: updnfyxt.c .... an example notify exit...                  
*
* Copyright 2001-2012 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Purpose       As a test and an example for users.
*
* Description   stuff for notify.
*
* History Information (using JIRA issue numbers)
*
* Revision    Date       Issue      DID      Comments
* ----------- ---------- ---------- -------- ----------------------------------
* 14.10.00.01 10/04/2012 TPT-14472  TSL      Support ExitEON option
* 14.00.00.01 11/02/2011 TPT-174    TSL      Support 64-bit row count variables
*
* History Information (using DR numbers)
*
* Revision    Date       DR     DID    Comments                           
* ----------- ---------- ------ ------ -----------------------------------
* 13.00.00.01 10/04/2007 115565 TSL    Copyright changes
* 05.02.00.01 01/26/2006 60326 SF3     Native 64-bit port
* 05.00.00.01 01/08/2005 91955 SF3     Fix UserString display
*                                      Display operator handle
* 01.04.00.01 04/04/2003 67848 SF3     Notify Feature Implementation
*
* Notes       The procedure must be called "_dynamn"
*
***************************************************************************/
#include <stdio.h>
#include <string.h>

typedef int             Int32;                                    /* DR60326 */
typedef unsigned int    UInt32;                                   /* DR60326 */
#ifdef WIN32
typedef unsigned __int64   UInt64;                                /* TPT-174 */
#else
typedef unsigned long long UInt64;                                /* TPT-174 */
#endif

typedef enum {
   NMEventInitialize     =  0,
   NMEventFileInmodOpen  =  1,
   NMEventPhaseIBegin    =  2,
   NMEventCheckPoint     =  3,
   NMEventPhaseIEnd      =  4,
   NMEventPhaseIIBegin   =  5,
   NMEventPhaseIIEnd     =  6,
   NMEventErrorTableI    =  7,
   NMEventErrorTableII   =  8,
   NMEventDBSRestart     =  9,
   NMEventCLIError       = 10,
   NMEventDBSError       = 11,
   NMEventExit           = 12,
   NMEventAmpsDown       = 21,
   NMEventImportBegin    = 22,
   NMEventImportEnd      = 23,
   NMEventDeleteInit     = 24,
   NMEventDeleteBegin    = 25,
   NMEventDeleteEnd      = 26,
   NMEventDeleteExit     = 27,
   NMEventCheckPoint64   = 28,
   NMEventPhaseIEnd64    = 29,
   NMEventImportEnd64    = 30,
   NMEventPhaseIIEnd64   = 31,
   NMEventErrorTableI64  = 32,
   NMEventErrorTableII64 = 33,
   NMEventInitializeEON  = 34,
   NMEventPhaseIBeginEON = 35,
   NMEventDeleteBeginEON = 36
} NfyUPDEvent;

/**************************************/
/* Structure for User Exit Interface  */
/**************************************/

#define NOTIFYID_LOAD          1
#define NOTIFYID_UPDATE        2
#define NOTIFYID_EXPORT        3
#define NOTIFYID_INSERT        4
#define NOTIFYID_STREAM        5

#define MAXVERSIONIDLEN       32
#define MAXUTILITYNAMELEN     32
#define MAXUSERNAMELEN        64
#define MAXUSERSTRLEN         80
#define MAXTABLENAMELEN      128
#define MAXFILENAMELEN       256

typedef struct _UPNotifyExitParm {
   UInt32 Event;                    /* should be NfyUPDEvent values */
   union {
      struct {
         UInt32 VersionLen;
         char   VersionId[MAXVERSIONIDLEN];
         UInt32 UtilityId;
         UInt32 UtilityNameLen;
         char   UtilityName[MAXUTILITYNAMELEN];
         UInt32 UserNameLen;        
         char   UserName[MAXUSERNAMELEN];
         UInt32 UserStringLen; 
         char   UserString[MAXUSERSTRLEN];
         UInt32 OperatorHandle; 
      } Initialize;
      struct {
         UInt32  VersionLen;
         char   *VersionId;
         UInt32  UtilityId;
         UInt32  UtilityNameLen;
         char   *UtilityName;
         UInt32  UserNameLen;        
         char   *UserName;
         UInt32  UserStringLen; 
         char   *UserString;
         UInt32  OperatorHandle; 
      } InitializeEON;
      struct {
         UInt32 FileNameLen;
         char   FileOrInmodName[MAXFILENAMELEN];
         UInt32 ImportNo;
      } FileInmodOpen ;
      struct {
         UInt32 TableNameLen;
         char   TableName[MAXTABLENAMELEN];
         UInt32 TableNo;
      } PhaseIBegin;
      struct {
         UInt32  TableNameLen;
         char   *TableName;
         UInt32  TableNo;
      } PhaseIBeginEON;
      struct {
         UInt32 RecordCount;
      } CheckPoint;
      struct {
         UInt64 RecordCount;
      } CheckPoint64;
      struct {
         UInt32 RecsRead;
         UInt32 RecsSkipped;
         UInt32 RecsRejected;
         UInt32 RecsSent;
      } PhaseIEnd ;
      struct {
         UInt64 RecsRead;
         UInt64 RecsSkipped;
         UInt64 RecsRejected;
         UInt64 RecsSent;
      } PhaseIEnd64 ;
      struct {
         UInt32 dummy;
      } PhaseIIBegin;
      struct {
         UInt32 Inserts;
         UInt32 Updates;
         UInt32 Deletes;
         UInt32 TableNo;
      } PhaseIIEnd;
      struct {
         UInt64 Inserts;
         UInt64 Updates;
         UInt64 Deletes;
         UInt32 TableNo;
      } PhaseIIEnd64;
      struct {
         UInt32 Rows;
         UInt32 TableNo;
      } ErrorTableI;
      struct {
         UInt64 Rows;
         UInt32 TableNo;
      } ErrorTableI64;
      struct {
         UInt32 Rows;
         UInt32 TableNo;
      } ErrorTableII ;
      struct {
         UInt64 Rows;
         UInt32 TableNo;
      } ErrorTableII64 ;
      struct {
         UInt32 dummy;
      } DBSRestart;
      struct {
         UInt32 ErrorCode;
      } CLIError;
      struct {
         UInt32 ErrorCode;
      } DBSError;
      struct {
         UInt32 ReturnCode;
      } Exit;
      struct {
         UInt32 dummy;
      } AmpsDown;
      struct {
         UInt32 ImportNo;
      } ImportBegin ;
      struct {
         UInt32 RecsRead;
         UInt32 RecsSkipped;
         UInt32 RecsRejected;
         UInt32 RecsSent;
         UInt32 ImportNo;
      } ImportEnd ;
      struct {
         UInt64 RecsRead;
         UInt64 RecsSkipped;
         UInt64 RecsRejected;
         UInt64 RecsSent;
         UInt32 ImportNo;
      } ImportEnd64 ;
      struct {
         UInt32 dummy;
      } DeleteInit;
      struct {
         UInt32 TableNameLen;
         char   TableName[MAXTABLENAMELEN];
         UInt32 TableNo;
      } DeleteBegin;
      struct {
         UInt32  TableNameLen;
         char   *TableName;
         UInt32  TableNo;
      } DeleteBeginEON;
      struct {
         UInt32 Deletes;
         UInt32 TableNo;
      } DeleteEnd;
      struct {
         UInt32 ReturnCode;
      } DeleteExit;
   } Vals;
} UPNotifyExitParm;
 
/*************************************************************************
*
*   CODE STARTS HERE
*
*************************************************************************/
#ifdef WIN32
__declspec(dllexport) Int32 _dynamn(UPNotifyExitParm *P)
#else
Int32 _dynamn(UPNotifyExitParm *P)
#endif
{
   char   tempstr[MAXUSERSTRLEN+1];
   static FILE *fp;

   if (!fp) {
#ifdef __MVS__
      if (!(fp = fopen("DD:NFYEXIT", "w")))
         return(1);
#else
      if (!(fp = fopen("NFYEXIT.OUT", "w")))
         return(1);
#endif
   }

   switch(P->Event) {

      case NMEventInitialize :
         fprintf(fp, "exit called @ Update Operator Notify init.\n");
         fprintf(fp,
                 "              Version Id:   %s.\n",
                 P->Vals.Initialize.VersionId);
         fprintf(fp,
                 "              Utility Id:   %d.\n",
                 P->Vals.Initialize.UtilityId);
         fprintf(fp,
                 "              Utility Name: %s.\n",
                 P->Vals.Initialize.UtilityName);
         fprintf(fp,
                 "              User Name:    %s.\n",
                 P->Vals.Initialize.UserName);
         if (P->Vals.Initialize.UserStringLen) {
            memset(tempstr, 0x0, sizeof(tempstr));
            memcpy(tempstr, P->Vals.Initialize.UserString, 
                   P->Vals.Initialize.UserStringLen);
            fprintf(fp, "              UserString:   '%s'\n", tempstr);
         }
         fprintf(fp,
                 "              Oper Handle:  0x%x\n",
                 P->Vals.Initialize.OperatorHandle);
         break;

      case NMEventInitializeEON :
         fprintf(fp, "exit called @ Update Operator Notify init EON.\n");
         fprintf(fp,
                 "              Version Id:   %s.\n",
                 P->Vals.InitializeEON.VersionId);
         fprintf(fp,
                 "              Utility Id:   %d.\n",
                 P->Vals.InitializeEON.UtilityId);
         fprintf(fp,
                 "              Utility Name: %s.\n",
                 P->Vals.InitializeEON.UtilityName);
         fprintf(fp,
                 "              User Name:    %s.\n",
                 P->Vals.InitializeEON.UserName);
         if (P->Vals.InitializeEON.UserStringLen)
            fprintf(fp, "              UserString:   '%s'\n",
                    P->Vals.InitializeEON.UserString);
         fprintf(fp,
                 "              Oper Handle:  0x%x\n",
                 P->Vals.InitializeEON.OperatorHandle);
         break;

      case NMEventFileInmodOpen:
         fprintf(fp, "exit called @ File open: import[%d]: %s\n",
                 P->Vals.FileInmodOpen.ImportNo,
                 P->Vals.FileInmodOpen.FileOrInmodName);
         break;
 
      case NMEventPhaseIBegin :
         fprintf(fp, "exit called @ Acquistion start: tablename[%d] : %s.\n",
                 P->Vals.PhaseIBegin.TableNo,
                 P->Vals.PhaseIBegin.TableName);
         break;

      case NMEventPhaseIBeginEON :
       fprintf(fp, "exit called @ Acquistion start EON: tablename[%d] : %s.\n",
               P->Vals.PhaseIBeginEON.TableNo,
               P->Vals.PhaseIBeginEON.TableName);
         break;

      case NMEventCheckPoint :
         fprintf(fp, "exit called @ Checkpoint : %d records loaded.\n",
                 P->Vals.CheckPoint.RecordCount);
         break;
 
      case NMEventCheckPoint64 :
         fprintf(fp, "exit called @ Checkpoint : %llu records loaded.\n",
                 (UInt64)P->Vals.CheckPoint64.RecordCount);
         break;
 
      case NMEventPhaseIEnd :
         fprintf(fp, "exit called @ Acquistion end.\n");
         fprintf(fp, "   Records Read:     %d\n", 
                 P->Vals.PhaseIEnd.RecsRead);
         fprintf(fp, "   Records Skipped:  %d\n", 
                 P->Vals.PhaseIEnd.RecsSkipped);
         fprintf(fp, "   Records Rejected: %d\n", 
                 P->Vals.PhaseIEnd.RecsRejected);
         fprintf(fp, "   Records Sent:     %d\n", 
                 P->Vals.PhaseIEnd.RecsSent);
         break;
 
      case NMEventPhaseIEnd64 :
         fprintf(fp, "exit called @ Acquistion end.\n");
         fprintf(fp, "   Records Read:     %llu\n", 
                 (UInt64)P->Vals.PhaseIEnd64.RecsRead);
         fprintf(fp, "   Records Skipped:  %llu\n", 
                 (UInt64)P->Vals.PhaseIEnd64.RecsSkipped);
         fprintf(fp, "   Records Rejected: %llu\n", 
                 (UInt64)P->Vals.PhaseIEnd64.RecsRejected);
         fprintf(fp, "   Records Sent:     %llu\n", 
                 (UInt64)P->Vals.PhaseIEnd64.RecsSent);
         break;
 
      case NMEventPhaseIIBegin :
         fprintf(fp, "exit called @ Application start\n");
         break;

      case NMEventPhaseIIEnd :
         fprintf(fp, "exit called @ Application complete for table %d.\n",
                 P->Vals.PhaseIIEnd.TableNo);
         fprintf(fp, "   %d updates, %d inserts, %d deletes\n",
                 P->Vals.PhaseIIEnd.Updates,
                 P->Vals.PhaseIIEnd.Inserts,
                 P->Vals.PhaseIIEnd.Deletes);
         break;

      case NMEventPhaseIIEnd64 :
         fprintf(fp, "exit called @ Application complete for table %d.\n",
                 P->Vals.PhaseIIEnd64.TableNo);
         fprintf(fp, "   %llu updates, %llu inserts, %llu deletes\n",
                 (UInt64)P->Vals.PhaseIIEnd64.Updates,
                 (UInt64)P->Vals.PhaseIIEnd64.Inserts,
                 (UInt64)P->Vals.PhaseIIEnd64.Deletes);
         break;

      case NMEventErrorTableI :
         fprintf(fp, 
               "exit called @ ET Table[%d] Drop : %d records in table.\n",
                P->Vals.ErrorTableI.TableNo, P->Vals.ErrorTableI.Rows);
         break;

      case NMEventErrorTableI64 :
         fprintf(fp, 
               "exit called @ ET Table[%d] Drop : %llu records in table.\n",
                P->Vals.ErrorTableI64.TableNo, 
                (UInt64)P->Vals.ErrorTableI64.Rows);
         break;

      case NMEventErrorTableII :
         fprintf(fp, 
                 "exit called @ UV Table[%d] Drop : %d records in table.\n",
                 P->Vals.ErrorTableII.TableNo, P->Vals.ErrorTableII.Rows);
         break;

      case NMEventErrorTableII64 :
         fprintf(fp, 
                 "exit called @ UV Table[%d] Drop : %llu records in table.\n",
                 P->Vals.ErrorTableII64.TableNo,
                 (UInt64)P->Vals.ErrorTableII64.Rows);
         break;

      case NMEventDBSRestart :
         fprintf(fp, "exit called @ RDBMS restarted\n");
         break;

      case NMEventCLIError :
         fprintf(fp, "exit called @ CLI error %d\n",
                 P->Vals.CLIError.ErrorCode);
         break;

      case NMEventDBSError :
         fprintf(fp, "exit called @ DBS error %d\n",
                 P->Vals.DBSError.ErrorCode);
         break;
 
      case NMEventExit :
         fprintf(fp, "exit called @ Notify out of scope: return code %d.\n",
                 P->Vals.Exit.ReturnCode);
         fclose(fp);
         break;

      case NMEventAmpsDown :
         fprintf(fp, "exit called @ Down amps have been detected\n");
         break;

      case NMEventImportBegin :
         fprintf(fp, "exit called @ Import %d starting\n",
                 P->Vals.ImportBegin.ImportNo);
         break;

      case NMEventImportEnd :
         fprintf(fp, "exit called @ Import %d ending.\n",
                      P->Vals.ImportEnd.ImportNo);
         fprintf(fp, "   Records Read:     %d\n", 
                 P->Vals.ImportEnd.RecsRead);
         fprintf(fp, "   Records Skipped:  %d\n", 
                 P->Vals.ImportEnd.RecsSkipped);
         fprintf(fp, "   Records Rejected: %d\n", 
                 P->Vals.ImportEnd.RecsRejected);
         fprintf(fp, "   Records Sent:     %d\n", 
                 P->Vals.ImportEnd.RecsSent);
         break;
 
      case NMEventImportEnd64 :
         fprintf(fp, "exit called @ Import %d ending.\n",
                      P->Vals.ImportEnd64.ImportNo);
         fprintf(fp, "   Records Read:     %llu\n", 
                 (UInt64)P->Vals.ImportEnd64.RecsRead);
         fprintf(fp, "   Records Skipped:  %llu\n", 
                 (UInt64)P->Vals.ImportEnd64.RecsSkipped);
         fprintf(fp, "   Records Rejected: %llu\n", 
                 (UInt64)P->Vals.ImportEnd64.RecsRejected);
         fprintf(fp, "   Records Sent:     %llu\n", 
                 (UInt64)P->Vals.ImportEnd64.RecsSent);
         break;
 
      case NMEventDeleteInit : /* nothing */
         fprintf(fp, "exit called @ Delete task init.\n");
         break;

      case NMEventDeleteBegin :
         fprintf(fp, "exit called @ Delete app start for table[%d]: %s.\n",
                 P->Vals.DeleteBegin.TableNo, P->Vals.DeleteBegin.TableName);
         break;
 
      case NMEventDeleteBeginEON :
         fprintf(fp, "exit called @ Delete app start EON for table[%d]: %s.\n",
                 P->Vals.DeleteBeginEON.TableNo,
                 P->Vals.DeleteBeginEON.TableName);
         break;
 
      case NMEventDeleteEnd :
         fprintf(fp, "exit called @ Delete app done for table[%d]: %d rows.\n",
                 P->Vals.DeleteEnd.TableNo, P->Vals.DeleteEnd.Deletes);
         break;

      case NMEventDeleteExit :
         fprintf(fp, "exit called @ Notify out of scope: return code %d.\n",
                 P->Vals.DeleteExit.ReturnCode);
         fclose(fp);
         break;
    }

    return(0);
}
