/*************************************************************************
*
* Title: BLKEXITR - sample INMOD routine
*
* COPYRIGHT (C) NCR Corporation. 1996
*
* This copyrighted material is the Confidential, Unpublished
* Property of the NCR Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*
* Description  This file contains a sample INMOD, written in C.
*              This file supports DBS restarts.
*
* Purpose
*
*
* History Information
*
* Revision    Date     DCR   DID     Comments
* ----------- -------- ----- ------- ---------------------------------------
* 07.01.00.00 07/22/98 43138 SF3     Test for CLI 350 Error
*
*
* How to build this INMOD 
*
*
*    MPRAS and SOLARIS:
*
*       cc -G -KPIC FAST26.c -o <shared-object-name>
*
*    IBM-AIX:
*
*       /usr/vac/bin/cc -qmkshrobj -o <shared-object-name> FAST26.c
*
*    Windows:
*
*       cl.exe /DWIN32 /LD FAST26.c
*
*    HPUX:
*
*       cc -n -b +DAportable +DS1.1 -o <shared-object-name> FAST26.c
*
*    Linux:
*
*       cc -m32 -shared -o <shared-object-name> FAST26.c
*
*    OS-390/USS:
*
*       cc -W l,dll -W c,dll,expo -o <shared-object-name> FAST26.c
*
* How to use this program:
*
*   This INMOD routine will generate 2 columns of data
*
*      2  integers
*
****************************************************************/
 
#include <errno.h>
#include <stdio.h>
#include <string.h>
 
#define FILEOF     401
#define EM_OK      0
 
#define NUMROWS    300
#define NUMCOLS    2
 
#define ROWSIZE    8
 
typedef int          Int32;
typedef unsigned int UInt32;
typedef short         Int16;
 
typedef struct inmod_struct {
   Int32  ReturnCode;
   UInt32 Length;
   char   Body[ROWSIZE];
} *inmdptr;
 
inmdptr inmodptr;
 
unsigned int reccnt = 0;
Int32 chkpnt;
 
/****************************************************************
*
*   MakeRecord - Generate a record
*
*      This module randomly generates data a column at a time.
*      Once a row has been successfully built and saved to a row
*      buffer, the row buffer is then copied to the main buffer
*      and returnd to Fastload.
*
****************************************************************/
Int32 MakeRecord()
{
   char  *p1;
   Int32  i;
   Int16  len;
 
   p1 = inmodptr->Body;
 
   /* place column 1, a unique primary index */
 
   memcpy(p1, &reccnt, sizeof(reccnt));
   p1 += sizeof(reccnt);
 
   /* place column 2 */
 
   memcpy(p1, &reccnt, sizeof(reccnt));
   p1 += sizeof(reccnt);
 
   inmodptr->ReturnCode = 0;
   inmodptr->Length = p1 - inmodptr->Body;     /* correct length */
 
   if ( reccnt == 1 )
       printf("INMOD: Data row length: %d\n", inmodptr->Length);
 
   reccnt++;
 
   return(EM_OK);
}
 
 
/****************************************************************
 * HostRestart - Host restarted, reset and start sending data
 *               from the begining.
 ****************************************************************/
Int32 HostRestart()
{
   printf("INMOD: I am told to reposition after Host restart.\n");
   reccnt = chkpnt;
   return(EM_OK);
}
 
/****************************************************************
 * CheckPoint - Save checkpoint
 ****************************************************************/
Int32 CheckPoint()
{
   printf("INMOD: I am told to take a checkpoint.\n");
   chkpnt = reccnt;
   return(EM_OK);
}
 
/****************************************************************
 * DBCRestart - DBC restarted, do what you have to do.
 *              Reset record counter to checkpoint value
 ****************************************************************/
Int32 DBCRestart()
{
   printf("INMOD: I am told to reposition after DBS restart.\n");
   reccnt = chkpnt;
   return(EM_OK);
}
 
/****************************************************************
 * CleanUp - Do cleanup.
 ****************************************************************/
Int32 CleanUp()
{
   printf("INMOD: I am told to do cleanup.\n");
   return(EM_OK);
}
 
/****************************************************************
 * InvalidCode - Invalid inmod code returned.
 ****************************************************************/
Int32 InvalidCode()
{
   fprintf(stderr, "*** Invalid Inmod code\n");
   return(EM_OK);
}
 
/****************************************************************
*
* Init - initialize some stuff
*
*    get the ending record number from FastLoad
*
****************************************************************/
Int32 Init()
{
   printf("INMOD: I am told to initialize.\n");
   return(EM_OK);
}
 
/****************************************************************
*
*   BLKEXIT - Start processing
*
*      This is the main module which contains the checks for
*      number of records generated and buffer filling.  This
*      module also sends the filled buffer to the DBC.
*
****************************************************************/

#if defined WIN32
__declspec(dllexport) Int32 BLKEXIT(char *tblptr)
#elif defined I370
Int32 _dynamn(char *tblptr)
#else
Int32 BLKEXIT(char *tblptr)
#endif
{
   Int32 result;
 
   inmodptr = (struct inmod_struct *)tblptr;
 
   if (reccnt >= NUMROWS) {
      inmodptr->ReturnCode = FILEOF;
      return(EM_OK);
   }
 
   switch (inmodptr->ReturnCode) {
      case 0:  result = Init();
      case 1:  result = MakeRecord();
               break;
      case 2:  result = HostRestart();
               break;
      case 3:  result = CheckPoint();
               break;
      case 4:  result = DBCRestart();
               break;
      case 5:  result = CleanUp();
               break;
      default: result = InvalidCode();
   }
 
   return(result);
}
 
