/*************************************************************************
*
* Title: Schema Class Definition
*
*
* Copyright 2005-2016 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description: 
*
* The Schema Class is a simple structure used to represent table information. 
* Each instance has zero or more Column objects, a type ( input/output ), and a
* name.
*
* Purpose:
*
* Store table schema information.
*
* History Information
*
* Revision    Date     DCR      DID      Comments
* ----------- -------- -------- -------- ---------------------------------------
* 16.00.00.01 02242016 TPT-26911 pg255017 TPT API support for XML datatype
* 16.00.00.00 05262015 TPT-19675 by255002 Allow temporal keywords to follow
*                                         temporal columns in TPTAPI schemas
* 15.10.00.00 05082014 TPT-19321 by255002 TPT API support for JSON
*                                         (JavaScript Object Notation) datatype
* 15.00.00.02 12262013 TPT-20211 kb186006  Copyright year change
* 15.00.00.01 10302013 TPT-19505 vm185012 TPTAPI to support CLOB/BLOB data types
* 14.10.00.02 10312012 TPT16579 by255002 Design change (renaming the overloaded
*                                        AddColumn function) in the TPT-388
*                                        enhancement
* 14.10.00.01 03132012 TPT3525 by255002 RFC: Enhance TPTAPI to get the actual column
*                                       type (no translation of TIMESTAMP to CHAR) to
*                                       from Export Driver
* 14.10.00.00 03052012 TPT388 by255002 MacroCharSet Support for TPTAPI
* 14.00.00.06 05062011 147441 vm185012 TPTAPI to support Number data type
* 14.00.00.05 04152011 148806 mv121803 Dynamic schema didn't work when TELAPI
*                                      is loaded dynamically
* 14.00.00.04 01222011 146403 jk186005 Remove the TD_NO_OPERATOR references
* 14.00.00.03 01042011 119676 vm185012 Dynamic schema with multiple instances support
* 14.00.00.02 12092010 145813 jk186005 TELAPI_EXPORT has been added to export the functions
* 14.00.00.01 25102010 145814 an185020 Add TELAPI_EXPORT for functions exported
*                                      on windows by DR144293
* 13.00.01.00 03182009 130830 vm185012 Re-Add ncr namespace
* 13.00.00.01 01232008 116135 ec185008 Update namespace
* 13.00.00.00 01182008 116133 jg185041 Add 5 new PERIOD data types
* 09.00.00.01 01192007 110121 kb185011 Add Error Limit Support
* 09.00.00.00 11172006  89727 jg185041 Add network UTF16 support
* 08.02.00.01 04072006  98234 jg185041 Add Large Decimal/BIGINT Support
* 08.02.00.00 03292006 102610 ls185040 Port to MP-RAS
* 01.00.00.10 11012005  97368 jg185041 Add current copyright notice
* 01.00.00.09 10052005  96274 kb185011 Remove generic typedefs
* 01.00.00.08 09152005  98513 lc185033 Added block loading
* 01.00.00.07 09012005  89728 jg185041 Update Event method support
* 01.00.00.06 08242005  98108 lc185033 Deleted unsupported data types
* 01.00.00.05 08232005  96274 jg185041 Clean up shipped header files
* 01.00.00.04 06082005  96168 sl185022 Add typedefs for user integer data types
* 01.00.00.03 03302005  94012 jg185041 Add TELAPI_EXPORT macro to enable dll linking
* 01.00.00.02 03282005  89756 kb185011 Replace All printf/cout with TELAPIDBG Macro
* 01.00.00.01 03282005  89698 kb185011 Modify Constructor & Add SetType Function
* 01.00.00.01 03282005  89698 kb185011 Add Copy Constructor
* 01.00.00.01 03282005  89698 kb185011 Create New AddColumn function
* 01.00.00.01 03282005  89698 kb185011 Add Include TelapiConstants.h
* 01.00.00.00 02012005  89676 dr185013 Initial Creation
* 
*
*************************************************************************/

#ifndef SCHEMA_H
#define SCHEMA_H

/* DR96274 BEGIN */

typedef enum PX_SchemaType
 {
   PX_NoSchema = 0,
   PX_InputSchema,
   PX_InputSchemaAny,
   PX_OutputSchema,
   PX_OutputSchemaAny,
   PX_InputOutputSchema,
   PX_InputOutputSchemaAny

 } PX_SchemaType;

//#include "column.h"
//#include "pxtypes.h"
//#include "TelapiConstants.h" // DR89698 //
//#include "platform.h"  // DR94012 //

#include <vector>

/* DR89709--start */
#include <stdlib.h>    
#if defined(__linux__)  
 #include<string> 
#else
#include <string.h> 
#endif
 /* DR89709 --END  */

/* DR102610 BEGIN */
#if defined(__HIGHC__) || defined(_ATT4)
#else
using namespace std;
#endif
/* DR102610 END */

// TelapiConstants.h

// DR89756 BEGIN //

// *********************************************************
// *	Constants used along with TD_TRACE_LEVEL
// *
// *	TD_OPER			 - Operator Level General Trace
// *	TD_OPER_CLI		 - Operator Level CLI Trace 
// *	TD_OPER_NOTIFY	 - Operator Level Notify Method Trace
// *	TD_OPER_OPCOMMON - Operator Level Common Library Trace 
// *	TD_OPER_ALL		 - All Operator Level Traces
// *	TD_GENERAL		 - Telapi Layer General Trace
// *	TD_ROW			 - PutRow/GetRow Trace 
// *	TD_MSG_CODES	 - Operator Log Message Info
// ********************************************************** //

typedef enum TD_TraceLevel {
	TD_OFF = 1, // DR89698 //
	TD_OPER,
	TD_OPER_CLI,
	TD_OPER_NOTIFY,
	TD_OPER_OPCOMMON,
	TD_OPER_SPECIAL, //Special
	TD_OPER_ALL,
	TD_GENERAL,
	TD_ROW,
	TD_MSG_CODES
} TD_TraceLevel;

// DR89756 END //

// DR89698 BEGIN //

// ****************************************************
// * Operator Type Values for use with TD_SYSTEM_OPERATOR
// ***************************************************** //

typedef enum TD_OperatorType {
	TD_LOAD = 1,                          /* DR146403 */
	TD_EXPORT,
	TD_UPDATE,
	TD_STREAM
}TD_OperatorType;

// DR89698 END //

// *********************************************
// * TELAPI Return Values
// ********************************************** //

typedef enum TD_StatusCode {
	TD_Success = 0,			// 000 //
	TD_SYNC_Barrier,		// 001 //
	TD_SYNC_TELINFO,		// 002 //
	TD_END_Method,			// 003 //
	TD_Unavailable,         // 004 //                           /* DR89728 */
    TD_SYNC_Schema,         // 005                              // DR119676
	TD_Error = 99,			// 099 //
	TD_Call_EndAcq = 25000  // 25000 //                         /* DR110121 */
}TD_StatusCode;

// DR96168 BEGIN //

typedef unsigned int TD_Length;
typedef int          TD_IntValue;
typedef unsigned int TD_Count;
typedef unsigned int TD_Index;
typedef unsigned int TD_ErrorType; // DR96160 //

typedef unsigned int TD_ColumnSize;
typedef unsigned int TD_ColumnPrecision;
typedef unsigned int TD_ColumnScale;
typedef char         TD_Boolean;   /* DR98513 */

// DR96168 END //

typedef enum TD_DataType {
	TD_NONE = 0,
	TD_INTEGER,
	TD_SMALLINT,
	TD_FLOAT,
	TD_DECIMAL,
	TD_CHAR,
	TD_BYTEINT,
	TD_VARCHAR,
	TD_LONGVARCHAR,
	TD_BYTE,
	TD_VARBYTE,
	TD_DATE,
	TD_GRAPHIC,
	TD_VARGRAPHIC,
	TD_LONGVARGRAPHIC,
	TD_DATE_ANSI,
    TD_TIME = 35,                                               //TPT-3525
    TD_TIME_WITHTIMEZONE = 36,                                  //TPT-3525
    TD_BIGINT = 37,                                             /* DR98234 */
    TD_BLOB   = 39,                                             // TPT-19505
    TD_CLOB   = 40,                                             // TPT-19505
    TD_BLOB_AS_DEFERRED_BY_NAME = 41,                           // TPT-19505
    TD_CLOB_AS_DEFERRED_BY_NAME = 42,                           // TPT-19505
    TD_TIMESTAMP = 43,                                            //TPT-3525
    TD_TIMESTAMP_WITHTIMEZONE = 44,                               //TPT-3525
    TD_INTERVAL_YEAR = 45,                                        //TPT-3525
    TD_INTERVAL_YEARTOMONTH = 46,                                 //TPT-3525
    TD_INTERVAL_MONTH = 47,                                       //TPT-3525
    TD_INTERVAL_DAY = 48,                                         //TPT-3525
    TD_INTERVAL_DAYTOHOUR = 49,                                   //TPT-3525
    TD_INTERVAL_DAYTOMINUTE = 50,                                 //TPT-3525
    TD_INTERVAL_DAYTOSECOND = 51,                                 //TPT-3525
    TD_INTERVAL_HOUR = 52,                                        //TPT-3525
    TD_INTERVAL_HOURTOMINUTE = 53,                                //TPT-3525
    TD_INTERVAL_HOURTOSECOND = 54,                                //TPT-3525
    TD_INTERVAL_MINUTE = 55,                                      //TPT-3525
    TD_INTERVAL_MINUTETOSECOND = 56,                              //TPT-3525
    TD_INTERVAL_SECOND = 57,                                      //TPT-3525
    TD_PERIOD_DATE = 58,                                        /* DR116133 */
    TD_PERIOD_TIME,                                             /* DR116133 */
    TD_PERIOD_TIME_TZ,                                          /* DR116133 */
    TD_PERIOD_TS,                                               /* DR116133 */
    TD_PERIOD_TS_TZ,                                             /* DR116133 */
    TD_NUMBER,                                                   // DR 147441
    TD_XML = 65,                                                 // TPT-26911
    TD_XML_AS_DEFERRED_BY_NAME = 66,                             // TPT-26911
    TD_JSON  = 67,                                               // TPT-19321
    TD_JSON_AS_DEFERRED_BY_NAME = 68                             // TPT-19321
	/* DR98108 */
}TD_DataType;
// DR89698 END //

/* DR89727 BEGIN */

typedef enum TD_Encoding {
	TD_UTF8_ENCODING = 0,
	TD_UTF16_ENCODING,
	TD_EBCDIC_ENCODING
}TD_Encoding;

/* DR89727 END */

//types.h

/* DR96274 BEGIN*/

//typedef						void				Void;

//typedef						char				Char;
//typedef						char				Char8;
//typedef						short				Char16;
//typedef						int					Char32;

//typedef						char				Bool;
//typedef						char				Boolean;

//typedef signed		int					Int;
//typedef signed		char				Int8;
//typedef signed		short				Int16;
//typedef signed		int					Int32;
//typedef signed		long long		Int64;

//typedef unsigned	long				UInt;
//typedef unsigned	char				UInt8;
//typedef unsigned	short				UInt16;
//typedef unsigned	int					UInt32;
//typedef unsigned	long long		UInt64;

//typedef 					char 				Byte;
//typedef 					Int 				StatusCode;

/* DR96274 END */

//plaform.h
#ifndef TELAPI_EXPORT
#  ifdef WIN32
#    ifdef _TELAPI_DLL
#      define TELAPI_EXPORT __declspec (dllexport)
#    else
#      define TELAPI_EXPORT __declspec (dllimport)
#    endif
#  else
#    define TELAPI_EXPORT
#  endif
#endif

// column.h

//namespace teradata {
//namespace client {

	class Column
	{
		public:
			//Column(
			//	Char* name, 
			//	Char* datatype, 
			//	Int		size = 0,
			//	Int		precision = 0,
			//	Int		scale = 0
				//); // DR89698 //

			TELAPI_EXPORT Column(
				char* name, 
				TD_DataType dataType, 
				int	size = 0, 
				int	precision = 0, 
				int	scale = 0
			); // DR89698 //

            TELAPI_EXPORT Column(         //TPT-388
                char* name,
                TD_DataType dataType,
                char*  mcsName,
                int     size = 0,
                int     precision = 0,
                int     scale = 0
            ); /*DR89698*/               //TPT-388

			TELAPI_EXPORT ~Column();

			TELAPI_EXPORT Column(const Column& c); // DR89698 //
               /* Following public methods exported for Windows also DR 144293 */
                        TELAPI_EXPORT  char*     GetName();
                        TELAPI_EXPORT  int       GetDataType();
                        TELAPI_EXPORT  int       GetSize();
                        TELAPI_EXPORT  int       GetPrecision();
                        TELAPI_EXPORT  int       GetScale();
                        TELAPI_EXPORT  char*     GetMCSname();     //TPT-388
                        TELAPI_EXPORT  int       GetOffset();
                        TELAPI_EXPORT  void      SetOffset(int offset);
                        TELAPI_EXPORT  void      SetColAttributes(char*,void* ,int,int);  /* TPT-19675 */
                        TELAPI_EXPORT  char*     GetColAttribute();                       /* TPT-19675 */
                        TELAPI_EXPORT  char*     GetColAttributeVal();                    /* TPT-19675 */
                        TELAPI_EXPORT  int       GetColAttributeLen();                    /* TPT-19675 */
                        TELAPI_EXPORT  int       GetColAttributeType();                   /* TPT-19675 */
						TELAPI_EXPORT  int       GetColErr();                             /* TPT-19675 */


			TELAPI_EXPORT bool		IsNullable(); 
			TELAPI_EXPORT void		SetNullable(); 


		private:
			char*		mName; 
			int			mSize; 
			int			mPrecision; 
			int			mScale; 
			int			mDatatype; 
			bool		mNullable;
			int			mOffset;
            char*       mMCSName;  //TPT-388
			void*       mColumnVars;   /* TPT-19675 */

	};

//} // namespace client
//} // namespace teradata

/* DR96274 END */
namespace ncr {         /* DR130830 */ 
namespace teradata {
namespace client {
namespace API {                                                 /* DR96274 */

class Schema
{
     public:
	TELAPI_EXPORT Schema(char* stype);                  // DR94012 //
	TELAPI_EXPORT Schema(TD_Encoding encoding);         // DR89727 //
	TELAPI_EXPORT ~Schema();                            // DR94012 //
	TELAPI_EXPORT Schema(const Schema& s); // DR89698 //

     public:
        TELAPI_EXPORT void	AddColumn(char* colName,        // DR94012 // 
					TD_DataType, 				
					TD_ColumnSize	size = 0, // DR96168 //
					TD_ColumnPrecision	precision = 0,         // DR96168 //
					TD_ColumnScale	scale = 0 ); // DR89698 // // DR96168 //
        TELAPI_EXPORT void   AddColumnWithCharSet(char*  colName,    //TPT16579
                                  TD_DataType,
                                  char *mcsName,
                                  TD_ColumnSize size = 0,
                                  TD_ColumnPrecision  precision = 0,
                                  TD_ColumnScale = 0 );    //TPT-388
        TELAPI_EXPORT void  AddAttributeForColumn(char* colName,                /* TPT-19675 */
                                           char* colAttrName,                   /* TPT-19675 */
                                           void* colAttrValue,                  /* TPT-19675 */
                                           TD_Length colAttrLength,             /* TPT-19675 */
                                           TD_DataType colAttrDataType);        /* TPT-19675 */
	TELAPI_EXPORT bool       getIsError();                                      /* TPT-19675 */               
	TELAPI_EXPORT int        getErrCode();										/* TPT-19675 */
	TELAPI_EXPORT char*      getErrMsg();										/* TPT-19675 */

        TELAPI_EXPORT void  AddColumn(Column*);
	TELAPI_EXPORT TD_Count	      GetColumnCount();     // DR94012  DR96168
        TELAPI_EXPORT Column*         GetColumn(int index); /* DR144293 */
        TELAPI_EXPORT PX_SchemaType   GetSchemaType(); /* DR144293 */
        TELAPI_EXPORT vector<Column*> GetColumns();    /* DR144293 */
        TELAPI_EXPORT TD_Encoding     GetEncoding();   /* DR89727  DR144293 */

	TELAPI_EXPORT void	SetName(string name);           // DR94012 //
	TELAPI_EXPORT void	SetType(char* stype); //DR89676 UI Change//
			                              // DR94012 //

	TELAPI_EXPORT string GetName();               // DR94012 //

     //DR148806 private:
     protected:                                                  //DR148806
	vector<Column*> columns;
	PX_SchemaType   schemaType;
	string tableName;
	TD_Encoding mEncoding;                              /* DR89727 */
	void* mSchemaVars;                                  /* DR89727 */

};

} // namespace API                                              /* DR96274 */
} // namespace client
} // namespace teradata
} // namespace ncr
namespace teradata = ncr::teradata;    /* DR130830 */ 
#endif

