/*************************************************************************
*
* Title: TEL API SchemaX Class Definition
*
*
* Copyright 2010-2015 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description: 
*
* This class enables access to the Schema class when the TPTAPI library
* is loaded dynamically during the runtime.
* The class methods will call their equivalent methods of Schema class.

* Purpose:
*
* Store table schema information.
*
* History Information
*
* Revision    Date     DCR    DID      Comments
* ----------- -------- ------ -------- ---------------------------------------
* 16.00.00.00 05262015 TPT-19675 by255002 Allow temporal keywords to follow
*                                         temporal columns in TPTAPI schemas
* 15.00.00.00 12262013 TPT-20211 kb186006  Copyright year change
* 14.10.00.01 10312012 TPT16579 by255002 Design change (renaming the overloaded
*                                        AddColumn function) in the TPT-388
*                                        enhancement
* 14.10.00.00 03052012 TPT388 by255002 MacroCharSet Support for TPTAPI
* 14.00.00.01 04152011 148806 mv121803 Dynamic schema didn't work when TELAPI
*                                      library is loaded dynamically
* 14.00.00.00 12092010 145813 jk186005 TELAPI_EXPORT has been added to export the functions
* 13.10.00.01 05211010 XXXXXX mv121803 Initial Creation
* 
*
*************************************************************************/

#ifndef SCHEMAX_H
#define SCHEMAX_H

#include "schema.h"

namespace ncr {  
namespace teradata {
namespace client {
namespace API { 

class SchemaX : public Schema
{
    public:
	TELAPI_EXPORT SchemaX(char* stype);          
	TELAPI_EXPORT SchemaX(TD_Encoding encoding); 
        TELAPI_EXPORT SchemaX(const SchemaX& s);                 //DR148806
	TELAPI_EXPORT virtual ~SchemaX();                  

	TELAPI_EXPORT virtual void  AddColumn(char* colName,  
	  		 	          TD_DataType, 				
				          TD_ColumnSize	size = 0, 
				          TD_ColumnPrecision  precision = 0,
				          TD_ColumnScale      scale = 0 );
    TELAPI_EXPORT virtual void  AddColumnWithCharSet(char* colName,      //TPT16579
                          TD_DataType,
                          char* mcsName,
                          TD_ColumnSize size = 0,
                          TD_ColumnPrecision  precision = 0,
                          TD_ColumnScale      scale = 0 );          //TPT-388
    TELAPI_EXPORT virtual void  AddAttributeForColumn(char* colName,            /* TPT-19675 */
                                           char* colAttrName,                   /* TPT-19675 */
                                           void* colAttrValue,                  /* TPT-19675 */
                                           TD_Length colAttrLength,             /* TPT-19675 */
                                           TD_DataType colAttrDataType);        /* TPT-19675 */
	TELAPI_EXPORT virtual bool       getIsError();                                      /* TPT-19675 */               
	TELAPI_EXPORT virtual int        getErrCode();                                      /* TPT-19675 */
	TELAPI_EXPORT virtual char*      getErrMsg();                                       /* TPT-19675 */ 
	TELAPI_EXPORT virtual TD_Count	GetColumnCount();        
        TELAPI_EXPORT virtual string    GetName();
	TELAPI_EXPORT virtual void	SetName(string name);  
        TELAPI_EXPORT virtual void      SetType(char* stype);

	TELAPI_EXPORT Column*		GetColumn(int index);
	TELAPI_EXPORT PX_SchemaType	GetSchemaType();
	TELAPI_EXPORT vector<Column*> GetColumns();
        TELAPI_EXPORT TD_Encoding     GetEncoding();   
        TELAPI_EXPORT void*           GetSchemaVars();          //DR148806

};

//
// Class Factory "C" functions which inistantiates the SchemaX class
//
typedef SchemaX*     (*pfn_newSchemaX)(char *);
typedef SchemaX*     (*pfn_newSchemaXEncoding)(TD_Encoding);
typedef SchemaX*     (*pfn_copySchemaX)(SchemaX *, SchemaX *);
typedef void         (*pfn_delSchemaX)(SchemaX *);

} // namespace API  
} // namespace client
} // namespace teradata
} // namespace ncr 
namespace teradata = ncr::teradata;  

#endif

