/*************************************************************************
*   
* Copyright 2005-2009 by Teradata Corporation.
* Copyright 2005-2010, 2014 by Teradata Corporation. 
* All Rights Reserved.
* TERADATA CONFIDENTIAL TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* BlockLoad sample:
* Load opearator loads 5000 rows from inputFile to the Database Table.
* Update opearator inserts 5000 rows from inputFile to the Database Table.
* Stream oparator inserts 5000 rows from inputFile to the Database Table. 
*
*************************************************************************/

#include "BlockLoad.h"
#include "OptionsManager.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "string.h"

using namespace ncr::teradata::client::API;

#if defined(__HIGHC__) || defined(_ATT4)
#else
  using namespace std;
#endif

BlockLoad::BlockLoad(OptionsManager * om): m_om(om)
{
  inputStream = NULL;
  outputStream = NULL;
  inputFileName = NULL;
  outputFileName = NULL;
  mbuffer = NULL;
  curpos = NULL;
}

BlockLoad::~BlockLoad(void)
{
  if( inputFileName != NULL )
  {
    free(inputFileName);
  }

  if( outputFileName != NULL )
  {
    free(outputFileName);
  }
}

void BlockLoad::checkbuffer(char indicator)
//does some simple data checking on the buffer
//this function assumes that the datalength including the indicator byte is 101
{
  unsigned short int tmplen;
  char *myptr = mbuffer[(curbuff+1)%2];
  char *endptr = myptr + sendbuffersize - buftrailersize;
  int i = 0;

  while(myptr < endptr)
  {
    i++;
    myptr += parcesize;
    tmplen = *((unsigned short*)myptr);
    if(indicator)
    {
      if(tmplen != 101)  //error on len
      {
        cout << i << " got len " << tmplen << endl;
      }
    }
    else
    {
      if(tmplen != 100)
      cout << i << " got len" << tmplen << endl;
    }
    records++;
    myptr += rowlengthsize;
    myptr += tmplen;
  }
  //myptr and endptr should be the same value
  cout << "buffer ok " << myptr << " " << endptr << "\n";
  cout << "\t" << i << " " << records << " " << \
  (unsigned int)(endptr - mbuffer[(curbuff+1)%2]) << "\n";
}

/***************************************************
* Method for testing Load Driver
***************************************************/

void BlockLoad::testLoad()
{
  time_t now;
  time(&now);
  int returnValue = 0;
  //char indicator = 0;
  char indicator = 1;   // vijai
  char *errorMsg = NULL;
  TD_ErrorType errorType;

  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *error_table_1 = strdup(m_om -> getErrorTable1().c_str());
  char *error_table_2 = strdup(m_om -> getErrorTable2().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());

  char * dml_statment = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s(:col1, :col2);", target_table);

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description :BlockLoad sample" << endl;
  cout << "Operator type    :LOAD" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name<< endl;
  cout << "TDP password     :<not displayable>" << endl;
  cout << "Target Table     :" << target_table << endl;
  cout << "Input File       :" << inputFileName  << endl;
  cout << "DML Statement    :" << dml_statment << endl;

  Connection *conn = new Connection();

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"load.txt");  //TPT-17718
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,\
  //TD_GENERAL, NULL);

  conn -> AddAttribute(TD_BUFFER_MODE, (char*)"yes");  //TPT-17718

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddAttribute(TD_ERROR_TABLE_1, error_table_1);
  conn -> AddAttribute(TD_ERROR_TABLE_2, error_table_2);

  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);
  //delete schema;

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;
  DMLGroup* dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statment);
  conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate
  *********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();

  char *dataptr;
  TD_Length datalen;
  if(returnValue < TD_Error)
  {
    cout << "Driver Initiated Successfully" << endl;
    returnValue = conn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);
    buffersize    = *((int*)dataptr);
    rowheadersize = *((int*)(dataptr + sizeof(int)));
    rowlengthsize = *((int*)(dataptr+2*sizeof(int)));
    buftrailersize= *((int*)(dataptr+3*sizeof(int)));
    parcesize     = rowheadersize - rowlengthsize;
 
    mbuffer    = new char*[2];
    mbuffer[0] = new char[buffersize];
    mbuffer[1] = new char[buffersize];
    sendbuffersize = 0;  //size of buffer to be sent
    byteswritten = 0;    //bytes written to current buffer
    curpos = mbuffer[0]; //current cursor position of buffer
    curbuff = 0;         //current buffer number
  }
  if ( returnValue < TD_Error)
  {
    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/

    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  :";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version:";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else 
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

    /**********************************************
    * Acquisition
    **********************************************/

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    int loadStatus = 0;
    //TD_Length bytes;
    int count = 0;
    records = 0;
    //byteswritten += rowheadersize;
    while( loadStatus != 1 )
    {
      loadStatus = bufGetRow1MBSupport();

      if( loadStatus != -1 )
      {
        //if a buffer is ready to send
        //check buffer integrity
        //checkbuffer();
        count++;

        returnValue = conn -> PutBuffer(mbuffer[(curbuff+1)%2],\
        sendbuffersize, indicator);
      
        if(returnValue < TD_Error)
        {
          cout << "sent buffer " << count << " indicator mode " \
          << (int)indicator << "\n";
          indicator = (indicator +1 )%2;
        }
        else
        {
          //Get Error Information

          cout << "Error occured during PutBuffer" << endl;
          conn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
    }
    if ( returnValue < TD_Error )
    {

      /**********************************************
      * End Acquisition
      **********************************************/
                        
      returnValue = conn -> EndAcquisition();

      if ( returnValue < TD_Error )
      {
        cout << "Acquisition completed Successfully" << endl;

        /********************************************
        * Application
        ********************************************/

        cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
        cout << "             APPLICATION PHASE             " << endl;
        cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

        returnValue = conn -> ApplyRows();

        if ( returnValue < TD_Error )
        {
          cout << "Rows Applied Successfully" << endl;
          cout << "Load completed successfully" << endl;
        }
        else
        {
          //Get Error Information

          cout << "Error occured during Application" << endl;
          conn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
	        cout << "Type: " << errorType << endl;
          }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
      else
      {
        //Get Error Information

        cout << "Error occured during EndAcquisition" << endl;
        conn -> GetErrorInfo(&errorMsg, &errorType);

        if ( errorMsg != NULL )
        {
          cout << errorMsg << endl;
          cout << "Type: " << errorType << endl;
        }
        else
        {
         cout << "No Error Info Available" << endl;
        }
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Acquisition" << endl;
      conn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Initiate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }

  /**********************************************
  * Terminate
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
   
  returnValue = conn -> Terminate();
  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfuly" << endl;
  } 

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting objects" << endl;

  delete dmlGr;
  delete schema;
  delete conn;

  if(mbuffer != NULL)
  {
    delete [] mbuffer[0];
    delete [] mbuffer[1];
    delete [] mbuffer;
  }

  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(log_table);
  free(error_table_1);
  free(error_table_2);
  free(dml_statment);

  cout << "*** Load Complete ***" << endl;
  cout <<"\n###################### OUTPUT END ######################" << endl;
  /* DR97368 */
}

/***************************************************
 * Method for testing Update Driver
***************************************************/

void BlockLoad::testUpdate()
{
  time_t now;
  time(&now);
  int returnValue = 0;
  char indicator = 1;     // pdate must be indicator mode
  char *errorMsg = NULL;
  TD_ErrorType errorType;

  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *error_table_1 = strdup(m_om -> getErrorTable1().c_str());
  char *error_table_2 = strdup(m_om -> getErrorTable2().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());

  char *dml_statment  = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);", target_table);
       
  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description :BlockLoad sample" << endl;
  cout << "Operator type    :UPDATE" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name << endl;
  cout << "TDP password     :<not displayable>" << endl;
  cout << "Target Table     :" << target_table << endl;
  cout << "Input File       :" << inputFileName << endl;
  cout << "DML Statement    :" << dml_statment << endl;

  /* DR97368 */
  Connection *conn = new Connection();

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_UPDATE);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"update.txt");  //TPT-17718
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, /
  //TD_GENERAL, NULL);
  //not real buffer mode, only using putbuffer function

  conn -> AddAttribute(TD_BUFFER_MODE, (char*)"yes");  //TPT-17718
	
  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddAttribute(TD_ERROR_TABLE_1, error_table_1);
  conn -> AddAttribute(TD_ERROR_TABLE_2, error_table_2);

  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);
  //delete schema;

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;
  DMLGroup* dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statment);
  conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();

  char *dataptr;
  TD_Length datalen;
  if(returnValue < TD_Error)
  {
    returnValue = conn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);
    buffersize = *((int*)dataptr);
    rowheadersize = *((int*)(dataptr + sizeof(int)));
    rowlengthsize = *((int*)(dataptr+2*sizeof(int)));
    buftrailersize= *((int*)(dataptr+3*sizeof(int)));
    parcesize = rowheadersize - rowlengthsize;

    mbuffer = new char*[2];
    mbuffer[0] = new char[buffersize];
    mbuffer[1] = new char[buffersize];
    sendbuffersize = 0;  //size of buffer to be sent
    byteswritten = 0;    //bytes written to current buffer
    curpos = mbuffer[0]; //current cursor position of buffer
    curbuff = 0;         //current buffer number
  }

  if ( returnValue < TD_Error )
  {
    cout << "Driver Initiated Successfully" << endl;

    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  : ";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else 
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

    /**********************************************
    * Acquisition
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    int loadStatus = 0;
    //TD_Length bytes;
    int count = 0;
    records = 0;

    while( loadStatus != 1 )
    {
      loadStatus = bufGetRow1MBSupport();

      if( loadStatus != -1 )
      {
        //if a buffer is ready to send
        //check buffer integrity
        //checkbuffer();
        count++;

        returnValue = conn -> PutBuffer(mbuffer[(curbuff+1)%2],\
        sendbuffersize, indicator);

        if(returnValue < TD_Error)
        {
          cout << "sent buffer " << count << " indicator mode " \
          << (int)indicator << "\n";
          //indicator = (indicator +1 )%2;
        }
        else
        {
          //Get Error Information

          cout << "Error occured during PutBuffer" << endl;
          conn -> GetErrorInfo(&errorMsg, &errorType);

          if ( errorMsg != NULL )
          {
	        cout << errorMsg << endl;
	        cout << "Type: " << errorType << endl;
	      }
          else
          {
            cout << "No Error Info Available" << endl;
          }
        }
      }
    }
    if ( returnValue < TD_Error )
    {

      /**********************************************
      * End Acquisition
      **********************************************/

      returnValue = conn -> EndAcquisition();
      if ( returnValue < TD_Error )
      {
        cout << "Acquistion completed Sucessfully" << endl;

        /**********************************************
        * Application
        **********************************************/

        cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
        cout << "             APPLICATION PHASE             " << endl;
        cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      
        returnValue = conn -> ApplyRows();

        if ( returnValue < TD_Error )
        {
          cout << "Rows Applied Successfully" << endl;
	      cout << "Update completed successfully" << endl;
        }
        else
        {
	      //Get Error Information

	      cout << "Error occured during Application" << endl;
	      conn -> GetErrorInfo(&errorMsg, &errorType);

	      if ( errorMsg != NULL )
          {
	        cout << errorMsg << endl;
	        cout << "Type: " << errorType << endl;
          }
          else
          {
	        cout << "No Error Info Available" << endl;
          }
        }
      }
      else
      {
        //Get Error Information

        cout << "Error occured during EndAcquisition" << endl;
        conn -> GetErrorInfo(&errorMsg, &errorType);

        if ( errorMsg != NULL )
        {
          cout << errorMsg << endl;
          cout << "Type: " << errorType << endl;
        }
        else
        {
          cout << "No Error Info Available" << endl;
        }
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Acquisition" << endl;
      conn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Initiate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL) 
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }

  /**********************************************
  * Terminate
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Terminate();

  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
       cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfuly" << endl;
  }

  /**********************************************
  * Clean Up
  **********************************************/
  cout << "Deleting objects" << endl;

  delete dmlGr;
  delete schema;
  delete conn;

  if(mbuffer != NULL)
  {
    delete [] mbuffer[0];
    delete [] mbuffer[1];
    delete [] mbuffer;
  }

   free(user_name);
   free(user_password);
   free(target_table);
   free(tdp_id);
   free(log_table);
   free(error_table_1);
   free(error_table_2);
   free(dml_statment);

   cout << "*** Update Complete ***" << endl;
   cout <<"\n###################### OUTPUT END #####################" << endl;
   /* DR97368 */
 }

/***************************************************
 * Method for testing Stream Driver
***************************************************/

void BlockLoad::testStream()
{
  time_t now;
  time(&now);
  int returnValue = 0;
  char indicator = 1;    // stream must be indicator mode
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *error_table_1 = strdup(m_om -> getErrorTable1().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());

  char * dml_statment = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);", target_table);
     
  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description :BlockLoad sample" << endl;
  cout << "Operator type    :STREAM" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name << endl;
  cout << "TDP password     :<not displayable>" << endl;
  cout << "Target Table     :" << target_table << endl;
  cout << "Input File       :" << inputFileName << endl;
  cout << "DML Statement    :" << dml_statment << endl;
         
  Connection *conn = new Connection();

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"stream.txt");  //TPT-17718
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
  //TD_GENERAL, NULL);
  //not real buffer mode, only using putbuffer function

  conn -> AddAttribute(TD_BUFFER_MODE, (char*)"yes");  //TPT-17718
	
  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddAttribute(TD_ERROR_TABLE, error_table_1);
 
  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);
  //delete schema;

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;
  DMLGroup* dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statment);
  conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();

  char *dataptr;
  TD_Length datalen;
  if(returnValue < TD_Error)
  {
    returnValue = conn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);
    buffersize = *((int*)dataptr);
    rowheadersize = *((int*)(dataptr + sizeof(int)));
    rowlengthsize = *((int*)(dataptr+2*sizeof(int)));
    buftrailersize= *((int*)(dataptr+3*sizeof(int)));
    parcesize = rowheadersize - rowlengthsize;

    mbuffer = new char*[2];
    mbuffer[0] = new char[buffersize];
    mbuffer[1] = new char[buffersize];
    sendbuffersize = 0;          //size of buffer to be sent
    byteswritten = 0;            //bytes written to current buffer
    curpos = mbuffer[0];         //current cursor position of buffer
    curbuff = 0;                 //current buffer number
  }

  if ( returnValue < TD_Error )
  {
    cout << "Driver Initiated Successfully" << endl;

    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/

    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  : ";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else 
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

    /**********************************************
    * Acquisition
    **********************************************/

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    int loadStatus = 0;
    //TD_Length bytes;
    int count = 0;
    records = 0;

    while( loadStatus != 1 )
    {
      loadStatus = bufGetRow1MBSupport();
      if( loadStatus != -1 )
      {
        //if a buffer is ready to send
        //check buffer integrity
        //checkbuffer();
        count++;

        returnValue = conn -> PutBuffer(mbuffer[(curbuff+1)%2],\
        sendbuffersize, indicator);
     
        if(returnValue < TD_Error)
        {
          cout << "sent buffer " << count << " indicator mode " \
          << (int)indicator << "\n";
	      //indicator = (indicator +1 )%2;
	    }
	    else
	    {
	      //Get Error Information

	      cout << "Error occured during PutBuffer" << endl;
	      conn -> GetErrorInfo(&errorMsg, &errorType);

	      if ( errorMsg != NULL )
          {
	        cout << errorMsg << endl;
	        cout << "Type: " << errorType << endl;
	      }
          else
          {
	        cout << "No Error Info Available" << endl;
	      }
        }
      }
    }
    if ( returnValue < TD_Error )
    {
      /**********************************************
      * End Acquisition
      **********************************************/

      returnValue = conn -> EndAcquisition();
      if ( returnValue < TD_Error )
      {
        cout << "Acquistion completed Sucessfully" << endl;
	    cout << "Stream completed successfully" << endl;
      }
      else
      {
        //Get Error Information

        cout << "Error occured during EndAcquisition" << endl;
        conn -> GetErrorInfo(&errorMsg, &errorType);

        if ( errorMsg != NULL )
        {
          cout << errorMsg << endl;
          cout << "Type: " << errorType << endl;
        }
        else
        {
          cout << "No Error Info Available" << endl;
        }
      }
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Acquisition" << endl;
      conn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Initiate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }    
  }

  /**********************************************
  * Terminate
  **********************************************/

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    
  returnValue = conn -> Terminate();
  if( returnValue >= TD_Error)
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }	
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfuly" << endl;
  }
      
  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting objects" << endl;
  delete dmlGr;
  delete schema;
  delete conn;

  if(mbuffer != NULL)
  {
    delete [] mbuffer[0];
    delete [] mbuffer[1];
    delete [] mbuffer;
  }

  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(log_table);
  free(error_table_1);
  free(dml_statment);

  cout << "*** Stream Complete ***" << endl;                 
  cout <<"\n###################### OUTPUT END #######################" << endl; 
  /* DR97368 */
}

/*************************** Utility Functions *******************************/

/***************************************
* Function sets input file name
***************************************/
void BlockLoad::setInput(char* fname)
{
  if( inputFileName != NULL )
  {
    delete inputFileName;
  }
  inputFileName = strdup(fname);
}

/***************************************
* Function sets output file name
***************************************/
void BlockLoad::setOutput(char* fname)
{
  if( outputFileName != NULL )
  {
    delete outputFileName;
  }
  outputFileName = strdup(fname);
}

/***************************************
* Function reads in a row of data and place it into buffer
* Supports 1MB database
***************************************/
int BlockLoad::bufGetRow1MBSupport()
{
  char dummy;
  //Open File if not already open
  if( inputStream == NULL )
  {
    inputStream = fopen(inputFileName, "rb");
    if( inputStream == NULL )
    {
      cout << "Error opening file " << inputFileName << endl;
      return -1;
    }
  }
  //leave 2/4 bytes empty (rowheadersize - rowlengthsize)
  curpos += parcesize;

  //Read  the length of each record..read first two bytes from the datafile
  unsigned short int tmplen;
  size_t recLen = fread(&tmplen, 1, (size_t)2, inputStream);
	
  int x = 0;
  x = tmplen;
  if(rowlengthsize == 4) // If the database supports 1MB row
  {
    memcpy(curpos, &x, rowlengthsize);
  }
  else   // Doesn't support 1MB row
  {
    memcpy(curpos, &tmplen, rowlengthsize);
  }

  if(recLen != 2)
  {
    //calculate and set the buffer size
    sendbuffersize = byteswritten + buftrailersize;
    curbuff = (curbuff + 1) % 2;   //switch the current buffer
    fclose(inputStream);
    inputStream = NULL;
    return 1;		           //end of data
  }
	
  //check if row will fit
  if( tmplen + rowheadersize + buftrailersize + byteswritten < buffersize )
  {
    curpos += rowlengthsize;
    byteswritten += rowheadersize;

    fread(curpos, 1, tmplen, inputStream);
    fread(&dummy, 1, 1, inputStream);
    curpos += tmplen;           //set the new position in the buffer
    byteswritten += tmplen;     //update the number of bytes written
    return -1;			//buffer not full return code
  }
  else
  {
    char *nbuff = mbuffer[(curbuff+1)%2];
    //calculate and set previous buffer size
    sendbuffersize = byteswritten+buftrailersize;
    nbuff += parcesize;
    memcpy(nbuff, &tmplen, rowlengthsize);
    curpos = nbuff + rowlengthsize;

    fread(curpos, 1, tmplen, inputStream);
    fread(&dummy, 1, 1, inputStream);    
    byteswritten = rowheadersize + tmplen;
    curpos += tmplen;                //set the current position in the buffer
    curbuff = (curbuff+1)%2;         //switch the current buffer
    return 0;		             //buffer full
  }
}

