/*************************************************************************
*
* Copyright 2005-2011, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*
* CheckPoint sample:
* Stream operator inserts 10 rows to the Database.
*
*************************************************************************/

#include "checkpoint.h"
#include "OptionsManager.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "connection.h"
#include "string.h"

using namespace teradata::client::API;

checkpoint::checkpoint(OptionsManager * om): m_om(om)
{
  inputStream = NULL;
  outputStream = NULL;
  inputFileName = NULL;
  outputFileName = NULL;
}

checkpoint::~checkpoint(void)
{
  if( inputFileName != NULL )
  {
    free(inputFileName);
  }

  if( outputFileName != NULL )
  {
    free(outputFileName);
  }
}

/***********************************************************************
* Function Shows Example of Using Checkpoint/Restart With Stream Driver
************************************************************************/

void checkpoint::checkpointTest()
{
  time_t now;
  time(&now);
  int returnValue = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char * tdp_id        = strdup(m_om -> getTdpId().c_str());
  char * user_name     = strdup(m_om -> getUserName().c_str());
  char * user_password = strdup(m_om -> getUserPassword().c_str());
  char * target_table  = strdup(m_om -> getTargetTable().c_str());
  char * error_table   = strdup(m_om -> getErrorTable().c_str());
  char * log_table     = strdup(m_om -> getLogTable().c_str());

  char * dml_statment  = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :Associate_Id, :Associate_Name,\
  :Salary, :DOJ, :Designation, :Loan_Amount, :Martial_Status,\
  :No_Of_Dependents);", target_table);

  cout << "######################### OUTPUT START #########################" \
  << endl; 
  cout << "\nTest Description :CheckPoint sample" << endl;
  cout << "Operator type    :STREAM" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name << endl;
  cout << "TDP password     :<not displayable>" << endl;
  cout << "Target Table     :" << target_table << endl;
  cout << "Input File       :" << inputFileName << endl;
  cout << "DML Statement    :" << dml_statment << endl;
  
  Connection *conn = new Connection();
  TD_Index dmlNumbers[2];
  /* DR97368 */

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"stream.txt");  //TPT-17718
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);

  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddAttribute(TD_ERROR_TABLE, error_table);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);

  conn -> AddAttribute(TD_MAX_SESSIONS, 4);
  conn -> AddAttribute(TD_MIN_SESSIONS, 1);
  conn -> AddAttribute(TD_TENACITY_HOURS, 2);
  conn -> AddAttribute(TD_TENACITY_SLEEP, 3);
  conn -> AddAttribute(TD_ROBUST, (char*)"No");  //TPT-17718
  conn -> AddAttribute(TD_RESTARTMODE, 1);
  conn -> AddAttribute(TD_MAX_INSTANCES, 1);
  conn -> AddAttribute(TD_INSTANCE_NUM, 1);
  conn -> AddAttribute(TD_CHARSET, (char*)"ASCII");  //TPT-17718

  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
  schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
  schema -> AddColumn((char*)"Salary",TD_FLOAT, 8, 8, 1);  //TPT-17718
  schema -> AddColumn((char*)"DOJ", TD_DATE, 4);  //TPT-17718
  schema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718
  schema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
  schema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
  schema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1);  //TPT-17718
  conn -> AddSchema(schema);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  DMLGroup * dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statment);

  dmlGr -> AddSerializeOn(8, "Associate_Id", "Associate_Name", \
  "Salary", "DOJ", "Designation", "Loan_Amount", \
  "Martial_Status", "No_Of_Dependents", NULL);

  dmlGr -> AddUseList(8, "Associate_Id", "Associate_Name", \
  "Salary", "DOJ", "Designation", "Loan_Amount", \
  "Martial_Status", "No_Of_Dependents", NULL);

  conn -> AddDMLGroup(dmlGr, &dmlNumbers[0]);
  conn -> UseDMLGroups(dmlNumbers, 1);

  /**********************************************
  * Initiate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();
  if ( returnValue < TD_Error )
  {
    cout << "Driver Initiated Successfully" << endl;
    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  : ";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else 
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

    /**********************************************
    * Acquisition - Load 3 Rows
    **********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    char rowBuffer[78]; /* must include the EOL byte */
    int loadStatus = 0;
    TD_Length bytes;
    int count = 0;

    while( loadStatus != -1 )
    {
      loadStatus = GetRowData(rowBuffer, 78);
      if( loadStatus != -1 )
      {
        /* pick up the first 2 bytes as the row length \
        of the indicator mode record */

        bytes = *((unsigned short *)rowBuffer);

        returnValue = conn -> PutRow( rowBuffer + 2 , bytes);
        if ( returnValue >= TD_Error )
        {
          cout << "PutRow failed on row " << count+1 \
          << " with status " << returnValue << endl;
          loadStatus = GetRowData(NULL,0); //Abort load
        }
        else
        {
          count++;
          if ( count == 3 )
          {
            break;
          }
        }
      }
    }
    if(returnValue == 0)
    {
      cout << "First three rows loaded Successfully" << endl;
    }
    if ( returnValue < TD_Error )
    {
      /**********************************************
      * Checkpoint
      **********************************************/

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "                CHECKPOINT                  " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      char* cp_data = NULL;
      TD_Length cp_data_len = 0;

      returnValue = conn -> Checkpoint(&cp_data, &cp_data_len);
      if ( returnValue < TD_Error )
      {
        cout << "CheckPoint taken after three rows loaded" << endl;

        /**********************************************
        *continue Acquisition - Load 2 Rows
        **********************************************/

        cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
        cout << "        CONTINUE ACQUISITION PHASE            " << endl;
        cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

        while( loadStatus != -1 )
        {
          loadStatus = GetRowData(rowBuffer, 78);
	      if( loadStatus != -1 )
	      {
	        /* pick up the first 2 bytes as the row length 
            of the indicator mode record */

	        bytes = *((unsigned short *)rowBuffer);
	        returnValue = conn -> PutRow( rowBuffer + 2 , bytes);

	        if ( returnValue >= TD_Error )
	        {
	          cout << "PutRow failed on row " << count+1 <<  \
              " with status " << returnValue << endl;
	          loadStatus = GetRowData(NULL,0); //Abort load
	        }
            else
            {
	          count++;
	          if ( count == 5 )
              {
	            break;
	          }
            }
          }
        }
        if(returnValue == 0)
        {
          cout << "Next two rows loaded" << endl;
        }
        if ( returnValue < TD_Error )
        {
	      /**********************************************
	      * Terminate
	      **********************************************/

          cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
          cout << "              TERMINATE PHASE                " << endl;
          cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

	      returnValue = conn -> Terminate();
          cout << "Driver Prematurely terminated" << endl;
	      if ( returnValue < TD_Error )
	      {
	        /**********************************************
	        * Re-Initiate
	        **********************************************/
            cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
            cout << "            RE-INITIATE PHASE               " << endl;
            cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

            returnValue = conn -> Initiate();
	        if ( returnValue < TD_Error )
	        {
              cout << "Driver re_Initiated Successfully" << endl;

 	          /**********************************************
	          * Restart
	          **********************************************/

	          returnValue = conn -> Restart(cp_data,cp_data_len);
              if ( returnValue < TD_Error )
              {
                cout << "Driver restarted Successfully" << endl;

	            /**********************************************
	            * Resubmit lost rows and finish Acquisition
	            **********************************************/

                cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
                cout << "             ACQUISITION PHASE            " << endl;
                cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

	            loadStatus = restartRead(78*3);
	            while( loadStatus != -1 )
	            {
	              loadStatus = GetRowData(rowBuffer, 78);
	              if( loadStatus != -1 )
                  {
		            /* pick up the first 2 bytes as the row length \
                    of the indicator mode record */

		            bytes = *((unsigned short *)rowBuffer);
		            returnValue = conn -> PutRow( rowBuffer + 2 , bytes);

	                if ( returnValue >= TD_Error )
	                {
                      cout << "PutRow failed on row " << count+1 \
                      << " with status " << returnValue << endl;
		              loadStatus = GetRowData(NULL, 0); //Abort load
	                }
                    else
                    {
		              count++;
	                }
	              }
                }
                if(returnValue == 0 )
                {
                  cout << "Lost rows and remaining rows sent" << endl;
                }
	            if ( returnValue < TD_Error )
	            {
		          cout << "Total rows sent " << count << endl;

		          /**********************************************
	              * End Acquisition
		          **********************************************/

	              returnValue = conn -> EndAcquisition();
	              if ( returnValue < TD_Error )
	              {
                    cout << "Acquistion completed Sucessfully" << endl;

	                /**********************************************
	                * Application
	                **********************************************/

	                if ( returnValue < TD_Error )
                    {
                      cout << "Stream  completed successfully" << endl;
	                }
                    else
                    {  
                      //Get Error Information

                      cout << "Error occured during Application" << endl;
                      conn -> GetErrorInfo(&errorMsg, &errorType);

                      if ( errorMsg != NULL )
                      {
                        cout << errorMsg << endl;
                        cout << "Type: " << errorType << endl;
                      }
                      else
                      {
                        cout << "No Error Info Available" << endl;
                      }
                    }
                  }
                  else
                  {
		            //Get Error Information

	                cout << "Error occured during EndAcquisition" << endl;
		            conn -> GetErrorInfo(&errorMsg, &errorType);

	                if ( errorMsg != NULL )
                    {
		              cout << errorMsg << endl;
		              cout << "Type: " << errorType << endl;
		            }
                    else
                    {
		              cout << "No Error Info Available" << endl;
	                }
	              }
	            }
                else
                {
	              //Get Error Information

	              cout << "Error occured during Acquisition" << endl;
                  conn -> GetErrorInfo(&errorMsg, &errorType);

	              if ( errorMsg != NULL )
                  {
                    cout << errorMsg << endl;
		            cout << "Type: " << errorType << endl;
	              }
                  else
                  {
	                cout << "No Error Info Available" << endl;
                  }
	            }
	          }
              else
              {
	            //Get Error Information

                cout << "Error occured during Restart" << endl;
	            conn -> GetErrorInfo(&errorMsg, &errorType);

	            if ( errorMsg != NULL )
                {
                  cout << errorMsg << endl;
                  cout << "Type: " << errorType << endl;
	            }
                else
                {
                  cout << "No Error Info Available" << endl;
	            }
              }
	        }
            else
            {
	          //Get Error Information

              cout << "Error occured during Re-Initiate" << endl;
              conn -> GetErrorInfo(&errorMsg, &errorType);

	          if ( errorMsg != NULL )
              {
	            cout << errorMsg << endl;
	            cout << "Type: " << errorType << endl;
	          }
              else
              {
	            cout << "No Error Info Available" << endl;
              }
            }
          }
          else
          {
	        //Get Error Information

	        cout << "Error occured during Terminate" << endl;
	        conn -> GetErrorInfo(&errorMsg, &errorType);

	        if ( errorMsg != NULL )
            {
	          cout << errorMsg << endl;
	          cout << "Type: " << errorType << endl;
	        }
            else
            {
	          cout << "No Error Info Available" << endl;
	        }
          }
        }
        else
        {
          //Get Error Information

	      cout << "Error occured during Acquisition" << endl;
	      conn -> GetErrorInfo(&errorMsg, &errorType);

	      if ( errorMsg != NULL )
          {
	        cout << errorMsg << endl;
	        cout << "Type: " << errorType << endl;
	      }
          else
          {
            cout << "No Error Info Available" << endl;
	      }
        }
      }
      else
      {
        //Get Error Information

        cout << "Error occured during Checkpoint" << endl;
        conn -> GetErrorInfo(&errorMsg, &errorType);

        if ( errorMsg != NULL )
        {
	      cout << errorMsg << endl;
	      cout << "Type: " << errorType << endl;
        }
        else
        {
	      cout << "No Error Info Available" << endl;
        }
      } 
    }
    else
    {
      //Get Error Information

      cout << "Error occured during Acquisition" << endl;
      conn -> GetErrorInfo(&errorMsg, &errorType);

      if ( errorMsg != NULL )
      {
        cout << errorMsg << endl;
        cout << "Type: " << errorType << endl;
      }
      else
      {
        cout << "No Error Info Available" << endl;
      }
    }
  }
  else
  {
    //Get Error Information

    cout << "Error occured during Initiate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    } 
  }

  /**********************************************
  * Terminate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Terminate();
  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfully" << endl;
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting Objects" << endl;

  delete dmlGr;
  delete schema;
  delete conn;

  free(user_name);
  free(user_password);
  free(tdp_id);
  free(target_table);
  free(log_table);
  free(error_table);
  free(dml_statment);

  cout << "*** CheckPoint-Restart Complete ***" << endl;   
  cout << "\n###################### OUTPUT END #####################" << endl;
  /* DR97368 */
}

/**************************** Utility Functions ******************************/

/***************************************
* Function sets input file name
***************************************/
void checkpoint::setInput(char* fname)
{
  if( inputFileName != NULL )
  {
    delete inputFileName;
  }
  inputFileName = strdup(fname);
}

/***************************************
* Function sets output file name
***************************************/
void checkpoint::setOutput(char* fname)
{
  if( outputFileName != NULL )
  {
    delete outputFileName;
  }
  outputFileName = strdup(fname);
}

/***************************************
* Function reads in a row of data
***************************************/
int checkpoint::GetRowData(char* rowBuffer, size_t rowLength)
{
  //Close File if End Signal Received
  if ( rowLength <= 0 )
  {
    if ( inputStream != NULL )
    {
      fclose(inputStream);
    }
    return -1;
  }
  //Open File if not already open
  if( inputStream == NULL )
  {
    inputStream = fopen(inputFileName, "rb");
    if( inputStream == NULL )
    {
      cout << "Error opening file " << inputFileName << endl;
      return -1;
    }
  }
  //Read a row of data
  size_t recLen = fread(rowBuffer, 1, rowLength, inputStream);

  if ( recLen < 1 )
  {
    if ( feof(inputStream) )  
    {
      cout << "End of file reached" << endl;
      if( inputStream != NULL )
      {
        fclose(inputStream);
      }
      return -1;
    }
    else
    {
      /* return some read error here and exit */
      cout << "read error" << endl;

      if( inputStream != NULL )
      {
        fclose(inputStream);
      }
      return -1;
    }
  }
  return 0;
}

/****************************************************
* Function writes a row of data to an external file
****************************************************/
int checkpoint::WriteRowData(char* rowBuffer)
{
  if( rowBuffer != NULL )
  {
    //Open File if not already open
    if( outputStream == NULL )
    {
      outputStream = fopen(outputFileName, "w");
      if( outputStream == NULL )
      {
	    cout << "Error opening file " << outputFileName << endl;
	    return -1;
      }
    }
    fprintf(outputStream, "%s\n",rowBuffer+1); // account for indicator byte
  }
  else
  {
    fclose(outputStream);
    return -1;
  }
  return 0;
}

/***************************************
* Function resets read file for restart
***************************************/
int checkpoint::restartRead(size_t offset)
{
  //Close file if open
  if( inputStream != NULL )
  {
    fclose(inputStream);
  }
  //Open file
  inputStream = fopen(inputFileName, "rb");

  if( inputStream == NULL )
  {
    cout << "Error opening file " << inputFileName << endl;
    return -1;
  }

  //Set File Position
  fseek(inputStream,offset,SEEK_SET);

  return 0;
}
