/*************************************************************************
*
* Copyright 2005-2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/


#ifndef __MVS__

  #include "OptionsManager.h"
  #include <fstream>
  #include <string>
  #include <iostream>

#else

  #include "PTCOPTMA.h"
  #include <fstream>
  #include <string>
  #include <iostream>

#endif//__MVS__

//#if defined(__HIGHC__) || defined(_ATT4)
#if defined(OS_MPRAS)
#else
using namespace std;
#endif

OptionsManager::OptionsManager()                     //TPT-17718
: oper_type(TD_LOAD), num_tests(1), num_sessions(2),
test_to_run(SAM_DEFAULT),user_name(SAM_DEFAULT),
user_password(SAM_DEFAULT), tdp_id(SAM_DEFAULT), 
target_table(SAM_DEFAULT), target_table_export(SAM_DEFAULT), 
target_table_stream(SAM_DEFAULT), log_table(SAM_DEFAULT), 
log_table_export(SAM_DEFAULT), log_table_stream(SAM_DEFAULT),
error_table(SAM_DEFAULT), error_table_1(SAM_DEFAULT), 
error_table_2(SAM_DEFAULT), error_table_export(SAM_DEFAULT), 
error_table_stream(SAM_DEFAULT), work_table(SAM_DEFAULT), 
input_file_name(SAM_DEFAULT), output_file_name(SAM_DEFAULT)
{
	target_table_array = NULL;
	error_table_1_array = NULL;
	error_table_2_array = NULL;
	work_table_array = NULL;
}


OptionsManager::~OptionsManager()
{
    if (target_table_array != NULL)
    {
        delete [] target_table_array;
    }
    if (error_table_1_array != NULL)
    {
        delete [] error_table_1_array;
    }
    if (error_table_2_array != NULL)
    {
        delete [] error_table_2_array;
    }
    if (work_table_array != NULL)
    {
        delete [] work_table_array;
    }
}


void OptionsManager::readOptionsFromFile(const char * fileName)
{
    ifstream infile(fileName);
    string inLine;
    string inParameter;
    string inValue;
    char inChars[256];
    size_t paramBeginPos;
    size_t paramEndPos;
    size_t equalPos;
    size_t valueBeginPos;
    size_t valueEndPos;

#ifndef OS_MPRAS
    if (infile.is_open()) 
    { 
        getline(infile, inLine);
#else
	infile.getline(inChars, 256);
        inLine = inChars;
#endif
        while (infile.good())
        {
            paramBeginPos = inLine.find_first_not_of(" ", 0);
            if (inLine[paramBeginPos] == '#')
            {
#ifndef OS_MPRAS
                getline(infile, inLine);
#else
		infile.getline(inChars, 256);
		inLine = inChars;
#endif
                continue;
            }
            // can handle ???=???, ??? =??? or ???= ??? cases
            equalPos = inLine.find_first_of("=", paramBeginPos);
            paramEndPos = inLine.find_last_not_of(" ", equalPos - 1);
#ifdef OS_MPRAS
	    ++paramEndPos;
#endif
	    inParameter = inLine.substr(paramBeginPos,
 paramEndPos - paramBeginPos + 1);
            valueBeginPos = inLine.find_first_not_of(" ", 
equalPos + 1);
            valueEndPos = inLine.find_last_not_of(" ");
            inValue = inLine.substr(valueBeginPos,
 valueEndPos - valueBeginPos + 1);
            
            if (inParameter == SAM_SYSTEM_OPERATOR) 
            {
                if (inValue == SAM_LOAD) 
                {
                    oper_type = TD_LOAD;
                }
                else if (inValue == SAM_EXPORT)
                {
                    oper_type = TD_EXPORT;
                }
                else if (inValue == SAM_UPDATE)
                {
                    oper_type = TD_UPDATE;
                }
                else if (inValue == SAM_STREAM)
                {
                    oper_type = TD_STREAM;
                }
            }
            else if (inParameter == SAM_TEST_TO_RUN )
            {
                test_to_run = inValue;
            }
            else if (inParameter == SAM_NUM_TESTS) 
            {
                num_tests = atoi(inValue.c_str());
            }
            else if (inParameter == SAM_NUM_SESSIONS) 
            {
                num_sessions = atoi(inValue.c_str());
            }
            else if (inParameter == SAM_USER_NAME) 
            {
                user_name = inValue;
            }
            else if (inParameter == SAM_USER_PASSWORD)
            {
                user_password = inValue;
            }
            else if (inParameter == SAM_TDP_ID)
            {
                tdp_id = inValue;
            }
            else if (inParameter == SAM_TARGET_TABLE)
            {
                target_table = inValue;
            }
            else if (inParameter == SAM_TARGET_TABLE_EXPORT)
            {
                target_table_export = inValue;
            }
            else if (inParameter == SAM_TARGET_TABLE_STREAM)
            {
                target_table_stream = inValue;
            }
            else if (inParameter == SAM_TARGET_TABLE_ARRAY)
            {
                size_t tableNameBeginPos;
                size_t nextTableNameBeginPos;
                size_t tablesEndPos;
                int table_array_size;
                tablesEndPos = inValue.find_first_of(";", 0);
                table_array_size = atoi(inValue.substr(0,
 tablesEndPos).c_str());
                target_table_array = new string[table_array_size];
                tableNameBeginPos = inValue.find_first_of(";",
 tablesEndPos) + 1;
                for (int i = 0; i < table_array_size; i++) {
                    nextTableNameBeginPos = inValue.find_first_of(";",
 tableNameBeginPos);
                    target_table_array[i] =
 inValue.substr(tableNameBeginPos, nextTableNameBeginPos - tableNameBeginPos);
                    tableNameBeginPos = nextTableNameBeginPos + 1;
                }
            }
            else if (inParameter == SAM_LOG_TABLE)
            {
                log_table = inValue;
            }
            else if (inParameter == SAM_LOG_TABLE_EXPORT)
            {
                log_table_export = inValue;
            }
            else if (inParameter == SAM_LOG_TABLE_STREAM)
            {
                log_table_stream = inValue;
            }
            else if (inParameter == SAM_ERROR_TABLE)
            {
                error_table = inValue;
            }
            else if (inParameter == SAM_ERROR_TABLE_1)
            {
                error_table_1 = inValue;
            }
            else if (inParameter == SAM_ERROR_TABLE_2)
            {
                error_table_2 = inValue;
            }
            else if (inParameter == SAM_ERROR_TABLE_EXPORT)
            {
                error_table_export = inValue;
            }
            else if (inParameter == SAM_ERROR_TABLE_STREAM)
            {
                error_table_stream = inValue;
            }
            else if (inParameter == SAM_ERROR_TABLE_1_ARRAY)
            {
                size_t tableNameBeginPos;
                size_t nextTableNameBeginPos;
                size_t tablesEndPos;
                int table_array_size;
                tablesEndPos = inValue.find_first_of(";", 0);
                table_array_size = atoi(inValue.substr(0,
 tablesEndPos).c_str());
                error_table_1_array = new string[table_array_size];
                tableNameBeginPos = inValue.find_first_of(";", 
tablesEndPos) + 1;
                for (int i = 0; i < table_array_size; i++) {
                    nextTableNameBeginPos = inValue.find_first_of(";",
 tableNameBeginPos);
                    error_table_1_array[i] =
 inValue.substr(tableNameBeginPos, nextTableNameBeginPos - tableNameBeginPos);
                    tableNameBeginPos = nextTableNameBeginPos + 1;
                }
            }
            else if (inParameter == SAM_ERROR_TABLE_2_ARRAY)
            {
                size_t tableNameBeginPos;
                size_t nextTableNameBeginPos;
                size_t tablesEndPos;
                int table_array_size;
                tablesEndPos = inValue.find_first_of(";", 0);
                table_array_size = atoi(inValue.substr(0,
 tablesEndPos).c_str());
                error_table_2_array = new string[table_array_size];
                tableNameBeginPos = inValue.find_first_of(";",
 tablesEndPos) + 1;
                for (int i = 0; i < table_array_size; i++) {
                    nextTableNameBeginPos =
 inValue.find_first_of(";", tableNameBeginPos);
                    error_table_2_array[i] =
 inValue.substr(tableNameBeginPos, nextTableNameBeginPos - tableNameBeginPos);
                    tableNameBeginPos = nextTableNameBeginPos + 1;
                }
            }
            else if (inParameter == SAM_WORK_TABLE)
            {
                work_table = inValue;
            }
            else if (inParameter == SAM_WORK_TABLE_ARRAY)
            {
                size_t tableNameBeginPos;
                size_t nextTableNameBeginPos;
                size_t tablesEndPos;
                int table_array_size;
                tablesEndPos = inValue.find_first_of(";", 0);
                table_array_size = atoi(inValue.substr(0,
 tablesEndPos).c_str());
                work_table_array = new string[table_array_size];
                tableNameBeginPos = inValue.find_first_of(";",
 tablesEndPos) + 1;
                for (int i = 0; i < table_array_size; i++) {
                    nextTableNameBeginPos = inValue.find_first_of(";",
 tableNameBeginPos);
                    work_table_array[i] =
 inValue.substr(tableNameBeginPos, nextTableNameBeginPos - tableNameBeginPos);
                    tableNameBeginPos = nextTableNameBeginPos + 1;
                }
            }
#if defined(ZLINUX)
        else if (inParameter == BIG_ENDIAN_INPUT_FILE)

#else
#if defined(OS_WINDOWS) || defined(OS_LINUX) \
 || defined(OS_MPRAS) || defined(OS_SOLARIS_OPTERON) \
 || defined(OS_OSX) //TPT-17719
            else if (inParameter == LITTLE_ENDIAN_INPUT_FILE)
#else
            else if (inParameter == BIG_ENDIAN_INPUT_FILE)
#endif
#endif
            {
                input_file_name = inValue;
            }

#if defined(ZLINUX)
        else if (inParameter == BIG_ENDIAN_OUTPUT_FILE)

#else
#if defined(OS_WINDOWS) || defined(OS_LINUX) \
 || defined(OS_MPRAS) || defined(OS_SOLARIS_OPTERON) \
 || defined(OS_OSX) //TPT-17719
            else if (inParameter == LITTLE_ENDIAN_OUTPUT_FILE)
#else
            else if (inParameter == BIG_ENDIAN_OUTPUT_FILE)
#endif
#endif
            {
                output_file_name = inValue;
            }

#ifndef OS_MPRAS
	    getline(infile, inLine);
#else
	    infile.getline(inChars, 256);
	    inLine = inChars;
#endif
        }

        /* parameter error checking here */

        infile.close();
#ifndef OS_MPRAS
    } 
    else { 
        cout << "Error opening config file. Using default options.\n"; 
    }  
#endif
}

TD_OperatorType OptionsManager::getOperType(void) 
{
    return oper_type;
}

int OptionsManager::getNumTests(void) 
{
    return num_tests;
}

int OptionsManager::getNumSessions(void) 
{
    return num_sessions;
}

string OptionsManager::getTestToRun(void)
{
   return test_to_run;
}

string OptionsManager::getUserName(void) 
{
    return user_name;
}

string OptionsManager::getUserPassword(void) 
{
    return user_password;
}

string OptionsManager::getTdpId(void) 
{
    return tdp_id;
}

string OptionsManager::getTargetTable(void) 
{
    return target_table;
}

string OptionsManager::getTargetTable(int item) 
{
    return target_table_array[item];
}

string OptionsManager::getTargetTableExport(void) 
{
    return target_table_export;
}

string OptionsManager::getTargetTableStream(void) 
{
    return target_table_stream;
}

string OptionsManager::getLogTable(void) 
{
    return log_table;
}

string OptionsManager::getLogTableExport(void) 
{
    return log_table_export;
}

string OptionsManager::getLogTableStream(void) 
{
    return log_table_stream;
}

string OptionsManager::getErrorTable(void) 
{
    return error_table;
}

string OptionsManager::getErrorTable1(void) 
{
    return error_table_1;
}

string OptionsManager::getErrorTable1(int item) 
{
    return error_table_1_array[item];
}

string OptionsManager::getErrorTable2(void) 
{
    return error_table_2;
}

string OptionsManager::getErrorTable2(int item) 
{
    return error_table_2_array[item];
}

string OptionsManager::getErrorTableExport(void) 
{
    return error_table_export;
}

string OptionsManager::getErrorTableStream(void) 
{
    return error_table_stream;
}

string OptionsManager::getWorkTable(void) 
{
    return work_table;
}

string OptionsManager::getWorkTable(int item) 
{
    return work_table_array[item];
}

string OptionsManager::getInputFileName(void) 
{
    return input_file_name;
}

string OptionsManager::getOutputFileName(void) 
{
    return output_file_name;
}








