/*************************************************************************
*
* Copyright 2005-2009, 2012-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/

#ifndef OPTIONSMANAGER_H
#define OPTIONSMANAGER_H

#include <string>
#include "schema.h"

#include <cstring>       // TPT-11431

/* platform macros */
#if defined( __MVS__) || defined(WIN32)
#else 
#define _OSUNIX_
#endif

#if defined(__MVS__)
#define OS_MVS
#elif defined(WIN32) || defined(WIN64)
#define OS_WINDOWS
#elif defined(__sun)
#define OS_SOLARIS
   #if defined(i386) || defined(__amd64)
   #define OS_SOLARIS_OPTERON
   #endif
#elif defined(_AIX)
#define OS_AIX
#elif defined(HPUX)
#define OS_HPUX
#elif defined(__linux__)
#define OS_LINUX
#elif defined(__HIGHC__) || defined(_ATT4)
#define OS_MPRAS
#elif defined(__APPLE__)    // TPT-17719
#define OS_OSX              // TPT-17719
#else
#error Building on a unknown platform
#endif

#if defined(OS_SOLARIS) || defined(OS_AIX) || defined(OS_HPUX) \
|| defined(OS_LINUX) || defined(OS_MPRAS) || defined(OS_OSX) // TPT-17719
#define OS_UNIX
#endif


#define SAM_DEFAULT "default"
#define SAM_LOAD "TD_LOAD"
#define SAM_EXPORT "TD_EXPORT"
#define SAM_UPDATE "TD_UPDATE"
#define SAM_STREAM "TD_STREAM"

#define SAM_NUM_TESTS "NUM_TESTS"
#define SAM_NUM_SESSIONS "NUM_SESSIONS"

#define SAM_SYSTEM_OPERATOR "TD_SYSTEM_OPERATOR"
#define SAM_TEST_TO_RUN "TEST_TO_RUN" /* DR105027 */
#define SAM_USER_NAME "TD_USER_NAME"
#define SAM_USER_PASSWORD "TD_USER_PASSWORD"
#define SAM_TDP_ID "TD_TDP_ID"
#define SAM_TARGET_TABLE "TD_TARGET_TABLE"
#define SAM_TARGET_TABLE_EXPORT "TD_TARGET_TABLE_EXPORT"
#define SAM_TARGET_TABLE_STREAM "TD_TARGET_TABLE_STREAM"
#define SAM_TARGET_TABLE_ARRAY "TD_TARGET_TABLE_ARRAY"
#define SAM_LOG_TABLE "TD_LOG_TABLE"
#define SAM_ERROR_TABLE_1 "TD_ERROR_TABLE_1"
#define SAM_ERROR_TABLE_2 "TD_ERROR_TABLE_2"
#define SAM_LOG_TABLE_EXPORT "TD_LOG_TABLE_EXPORT"
#define SAM_LOG_TABLE_STREAM "TD_LOG_TABLE_STREAM"
#define SAM_ERROR_TABLE "TD_ERROR_TABLE"
#define SAM_ERROR_TABLE_EXPORT "TD_ERROR_TABLE_EXPORT"
#define SAM_ERROR_TABLE_STREAM "TD_ERROR_TABLE_STREAM"
#define SAM_ERROR_TABLE_1_ARRAY "TD_ERROR_TABLE_1_ARRAY"
#define SAM_ERROR_TABLE_2_ARRAY "TD_ERROR_TABLE_2_ARRAY"
#define SAM_WORK_TABLE "TD_WORK_TABLE"
#define SAM_WORK_TABLE_ARRAY "TD_WORK_TABLE_ARRAY"

#define LITTLE_ENDIAN_INPUT_FILE "LITTLE_ENDIAN_INPUT_FILE"
#define BIG_ENDIAN_INPUT_FILE "BIG_ENDIAN_INPUT_FILE"
#define LITTLE_ENDIAN_OUTPUT_FILE "LITTLE_ENDIAN_OUTPUT_FILE"
#define BIG_ENDIAN_OUTPUT_FILE "BIG_ENDIAN_OUTPUT_FILE"

#if defined(__HIGHC__) || defined(_ATT4)
#else
using namespace std;
#endif

class OptionsManager
{
public:
    OptionsManager(void);
    ~OptionsManager(void);

    void readOptionsFromFile(const char * fileName);

    TD_OperatorType getOperType(void);
    int getNumTests(void);
    int getNumSessions(void);
    string getTestToRun(void); /* DR105027 */
    string getUserName(void);
    string getUserPassword(void);
    string getTdpId(void);
    string getTargetTable(void);
    string getTargetTable(int item);
    string getTargetTableExport(void);
    string getTargetTableStream(void);
    string getLogTable(void);
    string getLogTableExport(void);
    string getLogTableStream(void);
    string getErrorTable(void);
    string getErrorTable1(void);
    string getErrorTable1(int item);
    string getErrorTable2(void);
    string getErrorTable2(int item);
    string getErrorTableExport(void);
    string getErrorTableStream(void);
    string getWorkTable(void);
    string getWorkTable(int item);
    string getInputFileName(void);
    string getOutputFileName(void);

private:
    TD_OperatorType oper_type;
    int num_tests;
    int num_sessions;
    string test_to_run; /* DR105027 */
    string user_name;
    string user_password;
    string tdp_id;
    string target_table;
    string target_table_export;
    string target_table_stream;
    string * target_table_array;
    string log_table;
    string log_table_export;
    string log_table_stream;
    string error_table;
    string error_table_1;
    string error_table_2;
    string error_table_export;
    string error_table_stream;
    string * error_table_1_array;
    string * error_table_2_array;
    string work_table;
    string * work_table_array;
    string input_file_name;
    string output_file_name;
};

#endif 
