/*************************************************************************
*
* Copyright 2010,2014 by Teradata Corporation. 
* All Rights Reserved.
* TERADATA CONFIDENTIAL TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* History Information:
*
* Revision     Date      DR      DID       Comments
* ---------------------------------------------------------------------
* 15.00.00.01  24032014  TPT21257 KB186006 dynlnk_sample compile errors in MVS
* 15.00.00.00  12262013  TPT20211 kb186006 Copyright year change
* 14.00.00.01  10252010  144824  mv121803  Support MVS
* 13.00.00.01  06182010  142720  mv121803  Support TELAPI dynamic loading
*
*************************************************************************/

#ifndef __MVS__

  #include "GenericDriver.h"
  #include "OptionsManager.h"
  #include "dynlib.h"                 //DR142720
  #include <stdio.h>
  #include <stdlib.h>
  #include <time.h>
  #include "string.h"

#else

  #include "PTCGENDR.h"              //DR144824
  #include "PTCOPTMA.h"              //DR144824
  #include "PTCDLLIB.h"              //DR144824
  #include <stdio.h>
  #include "string.h"

#endif //__MVS__


using namespace teradata::client::API;

GenericDriver::GenericDriver(OptionsManager * om)
{
  inputStream = NULL;
  outputStream = NULL;
  inputFileName = NULL;
  outputFileName = NULL;
  m_om = om;
}

GenericDriver::~GenericDriver(void)
{
  if( inputFileName != NULL )
  {
	free(inputFileName);
  }

  if( outputFileName != NULL )
  {
    free(outputFileName);
  }
}

#ifdef __MVS__
int GenericDriver::getBufferSize() 
{
  int rc;
  char filename[100];
  fldata_t fileinfo;

  //Open File if not already open
  if( inputStream == NULL )
  {
    inputStream = fopen(inputFileName, "rb, type=record");
     
    if( inputStream == NULL )
	{
	  cout << "Error opening file " << inputFileName << endl;
	  return -1;
	}
      
    rc = fldata(inputStream,filename,&fileinfo);
    if (rc != 0)
	  printf("flfldata failed\n");
    else
	  printf("max rec length is %d\n",fileinfo.__maxreclen);
  }
  return (int)fileinfo.__maxreclen;
}
#endif //__MVS__

/***************************************************
* Method for testing Load Driver
***************************************************/

void GenericDriver::testLoad()
{
  time_t now;
  time(&now);
  DynamicLibrary *lib_hdl;
  int returnValue = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());

  char * dml_statment = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :Associate_Id,\
  :Associate_Name, :Salary, :DOJ, :Designation, :Loan_Amount,\
  :Martial_Status, :No_Of_Dependents);", target_table);

     
  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description  :Generic sample" << endl;
  cout << "Operator type     :LOAD" << endl;
  cout << "Date              :" << ctime(&now);
  cout << "TDP ID            :" << tdp_id << endl;
  cout << "TDP username      :" << user_name << endl;
  cout << "TDP password      :<not displayable>" << endl;
  cout << "Target Table      :" << target_table << endl;
  cout << "Input File        :" << inputFileName << endl;
  cout << "DML Statement     :" << dml_statment << endl;

  lib_hdl = DynamicLibrary::Instance();                 //DR142720
  ConnectionX *conn = lib_hdl -> newConnectionX();      //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn->AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
  //conn->AddAttribute(TD_TRACE_OUTPUT, "load.txt");
  //conn->AddArrayAttribute(TD_TRACE_LEVEL, 2,
  //TD_OPER_ALL, TD_GENERAL, NULL);
  //TD_OPER_ALL, TD_OFF, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *schema = lib_hdl->newSchemaX("input");      //DR142720
  schema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
  schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
  schema -> AddColumn((char*)"Salary", TD_FLOAT, 8, 8, 1);  //TPT-17718
  schema -> AddColumn((char*)"DOJ", TD_DATE, 4);  //TPT-17718
  schema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718
#ifndef __MVS__
  schema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
  schema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif //__MVS__
  schema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
  schema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1);  //TPT-17718
  conn -> AddSchema(schema);
  //delete schema;

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;

  DMLGroupX* dmlGr = lib_hdl -> newDMLGroupX();      //DR142720
  dmlGr -> AddStatement(dml_statment);
  conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate
  **********************************************/
    
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();

  if ( returnValue < TD_Error )
  {
    cout << "Driver Initiated Successfully" << endl;

    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/

    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  : ";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

	/**********************************************
	* Acquisition
	**********************************************/
        
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

#ifndef __MVS__
    char rowBuffer[78]; /* must include the EOL byte */
#else
    int maxRecLen = getBufferSize();
    char *rowBuffer = new char[maxRecLen];
#endif		
    int loadStatus = 0;
    TD_Length bytes;
    int count = 0;

	while( loadStatus != -1 )
	{
#ifndef __MVS__
	  loadStatus = GetRowData(rowBuffer,78);
#else
	  loadStatus = GetRowData(rowBuffer,maxRecLen);
#endif

	  if( loadStatus != -1 )
	  {
		/* pick up the first 2 bytes as
        the row length of the indicator mode record */
#ifndef __MVS__
	    bytes = *((unsigned short *)rowBuffer);
		returnValue = conn -> PutRow( rowBuffer + 2 , bytes);
#else
		returnValue = conn -> PutRow( rowBuffer , loadStatus );
#endif
		if ( returnValue >=TD_Error )
		{
		  cout << "PutRow  failed on row " << count+1 
          << " with status " << returnValue << endl;
		  loadStatus = GetRowData(NULL,0); //Abort load
		}
        else
        {
		  count++;
		}
	  }
	}
	if ( returnValue < TD_Error )
	{
	  cout << "Sent " << count << " rows" << endl;

	  /**********************************************
	  * End Acquisition
	  **********************************************/

	  returnValue = conn -> EndAcquisition();

	  if ( returnValue < TD_Error )
	  {
        cout << "Acquistion completed Sucessfully" << endl;

	    /**********************************************
	    * Application
	    **********************************************/

        cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
        cout << "             APPLICATION PHASE             " << endl;
        cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

		returnValue = conn->ApplyRows();

		if ( returnValue < TD_Error )
		{
          cout << "Rows Applied Successfully" << endl;
		  cout << "Load completed successfully" << endl;
		}
        else
        {
		  //Get Error Information
		  cout << "Error occured during Application" << endl;
          conn -> GetErrorInfo(&errorMsg, &errorType);

		  if ( errorMsg != NULL )
          {
            cout << errorMsg << endl;
            cout << "Type: " << errorType << endl;
		  }
          else
          {
            cout << "No Error Info Available" << endl;
		  }
		}
	  }
      else
      {
		//Get Error Information
        cout << "Error occured during EndAcquisition" << endl;
        conn -> GetErrorInfo(&errorMsg, &errorType);

		if ( errorMsg != NULL )
        {
		  cout << errorMsg << endl;
          cout << "Type: " << errorType << endl;
		}
        else
        {
          cout << "No Error Info Available" << endl;
	    }
	  }
	}
    else
    {
	  //Get Error Information
      cout << "Error occured during Acquisition" << endl;
	  conn -> GetErrorInfo(&errorMsg, &errorType);

	  if ( errorMsg != NULL )
      {
		cout << errorMsg << endl;
		cout << "Type: " << errorType << endl;
	  }
      else
      {
        cout << "No Error Info Available" << endl;
	  }
	}
  }
  else
  {
	//Get Error Information
	cout << "Error occured during Initiate" << endl;
	conn -> GetErrorInfo(&errorMsg, &errorType);

	if ( errorMsg != NULL )
    {
	  cout << errorMsg << endl;
	  cout << "Type: " << errorType << endl;
	}
    else
    {
	  cout << "No Error Info Available" << endl;
	}
  }

  /**********************************************
  * Terminate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Terminate();

  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfuly" << endl;
  }
  
  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting objects" << endl;
  lib_hdl -> delConnectionX(conn);                        //DR142720
  lib_hdl -> delSchemaX(schema);                          //DR142720
  lib_hdl -> delDMLGroupX(dmlGr);                         //DR142720
  lib_hdl -> Delete();                                    //DR142720

#ifdef __MVS__
  //if (rowBuffer != NULL)
  //delete [] rowBuffer;
#endif
  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(log_table);
  free(dml_statment);

  cout << "*** Load Complete ***" << endl;                   
  cout <<"\n######################## OUTPUT END ###########################" \
  << endl;  /* DR97368 */
}

/***************************************************
* Method for testing Export Driver
***************************************************/
void GenericDriver::testExport()
{
  time_t now;
  time(&now);
  DynamicLibrary  *lib_hdl;               //DR142720
  int returnValue = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *tdp_id        = strdup(m_om -> getTdpId().c_str());

  char *sel_statment  = (char*)malloc(1024);
  sprintf(sel_statment, "sel Associate_Name from %s;", target_table);

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description :Generic sample" << endl;
  cout << "Operator type    :EXPORT" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name << endl;
  cout << "TDP password     :<not displayable>" << endl;
  cout << "Target Table     :" << target_table << endl;
  cout << "Output File      :" << outputFileName << endl;
  cout << "DML Statement    :" << sel_statment << endl;

  /* DR97368 */

  lib_hdl = DynamicLibrary::Instance();                      //DR142720
  ConnectionX *conn = lib_hdl -> newConnectionX();             //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_SELECT_STMT, sel_statment);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_TDP_ID, tdp_id);

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *schema = lib_hdl -> newSchemaX("output");      //DR142720
  schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718

  conn -> AddSchema(schema);

  /**********************************************
  * Initiate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn->Initiate();

  if ( returnValue < TD_Error )
  {
    cout << "Driver Initiated Successfully" << endl;
    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/

    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  : ";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

	/**********************************************
	* Acquisition
	**********************************************/
        
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

	int count = 0;
	char* data;
	TD_Length dataLen = 0;
	int exportStatus = 0;

	while( exportStatus != -1)
    {
	  returnValue = conn -> GetRow(&data, &dataLen);

      if ( returnValue >= TD_Error )
	  {
		cout << "GetRow failed on row " <<
        count+1 << " with status " << returnValue << endl;
		exportStatus = WriteRowData(NULL, 0); //Abort write

      }
      else if ( returnValue == TD_END_Method )
      {
		cout << "End of Data Reached" << endl;
		exportStatus = WriteRowData(NULL,0);
	  }
      else
      {
		exportStatus = WriteRowData(data,dataLen);
		count++;
	  }
	}
    cout << count << " row(s) exported\n";

	if ( returnValue < TD_Error )
	{
	  cout << "Export completed successfully" << endl;
	}
    else
    {
	  //Get Error Information
	  cout << "Error occured during Export" << endl;
	  conn -> GetErrorInfo(&errorMsg, &errorType);

	  if ( errorMsg != NULL )
      {
		cout << errorMsg << endl;
		cout << "Type: " << errorType << endl;
	  }
      else
      {
		cout << "No Error Info Available" << endl;
	  }
	}
  }
  else
  {
	//Get Error Information
	cout << "Error occured during Initiate" << endl;
	conn -> GetErrorInfo(&errorMsg, &errorType);

	if ( errorMsg != NULL )
    {
	  cout << errorMsg << endl;
	  cout << "Type: " << errorType << endl;
	}
    else
    {
	  cout << "No Error Info Available" << endl;
	}
  }

  /**********************************************
  * Terminate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Terminate();

  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfully " << endl;
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting Objects" << endl;
  lib_hdl->delConnectionX(conn);            //DR142720
  lib_hdl->delSchemaX(schema);              //DR142720
  lib_hdl->Delete();                        //DR142720

  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(sel_statment);

  cout << "*** Export Complete ***" << endl;            
  cout <<"\n######################## OUTPUT END ###########################" \
  << endl;  /* DR97368 */
}

/************************************
* Function demonstrates basic usage of
* Update Operator along with DMLGroups
* and DMLOptions
************************************/
void GenericDriver::testUpdate()
{
  time_t now;
  time(&now);
  DynamicLibrary  *lib_hdl;                    //DR142720
  int returnValue = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char *user_name      = strdup(m_om -> getUserName().c_str());
  char *user_password  = strdup(m_om -> getUserPassword().c_str());
  char *tdp_id         = strdup(m_om -> getTdpId().c_str());
  char *log_table      = strdup(m_om -> getLogTable().c_str());
  char *target_tableA  = strdup(m_om -> getTargetTable(0).c_str());
  char *target_tableB  = strdup(m_om -> getTargetTable(1).c_str());
  char *work_tableA    = strdup(m_om -> getWorkTable(0).c_str());
  char *work_tableB    = strdup(m_om -> getWorkTable(1).c_str());
  char *error_table_1A = strdup(m_om -> getErrorTable1(0).c_str());
  char *error_table_1B = strdup(m_om -> getErrorTable1(1).c_str());
  char *error_table_2A = strdup(m_om -> getErrorTable2(0).c_str());
  char *error_table_2B = strdup(m_om -> getErrorTable2(1).c_str());

  char *dml_statmentA  = (char*)malloc(1024);
  sprintf(dml_statmentA, "INSERT INTO %s( :Associate_Id,\
  :Associate_Name, :Salary, :DOJ, :Designation, :Loan_Amount,\
  :Martial_Status, :No_Of_Dependents);", target_tableA);

  char *dml_statmentB = (char*)malloc(1024);
  sprintf(dml_statmentB, "INSERT INTO %s( :Associate_Id,\
  :Associate_Name, :Salary, :DOJ, :Designation, :Loan_Amount,\
  :Martial_Status, :No_Of_Dependents);",target_tableB);


  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description :Generic sample" << endl;
  cout << "Operator type    :UPDATE" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name << endl;
  cout << "TDP password     :<not displyable>" << endl;
  cout << "Target TableA    :" << target_tableA << endl;
  cout << "Target TableB    :" << target_tableB << endl;
  cout << "Input File       :" << inputFileName << endl;
  cout << "DML StatementA   :" << dml_statmentA << endl;
  cout << "DML StatementB   :" << dml_statmentB << endl;
   
  /* DR97368 */

  lib_hdl = DynamicLibrary::Instance();                          //DR142720
  ConnectionX *conn = lib_hdl -> newConnectionX();               //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_UPDATE);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, "update.txt");
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);

  conn -> AddArrayAttribute(TD_TARGET_TABLE, 2,target_tableA, 
  target_tableB, NULL);
  conn -> AddArrayAttribute(TD_WORK_TABLE, 2,work_tableA,
  work_tableB, NULL);
  conn -> AddArrayAttribute(TD_ERROR_TABLE_1,2,error_table_1A,
  error_table_1B, NULL);
  conn -> AddArrayAttribute(TD_ERROR_TABLE_2,2,error_table_2A,
  error_table_2B, NULL);

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *schema = lib_hdl -> newSchemaX("input");      //DR142720
  schema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
  schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
  schema -> AddColumn((char*)"Salary", TD_FLOAT,8,8,1);  //TPT-17718
  schema -> AddColumn((char*)"DOJ", TD_DATE, 4);  //TPT-17718
  schema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718

#ifndef __MVS__
  schema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
  schema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif //__MVS__

  schema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
  schema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1);  //TPT-17718
  conn -> AddSchema(schema);

  /**********************************************
  * Set DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex[2];

  DMLGroupX * dmlGr = lib_hdl -> newDMLGroupX();      //DR142720
  dmlGr -> AddStatement(dml_statmentA);
  dmlGr -> AddDMLOption(MARK_DUPLICATE_ROWS);
  returnValue = conn -> AddDMLGroup(dmlGr, &dmlGroupIndex[0]);
  lib_hdl -> delDMLGroupX(dmlGr);                     //DR142720

  dmlGr = lib_hdl -> newDMLGroupX();                  //DR142720
  dmlGr -> AddStatement(dml_statmentB);
  dmlGr -> AddDMLOption(IGNORE_DUPLICATE_ROWS);
  returnValue = conn -> AddDMLGroup(dmlGr, &dmlGroupIndex[1]);
  lib_hdl->delDMLGroupX(dmlGr);                     //DR142720

  cout << "DMLGroups added with status " << returnValue << endl;

  if ( returnValue < TD_Error )
  {
    /**********************************************
    * Initiate
    **********************************************/

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              INITIATE PHASE               " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    
    returnValue = conn->Initiate();

    if( returnValue < TD_Error )
	{
      cout << "Driver Initiated Successfully" << endl;
      char *dataptr;
      TD_Length datalen;
      int queryStatus = 0;

      /**********************************************
      * Query Version Number
      **********************************************/

      queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              VERSION INFO                  " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "TPTAPI Version  : ";
        cout << *((char**)dataptr) << endl;
        cout << "Operator Version: ";
        cout << *((char**)(dataptr + sizeof(char**))) << endl;
      }
      else
      {
        printf("TD_Evt_Version event failed with status = %d", queryStatus);
      }

	  /**********************************************
	  * Acquisition
	  **********************************************/
      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "             ACQUISITION PHASE            " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

#ifndef __MVS__
	  char rowBuffer[78]; 
      /* must include the EOL byte */
#else
	  int maxRecLen = getBufferSize();
	  char *rowBuffer = new char[maxRecLen];
#endif		
	  TD_Length bytes;
	  int loadStatus = 0;

	  //Get A Row of Data for Insertion
#ifndef __MVS__
	  loadStatus = GetRowData(rowBuffer, 78);
#else
	  loadStatus = GetRowData(rowBuffer, maxRecLen);
#endif
	  if( loadStatus != -1 )
      {
#ifndef __MVS__
	   bytes = *((unsigned short *)rowBuffer);
#endif 

       //Insert the row twice using each DML group
 	   for( int i = 0; i < 2 ; i++ )
       {
		 returnValue = conn -> UseDMLGroups(&dmlGroupIndex[i], 1);

		 if( returnValue >= TD_Error)
         {
		   break;
		 }
         cout << "Sending First Row" << endl;
#ifndef __MVS__
					
         returnValue = conn -> PutRow( rowBuffer + 2 , bytes);
#else
					
         returnValue = conn -> PutRow( rowBuffer , loadStatus);
#endif //__MVS__

		 if( returnValue >= TD_Error)
         {
		   break;
		 }

         cout << "Sending Duplicate Row" << endl;
#ifndef __MVS__
					
         returnValue = conn -> PutRow( rowBuffer + 2, bytes);
#else
					
         returnValue = conn -> PutRow( rowBuffer, loadStatus);
#endif //__MVS__

		 if( returnValue >= TD_Error)
         {
		   break;
	   	 }
	   }
	 }
     cout << "Rows sent with status " << returnValue << endl;
			
     loadStatus = GetRowData(NULL, 0); //End Read and close file

     if( returnValue < TD_Error )
     {
	   /**********************************************
	   * End Acquisition
	   **********************************************/
	   returnValue = conn->EndAcquisition();

	   if( returnValue < TD_Error )
	   {
         cout << "Acquistion completed Sucessfully" << endl;

		 /**********************************************
	     * Application
		 **********************************************/
					
         returnValue = conn -> ApplyRows();

		 if ( returnValue < TD_Error )
		 {
           cout << "Acquistion completed Sucessfully" << endl;
           cout << "Update completed successfully" << endl;
		 }
         else
         {
		   //Get Error Information
           cout << "Error occured during Application" << endl;
           conn -> GetErrorInfo(&errorMsg, &errorType);

           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
		   }
           else
           {
             cout << "No Error Info Available" << endl;
		   }
	     }
	   }
       else
       {
		 //Get Error Information
         cout << "Error occured during EndAcquisition" << endl;
         conn -> GetErrorInfo(&errorMsg, &errorType);

	     if ( errorMsg != NULL )
         {
           cout << errorMsg << endl;
           cout << "Type: " << errorType << endl;
		 }
         else
         {
           cout << "No Error Info Available" << endl;
	     }
	   }
	 }
     else
     {
	   //Get Error Information
       cout << "Error occured during Acquisition" << endl;
       conn -> GetErrorInfo(&errorMsg, &errorType);

	   if ( errorMsg != NULL )
       {
	 	 cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
	   }
       else
       {
         cout << "No Error Info Available" << endl;
	   }
 	 }
   }
   else
   {
	 //Get Error Information
     cout << "Error occured during Initiate" << endl;
	 conn -> GetErrorInfo(&errorMsg, &errorType);

	 if ( errorMsg != NULL )
     {
	   cout << errorMsg << endl;
	   cout << "Type: " << errorType << endl;
	 }
     else
     {
       cout << "No Error Info Available" << endl;
	 }
   }	
 }
 else
 {
    //Get Error Information
    cout << "Error occured during AddDMLGroups" << endl;
    conn -> GetErrorInfo(&errorMsg,&errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
	  cout << "Type: " << errorType << endl;
    }
    else
    {
	  cout << "No Error Info Available" << endl;
    }
  }

  /**********************************************
  * Terminate
  **********************************************/
    
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Terminate();

  if( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg,&errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfuly" << endl;
  }


  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting Objects" << endl;
  lib_hdl -> delConnectionX(conn);                      //DR142720
  lib_hdl -> delSchemaX(schema);                        //DR142720
  lib_hdl -> Delete();                                  //DR142720

#ifdef __MVS__
	//if (rowBuffer != NULL)
	//delete [] rowBuffer;
#endif
   free(user_name);
   free(user_password);
   free(tdp_id);
   free(log_table);
   free(target_tableA);
   free(target_tableB);
   free(work_tableA);
   free(work_tableB);
   free(error_table_1A);
   free(error_table_1B);
   free(error_table_2A);
   free(error_table_2B);
   free(dml_statmentA);
   free(dml_statmentB);

   cout << "*** Update Complete ***" << endl;
   cout <<"\n######################## OUTPUT END ###########################" \
   << endl;
}

/***************************************
* Function demonstrates basic usage of
* Stream Operator
***************************************/

void GenericDriver::testStream()
{
  time_t now;
  time(&now);
  DynamicLibrary   *lib_hdl;                   //DR142720
  int returnValue = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());
  char *error_table   = strdup(m_om -> getErrorTable().c_str());

  char * dml_statment = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :Associate_Id,\
  :Associate_Name, :Salary, :DOJ, :Designation, :Loan_Amount,\
  :Martial_Status, :No_Of_Dependents);", target_table);

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description :Generic sample" << endl;
  cout << "Operator type    :STREAM" << endl;
  cout << "Date             :" << ctime(&now);
  cout << "TDP ID           :" << tdp_id << endl;
  cout << "TDP username     :" << user_name << endl;
  cout << "TDP password     :<not displayable>" << endl;
  cout << "Target Table     :" << target_table << endl;
  cout << "Input File       :" << inputFileName << endl;
  cout << "DML Statement    :" << dml_statment << endl;
    
  /* DR97368 */

  lib_hdl = DynamicLibrary::Instance();                         //DR142720
  ConnectionX *conn = lib_hdl -> newConnectionX();              //DR142720

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);
  //conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"stream.txt");
  //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL,
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddAttribute(TD_ERROR_TABLE, error_table);
  conn -> AddAttribute(TD_MAX_SESSIONS, 4);
  conn -> AddAttribute(TD_MIN_SESSIONS, 1);
  conn -> AddAttribute(TD_TENACITY_HOURS, 2);
  conn -> AddAttribute(TD_TENACITY_SLEEP, 3);
  conn -> AddAttribute(TD_ROBUST, "No");

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *schema = lib_hdl->newSchemaX("input");      //DR142720
  schema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
  schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
  schema -> AddColumn((char*)"Salary", TD_FLOAT, 8, 8, 1);  //TPT-17718
  schema -> AddColumn((char*)"DOJ", TD_DATE, 4);  //TPT-17718
  schema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718

#ifndef __MVS__
  schema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
  schema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif

  schema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
  schema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT,1);  //TPT-17718
  conn -> AddSchema(schema);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;

  DMLGroupX * dmlGr = lib_hdl->newDMLGroupX();      //DR142720
  dmlGr -> AddStatement(dml_statment);

  dmlGr -> AddSerializeOn(8, "Associate_Id", "Associate_Name", 
  "Salary", "DOJ", "Designation", "Loan_Amount", "Martial_Status",
  "No_Of_Dependents", NULL);

  dmlGr->AddUseList(8, "Associate_Id", "Associate_Name",
  "Salary", "DOJ", "Designation", "Loan_Amount", "Martial_Status",
  "No_Of_Dependents", NULL);

  returnValue = conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              INITIATE PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn -> Initiate();

  if ( returnValue < TD_Error )
  {
    cout << "Driver Initiated Successfully" << endl;
    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/

    queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version  : ";
      cout << *((char**)dataptr) << endl;
      cout << "Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

	/**********************************************
	* Acquisition
	**********************************************/
    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "             ACQUISITION PHASE            " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
#ifndef __MVS__
    char rowBuffer[78]; /* must include the EOL byte */
#else
    int maxRecLen = getBufferSize();
    char *rowBuffer = new char[maxRecLen];
#endif		
	int loadStatus = 0;
	TD_Length bytes;
	int count = 0;

	while( loadStatus != -1 )
	{
#ifndef __MVS__
	  loadStatus = GetRowData(rowBuffer, 78);
#else
	  loadStatus = GetRowData(rowBuffer, maxRecLen);
#endif //__MVS__

	  if( loadStatus != -1 )
	  {
		/* pick up the first 2 bytes
        as the row length of the indicator mode record */
#ifndef __MVS__
	    bytes = *((unsigned short *)rowBuffer);
		returnValue =  conn -> PutRow( rowBuffer + 2 , bytes);
#else
        returnValue = conn -> PutRow( rowBuffer, loadStatus ); //TPT-21257
#endif //__MVS__

		if ( returnValue >=TD_Error )
		{
		  cout << "PutRow failed on row "; 
		  cout << count+1 << " with status "; 
          cout << returnValue << endl;
		  loadStatus =  GetRowData(NULL, 0); //Abort load 
		}
        else
        {
		  count++;
		}
	  }
	}
	if ( returnValue < TD_Error )
	{
	  cout << "Sent " << count << " rows" << endl;

	  /**********************************************
	  * End Acquisition
	  **********************************************/

	  returnValue = conn -> EndAcquisition();

	  if ( returnValue < TD_Error )
	  {
        cout << "Acquistion completed Sucessfully" << endl;
		cout << "Stream completed successfully" << endl;
	  }
      else
      {
		//Get Error Information
		cout << "Error occured during  EndAcquisition" << endl;
		conn -> GetErrorInfo(&errorMsg, &errorType);

		if ( errorMsg != NULL )
        {
		  cout << errorMsg << endl;
		  cout << "Type: " << errorType << endl;
		}
        else
        {
		  cout << "No ErrorInfo Available" << endl;
		}
	  }
	}
    else
    {
	  //Get Error Information
	  cout << "Error occured during Acquisition" << endl;
	  conn -> GetErrorInfo(&errorMsg, &errorType);

	  if ( errorMsg != NULL )
      {
		cout << errorMsg << endl;
		cout << "Type: " << errorType << endl;
	  }
      else
      {
		cout << "No Error Info Available" << endl;
	  }
	}
  }
  else
  {
    //Get Error Information
	cout << "Error occured during Initiate" << endl;
	conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
	  cout << errorMsg << endl;
	  cout << "Type: " << errorType << endl;
	}
    else
    {
	  cout << "No Error Info Available" << endl;
	}
  }

  /**********************************************
  * Terminate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = conn->Terminate();

  if ( returnValue >= TD_Error)
  {
    //Get Error Information

    cout << "Error occured during Terminate" << endl;
    conn -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Driver Terminated Successfuly" << endl;
  }

  /**********************************************
  * Clean Up
  **********************************************/

  cout << "Deleting Objects" << endl;
  lib_hdl -> delDMLGroupX(dmlGr);                   //DR142720
  lib_hdl -> delSchemaX(schema);                    //DR142720
  lib_hdl -> delConnectionX(conn);                  //DR142720
  lib_hdl -> Delete();                              //DR142720

#ifdef __MVS__
   //if (rowBuffer != NULL)
   //delete [] rowBuffer;
#endif
   free(user_name);
   free(user_password);
   free(target_table);
   free(tdp_id);
   free(log_table);
   free(error_table);

   cout << "*** Stream Complete ***" << endl;
   cout <<"\n######################## OUTPUT END ###########################" \
   << endl;
              
    /* DR97368 */
}


/*************************** Utility Functions ******************************/

/***************************************
* Function sets input file name
***************************************/
void GenericDriver::setInput(char* fname)
{
  if( inputFileName != NULL )
  {
    delete inputFileName;
  }
  inputFileName = strdup(fname);
}


/***************************************
* Function sets output file name
***************************************/
void GenericDriver::setOutput(char* fname)
{
  if( outputFileName != NULL )
  {
    delete outputFileName;
  }
  outputFileName = strdup(fname);
}


/***************************************
* Function reads in a row of data
***************************************/
int GenericDriver::GetRowData(char* rowBuffer, size_t rowLength)
{
  unsigned short rlength;
       
  //Close File if End Signal Received
  if ( rowLength <= 0 )
  {
	if ( inputStream != NULL )
	{
	  fclose(inputStream);
	}
	return -1;
  }

  //Open File if not already open
  if( inputStream == NULL )
  {
#ifndef __MVS__
	inputStream = fopen(inputFileName, "rb");
#else 
	inputStream = fopen(inputFileName, "rb, type=record");
#endif //__MVS__

	if( inputStream == NULL )
	{
	  cout << "Error opening file " << inputFileName << endl;
	  return -1;
	}
  }

  //Read  the lenght of each record..read first two bytes
#ifndef __MVS__
  size_t recLen =fread(rowBuffer, 1, (size_t)2, inputStream);
#else
  size_t recLen =fread(rowBuffer, 1, rowLength, inputStream);
#endif //__MVS__

#ifndef __MVS__
  if ( recLen < 1 )
  {
#else
	if ( recLen < 2 )
    {
#endif //__MVS__
		if ( feof(inputStream) )  
        {
		  cout << "End of file reached" << endl;
		  if( inputStream != NULL )
          {
			fclose(inputStream);
		  }
		  return -1;
		}
		else
        {
		  /* return some read error here and exit */
		  cout << "read error" << endl;
		  if( inputStream != NULL )
          {
			fclose(inputStream);
		  }
		  return -1;
		}
	  }
      //read row of data with size rlength

#ifndef __MVS__
      rlength = *((unsigned short *)rowBuffer);
      size_t recoLen = fread(rowBuffer+2, 1, (size_t)rlength+1, inputStream);
	  return 0;
#else
	  return recLen;
#endif //__MVS__

}


/*****************************************************
* Function writes out a row of data in indicator mode
******************************************************/
int GenericDriver::WriteRowData(char* data, TD_Length dataLen)
{
  if( data != NULL )
  {
	//Open File if not already open
	if( outputStream == NULL )
	{
#ifndef __MVS__
	  outputStream = fopen(outputFileName, "wb");
#else
	  outputStream = fopen(outputFileName, "wb, type=record");
#endif //__MVS__

	  if( outputStream == NULL )
	  {
		cout << "Error opening file " << outputFileName << endl;
		return -1;
	  }
	}
	unsigned long i = 0;
	unsigned short shortdataLen = (unsigned short)dataLen;
	char* rowBuffer = (char*)malloc(dataLen+20);

#ifndef __MVS__
	memcpy(rowBuffer, &shortdataLen, 2);

	for(i = 0; i < dataLen; i++)
	{
	  rowBuffer[i+2] = data[i];
	}
	rowBuffer[i+2] = 0x0a;

	//fprintf (stream, "%s\n",rowBuffer+1); 
	// account for indicator byte

	fwrite(rowBuffer, dataLen+3, 1, outputStream);

#else
	for(i = 0; i < dataLen; i++)
	{
	  rowBuffer[i] = data[i];
	}

	//fprintf (stream, "%s\n",rowBuffer+1);
    // account for indicator byte

	fwrite(rowBuffer, 1, dataLen, outputStream);
#endif //__MVS__

	free(rowBuffer);
  }
  else
  {
	if( outputStream != NULL )
	{
	  fclose(outputStream);
	}
	return -1;
  }
  return 0;
}

















