/*************************************************************************
*
* Copyright 2005-2008, 2011, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/



#include "fileio.h"

fileio::fileio()
{
  inputbuf = new char[256];
  ofile = NULL;
#ifndef __USEMMAP__
  ifile = NULL;
#else
  ifile = 0;
#endif
  end = 0x0a;
  cur_page = -1;
#ifdef __USEMMAP__
  pageptr = curptr = NULL;
  pagesize = getpagesize() * 128;
  //std::cout << "USING PAGE SIZE " << pagesize << "\n";
#endif
}

fileio::~fileio()
{
  if(inputbuf != NULL)
    delete [] inputbuf;
  //  closeinput();
  //  closeoutput();
}


#ifdef __USEMMAP__
int fileio::openinput(string name)
{
  if(ifile != 0)
    return -1;
  ifile = open(name.c_str(), O_RDONLY);
  if(ifile == 0)
    return -1;
  struct stat filestat;
  fstat(ifile, &filestat);
  filesize = filestat.st_size;
  openpage(0);
  if(pageptr == NULL)
    {
      cout << "mmap error\n";
      return -1;
    }
  return 0;
}
#elif defined(__MVS__)
int fileio::openinput(string name)
{
  if(ifile != NULL)
    return -1;
  ifile = fopen(name.c_str(), "rb, type=record");
  if(ifile == NULL)
    return -1;
  return 0;
}
#else
int fileio::openinput(string name)
{
  if(ifile != NULL)
    return -1;
  ifile = fopen(name.c_str(), "rb");
  if(ifile == NULL)
    return -1;
  return 0;
}
#endif
int fileio::openoutput(string name)
{
  if(ofile != NULL)
    return -1;
#ifndef __MVS__
  ofile = fopen(name.c_str(), "wb");
#else
  ofile = fopen(name.c_str(), "wb, type=record");
#endif
  if(ofile == NULL)
    return -1;
  return 0;
}

int fileio::writerow(char *data, unsigned short int len)
{
#ifndef __MVS__
  fwrite(reinterpret_cast<char*>(&len),
	 sizeof(unsigned short int), 1, ofile);
#endif
  fwrite(data, 1, len, ofile);
#ifndef __MVS__
  fwrite(&end, sizeof(char), 1, ofile);
#endif
  return 0;
}

#if !defined(__USEMMAP__) && !defined(__MVS__)
int fileio::getrow(char **bufptr, unsigned short int &len)
{
  fread(&len, sizeof(unsigned short), 1, ifile);
  if(feof(ifile))
    return -1;
  if(len + 1 > buffersize)
    //if buffer is too small allocate more
    {
      buffersize = len+1;
      delete [] inputbuf;
      inputbuf = new char[buffersize];
    }
  fread(inputbuf, len+1, 1, ifile);
  *bufptr = inputbuf;
  return 0;
}
#elif defined(__MVS__)
int fileio::getrow(char **bufptr, unsigned short int &len)
{
  int rowLength = 101;
  len = fread(inputbuf, 1, rowLength, ifile);
  if(feof(ifile))
    return -1;
  *bufptr = inputbuf;
  return 0;
}
#else
int fileio::getrow(char **bufptr, unsigned short int &len)
  /*
    file input using memory mapped file.
    bufptr will point to a section in memory that is readonly
   */
{
  //read in length
  //if(2 <= (unsigned int)pageend - (unsigned int)curptr) 
       /* DR100926 */
  if(2 <= (char*)pageend - curptr)                          
     /* DR100926 */
    {
      memcpy(&len, curptr, sizeof(unsigned short));
      curptr += sizeof(unsigned short);
    }
  //else if((unsigned int)pageend - (unsigned int)curptr == 0)  
 /* DR100926 */
  else if((char*)pageend - curptr == 0)                        
  /* DR100926 */
    //page end open next page
    {
      int tmp = cur_page+1;
      openpage(tmp);
      if(pageptr == NULL)
	return -1;
      memcpy(&len, curptr, sizeof(unsigned short));
      curptr += sizeof(unsigned short);
    }
  else
    //length variable split on two pages
    {
      inputbuf[0] = *curptr;
      int tmp = cur_page+1;
      openpage(tmp);
      if(pageptr == NULL) ///end of file
	return -1;
      inputbuf[1] = *curptr;
      memcpy(&len, inputbuf, sizeof(unsigned short));
      curptr += 1;
    }
  if(len == 0)
    return -1;
  int reallen = len+1;
  //if(reallen > (unsigned int)pageend - (unsigned int)curptr)  
      /* DR100926 */
  if(reallen > (char*)pageend - curptr)                         
      /* DR100926 */
    //data split on two pages
    {
      //int bytescurp = (unsigned int)pageend - (unsigned int)curptr;
 /* DR100926 */
      int bytescurp = (char*)pageend - curptr;                      
  /* DR100926 */
      int nextpage = cur_page + 1;
      int bytesnextp = reallen - bytescurp;
      if(buffersize < reallen)
	{
	  inputbuf = new char[reallen+3];
	  buffersize = reallen+3;
	}
      memcpy(inputbuf, curptr, bytescurp);
      openpage(nextpage);
      memcpy(inputbuf+bytescurp,
	     curptr, bytesnextp);
      *bufptr = inputbuf;
      curptr += bytesnextp;
      return 0;
    }
  else
    {
      *bufptr = curptr;
      curptr += reallen;
      return 0;
    }
}

int fileio::openpage(int i)
{
  if(pageptr != NULL)
    munmap((char*)pageptr, pagesize);
  if(filesize < pagesize*i)
    {
      //end of file
      pageptr = curptr = NULL;
      cur_page = -1;
      return -1;
    }
  if(filesize < pagesize*(i+1))
    pageptr = mmap(NULL, filesize%pagesize, 
		   (PROT_READ), MAP_SHARED,
		   (int)ifile, i*pagesize);
  else
    pageptr = mmap(NULL, pagesize,
		   (PROT_READ), MAP_SHARED,
		   (int)ifile, i*pagesize);
  //if(pageptr == NULL || (int)pageptr == -1)         
      /* DR100926 */
  if(pageptr == NULL || pageptr == (char*)-1)          
     /* DR100926 */
    {
      cout << "Error loading page";
      curptr = NULL;
      return -1;
    }
#if defined(__HIGHC__) || defined(_ATT4)  
  // mpras doesn't have madvise in sys/mman.h??? 
#else
  madvise((char*)pageptr, pagesize, MADV_SEQUENTIAL);
#endif
  cur_page = i;
  curptr = (char*)pageptr;
  pageend = curptr+pagesize;
  return 0;
}
#endif

#ifndef __USEMMAP__
int fileio::closeinput()
{
  if(ifile != NULL)
    fclose(ifile);
  ifile = NULL;
  return 0;
}
#else
int fileio::closeinput()
{
  if(ifile != 0)
    close(ifile);
  ifile = 0;
  return 0;
}
#endif
int fileio::closeoutput()
{
  if(ofile != NULL)
    fclose(ofile);
  ofile = NULL;
  return 0;
}





















































































































































