/*************************************************************************
*
* Copyright 2005-2008, 2011, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/


#ifndef __MVS__
#include "multitest.h"
#include "OptionsManager.h"
#else
#include "multites.h"
#include "OptionsM.h"
#endif//__MVS__

multitest::multitest(OptionsManager * om)
: m_om(om)
{
}

multitest::~multitest()
{
}

int multitest::set_masterip(string ip)
{
  masterip = ip;
  return mysync.set_masterip(ip);
}

int multitest::set_numslaves(int num)
{
  numslaves = num;
  return 0;
}

int multitest::set_master(bool flag)
{
  master = flag;
  if(flag == true)
    return mysync.set_numslaves(numslaves);
  else
    return mysync.set_numslaves(-1);
}

int multitest::set_inputfile(string name)
{
  inputfile = name;
  return 0;
}

int multitest::set_outputfile(string name)
{
  outputfile = name;
  return 0;
}

int multitest::runexport_load(int sessions)
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();        //DR148806

  if(mysync.initialize() != 0)
  {
      cout << " ERROR communicating to slave or master\n";
      return -1;
  }

  conn     = lib_hdl->newConnectionX();                         //DRXXXXXX
  connLod  = lib_hdl->newConnectionX();                         //DRXXXXXX

  //set attributes
  char * tdp_id        = strdup(m_om -> getTdpId().c_str());
  char * user_name     = strdup(m_om -> getUserName().c_str());
  char * user_password = strdup(m_om -> getUserPassword().c_str());

  char * target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char * target_table        = strdup(m_om -> getTargetTable().c_str());

  char * log_table_load      = strdup(m_om -> getLogTable().c_str());

  char *sel_statment  = (char *)malloc(1024);
  sprintf(sel_statment, "sel col1, col2(CHAR(96)) from %s;", target_table_export);

  char *ins_statment  = (char *)malloc(1024);
  sprintf(ins_statment, "insert into %s (:col1, :col2);", target_table);

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn    -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  connLod -> AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
  
  /*
  if(master)  
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, "m_export.txt");
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);

    connLod -> AddAttribute(TD_TRACE_OUTPUT, "m_load.txt");
    connLod -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);
    
  }
  else
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, "s_export.txt");
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);

    connLod -> AddAttribute(TD_TRACE_OUTPUT, "s_load.txt");
    connLod -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);
  }
  */
  
  /**********************************************
  * Add Attributes for export
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_SELECT_STMT, sel_statment);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table_export);
 
  //cout << " setting max session to " << numslaves+1 << "\n";  
  conn -> AddAttribute(TD_MAX_INSTANCES, ( numslaves + 1 ));
    
  //cout << " this is session " << mysync.getid()+2 << "\n";    
  conn -> AddAttribute(TD_INSTANCE_NUM, ( mysync.getid() + 2 ));
  conn -> AddAttribute(TD_TDP_ID, tdp_id);

  if(master) //if master set session
  {
    //cout << " setting sessions to:" << sessions << "\n";    
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }

  /**********************************************
  * Add Attributes for Load
  **********************************************/
  connLod -> AddAttribute(TD_USER_NAME, user_name);
  connLod -> AddAttribute(TD_USER_PASSWORD, user_password);
  connLod -> AddAttribute(TD_TDP_ID, tdp_id);

  connLod -> AddAttribute(TD_LOG_TABLE, log_table_load);
  connLod -> AddAttribute(TD_TARGET_TABLE, target_table);

  connLod -> AddAttribute(TD_MAX_INSTANCES, numslaves + 1);
  connLod -> AddAttribute(TD_INSTANCE_NUM, mysync.getid() + 2);

  /**********************************************
  * Add Schema - Using Dynamic schema
  **********************************************/
  SchemaX * schema = NULL;

  /*
  Schema * schema = new Schema("output");
  schema -> AddColumn("col1", TD_INTEGER, 4);
  schema -> AddColumn("col2", TD_CHAR, 96);
  conn -> AddSchema(schema);
  */

  /**********************************************
  * Initiate Connection for EXPORT
  **********************************************/

  cout <<"\n --------------------";
  cout << "\n EXPORT operator : ";
  cout <<"\n --------------------";
  cout << "\n TDP ID        : " << tdp_id;
  cout << "\n TDP Username  : " << user_name;
  cout << "\n TDP Password  : " << "<not displayable>";
  cout << "\n Target Table  : " << target_table_export ;
  cout << "\n DML Statement : " << "\"" << sel_statment << "\"" << endl;
  cout <<"\n /***************************************";
  cout <<"\n *******      EXPORT INITIATE      ******";
  cout <<"\n /***************************************\n";
  if(api_initiate(conn) != 0)                                    
  {
    cout << " ERROR in Connection Initiate\n";          
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connLod);                //DRXXXXXX
    return -1;
  }
  else
  {
    cout << "\n Export operator Initiated successfully. \n";
  }

  int queryStatus;
  char *dptr; 
  TD_Length dlen;
 
  cout <<"\n --------------------";
  cout << "\n EXPORT operator : ";
  cout <<"\n --------------------";
  queryStatus = conn -> GetEvent(TD_Evt_Version, &dptr, &dlen);
  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "\n TD_Evt_Version: TPTAPI Version   : ";
    cout << *((char **)dptr) << endl;
    cout << " TD_Evt_Version: Operator Version : ";
    cout << *((char **)(dptr + sizeof(char **))) << endl;
  }
  else 
  {
    printf("\n TD_Evt_Version event failed with status - %d \n", queryStatus);
  }

  // DML Group for Load 
  TD_Index dmlGroupIndex = 0;

  DMLGroupX *dmlGr = lib_hdl->newDMLGroupX();                   //DRXXXXXX
  dmlGr -> AddStatement(ins_statment);
  
  connLod -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Add Schema for LOAD operator 
  **********************************************/
  SchemaX *dynSchema = NULL; 

  int st = conn -> GetSchema(&dynSchema);
  if( st != TD_Success )
  {
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connLod);                //DRXXXXXX
    return -1;
  }


//string table = dynSchema->GetName();
//cout << "   ====>                     tableName: " << table << ".\n";
  connLod -> AddSchema(dynSchema);

  cout <<"\n ------------------";
  cout << "\n LOAD operator : ";
  cout <<"\n ------------------";
  cout << "\n TDP ID        : " << tdp_id;
  cout << "\n TDP Username  : " << user_name;
  cout << "\n TDP Password  : " << "<not displayable>";
  cout << "\n Target Table  : " << target_table ;
  cout << "\n DML Statement : " << "\"" << ins_statment << "\"" << endl;
  cout <<"\n /***************************************";
  cout <<"\n *******      LOAD INITIATE      ********";
  cout <<"\n /***************************************\n";
  // Initiate the load operator.
  if(api_initiate(connLod) != 0)
  {
    cout << " ERROR in Connection Initiate\n";
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connLod);                //DRXXXXXX
    return -1;
  }
  else
  {
    cout << "\n Load operator Initiated successfully. \n";
  }

  cout <<"\n ------------------";
  cout << "\n LOAD operator : ";
  cout <<"\n ------------------";
  queryStatus = connLod -> GetEvent(TD_Evt_Version, &dptr, &dlen);
  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "\n TD_Evt_Version: TPTAPI Version   : ";
    cout << *((char **)dptr) << endl;
    cout << " TD_Evt_Version: Operator Version : ";
    cout << *((char **)(dptr + sizeof(char **))) << endl;
  }
  else
  {
    printf("\n TD_Evt_Version event failed with status - %d \n", queryStatus);
  }

  // GetRow
  TD_StatusCode exp_reval = TD_Success;
  TD_StatusCode lod_reval = TD_Success;
  char *data;
  TD_Length datalen;
  rowrecv = 0;
  rowsent = 0;

  while( exp_reval != TD_END_Method )
  {

    /**************************************************
    ***************** GETROW **************************
    **************************************************/
    
    // Get a row from Export operator.
    exp_reval = conn -> GetRow(&data, &datalen);

    if( exp_reval == TD_Success )
    {
      rowrecv++;

      /**************************************************
      ***************** PUTROW **************************
      **************************************************/

      lod_reval = connLod -> PutRow(data, datalen);      
      if ( lod_reval == TD_Success )
      {
        rowsent++;
      }
      else
      {
        if( lod_reval >= TD_Error )
        {
          cout << "\n Error occured in PutRow. Aborting.... \n";
          aborttest(connLod);
          return -1;
        }
        else
        {
          if( process_returnvalue(connLod, lod_reval) != 0 )
          {
            aborttest(connLod);
            return -1;
          }
        }
      }
 
     /**************************************************
     ***************** PUTROW END  *********************
     **************************************************/
    }
    else
    {
      if( exp_reval >= TD_Error )
      {
        cout << "\n Error occured in GetRow. Aborting.... \n";
        aborttest(conn);
        return -1;
      }
      else
      {
        if( process_returnvalue(conn, exp_reval) != 0 )
        {
          aborttest(conn);
          return -1;
        }
      }
    }
    /**************************************************
    ***************** GETROW END  *********************
    **************************************************/

  }// End of while loop 
 
  cout << "\n ROWS EXPORTED : " << rowrecv ;
  cout << "\n ROWS SENT     : " << rowsent << endl;
  
  cout <<"\n /***************************************";
  cout <<"\n *******      LOAD ACQUISITION   ********";
  cout <<"\n /***************************************\n";
  //Load operator Acquisition phase
  if(api_endacquisition(connLod) != 0)
  {
      cout << " ERROR in Connection EndAcquisition\n";
      aborttest(connLod);
      return -1;
  }

  cout <<"\n /***************************************";
  cout <<"\n *******      LOAD APPLICATION   ********";
  cout <<"\n /***************************************\n";
  //Load operator Application phase
  if(api_applyrows(connLod) != 0)
  {
      cout << " ERROR in Connection ApplyRows\n";
      aborttest(connLod);
      return -1;
  }

  //Terminate

  cout <<"\n /***************************************";
  cout <<"\n *******      EXPORT TERMINATE   ********";
  cout <<"\n /***************************************\n";
  if(api_terminate(conn) != 0)                                    
  {
      cout << " ERROR in Connection Terminate\n";          
      aborttest(conn);
      return -1;
  }

  cout <<"\n /***************************************";
  cout <<"\n *******      LOAD TERMINATE      *******";
  cout <<"\n /***************************************\n";
  if(api_terminate(connLod) != 0)
  {
      cout << " ERROR in Connection Terminate\n";
      aborttest(connLod);
      return -1;
  }


  //delete conn;
  //delete connLod;

  lib_hdl->delConnectionX(conn);                   //DRXXXXXX
  lib_hdl->delConnectionX(connLod);                //DRXXXXXX

  free(tdp_id);
  free(user_name);
  free(user_password);
  free(target_table_export);
  free(target_table);
  free(log_table_load);
  free(sel_statment);
  free(ins_statment);

  return 0;
}
// End of function runexport_load

int multitest::runexport_update(int sessions)
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();        //DRXXXXXX

  if(mysync.initialize() != 0)
  {
      cout << " ERROR communicating to slave or master\n";
      return -1;
  }

  conn     = lib_hdl->newConnectionX();                         //DRXXXXXX
  connUpd  = lib_hdl->newConnectionX();                         //DRXXXXXX

  //set attributes
  char * tdp_id        = strdup(m_om -> getTdpId().c_str());
  char * user_name     = strdup(m_om -> getUserName().c_str());
  char * user_password = strdup(m_om -> getUserPassword().c_str());

  char * target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char * target_table        = strdup(m_om -> getTargetTable().c_str());

  char * log_table_load      = strdup(m_om -> getLogTable().c_str());

  char *sel_statment  = (char *)malloc(1024);
  sprintf(sel_statment, "sel col1, col2(CHAR(96)) from %s;", target_table_export);

  char *ins_statment  = (char *)malloc(1024);
  sprintf(ins_statment, "insert into %s (:col1, :col2);", target_table);

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn    -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  connUpd -> AddAttribute(TD_SYSTEM_OPERATOR, TD_UPDATE);

  /*
  if(master)
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, "m_export.txt");
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);

    connUpd -> AddAttribute(TD_TRACE_OUTPUT, "m_update.txt");
    connUpd -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);

  }
  else
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, "s_export.txt");
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);

    connUpd -> AddAttribute(TD_TRACE_OUTPUT, "s_update.txt");
    connUpd -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);
  }
  */

  /**********************************************
  * Add Attributes for export
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_SELECT_STMT, sel_statment);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table_export);

  //cout << " setting max session to " << numslaves+1 << "\n";
  conn -> AddAttribute(TD_MAX_INSTANCES, ( numslaves + 1 ));

  //cout << " this is session " << mysync.getid()+2 << "\n";
  conn -> AddAttribute(TD_INSTANCE_NUM, ( mysync.getid() + 2 ));
  conn -> AddAttribute(TD_TDP_ID, tdp_id);

  if(master) //if master set session
  {
    //cout << " setting sessions to:" << sessions << "\n";
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }

  /**********************************************
  * Add Attributes for Update
  **********************************************/
  connUpd -> AddAttribute(TD_USER_NAME, user_name);
  connUpd -> AddAttribute(TD_USER_PASSWORD, user_password);
  connUpd -> AddAttribute(TD_TDP_ID, tdp_id);

  connUpd -> AddAttribute(TD_LOG_TABLE, log_table_load);
  connUpd -> AddAttribute(TD_TARGET_TABLE, target_table);

  connUpd -> AddAttribute(TD_MAX_INSTANCES, numslaves + 1);
  connUpd -> AddAttribute(TD_INSTANCE_NUM, mysync.getid() + 2);

  /**********************************************
  * Add Schema - Using Dynamic schema
  **********************************************/
  SchemaX* schema = NULL;

  /*
  Schema * schema = new Schema("output");
  schema -> AddColumn("col1", TD_INTEGER, 4);
  schema -> AddColumn("col2", TD_CHAR, 96);
  conn -> AddSchema(schema);
  */

  /**********************************************
  * Initiate Connection for EXPORT
  **********************************************/

  cout <<"\n --------------------";
  cout << "\n EXPORT operator : ";
  cout <<"\n --------------------";
  cout << "\n TDP ID        : " << tdp_id;
  cout << "\n TDP Username  : " << user_name;
  cout << "\n TDP Password  : " << "<not displayable>";
  cout << "\n Target Table  : " << target_table_export ;
  cout << "\n DML Statement : " << "\"" << sel_statment << "\"" << endl;
  cout <<"\n /***************************************";
  cout <<"\n *******      EXPORT INITIATE      ******";
  cout <<"\n /***************************************\n";
  if(api_initiate(conn) != 0)
  {
    cout << " ERROR in Connection Initiate\n";
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connUpd);                //DRXXXXXX
    return -1;
  }
  else
  {
    cout << "\n Export operator Initiated successfully. \n";
  }

  int queryStatus;
  char *dptr;
  TD_Length dlen;

  cout <<"\n --------------------";
  cout << "\n EXPORT operator : ";
  cout <<"\n --------------------";
  queryStatus = conn -> GetEvent(TD_Evt_Version, &dptr, &dlen);
  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "\n TD_Evt_Version: TPTAPI Version   : ";
    cout << *((char **)dptr) << endl;
    cout << " TD_Evt_Version: Operator Version : ";
    cout << *((char **)(dptr + sizeof(char **))) << endl;
  }
  else
  {
    printf("\n TD_Evt_Version event failed with status - %d \n", queryStatus);
  }

  // DML Group for Update
  TD_Index dmlGroupIndex = 0;

  DMLGroupX *dmlGr = lib_hdl->newDMLGroupX();           //DRXXXXXX
  dmlGr -> AddStatement(ins_statment);

  connUpd -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Add Schema for UPDATE operator
  **********************************************/
  SchemaX *dynSchema = NULL;
  int st = conn -> GetSchema(&dynSchema);
  if( st != TD_Success )
  {
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connUpd);                //DRXXXXXX
    return -1;
  }
  connUpd -> AddSchema(dynSchema);

  cout <<"\n ------------------";
  cout << "\n UPDATE operator : ";
  cout <<"\n ------------------";
  cout << "\n TDP ID        : " << tdp_id;
  cout << "\n TDP Username  : " << user_name;
  cout << "\n TDP Password  : " << "<not displayable>";
  cout << "\n Target Table  : " << target_table ;
  cout << "\n DML Statement : " << "\"" << ins_statment << "\"" << endl;
  cout <<"\n /***************************************";
  cout <<"\n *******      UPDATE INITIATE    ********";
  cout <<"\n /***************************************\n";
  // Initiate the update operator.
  if(api_initiate(connUpd) != 0)
  {
    cout << " ERROR in Connection Initiate\n";
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connUpd);                //DRXXXXXX
    return -1;
  }
  else
  {
    cout << "\n Update operator Initiated successfully. \n";
  }

  cout <<"\n --------------------";
  cout << "\n UPDATE operator : ";
  cout <<"\n --------------------";
  queryStatus = connUpd -> GetEvent(TD_Evt_Version, &dptr, &dlen);
  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "\n TD_Evt_Version: TPTAPI Version   : ";
    cout << *((char **)dptr) << endl;
    cout << " TD_Evt_Version: Operator Version : ";
    cout << *((char **)(dptr + sizeof(char **))) << endl;
  }
  else
  {
    printf("\n TD_Evt_Version event failed with status - %d \n", queryStatus);
  }

  // GetRow
  TD_StatusCode exp_reval = TD_Success;
  TD_StatusCode lod_reval = TD_Success;
  char *data;
  TD_Length datalen;
  rowrecv = 0;
  rowsent = 0;

  while( exp_reval != TD_END_Method )
  {

    /**************************************************
    ***************** GETROW **************************
    **************************************************/

    // Get a row from Export operator.
    exp_reval = conn -> GetRow(&data, &datalen);

    if( exp_reval == TD_Success )
    {
      rowrecv++;

      /**************************************************
      ***************** PUTROW **************************
      **************************************************/

      lod_reval = connUpd -> PutRow(data, datalen);
      if ( lod_reval == TD_Success )
      {
        rowsent++;
      }
      else
      {
        if( lod_reval >= TD_Error )
        {
          cout << "\n Error occured in PutRow. Aborting.... \n";
          aborttest(connUpd);
          return -1;
        }
        else
        {
          if( process_returnvalue(connUpd, lod_reval) != 0 )
          {
            aborttest(connUpd);
            return -1;
          }
        }
      }

     /**************************************************
     ***************** PUTROW END  *********************
     **************************************************/
    }
    else
    {
      if( exp_reval >= TD_Error )
      {
        cout << "\n Error occured in GetRow. Aborting.... \n";
        aborttest(conn);
        return -1;
      }
      else
      {
        if( process_returnvalue(conn, exp_reval) != 0 )
        {
          aborttest(conn);
          return -1;
        }
      }
    }
    /**************************************************
    ***************** GETROW END  *********************
    **************************************************/

  }// End of while loop

  cout << "\n ROWS EXPORTED : " << rowrecv ;
  cout << "\n ROWS SENT     : " << rowsent << endl;

  cout <<"\n /***************************************";
  cout <<"\n *******      UPDATE ACQUISITION ********";
  cout <<"\n /***************************************\n";
  //Update operator Acquisition phase
  if(api_endacquisition(connUpd) != 0)
  {
      cout << " ERROR in Connection EndAcquisition\n";
      aborttest(connUpd);
      return -1;
  }

  cout <<"\n /***************************************";
  cout <<"\n *******     UPDATE APPLICATION  ********";
  cout <<"\n /***************************************\n";
  //update operator Application phase
  if(api_applyrows(connUpd) != 0)
  {
      cout << " ERROR in Connection ApplyRows\n";
      aborttest(connUpd);
      return -1;
  }

  //Terminate

  cout <<"\n /***************************************";
  cout <<"\n *******      EXPORT TERMINATE   ********";
  cout <<"\n /***************************************\n";
  if(api_terminate(conn) != 0)
  {
      cout << " ERROR in Connection Terminate\n";
      aborttest(conn);
      return -1;
  }

  cout <<"\n /***************************************";
  cout <<"\n *******      UPDATE TERMINATE    *******";
  cout <<"\n /***************************************\n";
  if(api_terminate(connUpd) != 0)
  {
      cout << " ERROR in Connection Terminate\n";
      aborttest(connUpd);
      return -1;
  }


  //delete conn;
  //delete connUpd;

  lib_hdl->delConnectionX(conn);                   //DRXXXXXX
  lib_hdl->delConnectionX(connUpd);                //DRXXXXXX

  free(tdp_id);
  free(user_name);
  free(user_password);
  free(target_table_export);
  free(target_table);
  free(log_table_load);
  free(sel_statment);
  free(ins_statment);

  return 0;
}
// End of function runexport_update

int multitest::runexport_stream(int sessions)
{
  DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();     //DRXXXXXX

  if(mysync.initialize() != 0)
  {
      cout << " ERROR communicating to slave or master\n";
      return -1;
  }

  conn     = lib_hdl->newConnectionX();                      //DRXXXXXX
  connStm  = lib_hdl->newConnectionX();                      //DRXXXXXX

  //set attributes
  char * tdp_id        = strdup(m_om -> getTdpId().c_str());
  char * user_name     = strdup(m_om -> getUserName().c_str());
  char * user_password = strdup(m_om -> getUserPassword().c_str());

  char * target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char * target_table        = strdup(m_om -> getTargetTable().c_str());

  char * log_table_load      = strdup(m_om -> getLogTable().c_str());

  char *sel_statment  = (char *)malloc(1024);
  sprintf(sel_statment, "sel col1, col2(CHAR(96)) from %s;", target_table_export);

  char *ins_statment  = (char *)malloc(1024);
  sprintf(ins_statment, "insert into %s (:col1, :col2);", target_table);

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn    -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  connStm -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);

/*
  if(master)
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, "m_export.txt");
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OFF, TD_GENERAL, NULL);

    connStm -> AddAttribute(TD_TRACE_OUTPUT, "m_stream.txt");
    connStm -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OFF, TD_GENERAL, NULL);

  }
  else
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, "s_export.txt");
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OFF, TD_GENERAL, NULL);

    connStm -> AddAttribute(TD_TRACE_OUTPUT, "s_stream.txt");
    connStm -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OFF, TD_GENERAL, NULL);
  }
*/

  /**********************************************
  * Add Attributes for export
  **********************************************/
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_SELECT_STMT, sel_statment);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table_export);

  //cout << " setting max session to " << numslaves+1 << "\n";
  conn -> AddAttribute(TD_MAX_INSTANCES, ( numslaves + 1 ));

  //cout << " this is session " << mysync.getid()+2 << "\n";
  conn -> AddAttribute(TD_INSTANCE_NUM, ( mysync.getid() + 2 ));
  conn -> AddAttribute(TD_TDP_ID, tdp_id);

  if(master) //if master set session
  {
    //cout << " setting sessions to:" << sessions << "\n";
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }

  /**********************************************
  * Add Attributes for Load
  **********************************************/
  connStm -> AddAttribute(TD_USER_NAME, user_name);
  connStm -> AddAttribute(TD_USER_PASSWORD, user_password);
  connStm -> AddAttribute(TD_TDP_ID, tdp_id);

  connStm -> AddAttribute(TD_LOG_TABLE, log_table_load);
  connStm -> AddAttribute(TD_TARGET_TABLE, target_table);

  connStm -> AddAttribute(TD_MAX_INSTANCES, numslaves + 1);
  connStm -> AddAttribute(TD_INSTANCE_NUM, mysync.getid() + 2);

  /**********************************************
  * Add Schema - Using Dynamic schema
  **********************************************/
  SchemaX* schema = NULL;

  /*
  Schema * schema = new Schema("output");
  schema -> AddColumn("col1", TD_INTEGER, 4);
  schema -> AddColumn("col2", TD_CHAR, 96);
  conn -> AddSchema(schema);
  */

  /**********************************************
  * Initiate Connection for EXPORT
  **********************************************/

  cout <<"\n --------------------";
  cout << "\n EXPORT operator : ";
  cout <<"\n --------------------";
  cout << "\n TDP ID        : " << tdp_id;
  cout << "\n TDP Username  : " << user_name;
  cout << "\n TDP Password  : " << "<not displayable>";
  cout << "\n Target Table  : " << target_table_export ;
  cout << "\n DML Statement : " << "\"" << sel_statment << "\"" << endl;
  cout <<"\n /***************************************";
  cout <<"\n *******      EXPORT INITIATE      ******";
  cout <<"\n /***************************************\n";
  if(api_initiate(conn) != 0)
  {
    cout << " ERROR in Connection Initiate\n";
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connStm);                //DRXXXXXX
    return -1;
  }
  else
  {
    cout << "\n Export operator Initiated successfully. \n";
  }

  int queryStatus;
  char *dptr;
  TD_Length dlen;

  cout <<"\n --------------------";
  cout << "\n EXPORT operator : ";
  cout <<"\n --------------------";
  queryStatus = conn -> GetEvent(TD_Evt_Version, &dptr, &dlen);
  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "\n TD_Evt_Version: TPTAPI Version   : ";
    cout << *((char **)dptr) << endl;
    cout << " TD_Evt_Version: Operator Version : ";
    cout << *((char **)(dptr + sizeof(char **))) << endl;
  }
  else
  {
    printf("\n TD_Evt_Version event failed with status - %d \n", queryStatus);
  }

  // DML Group for Load
  TD_Index dmlGroupIndex = 0;

  DMLGroupX *dmlGr = lib_hdl->newDMLGroupX();           //DRXXXXXX
  dmlGr -> AddStatement(ins_statment);

  connStm -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Add Schema for STREAM operator
  **********************************************/
  SchemaX *dynSchema = NULL;
  int st = conn -> GetSchema(&dynSchema);
  if( st != TD_Success )
  {
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connStm);                //DRXXXXXX
    return -1;
  }
  connStm -> AddSchema(dynSchema);

  cout <<"\n ------------------";
  cout << "\n STREAM operator : ";
  cout <<"\n ------------------";
  cout << "\n TDP ID        : " << tdp_id;
  cout << "\n TDP Username  : " << user_name;
  cout << "\n TDP Password  : " << "<not displayable>";
  cout << "\n Target Table  : " << target_table ;
  cout << "\n DML Statement : " << "\"" << ins_statment << "\"" << endl;
  cout <<"\n /***************************************";
  cout <<"\n *******      STREAM INITIATE    ********";
  cout <<"\n /***************************************\n";
  // Initiate the STREAM operator.
  if(api_initiate(connStm) != 0)
  {
    cout << " ERROR in Connection Initiate\n";
    lib_hdl->delConnectionX(conn);                   //DRXXXXXX
    lib_hdl->delConnectionX(connStm);                //DRXXXXXX
    return -1;
  }
  else
  {
    cout << "\n STREAM operator Initiated successfully. \n";
  }

  cout <<"\n --------------------";
  cout << "\n STREAM operator : ";
  cout <<"\n --------------------";
  queryStatus = connStm -> GetEvent(TD_Evt_Version, &dptr, &dlen);
  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "\n TD_Evt_Version: TPTAPI Version   : ";
    cout << *((char **)dptr) << endl;
    cout << " TD_Evt_Version: Operator Version : ";
    cout << *((char **)(dptr + sizeof(char **))) << endl;
  }
  else
  {
    printf("\n TD_Evt_Version event failed with status - %d \n", queryStatus);
  }

  // GetRow
  TD_StatusCode exp_reval = TD_Success;
  TD_StatusCode lod_reval = TD_Success;
  char *data;
  TD_Length datalen;
  rowrecv = 0;
  rowsent = 0;

  while( exp_reval != TD_END_Method )
  {

    /**************************************************
    ***************** GETROW **************************
    **************************************************/

    // Get a row from Export operator.
    exp_reval = conn -> GetRow(&data, &datalen);

    if( exp_reval == TD_Success )
    {
      rowrecv++;

      /**************************************************
      ***************** PUTROW **************************
      **************************************************/

      lod_reval = connStm -> PutRow(data, datalen);
      if ( lod_reval == TD_Success )
      {
        rowsent++;
      }
      else
      {
        if( lod_reval >= TD_Error )
        {
          cout << "\n Error occured in PutRow. Aborting.... \n";
          aborttest(connStm);
          return -1;
        }
        else
        {
          if( process_returnvalue(connStm, lod_reval) != 0 )
          {
            aborttest(connStm);
            return -1;
          }
        }
      }

     /**************************************************
     ***************** PUTROW END  *********************
     **************************************************/
    }
    else
    {
      if( exp_reval >= TD_Error )
      {
        cout << "\n Error occured in GetRow. Aborting.... \n";
        aborttest(conn);
        return -1;
      }
      else
      {
        if( process_returnvalue(conn, exp_reval) != 0 )
        {
          aborttest(conn);
          return -1;
        }
      }
    }
    /**************************************************
    ***************** GETROW END  *********************
    **************************************************/

  }// End of while loop

  cout << "\n ROWS EXPORTED : " << rowrecv ;
  cout << "\n ROWS SENT     : " << rowsent << endl;

  cout <<"\n /***************************************";
  cout <<"\n *******      STREAM ACQUISITION ********";
  cout <<"\n /***************************************\n";
  //STREAM operator Acquisition phase
  if(api_endacquisition(connStm) != 0)
  {
      cout << " ERROR in Connection EndAcquisition\n";
      aborttest(connStm);
      return -1;
  }

  cout <<"\n /***************************************";
  cout <<"\n *******     STREAM APPLICATION  ********";
  cout <<"\n /***************************************\n";
  //Stream operator Application phase
  /*
  if(api_applyrows(connStm) != 0)
  {
      cout << " ERROR in Connection ApplyRows\n";
      aborttest(connStm);
      return -1;
  }
  */
  cout << " No Application phase for Stream operator. \n";

  //Terminate

  cout <<"\n /***************************************";
  cout <<"\n *******      EXPORT TERMINATE   ********";
  cout <<"\n /***************************************\n";
  if(api_terminate(conn) != 0)
  {
      cout << " ERROR in Connection Terminate\n";
      aborttest(conn);
      return -1;
  }

  cout <<"\n /***************************************";
  cout <<"\n *******      STREAM TERMINATE    *******";
  cout <<"\n /***************************************\n";
  if(api_terminate(connStm) != 0)
  {
      cout << " ERROR in Connection Terminate\n";
      aborttest(connStm);
      return -1;
  }


  //delete conn;
  //delete connStm;

  lib_hdl->delConnectionX(conn);                   //DRXXXXXX
  lib_hdl->delConnectionX(connStm);                //DRXXXXXX

  free(tdp_id);
  free(user_name);
  free(user_password);
  free(target_table_export);
  free(target_table);
  free(log_table_load);
  free(sel_statment);
  free(ins_statment);

  return 0;
}
// End of function runexport_stream

int multitest::api_initiate(ConnectionX *conn)         //DRXXXXXX 
{
  int reval = TD_END_Method + 1;

  while(reval != TD_END_Method)
  {
    if(mysync.barrier(conn) != 0)
    {
      return -1;
    }
    
    reval = conn -> Initiate();
    if(reval >= TD_Error)
    {
      cout << " Connection Initiate returned error code: " << reval << "\n";
      process_returnvalue(conn, reval);
      return -1;
    }
		
    if(process_returnvalue(conn, reval) != 0)
    {
      return -1;
    }
  }
	
  cout << " Connection Initiate finished\n";            
  return 0;
}

int multitest::api_endacquisition(ConnectionX *conn)        //DRXXXXXX 
{
  //cout << " ending acquisition\n";                      
  TD_StatusCode reval = TD_Success;
  while(reval != TD_END_Method)
  {
    if(mysync.barrier(conn) != 0)
      return -1;
    reval = conn -> EndAcquisition();
    if(reval >= TD_Error)
    {
      return -1;
    }
    if(process_returnvalue(conn, reval) != 0)
      return -1;
  }
  cout << " Connection EndAcquisition complete\n";         
  return 0;
}

int multitest::process_returnvalue(ConnectionX *conn, int val)  //DRXXXXXX
{
  switch(val)
  {

    case TD_SYNC_Barrier:
                   cout << " Received TD_SYNC_Barrier\n";                
                   return mysync.barrier(conn);
                   break;

    // DR119676 : Begin
    case TD_SYNC_Schema:
                   if ( ( conn -> GetType() ) == TD_EXPORT )
                   {
                     cout <<" Received TD_SYNC_Schema\n";
                     return mysync.syncschema(conn);
                   }
                   else
                   {
                     return mysync.barrier(conn);
                   }
                   break;
    // DR119676 : End

    case TD_SYNC_TELINFO:
                   cout << " Received TD_SYNC_TELINFO\n";               
                   return mysync.synctelinfo(conn);
                   break;

    case TD_END_Method:
                   return mysync.switchbarrier();
                   cout << " Received TD_END_Method\n";                  
                   break;

    default:
                   cout << " ERROR processing return code:" << val << "\n";
                   char *errorMsg = NULL;
                   TD_ErrorType errorType;
                   conn -> GetErrorInfo(&errorMsg, &errorType);
                   if (errorMsg != NULL) 
                   {
	             cout << errorMsg << endl;
	             cout << " Type: " << errorType << endl;
                   }
                   else
                   {
	             cout << " No Error Info Available" << endl;
                   }
                   break;
  } // End of switch

  return 0;
}

int multitest::api_applyrows(ConnectionX *conn)    //DRXXXXXX 
{
  //cout << "apply rows\n";
  TD_StatusCode reval = TD_Success;
  while(reval != TD_END_Method)
  {
    if(mysync.barrier(conn) != 0)
      return -1;
    reval = conn -> ApplyRows();
    if(reval >= TD_Error)
    {
      return -1;
    }
    if(process_returnvalue(conn, reval)  != 0)
      return -1;
  }
  cout << " Connection ApplyRows complete\n";              
  return 0;
}

int multitest::api_terminate(ConnectionX *conn)      //DRXXXXXX 
{
  //cout << "terminating connection object\n";
  TD_StatusCode reval = TD_Success;
  while(reval != TD_END_Method)
  {
    if(mysync.barrier(conn) != 0)
     return -1;
    reval = conn -> Terminate();
    if(reval >= TD_Error)
    {
      return -1;
    }
    if(process_returnvalue(conn, reval) != 0)
      return -1;
  }
  cout << " Connection terminated\n";
  return 0;
}

void multitest::aborttest(ConnectionX *conn)         //DRXXXXXX 
{
  char *errorMsg = NULL;
  TD_ErrorType errorType;

  cout << "Test failed\n";
  mysync.abort();
  cout << "Terminating with return code " << ( conn -> Terminate() ) << "\n";

  conn -> GetErrorInfo(&errorMsg, &errorType);
  if( errorMsg != NULL )
  {
    cout << errorMsg << endl;
    cout << "Type: " << errorType << endl;
  }
  else
  {
    cout << "No Error Info Available" << endl;
  }
  //delete conn;
}

