/*************************************************************************
*
* Copyright 2010-2011, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/

#include "MultipleOperators.h"
#include "OptionsManager.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "connection.h"
#include "string.h"
#include "dynlib.h"                                          //DR142720

using namespace teradata::client::API;

MultipleOperators::MultipleOperators(OptionsManager * om): m_om(om)
{
}

MultipleOperators::~MultipleOperators(void) 
{
}

/***************************************
* Function demonstrates usage of Stream
* Operator with multiple sessions.
* Uses Export Operator to get data that
* will be loaded using Stream Operator
***************************************/
void MultipleOperators::testExportStream()
{
  DynamicLibrary   *lib_hdl;                             //DR142720
  time_t now;
  time(&now);

  lib_hdl = DynamicLibrary::Instance();                    //DR142720
  ConnectionX *connExp = lib_hdl -> newConnectionX();      //DR142720
  ConnectionX *connStr = lib_hdl -> newConnectionX();      //DR142720

  TD_Index dmlNumbers[2];
  int returnValue = 0;
  char* errorMsg = NULL;
  TD_ErrorType errorType;

  char *user_name           = strdup(m_om -> getUserName().c_str());
  char *user_password       = strdup(m_om -> getUserPassword().c_str());
  char *tdp_id              = strdup(m_om -> getTdpId().c_str());
  char *target_table_export = strdup(m_om -> getTargetTableExport().c_str());
  char *target_table_stream = strdup(m_om -> getTargetTableStream().c_str());
  char *error_table_stream  = strdup(m_om -> getErrorTableStream().c_str());
  char *log_table_stream    = strdup(m_om -> getLogTableStream().c_str());

  char *sel_statment        = (char*)malloc(1024);
  sprintf(sel_statment, "sel * from %s;", target_table_export);
  char *dml_statment        = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :Associate_Id, :Associate_Name, \
  :Salary, :DOJ, :Designation, :Loan_Amount, :Martial_Status, \
  :No_Of_Dependents);", target_table_stream);
   

  cout << "######################### OUTPUT START #########################" \
  << endl;
  cout << "\nTest Description      :Multiple sample" << endl;
  cout << "Date                  :" << ctime(&now);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "       EXPORT OPERATOR CONNECTION INFO      " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  cout << "Operator type       :EXPORT" << endl;
  cout << "TDP ID              :" << tdp_id << endl;
  cout << "TDP username        :" << user_name << endl;
  cout << "TDP password        :<not displayable>" << endl;
  cout << "Target Table        :"<< target_table_export << endl;
  cout << "DML Statement       :" << sel_statment << endl;

  /**********************************************
  * Export Operator
  **********************************************/

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  connExp -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
  //connExp -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
  //connExp -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  connExp -> AddAttribute(TD_USER_NAME, user_name);
  connExp -> AddAttribute(TD_USER_PASSWORD, user_password);
  connExp -> AddAttribute(TD_TDP_ID, tdp_id);
  connExp -> AddAttribute(TD_SELECT_STMT, sel_statment);
  connExp -> AddAttribute(TD_TARGET_TABLE, target_table_export);

  /**********************************************
  * Schema Definitions
  **********************************************/
  SchemaX *schemaExp = lib_hdl->newSchemaX("output");      //DR142720
  schemaExp -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4);  //TPT-17718
  schemaExp -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
  schemaExp -> AddColumn((char*)"Salary", TD_FLOAT, 8);  //TPT-17718
  schemaExp -> AddColumn((char*)"DOJ", TD_DATE, 4);  //TPT-17718
  schemaExp -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718
  schemaExp -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);//TPT-17718
  schemaExp -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
  schemaExp -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1);//TPT-17718
  connExp -> AddSchema(schemaExp);

  /**********************************************
  * Stream Operator
  **********************************************/

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  connStr -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);
  //connStr -> AddAttribute(TD_TRACE_OUTPUT, "stream.txt");
  //connStr -> AddAttribute(TD_PRIVATE_LOG_NAME, "private_log_stream.txt");
  //connStr -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
  //TD_GENERAL, NULL);

  /**********************************************
  * Add Attributes
  **********************************************/
  connStr -> AddAttribute(TD_USER_NAME, user_name);
  connStr -> AddAttribute(TD_USER_PASSWORD, user_password);
  connStr -> AddAttribute(TD_TDP_ID, tdp_id);
  connStr -> AddAttribute(TD_LOG_TABLE, log_table_stream);
  connStr -> AddAttribute(TD_ERROR_TABLE, error_table_stream);
  connStr -> AddAttribute(TD_TARGET_TABLE, target_table_stream);
  connStr -> AddAttribute(TD_MAX_SESSIONS, 4);
  connStr -> AddAttribute(TD_MIN_SESSIONS, 1);
  connStr -> AddAttribute(TD_TENACITY_HOURS, 2);
  connStr -> AddAttribute(TD_TENACITY_SLEEP, 3);
  connStr -> AddAttribute(TD_ROBUST, "No");

  /**********************************************
  * Add Schema
  **********************************************/
  SchemaX *schemaStr = lib_hdl->newSchemaX("output");     //DR142720
  schemaStr -> AddColumn((char*)"Associate_Id", TD_INTEGER,4);  //TPT-17718
  schemaStr -> AddColumn((char*)"Associate_Name", TD_CHAR,25);  //TPT-17718
  schemaStr -> AddColumn((char*)"Salary", TD_FLOAT, 8);  //TPT-17718
  schemaStr -> AddColumn((char*)"DOJ", TD_DATE, 4);  //TPT-17718
  schemaStr -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718
  schemaStr -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);//TPT-17718
  schemaStr -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
  schemaStr -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1);//TPT-17718
  connStr -> AddSchema(schemaStr);

  /**********************************************
  * Add DMLGroups
  **********************************************/
  DMLGroupX* dmlGr = lib_hdl->newDMLGroupX();       //DR142720
  dmlGr -> AddStatement(dml_statment);

  dmlGr -> AddSerializeOn(8, "Associate_Id", "Associate_Name", "Salary", \
  "DOJ", "Designation", "Loan_Amount", "Martial_Status", \
  "No_Of_Dependents", NULL);

  dmlGr -> AddUseList(8, "Associate_Id", "Associate_Name", "Salary", \
  "DOJ", "Designation", "Loan_Amount", "Martial_Status", \
  "No_Of_Dependents", NULL);

  dmlGr -> AddDMLOption(IGNORE_DUPLICATE_ROWS);
  dmlGr -> AddDMLOption(IGNORE_MISSING_ROWS);
  connStr -> AddDMLGroup(dmlGr, &dmlNumbers[0]);
  // Must set the DML number to apply the group to a row
  connStr -> UseDMLGroups(dmlNumbers, 1);  

  /**********************************************
  * Initiate Export 
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "        INITIATE PHASE FOR EXPORT           " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = connExp -> Initiate();

  if ( returnValue < TD_Error )
  {
    cout << "Export Driver initiated Successfully" << endl;
    char *dataptr;
    TD_Length datalen;
    int queryStatus = 0;

    /**********************************************
    * Query Version Number
    **********************************************/
    queryStatus = connExp -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "              VERSION INFO                  " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
    {
      cout << "TPTAPI Version         : ";
      cout << *((char**)dataptr) << endl;
      cout << "Export Operator Version: ";
      cout << *((char**)(dataptr + sizeof(char**))) << endl;
    }
    else
    {
      printf("TD_Evt_Version event failed with status = %d", queryStatus);
    }

    cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
    cout << "       STREAM OPERATOR CONNECTION INFO      " << endl;
    cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

    cout << "Operator type      :STREAM"<< endl;
    cout << "TDP ID             :" << tdp_id << endl;
    cout << "TDP username       :" << user_name << endl;
    cout << "TDP password       :<not displayable>" << endl;
    cout << "Target Table       :" << target_table_stream << endl;
    cout << "DML Statement      :" << dml_statment << endl;


    /**********************************************
    * Initiate Stream
	**********************************************/
	returnValue = connStr -> Initiate();

	if ( returnValue < TD_Error )
	{
      cout << "Stream Driver initiated Successfully" << endl;

      /**********************************************
      * Query Version Number
      **********************************************/
      queryStatus = connStr -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "              VERSION INFO                  " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

      if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
      {
        cout << "TPTAPI Version         : ";
        cout << *((char**)dataptr) << endl;
        cout << "Stream Operator Version: ";
        cout << *((char**)(dataptr + sizeof(char**))) << endl;
      }
      else
      {
        printf("TD_Evt_Version event failed with status = %d", queryStatus);
      }

      /**********************************************
      * Acquisition
      **********************************************/
      cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
      cout << "             ACQUISITION PHASE            " << endl;
      cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

	  int count = 0;

	  cout << "----------- transferring data ------------" << endl;

	  char *data;
	  TD_Length dataLen = 0;
	  char *str_checkpointData;
	  TD_Length str_checkpointLength;

	  do 
      { 
		// Take a checkpoint every 5 rows
		if(count == 5)
		{
		  returnValue = connStr -> Checkpoint(&str_checkpointData,\
          &str_checkpointLength);

		  if ( returnValue >= TD_Error )
		  {
			cout << "Checkpoint failed on row " << count+1 \
            << " with status " << returnValue << endl;
			break;
		  }
		}

		//Get row from Export
		returnValue = connExp -> GetRow(&data, &dataLen);

		if ( returnValue >= TD_Error )
		{
		  cout << "GetRow failed on row " << count+1 \
          << " with status " << returnValue << endl;
		  break;
		}
        else if ( returnValue == TD_END_Method )
        {
		  cout << "End of Data Reached" << endl;
		}
        else
        {
		  //Put row using Stream
		  returnValue = connStr -> PutRow(data, dataLen);

		  if ( returnValue >= TD_Error )
	 	  {
			cout << "PutRow failed on row " << count+1 
            << " with status " << returnValue << endl;
			break;
		  }
		  count++;
		}
	  }
      while ( returnValue != TD_END_Method );

	  cout << "Data transfer ended with status " << returnValue << endl;

	  if ( returnValue < TD_Error )
	  {
	   	cout << "Transfered " << count << " rows" << endl;

		/**********************************************
		* End Acquisition
		**********************************************/
		returnValue = connStr -> EndAcquisition();

		if ( returnValue < TD_Error )
		{
          cout << "Acquisition completed Successfully" << endl;
		  cout << "Data transfer completed successfully " << endl;
		}
        else
        {
		  //Get Error Information
		  cout << "Error occured during EndAcquisition" << endl;
		  connStr -> GetErrorInfo(&errorMsg, &errorType);

		  if ( errorMsg != NULL )
          {
			cout << errorMsg << endl;
			cout << "Type: " << errorType << endl;
		  }
          else
          {
		    cout << "No Error Info Available" << endl;
		  }
		}
	  }  
      else
      {
		//Get Error Information
		cout << "Error occured during Acquisition" << endl;
		connStr -> GetErrorInfo(&errorMsg, &errorType);

		if ( errorMsg != NULL )
        {
		  cout << errorMsg << endl;
		  cout << "Type: " << errorType << endl;
		}
        else
        {
		  cout << "No Error Info Available" << endl;
		}
		connExp -> GetErrorInfo(&errorMsg,&errorType);

		if ( errorMsg != NULL) 
        {
		  cout << errorMsg << endl;
		  cout << "Type: " << errorType << endl;
		}
        else
        {
		  cout << "No Error Info Available" << endl;
		}
      }
	}
    else
    {
	  //Get Error Information
	  cout << "Error occured during Stream Initiate" << endl;
	  connStr -> GetErrorInfo(&errorMsg, &errorType);

	  if ( errorMsg != NULL )
      {
	    cout << errorMsg << endl;
	    cout << "Type: " << errorType << endl;
	  }
      else
      {
        cout << "No Error Info Available" << endl;
	  }
	}
  }
  else
  {
	//Get Error Information
	cout << "Error occured during Export Initiate" << endl;
	connExp -> GetErrorInfo(&errorMsg, &errorType);

	if ( errorMsg != NULL )
    {
	  cout << errorMsg << endl;
	  cout << "Type: " << errorType << endl;
	}
    else
    {
	  cout << "No Error Info Available" << endl;
	}
  }

  /**********************************************
  * Terminate
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              TERMINATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  returnValue = connExp -> Terminate();
  if ( returnValue >= TD_Error )
  {
    //Get Error Information

    cout << "Error occured during Export Terminate" << endl;
    connExp -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Export Terminated Successfully" << endl;
  }

  returnValue = connStr -> Terminate();
  if ( returnValue >= TD_Error)
  {
    //Get Error Information

    cout << "Error occured during Stream Terminate" << endl;
    connStr -> GetErrorInfo(&errorMsg, &errorType);

    if ( errorMsg != NULL )
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
  }
  else
  {
    cout << "Stream Terminated Successfully" << endl;
  }

  /**********************************************
  * Clean Up
  **********************************************/

  lib_hdl -> delConnectionX(connStr);                     //DR142720
  lib_hdl -> delConnectionX(connExp);                     //DR142720
  lib_hdl -> delSchemaX(schemaStr);                       //DR142720
  lib_hdl -> delSchemaX(schemaExp);                       //DR142720
  lib_hdl -> delDMLGroupX(dmlGr);                         //DR142720
  lib_hdl -> Delete();                                    //DR142720

  free(user_name);
  free(user_password);
  free(target_table_export);
  free(tdp_id);
  free(error_table_stream);
  free(log_table_stream);
  free(sel_statment);
  free(dml_statment);

  cout << endl << "*** Export to Stream Example Complete ***" << endl;
  cout <<"\n######################## OUTPUT END ###########################" \
  << endl;
}


