/* 
 * Copyright 2005-2008, 2010, 2014 by Teradata Corporation.
 * All Rights Reserved.
 * TERADATA CONFIDENTIAL AND TRADE SECRET
 *
 * This document, and the information contained herein, 
 * are the exclusive
 * property of Teradata. In no case shall this document or its contents be
 * reproduced or copied by any means, in whole or in part, 
 * or disseminated
 * outside of the company, without prior written permission
 * of a company
 * officer.
 *
 */



#ifndef __MVS__
  #include "common.h"
  #include "PerfTestMessage.h"
#else
  #include "COMMOND.H"               //DR144824
  #include "PTCPERTD.H"              //DR144824
#endif//__MVS__

/**
 * Initializes the handle to the message event.
 * Refer to the header file for a more detailed method description.
 */
PerfTestMessage::PerfTestMessage(int instanceID,
 Synchronizer* sync) : hmsg(NULL), nextWrite(0), nextRead(0) {
    //mutx = new Mutex();
    //if ((mutx != NULL) && mutx->isDefined()) {
    //    hmsg = CreateEvent(NULL, FALSE, FALSE, NULL);
    //}
	mInstance = instanceID;
	mSync = sync;

	pthread_mutex_init(&mutx,NULL);
}

/**
 * Closes the handle to the message event.
 * Refer to the header file for a more detailed method description.
 */
PerfTestMessage::~PerfTestMessage() {
    //CloseHandle(hmsg);
    //delete mutx;
	pthread_mutex_destroy(&mutx);
}

/**
 * Writes a message into the array of strings of performance
 test messages.
 * Refer to the header file for a more detailed method description.
 */
void PerfTestMessage::write(int msgType, const string &msg) {

	//Store message in our TestMessage
	/*if (nextWrite == msgs.size()) {
		pthread_mutex_lock(&mutx);
		msgs.resize(msgs.size()+100);
		msgTypes.resize(msgTypes.size()+100);
		pthread_mutex_unlock(&mutx);
	}*/

    /* DR99279 BEGIN */

    pthread_mutex_lock(&mutx);

	msgTypes[nextWrite] = msgType;
	msgs[nextWrite] = msg;
	nextWrite++;

    if( nextWrite == MAX_MESSAGE_SIZE )
    {
        nextWrite = 0;
    }

    pthread_mutex_unlock(&mutx);

    /* DR99279 END */

	if ( msgType != ID_MESSAGE ){
		mSync->signalManager(mInstance,msgType);
	}
}

/**
 * Reads a message from the array of stings of performance
 test messages.
 * Refer to the header file for a more detailed method description.
 */
string PerfTestMessage::read() {
    string temp;

    /* DR99279 BEGIN */

	pthread_mutex_lock(&mutx);

    temp = msgs[nextRead];
    nextRead++;

    if( nextRead == MAX_MESSAGE_SIZE )
    {
        nextRead = 0;
    }

    pthread_mutex_unlock(&mutx);

    /* DR99279 END */
    return temp;
}

/**
 * Returns the type of the next message
 */
int PerfTestMessage::getMessageType() {
	int temp;
	pthread_mutex_lock(&mutx);
	temp = msgTypes[nextRead];
	pthread_mutex_unlock(&mutx);
	return temp;
}

/**
 * Moves to next message for read
 */
void PerfTestMessage::skipRead() {
	nextRead++;
}








