/*****************************************************
********************
*
* Copyright 2005-2008, 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
***************************************************************
**********/

#ifndef __MVS__                              //DR144824
  #include "Synchronizer.h"
#else                                        //DR144824
  #include "PTCTSYND.h"                      //DR144824
#endif                                       //DR144824

Synchronizer::Synchronizer()
{
	
        //Initiate Synchronization Variables

        pthread_mutex_init(&msgMutex, NULL);
        pthread_cond_init (&msgCond, NULL);
        //managerMsgBox = new PerfTestMessage(); 
        managerWaiting = 0;
        
        for( int i = 0; i < 2; i++){
                pthread_cond_init(&syncCond[i],NULL);
                pthread_mutex_init(&syncMutex[i],NULL);
                signalStatus[i] = 0;
        }

	pthread_mutex_init(&exportMutex,NULL);
}

Synchronizer::~Synchronizer()
{

        //Delete Synchronization Variables
         
        //delete managerMsgBox;
                 
        pthread_mutex_destroy(&msgMutex);
        pthread_cond_destroy(&msgCond);
                 
        for(int i = 0; i < 2; i++ ){
                pthread_mutex_destroy(&syncMutex[i]);
                pthread_cond_destroy(&syncCond[i]);
        }

	pthread_mutex_destroy(&exportMutex);

}

int Synchronizer::waitForThreads()
{

        /***************************************************
        * Wait until a thread has signaled that there is a
        * message
        ***************************************************/
        pthread_mutex_lock(&msgMutex);
        
        while( managerMsgBox.empty() ){ 
                managerWaiting = 1;   
                pthread_cond_wait(&msgCond,&msgMutex);
                managerWaiting = 0;
        }

        /***************************************************
        * Message in manager's message box is the threads
        * reference ID
        ***************************************************/
        int tref = managerMsgBox.front();
	managerMsgBox.pop_front();

        pthread_mutex_unlock(&msgMutex);

	tref--;

        return tref;

}

void Synchronizer::awakenThreads(int currentCond)
{

        if( currentCond == 0 ){
 
                //Reset Next Conditional Variable
                pthread_mutex_lock(&syncMutex[1]);
                signalStatus[1] = 0;
                pthread_mutex_unlock(&syncMutex[1]);

                //Signal on Current Conditional Variable
                pthread_mutex_lock(&syncMutex[0]);
                signalStatus[0] = 1;
                pthread_mutex_unlock(&syncMutex[0]);
                pthread_cond_broadcast(&syncCond[0]);
        
        }else{
                
                //Reset Next Conditional Variable
                pthread_mutex_lock(&syncMutex[0]);
                signalStatus[0] = 0;
                pthread_mutex_unlock(&syncMutex[0]);
                
                //Signal on Current Conditional Variable
                pthread_mutex_lock(&syncMutex[1]);
                signalStatus[1] = 1;
                pthread_mutex_unlock(&syncMutex[1]);
                pthread_cond_broadcast(&syncCond[1]);
        } 

}

void Synchronizer::waitForManager(int currentCond)
{

	/**********************************************
	* If current condition variable is already
        * signaled then go - otherwise wait
      	**********************************************/
   	pthread_mutex_lock(&syncMutex[currentCond]);
                
     	if ( signalStatus[currentCond] == 0 ){
		pthread_cond_wait(&syncCond[currentCond],
&syncMutex[currentCond]);
  	}
                
	pthread_mutex_unlock(&syncMutex[currentCond]);
                

}

void Synchronizer::signalManager(int instanceID, int msgType)
{
                        
 	//Put instance id in manager's TestMessage   
                
    	pthread_mutex_lock(&msgMutex);
     	managerMsgBox.push_back(instanceID);
 
      	//Wait until manager is waiting unless this is
	// a status message
     	//while( managerWaiting == 0 && msgType != STATUS_MESSAGE){
        //   	pthread_mutex_unlock(&msgMutex);
        //    	yield();
        //    	pthread_mutex_lock(&msgMutex);
    	//}

	pthread_cond_signal(&msgCond);
	pthread_mutex_unlock(&msgMutex);

    	//Signal manager if waiting
     	//if ( managerWaiting == 1 ){
        //   	pthread_mutex_unlock(&msgMutex);
        //  	pthread_cond_signal(&msgCond);
      	//}else{
        //     	pthread_mutex_unlock(&msgMutex);
     	//}                        

}

int Synchronizer::writeOutRow(FILE* dest, char* data,
 TD_Length dataLen)
{
        //Prepare data for writing
        unsigned long i = 0;
        unsigned short shortdataLen = (unsigned short)dataLen;
        char* rowBuffer = (char*)malloc(dataLen+20);

#ifndef __MVS__
        memcpy(rowBuffer, &shortdataLen, 2);

        for(i = 0; i < dataLen; i++)
        {
                rowBuffer[i+2] = data[i];
        }
        rowBuffer[i+2] = 0x0a;
#else
	for(i = 0; i < dataLen; i++)
        {
                rowBuffer[i] = data[i];
        }
#endif //__MVS__

        //Obtain output file lock and write data
        pthread_mutex_lock(&exportMutex);

#ifndef __MVS__
        fwrite(rowBuffer, dataLen+3, 1, dest);
#else
	fwrite(rowBuffer, dataLen, 1, dest);
#endif //__MVS__

	pthread_mutex_unlock(&exportMutex);

        free(rowBuffer);

        return 0;

}
