/*************************************************************************
*
* Copyright 2010-2011, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* History Information:
*
* Revision     Date      DR      DID       Comments
* ---------------------------------------------------------------------
* 14.00.00.02  01222011  146403  jk186005  Remove the TD_NO_OPERATOR 
                                           references
* 14.00.00.01  10292010  144824  mv121803  Support MVS
* 13.00.00.01  06182010  142720  mv121803  Dynamically load TELAPI
*
*************************************************************************/


// TelapiThreadTest.cpp : Defines the entry point for the
// console application.
//

//#include "stdafx.h"
#ifndef __MVS__
#include "common.h"
#include "ThreadTest.h"
#include "OptionsManager.h"
#include "dynlib.h"                  //DR142720
#else
#include "COMMOND.H"                 //DR144824
#include "PTCTHDTD.h"                //DR144824
#include "PTCOPTMA.h"                //DR144824
#include "PTCDLLIB.H"                //DR144824
#endif//__MVS__

#define _XOPEN_SOURCE_EXTENDED 1
#include <string.h>

// Platform definitions

#if defined(WIN32) || defined(WIN64)
#define OS_WINDOWS
#elif defined(__sun)
#define OS_SOLARIS
#elif defined(__linux__)
#define OS_LINUX
#elif defined(HPUX)
#define OS_HPUX
#elif defined(_AIX)
#define OS_AIX
#endif


int main( int argc,char * argv[] )
{
    DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();       //DR142720

    OptionsManager  *om = new OptionsManager();
    if (argc != 2) {
        std::cout << "Usage:\n";
        std::cout << "TelapiThreadTest <config file>\n";
        return -1;
    }
    else 
    {
        om->readOptionsFromFile(argv[1]);
    }

    ThreadTestParameters* params = NULL;
    ThreadTest* t = NULL;
    SchemaX* schema = NULL;
    DMLGroupX* dmlGr = NULL;
    DMLGroupX* dmlGr2 = NULL;
    char input;
    char * user_name = strdup(om->getUserName().c_str());
    char * user_password = strdup(om->getUserPassword().c_str());
    char * target_table = strdup(om->getTargetTable().c_str());
    char * target_tableA = strdup(om->getTargetTable(0).c_str());
    char * target_tableB = strdup(om->getTargetTable(1).c_str());
    char * tdp_id = strdup(om->getTdpId().c_str());
    char * file_name = strdup(om->getInputFileName().c_str());
    char * outfile_name = strdup(om->getOutputFileName().c_str());
    char * dml_statment = (char*)malloc(1024);
    char * dml_statmentA = (char*)malloc(1024);
    char * dml_statmentB = (char*)malloc(1024);

int testToRun = om->getOperType();


	switch ( testToRun ){

		case LOAD_TEST:
			/**********************************************
			* Uses Load to load 100 rows
			**********************************************/
			params = new ThreadTestParameters(TD_LOAD,4);
			params->SetLogon(user_name,user_password, tdp_id);
			params->SetFile(file_name);
			params->SetTable(1,target_table);
			/* SetCheckpoint - num of rows/buffers
 to checkpoint at : restart tries */
			params->SetCheckpoint(10,1);
			params->SetSessions(4,4);
			/* SetBufferMode - 1 - on, 0 - off :
 then num of rows per buffer */
			//params->SetBufferMode(1,10); 

			//DR142720 schema = new Schema("input");
                        schema = lib_hdl->newSchemaX("input");      //DR142720
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_CHAR,96);  //TPT-17718
			params->mSchemaList.push_back(schema);

            sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);"
, om->getTargetTable().c_str());
			//DR142720 dmlGr = new DMLGroup();
                        dmlGr = lib_hdl->newDMLGroupX();            //DR142720
			dmlGr->AddStatement(dml_statment);
			params->mDMLGroupList.push_back(dmlGr);

			t = new ThreadTest(params);
			t->runLoad();
			break;

		case EXPORT_TEST:
			/*************************************
*********
			* Uses Export to export from target table
			***************************************
*******/
			params = new ThreadTestParameters(TD_EXPORT, 4);
			params->SetLogon(user_name,user_password,
tdp_id);
			params->SetFile(outfile_name);
			params->SetTable(1,target_table);
			params->SetSessions(4,4);

            sprintf(dml_statment, "sel * from %s;",
 om->getTargetTable().c_str());
			params->AddAttribute(TD_SELECT_STMT,1,dml_statment);

			//DR142720 schema = new Schema("input");
                        schema = lib_hdl->newSchemaX("input");      //DR142720
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_VARCHAR,32000);  //TPT-17718
			params->mSchemaList.push_back(schema);

			t = new ThreadTest(params);
			t->runExport();
			break;

		case UPDATE_TEST:
			/*************************************
*********
			* Uses Update to load 50 rows into one table
			* and then 50 rows into another table
			**************************************
********/
			params =
                        new ThreadTestParameters(TD_UPDATE,4);
			params->SetLogon(user_name, user_password,tdp_id);
			params->SetFile(file_name);
			params->SetTable(2,
target_tableA,target_tableB);
			/* SetCheckpoint - num of rows/buffers to
 checkpoint at : restart tries */
			params->SetCheckpoint(10,1);
			params->SetSessions(4,4);
			/* SetBufferMode - 1 - on, 0 - off :
 then num of rows per buffer */
			//params->SetBufferMode(1,10); 

			//DR142720 schema = new Schema("input");
                        schema = lib_hdl->newSchemaX("input");      //DR142720
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_CHAR,96);  //TPT-17718
			params->mSchemaList.push_back(schema);

            sprintf(dml_statmentA, "INSERT INTO %s( :col1, :col2);",
 om->getTargetTable(0).c_str());
            sprintf(dml_statmentB, "INSERT INTO %s( :col1, :col2);",
 om->getTargetTable(1).c_str());

			//DR142720 dmlGr = new DMLGroup();
                        dmlGr = lib_hdl->newDMLGroupX();            //DR142720
			dmlGr->AddStatement(dml_statmentA);
			params->mDMLGroupList.push_back(dmlGr);

			//DR142720 dmlGr2 = new DMLGroup();
                        dmlGr2 = lib_hdl->newDMLGroupX();            //DR142720
			dmlGr2->AddStatement(dml_statmentB);
			params->mDMLGroupList.push_back(dmlGr2);

			t = new ThreadTest(params);
			t->runUpdate();
			break;

		case STREAM_TEST:
			/***********************************
***********
			* Uses Stream to load 100 rows
			**************************************
********/
			params = new ThreadTestParameters(TD_STREAM, 4);
			params->SetLogon(user_name,user_password,
tdp_id);
			params->SetFile(file_name);
			params->SetTable(1,target_table);
			/* SetCheckpoint - num of rows/buffers to 
checkpoint at : restart tries */
			params->SetCheckpoint(10,1);
			params->SetSessions(4,4);
			/* SetBufferMode - 1 - on, 0 - off : 
then num of rows per buffer */
			//params->SetBufferMode(1,10); 

			//DR142720 schema = new Schema("input");
                        schema = lib_hdl->newSchemaX("input");      //DR142720
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_CHAR,96);  //TPT-17718
			params->mSchemaList.push_back(schema);

            sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);",
 om->getTargetTable().c_str());
			//DR142720 dmlGr = new DMLGroup();
                        dmlGr = lib_hdl->newDMLGroupX();            //DR142720
			dmlGr->AddStatement(dml_statment);
			params->mDMLGroupList.push_back(dmlGr);

			t = new ThreadTest(params);
			t->runStream();
			break;

		/* DR146403 - Remove the SIM_TEST case statement to 
         remove the TD_NO_OPERATOR references */

		default:
			cout << "No Test Specified" << endl;

	}


	if ( params != NULL )
		delete params;

	if ( t != NULL )
		delete t;

    lib_hdl->Delete();                           //DR142720 

    free(user_name);
    free(user_password);
    free(target_table);
    free(target_tableA);
    free(target_tableB);
    free(tdp_id);
    free(file_name);
    free(outfile_name);
    free(dml_statment);
    free(dml_statmentA);
    free(dml_statmentB);

	//system("PAUSE");

    cout << "*** Application Complete ***" << endl;

    pthread_exit(NULL);

    return 0;
}













