/*************************************************************************
*
* Copyright 2010-2011, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* History Information:
*
* Revision     Date      DR      DID       Comments
* ---------------------------------------------------------------------
* 14.00.00.02  01222011  146403  jk186005  Remove the TD_NO_OPERATOR
                                           references 
* 14.00.00.01  10292010  144824  mv121803  Support MVS
* 13.00.00.01  06182010  142720  mv121803  Dynamically load TELAPI
*
*************************************************************************/

#ifndef __MVS__                           //DR144824
  #include "TestBase.h"                   //DR144824
  #include "dynlib.h"                     //DR142720
#else                                     //DR144824
  #include "PTCBASTD.h"                   //DR144824
  #include "PTCDLLIB.H"                   //DR144824
#endif                                    //DR144824


TestBase::TestBase(ThreadTestParameters* params, int instance, 
Synchronizer* sync, long start, long end) {
	mInstanceID = instance;
	mParams = params; /* save for later */
	testMessage = NULL;
	mCurrentStatus = TD_Success;
	mErrorCode = -1;
	mErrorType = -1;
	mErrorMsg  = NULL;
	mRunStatus = NORMAL;
	mCurrentMethod = PRE_INITIATE_METHOD;
	mHaveFailed = 0;
	mEndAcquisitionDone = false;                       /* DR99279 */
	mRowsSentCount = 0;
	mCheckpointsTaken = 0;
	mBufferModeEnabled = mParams->mBufferModeEnabled;
	mBufferLayout.BufTrailerSize = 0;
	mBufferLayout.MaxMsgSize = 0;
	mBufferLayout.RowHeaderSize = 0;
	mBufferLayout.RowLengthSize = 0;
	mBuffer = NULL;
	mBufferLen = 0;
	mLeftOverRowLen = 0;
	mLeftOverRow = NULL;
	mBuffersSentCount = 0;
	mSkipRow = 0;
	mBufferRowCountLimit = mParams->mBufferRowCountLimit;

	//Store Manager's event handle
	mSync = sync;
	mActiveHandle = 0;

	//Initialize Test Message
	testMessage = new PerfTestMessage(mInstanceID,mSync);

	//Initialize Checkpoint variables
	mCheckPointData = NULL;
	mCheckPointLength = 0;
	mCheckPointInterval = params->mCheckpointInterval;
	mLastCheckpointPosition = SEEK_SET;

	// Initialize Source Info
	mSource = NULL;

	if ( params->mSourceFileName != NULL && params->mOperType !=
 TD_EXPORT )
	{
		mSource = fopen( params->mSourceFileName, "rb" );

		if( mSource == NULL ){
			mRunStatus = ABORT; //Notify manager
			// that file could not be opened
		}else{
			unsigned short rowSize;
#ifndef __MVS__
			size_t recLen = fread(&rowSize, 1, 2, mSource);
			mRowSize = rowSize+3; 
#else
			mRowSize = 101;
#endif //__MVS__
			fseek(mSource,SEEK_SET,SEEK_SET);
		}
	}

	mStart = start;
	mEnd   = end;
	//mRowSize = params->mRowSize;
	mOperType = params->mOperType;
	mConnection = NULL;
}

TestBase::~TestBase()
{
	//close source file
	if ( mSource != NULL ){
		fclose(mSource);
	}

	// close the phases file
	if ( mPhaseFile != NULL ){
		fclose(mPhaseFile);
	}

	if ( testMessage != NULL ){
		delete testMessage;
	}

	if ( mConnection != NULL ){
	   //delete mConnection;
           DynamicLibrary *lib_hdl = DynamicLibrary::Instance();   //DR142720
           lib_hdl->delConnectionX(mConnection);                   //DR142720
	}

	if ( mErrorMsg != NULL ){
		free(mErrorMsg);
	}

	if ( mCheckPointData != NULL ){
		free(mCheckPointData);
	}

	if( mBuffer != NULL ){
		free(mBuffer);
	}
}

void TestBase::createConnection()
{
    DynamicLibrary  *lib_hdl = DynamicLibrary::Instance();        //DR142720

	if ( mConnection != NULL )
	{
	   //delete mConnection;
           lib_hdl->delConnectionX(mConnection);                   //DR142720
	}

	if ( mParams->mOperType )                                   //DR146403
	{
		//Prepare Connection
		//mConnection = new Connection();
                mConnection = lib_hdl->newConnectionX();          //DR142720

		//Add Special Attributes
		switch( mParams->mOperType ){

			case TD_LOAD:
				
mConnection->AddAttribute(TD_SYSTEM_OPERATOR,TD_LOAD);
				if( mParams->mTraceLevel != TD_OFF ){
					if(mInstanceID == 1)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"load1.txt");
					}
					else if(mInstanceID == 2)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"load2.txt");
					}
					else if(mInstanceID == 3)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"load3.txt");
					}
					else if(mInstanceID == 4)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"load4.txt");

					}
				}
				break;
			case TD_UPDATE:
				
mConnection->AddAttribute(TD_SYSTEM_OPERATOR,TD_UPDATE);
				if( mParams->mTraceLevel != TD_OFF ){
					if(mInstanceID == 1)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"update1.txt");
					}
					else if(mInstanceID == 2)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"update2.txt");
					}
					else if(mInstanceID == 3)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"update3.txt");
					}
					else if(mInstanceID == 4)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"update4.txt");
					}
				}
				break;
			case TD_STREAM:
				
mConnection->AddAttribute(TD_SYSTEM_OPERATOR,TD_STREAM);
				if( mParams->mTraceLevel != TD_OFF ){
					if(mInstanceID == 1)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"stream1.txt");
					}
					else if(mInstanceID == 2)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"stream2.txt");
					}
					else if(mInstanceID == 3)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"stream3.txt");
					}
					else if(mInstanceID == 4)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"stream4.txt");
					}
				}
				break;
			case TD_EXPORT:
				
mConnection->AddAttribute(TD_SYSTEM_OPERATOR,TD_EXPORT);
				if( mParams->mTraceLevel != TD_OFF ){
					if(mInstanceID == 1)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"export1.txt");
					}
					else if(mInstanceID == 2)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"export2.txt");
					}
					else if(mInstanceID == 3)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"export3.txt");
					}
					else if(mInstanceID == 4)
					{
						
mConnection->AddAttribute(TD_TRACE_OUTPUT,"export4.txt");
					}
				}
				break;
			default:
				break;
		}

		if( mParams->mTraceLevel != TD_OFF ){
			
mConnection->AddArrayAttribute(TD_TRACE_LEVEL,2,TD_OPER_ALL,TD_GENERAL,NULL);
//mParams->mTraceLevel,TD_OFF,NULL);
		}

		//Add Unique Instance ID
		
mConnection->AddAttribute(TD_INSTANCE_NUM, mInstanceID);

		//Add Buffer Mode if Necessary
		if( mParams->mBufferModeEnabled )
		{
			
mConnection->AddAttribute(TD_BUFFER_MODE,"Yes");
		}

		//Add Attributes

		vector<TD_Attribute>::iterator attrIter = 
mParams->mAttributeTypes.begin();
		vector<int>::iterator countIter = 
mParams->mAttributeValueCount.begin();
		vector<char*>::iterator valueIter = 
mParams->mAttributeValues.begin();

		for(; attrIter!=mParams->
mAttributeTypes.end(); attrIter++, countIter++)
		{
			TD_Attribute aType = *attrIter;
			int numValues = *countIter;

			if ( numValues == 1 ){

				
mConnection->AddAttribute(aType,*valueIter++);

			}else if ( numValues == 2){

				char* val1 = *valueIter++;

				
mConnection->AddArrayAttribute( aType, numValues,
 val1, *valueIter++, NULL);

			}else if ( numValues == 3){

				char* val1 = *valueIter++;
				char* val2 = *valueIter++;

				
mConnection->AddArrayAttribute( aType, numValues, val1, 
val2, *valueIter++, NULL);

			}else if ( numValues == 4){

				char* val1 = *valueIter++;
				char* val2 = *valueIter++;
				char* val3 = *valueIter++;

				
mConnection->AddArrayAttribute( aType, numValues, val1, val2, val3,
 *valueIter++, NULL);

			}

		}

		// Add Schemas

		vector<SchemaX*>::iterator schemaIter =
 mParams->mSchemaList.begin();

		for(; schemaIter != mParams->mSchemaList.end();
 schemaIter++)
		{
			mConnection->AddSchema(*schemaIter);
		}

		// Add DMLGroups

		vector<DMLGroupX*>::iterator dmlIter = 
mParams->mDMLGroupList.begin();

		for(int i = 0; dmlIter != 
mParams->mDMLGroupList.end(); dmlIter++, i++)
		{
			
mConnection->AddDMLGroup(*dmlIter, &mIndexes[i]);
		}

	}else{
		mConnection = NULL;
	}
}

/**
 * Waits for Manager to signal
 */
TD_StatusCode TestBase::waitForManager()
{

	do{
		if ( mActiveHandle == 0 ){
			mActiveHandle = 1;

			
testMessage->write(STATUS_MESSAGE,"Waiting for manager [0]");
			
mSync->waitForManager(0);

		}else{
			mActiveHandle = 0;

			
testMessage->write(STATUS_MESSAGE,"Waiting for manager [1]");
			mSync->waitForManager(1);
		}

		//Resubmit message if waiting again
		if( mRunStatus == WAIT_AGAIN && mInstanceID != 1 ){
			
testMessage->write(mCurrentMessageType,"Slave waiting again");
		}

	}while( mRunStatus == WAIT_AGAIN && mInstanceID != 1 );

	if( mRunStatus != NORMAL && mCurrentMethod != 
TERMINATE_METHOD && mRunStatus != WAIT_AGAIN ){
		return TD_Error;
	}else{
		return mCurrentStatus;
	}
}


/**
 * Gets the handle to TestBase test messages.
 * Refer to the header file for a more detailed method description.
 */
//HANDLE TestBase::getMessageHandle() 
//{
//    if (testMessage == NULL) {
//        testMessage = new PerfTestMessage();
//    }
//    return testMessage->getMessageHandle();
//}

int	TestBase::getRow(char* rowBuffer)
{
	size_t recLen = 0;

	if ( mSource != NULL )
	{
		// If first time, set file position
		if ( ftell(mSource) == SEEK_SET )
		{
			fseek(mSource,mStart,SEEK_SET);
		}

		if ( ftell(mSource) < mEnd && !feof(mSource) )
		{
			//ReadRow
			recLen = fread(rowBuffer, 1,
 mRowSize, mSource);

			if ( recLen < mRowSize ){
				return READ_ERROR;
			}
		}else {
			return READ_DONE;
		}

		return READ_SUCCESS;
	}

	return READ_ERROR;
}

int	TestBase::getBuffer(char** returnBuffer,
TD_Length* returnBufferLen,int* count)
{
	char* row;
	TD_Length rowLen = 0;
	char* bufptr;
	char endChar;
	int readStatus = READ_SUCCESS;
	int haveBuffer = FALSE;
	int skipRead = FALSE;
	size_t readCount = 0;
	unsigned short len = 0;
	int bufferRowCount = 0;
	int parcelSize = mBufferLayout.RowHeaderSize -
 mBufferLayout.RowLengthSize;
	int padding = mBufferLayout.BufTrailerSize +
 mBufferLayout.RowHeaderSize;

	if ( mSource != NULL )
	{
		//If first time, set file position
		if ( ftell(mSource) == SEEK_SET )
		{
			fseek(mSource,mStart,SEEK_SET);
		}

		/* Initialize Buffer */
		if( mBuffer == NULL )
		{
			mBuffer =
 (char*)malloc(mBufferLayout.MaxMsgSize);
		}
		mBufferLen = 0;
		bufptr = mBuffer;

		/* Check For Left Over Data */
		if( mLeftOverRowLen > 0 )
		{
			len = (unsigned short)mLeftOverRowLen;
			row = mLeftOverRow;
			rowLen = len;

			mLeftOverRow = NULL;
			mLeftOverRowLen = 0;

			skipRead = TRUE;
		}

		while( !haveBuffer && readStatus == READ_SUCCESS )
		{
			if( !skipRead )
			{
				/* Get A Row */
				if ( ftell(mSource)
 < mEnd && !feof(mSource) )
				{
					/* Read Record Length */
					readCount = 
fread(&len,1,sizeof(unsigned short),mSource);
					rowLen = len;
					
					if( readCount < 1 )
					{
						
readStatus = READ_ERROR;
					}

					if( readStatus !=
 READ_ERROR && len > 0 )
					{
						/* Read Record Data */
						
row = (char*)malloc(len);
						
readCount = fread(row,1,len,mSource);

						if( readCount < 1 )
						{
							
readStatus = READ_ERROR;
						}

						
readCount = fread(&endChar,1,1,mSource);
					}

				}else {
					readStatus = READ_DONE;
				}
			}else
			{
				skipRead = FALSE;
			}

			if( readStatus == READ_SUCCESS )
			{
				if( !mSkipRow )
				{
					/* Check If Row Fits */
					
if( rowLen + mBufferLen + padding <= mBufferLayout.MaxMsgSize )
					{
						/* Skip Parcel */
						bufptr += parcelSize;
						
mBufferLen += parcelSize;

						/* Add Length */
						
if( mBufferLayout.RowLengthSize == 2 )
						{
							
memcpy(bufptr,&len,sizeof(unsigned short));
							bufptr += 2;
							
mBufferLen += 2;
						}
						
else if ( mBufferLayout.RowLengthSize == 4 )
						{
							
memcpy(bufptr,&rowLen,sizeof(unsigned int));
							bufptr += 4;
							
mBufferLen += 4;
						}

						/* Add Data */
						
memcpy(bufptr,row,rowLen);
						bufptr += rowLen;
						mBufferLen += rowLen;

						/* Increment Count */
						bufferRowCount++;

						
if( bufferRowCount >= mBufferRowCountLimit &&
 mBufferRowCountLimit != 0 )
						{
							
haveBuffer = TRUE;
						}
					}else
					{
						
/* Save Row For Next Time */
						
mLeftOverRow = (char*)malloc(rowLen);
						
memcpy(mLeftOverRow,row,rowLen);
						
mLeftOverRowLen = rowLen;

						haveBuffer = TRUE;
					}

				}else{
					mSkipRow = FALSE;
				}

				free(row);
			}
		}

	}else{
		readStatus = READ_ERROR;
	}

	if( readStatus != READ_ERROR && mBufferLen > 0 )
	{
		/* Add Trailer to Buffer */
		mBufferLen += mBufferLayout.BufTrailerSize;
		readStatus = READ_SUCCESS;

		*returnBuffer = mBuffer;
		*returnBufferLen = mBufferLen;
		*count = bufferRowCount;
	}
	else
	{
		*returnBuffer = NULL;
		*returnBufferLen = 0;
		*count = 0;
	}

	return readStatus;
}

int	TestBase::putRow(char* data, TD_Length dataLen)
{
	return 0;
}

void TestBase::signalError(TD_StatusCode status)
{
	char msg[256];

	//Set Error String Message
	switch( mCurrentMethod ){
		case PRE_INITIATE_METHOD:
			sprintf(msg,"Error occured before Initiate");
			break;
		case INITIATE_METHOD:
			sprintf(msg,"Error occured in Initiate");
			break;
		case ACQUISITION_METHOD:
			sprintf(msg,"Error occured in Acquisition");
			break;
		case END_ACQUISITION_METHOD:
			sprintf(msg,"Error occured in EndAcquisition");
			break;
		case APPLICATION_METHOD:
			sprintf(msg,"Error occured in Application");
			break;
		case TERMINATE_METHOD:
			sprintf(msg,"Error occured in Terminate");
			break;
		case CHECKPOINT_METHOD:
			sprintf(msg,"Error occured in Checkpoint");
			break;
		case RESTART_METHOD:
			sprintf(msg,"Error occured in Restart");
			break;
		default:
			sprintf(msg,"Error occured");
			break;
	}

	setErrorInfo(status);
	mCurrentStatus = status;
	testMessage->write(ERROR_MESSAGE,msg);	
	waitForManager();
}

void TestBase::setErrorCode(int eCode)
{
	mErrorCode = eCode;
}


void TestBase::setErrorType(int eType)
{
	mErrorType = eType;
}

void TestBase::setErrorMsg(const char* eMsg)
{
	if ( mErrorMsg != NULL ){
		free(mErrorMsg);
	}

	if ( eMsg != NULL ){
		mErrorMsg = strdup(eMsg);
	}else{
		mErrorMsg = strdup("Invalid Error Message");
	}
	//char* mErrorMsg = (char*)malloc(sizeof(eMsg)+256);
	//strcpy(mErrorMsg,eMsg);
}

void TestBase::setErrorInfo(int eCode){
	//Get remaining error info from connection
	char* errorMsg;
	TD_ErrorType errorType;

	if ( mConnection != NULL ){
		mConnection->GetErrorInfo(&errorMsg,&errorType);
	}else{
		errorMsg = (char*)"Error occured!";         //TPT-17718
		errorType = -1;
	}

	//Store error info
	setErrorCode(eCode);
	setErrorType(errorType);
	setErrorMsg(errorMsg);
}

TD_StatusCode TestBase::getTELINFO(char** telinfo, TD_Length* length)
{
	//char* buffer = strdup("TELINFO TELINFO TELINFO TELINFO");
	//*telinfo = buffer;
	//*length  = (TD_Length)strlen(buffer);
	//return TD_Success;
	return mConnection->GetTELINFO(telinfo,length);
}

TD_StatusCode TestBase::setTELINFO(char* telinfo, TD_Length length)
{
	return mConnection->PutTELINFO(telinfo,length);
	//return TD_Success;
}

void TestBase::setRunStatus(int status)
{
	if ( mRunStatus != RESTART ){
		mRunStatus = status;
	}
}

int TestBase::getRunStatus()
{
	return mRunStatus;
}

int	TestBase::getCurrentMethod()
{
	return mCurrentMethod;
}

void TestBase::setInstanceID(int id)
{
	mInstanceID = id;
}

int	TestBase::getInstanceID()
{
	return mInstanceID;
}

TD_StatusCode TestBase::getCurrentStatus()
{
	return mCurrentStatus;
}

TD_StatusCode TestBase::processStatusCode(TD_StatusCode status)
{
	TD_StatusCode rv = status;

	switch( rv ){

		case TD_SYNC_Barrier:
			mCurrentStatus = rv;
			mCurrentMessageType = SYNC_BARRIER;
			
testMessage->write(SYNC_BARRIER,"Barrier Message");
			rv = waitForManager();
			break;
		case TD_SYNC_TELINFO:
			mCurrentStatus = rv;
			mCurrentMessageType = SYNC_TELINFO;
			
testMessage->write(SYNC_TELINFO,"TELINFO Message");
			rv = waitForManager();
			break;
		case TD_END_Method:
			mCurrentStatus = rv;
			mCurrentMessageType = SYNC_BARRIER;
			
testMessage->write(SYNC_BARRIER,"End of Method");
			rv = waitForManager();
			break;
		case TD_Success:
			rv = TD_Success;
			mCurrentStatus = rv;
			mCurrentMessageType = STATUS_MESSAGE;
			
//testMessage->write(SYNC_BARRIER,"End of Method");
			
//testMessage->write(STATUS_MESSAGE,"Success getting a row");
			//rv = waitForManager();
			break;
		default:
			
if( mRunStatus == NORMAL || ( mRunStatus != NORMAL &&
 mCurrentMethod == TERMINATE_METHOD) ){
				mCurrentMessageType = ERROR_MESSAGE;
				signalError(rv);
			}
			break;
	}	

	return rv;
}


void TestBase::printTestOptions()
{}

TD_StatusCode TestBase::run()
{ return TD_Success; }

TD_StatusCode TestBase::runRestart()
{ return TD_Success; }

TD_StatusCode TestBase::Initiate()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = INITIATE_METHOD;
	char *dataptr;
	char* msg = (char*)malloc(256);
	TD_Length datalen;
	TD_StatusCode queryStatus = TD_Success;

	//Add Unique Instance ID
	//mConnection->AddAttribute(TD_INSTANCE_NUM, mInstanceID);

	while ( rv != TD_END_Method && rv < TD_Error )
	{
		rv = mConnection->Initiate();

		rv = processStatusCode(rv);	
	}

	if( rv < TD_Error){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Driver Initiated");

		if( mBufferModeEnabled )
		{
			queryStatus = 
mConnection->GetEvent(TD_Evt_BufferLayout,&dataptr,&datalen);

			if ( queryStatus <
 TD_Error && queryStatus != TD_Unavailable )
			{
				mBufferLayout.MaxMsgSize =
 *((int*)dataptr);
				mBufferLayout.RowHeaderSize =
 *((int*)(dataptr+4));
				mBufferLayout.RowLengthSize =
 *((int*)(dataptr+8));
				mBufferLayout.BufTrailerSize =
 *((int*)(dataptr+12));

				/* Report Buffer Layout */
				sprintf(msg,
"TD_Evt_BufferLayout reports %d max buffer size",
mBufferLayout.MaxMsgSize);
				testMessage->write(STATUS_MESSAGE,msg);
				sprintf(msg,
"TD_Evt_BufferLayout reports %d row header size",
mBufferLayout.RowHeaderSize);
				testMessage->write(STATUS_MESSAGE,msg);
				sprintf(msg,
"TD_Evt_BufferLayout reports %d row length size",
mBufferLayout.RowLengthSize);
				testMessage->write(STATUS_MESSAGE,msg);
				sprintf(msg,
"TD_Evt_BufferLayout reports %d trailer size",
mBufferLayout.BufTrailerSize);
				testMessage->write(STATUS_MESSAGE,msg);
			}else
			{
				sprintf(msg,
"TD_Evt_BufferLayout info unavailable with status %d",queryStatus);
				testMessage->write(STATUS_MESSAGE,
msg);				
			}
		}
	}

	free(msg);
	return rv;
}

TD_StatusCode TestBase::Acquisition()
{ return TD_Success; }

TD_StatusCode TestBase::Application()
{ return TD_Success; }

TD_StatusCode TestBase::Terminate()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = TERMINATE_METHOD;
	char* msg = (char*)malloc(256);

	while ( rv != TD_END_Method && rv < TD_Error )
	{
		testMessage->write(STATUS_MESSAGE,"Calling Terminate");

		rv = mConnection->Terminate();
	
		testMessage->write(STATUS_MESSAGE,
"Returned from Terminate");

		rv = processStatusCode(rv);
	}

	if( rv < TD_Error){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,
"Driver Terminated");

		if( mParams->mOperType != TD_EXPORT ){
			
sprintf(msg,"Total Number o\
f Rows Sent: %d",mRowsSentCount);
			testMessage->write(STATUS_MESSAGE,msg);
			
sprintf(msg,"Number of Checkpoints Taken: %d",mCheckpointsTaken);
			testMessage->write(STATUS_MESSAGE,msg);
		}else{
			
sprintf(msg,"Total Number of Rows Exported: %d",mRowsSentCount);
			testMessage->write(STATUS_MESSAGE,msg);
		}
	}

	if( mRunStatus == RESTART ){
		mRunStatus = NORMAL;
		rv = runRestart();
	}

	free(msg);
	mCurrentStatus = rv;
	testMessage->write(EXIT_MESSAGE,"Thread Exiting");

	return rv;
}

TD_StatusCode TestBase::Checkpoint()
{ 
	TD_StatusCode rv = TD_Success;
	TD_StatusCode queryStatus = TD_Success;
	mCurrentMethod = CHECKPOINT_METHOD;
	char* cpData = NULL;
	TD_Length cpLen = 0;
	char *dataptr;
	char* msg = (char*)malloc(256);
	TD_Length datalen;
	int dbserror = 0;

	/* Barrier Before Checkpoint */
	rv = processStatusCode(TD_SYNC_Barrier);

	while ( rv != TD_END_Method && rv < TD_Error )
	{
		rv = mConnection->Checkpoint(&cpData,&cpLen);

		rv = processStatusCode(rv);
	}

	if( rv < TD_Error){

		/* Copy in Checkpoint Data */
		mCheckPointLength = cpLen;

		if( mCheckPointData == NULL ){
			mCheckPointData = (char*)malloc(256);
		}

		if( cpLen != 0 ){
			memcpy(mCheckPointData,cpData,cpLen);
		}else{
			free(mCheckPointData);
			mCheckPointData = NULL;
			mCheckPointLength = 0;
		}

		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Checkpoint Taken");

		if( mCheckPointLength != 0 ){
			
sprintf(msg,"Checkpoint data: %d, length: %d",
*(int*)mCheckPointData,mCheckPointLength);
			testMessage->write(STATUS_MESSAGE,msg);
		}else{
			
sprintf(msg,"Checkpoint data: NULL, length: %d",mCheckPointLength);
			testMessage->write(STATUS_MESSAGE,msg);
		}

		mLastCheckpointPosition = ftell(mSource);
		mCheckpointsTaken++;

		/* Query Row Counts */
		
queryStatus = mConnection->GetEvent(TD_Evt_RowCounts,
 &dataptr, &datalen);	

		/* If there is information available, process it */
		
if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
		{
			
sprintf(msg,"TD_Evt_RowCounts reports %d rows received"
,*((int*)dataptr));
			testMessage->write(STATUS_MESSAGE,msg);
			
sprintf(msg,"TD_Evt_RowCounts\
 reports %d rows sent",*((int*)(dataptr+4)));
			testMessage->write(STATUS_MESSAGE,msg);
			
sprintf(msg,"TD_Evt_RowCounts reports\
 %d rows applied",*((int*)(dataptr+8)));
			testMessage->write(STATUS_MESSAGE,msg);
		}else{
			
sprintf(msg,"TD_Evt_RowCounts Data Unavailable");
			testMessage->write(STATUS_MESSAGE,msg);
		}

		if( mOperType == TD_STREAM )
		{
			/* Query Run Stats */
			
queryStatus = mConnection->GetEvent(TD_Evt_RunStats, &dataptr,
 &datalen, 1);	

			
/* If there is information available, process it */
			
if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
			{
				
sprintf(msg,"TD_Evt_RunStats reports\
 %d SQL statements sent",*((int*)dataptr));
				
testMessage->write(STATUS_MESSAGE,msg);
				
sprintf(msg,"TD_Evt_RunStats reports \
%d requests sent",*((int*)(dataptr+4)));
				
testMessage->write(STATUS_MESSAGE,msg);
				
sprintf(msg,"TD_Evt_RunStats reports\
 %d rows received",*((int*)(dataptr+8)));
				
testMessage->write(STATUS_MESSAGE,msg);
				
sprintf(msg,"TD_Evt_RunStats reports\
 %d rows sents",*((int*)(dataptr+12)));
				
testMessage->write(STATUS_MESSAGE,msg);
				
dbserror = *((int*)(dataptr+16));
				
if( dbserror > 0 )
				
  {
					
sprintf(msg,"TD_Evt_RunStats reports %d rows caused\
 DBS errors",*((int*)(dataptr+16)));
					
testMessage->write(STATUS_MESSAGE,msg);
				
}
				
else
				
{
					
sprintf(msg,"TD_Evt_RunStats reports\
 no rows caused DBS errors");
					
testMessage->write(STATUS_MESSAGE,msg);
				}
			}else{
				
sprintf(msg,"TD_Evt_RunStats Data Unavailable");
				
testMessage->write(STATUS_MESSAGE,msg);
			}
		}
	}

	free(msg);
	return rv;
}

TD_StatusCode TestBase::Restart()
{ 
	TD_StatusCode rv = TD_Success;
	TD_StatusCode queryStatus = TD_Success;
	mCurrentMethod = RESTART_METHOD;
	char *dataptr;
	char* msg = (char*)malloc(256);
	TD_Length datalen;
	int dbserror = 0;

	if( mCheckPointLength != 0 ){
		sprintf(msg,"Restart\
 data: %d, length: %d",*(int*)mCheckPointData,mCheckPointLength);
		testMessage->write(STATUS_MESSAGE,msg);
	}else{
		sprintf(msg,"Restart data: NULL,\
 length: %d",mCheckPointLength);
		testMessage->write(STATUS_MESSAGE,msg);
	}

	while ( rv != TD_END_Method && rv < TD_Error )
	{
		
rv = mConnection->Restart(mCheckPointData,mCheckPointLength);
	
		rv = processStatusCode(rv);
	}

	if( rv < TD_Error){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Driver Restarted");

		if( mSource != NULL ){
			
fseek(mSource,mLastCheckpointPosition,SEEK_SET);
		}

		if( mBufferModeEnabled ){
			mLeftOverRowLen = 0;
			if( mLeftOverRow != NULL ){
				free(mLeftOverRow);
				mLeftOverRow = NULL;
			}
		}

		/* Query Row Counts */
		
queryStatus = mConnection->GetEvent(TD_Evt_RowCounts,
 &dataptr, &datalen);	

		/* If there is information available, process it */
		if ( queryStatus < TD_Error &&
 queryStatus != TD_Unavailable )
		{
			sprintf(msg,"TD_Evt_RowCounts\
 reports %d rows received",*((int*)dataptr));
			testMessage->write(STATUS_MESSAGE,msg);
			sprintf(msg,"TD_Evt_RowCounts\
 reports %d rows sent",*((int*)(dataptr+4)));
			testMessage->write(STATUS_MESSAGE,msg);
			sprintf(msg,"TD_Evt_RowCounts\
 reports %d rows applied",*((int*)(dataptr+8)));
			testMessage->write(STATUS_MESSAGE,msg);
		}else{
			sprintf(msg,"TD_Evt_RowCounts\
 Data Unavailable");
			testMessage->write(STATUS_MESSAGE,msg);
		}

		if( mOperType == TD_STREAM )
		{
			/* Query Run Stats */
			queryStatus =
 mConnection->GetEvent(TD_Evt_RunStats, &dataptr, &datalen, 1);	

			/* If there is information available, 
process it */
			if ( queryStatus < TD_Error &&
 queryStatus != TD_Unavailable )
			{
				
			  sprintf(msg,
"TD_Evt_RunStats reports %d SQL statements sent",*((int*)dataptr));
				
testMessage->write(STATUS_MESSAGE,msg);
				
sprintf(msg,"TD_Evt_RunStats reports\
 %d requests sent",*((int*)(dataptr+4)));
				
testMessage->write(STATUS_MESSAGE,msg);
				
sprintf(msg,"TD_Evt_RunStats reports %d rows\
 received",*((int*)(dataptr+8)));
				
testMessage->write(STATUS_MESSAGE,msg);
				
sprintf(msg,"TD_Evt_RunStats reports %d rows\
 sents",*((int*)(dataptr+12)));
				
testMessage->write(STATUS_MESSAGE,msg);
				
dbserror = *((int*)(dataptr+16));
				if( dbserror > 0 )
				{
					
sprintf(msg,"TD_Evt_RunStats reports %d rows \
caused DBS errors",*((int*)(dataptr+16)));
					
testMessage->write(STATUS_MESSAGE,msg);
				}
				else
				{
					
sprintf(msg,"TD_Evt_RunStats reports no rows caused DBS errors");
					
testMessage->write(STATUS_MESSAGE,msg);
				}
			}else{
				
sprintf(msg,"TD_Evt_RunStats Data Unavailable");
				
testMessage->write(STATUS_MESSAGE,msg);
			}
		}
	}

	free(msg);
	return rv;
}

int TestBase::failReached(int rowCount)
{
	vector<int>::iterator instanceIter = 
mParams->mInstanceFail.begin();
	vector<int>::iterator countIter = 
mParams->mRowCountFail.begin();
	vector<int>::iterator failIter = 
mParams->mDidFail.begin();

	for(; instanceIter!=
mParams->mInstanceFail.end(); instanceIter++, countIter++, failIter++)
	{
		if( *instanceIter == mInstanceID )
		{
			if( *countIter ==
 mRowsSentCount && *failIter != 1 && !mBufferModeEnabled){
				*failIter = 1;
				return 1;
			}else if ( *countIter == 
mBuffersSentCount && *failIter != 1 && mBufferModeEnabled){
				*failIter = 1;
				return 1;
			}
		}
	}

	return 0;
}

int TestBase::limitReached(int rowCount)
{
	vector<int>::iterator instanceIter = 
mParams->mInstanceLimit.begin();
	vector<int>::iterator countIter = 
mParams->mRowCountLimit.begin();

	for(; instanceIter!=mParams->mInstanceLimit.end();
 instanceIter++, countIter++)
	{
		if( *instanceIter == mInstanceID )
		{
			if( rowCount >= *countIter  ){
				return 1;
			}
		}
	}

	return 0;
}











