/*************************************************************************
*
* Copyright 2005-2008, 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/

#ifndef __MVS__
#include "TestExport.h"
#else
#include "PTCEXPTD.h"                      //DR144824
#endif//__MVS__

TestExport::TestExport(ThreadTestParameters* params, int instance,
 Synchronizer* sync, long start,
 long end, FILE* file): TestBase(params,instance,sync,start,end)
{
	mOutFile = file;
}

TestExport::~TestExport()
{}

void TestExport::printTestOptions()
{
	cout << "Multi-Threaded Export Test" << endl;  
             /* DR97368 */
}

int	TestExport::putRow(char* data, TD_Length dataLen)
{
	return mSync->writeOutRow(mOutFile,data,dataLen);
}

TD_StatusCode TestExport::run()
{
	TD_StatusCode rv = TD_Success;
	createConnection();
	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Acquisition
		**********************************************/

		rv = Acquisition();

	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,
"Export Successful");
	}

	testMessage->write(EXIT_MESSAGE,"Export Thread Exiting");

	return rv;
}

TD_StatusCode TestExport::runRestart()
{ 
	TD_StatusCode rv = TD_Success;

	if( mParams->mCreateNewConnection ){
		createConnection();
		mConnection->AddAttribute(TD_RESTARTMODE,1);
	}

	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Restart
		**********************************************/

		rv = Restart();

		if ( rv < TD_Error )
		{
			/*******************************
***************
			* Acquisition
			***********************************
***********/

		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Load Successful");
	}

	return rv;
}

TD_StatusCode TestExport::Acquisition()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = ACQUISITION_METHOD;

	/**********************************************
	* Acquisition
	**********************************************/ 	
	int count = 0;
	int barrier_count = 0;
	int last_row = 0;
	char* data;
	TD_Length dataLen = 0;

	while( rv != TD_END_Method && rv < TD_Error ){

		rv = mConnection->GetRow(&data, &dataLen);

		if ( rv >= TD_Error )
		{
			char msg[256];
			sprintf(msg,"GetRow failed on row %d",
count+1);
			mCurrentStatus = rv;
			testMessage->write(STATUS_MESSAGE,msg);

		}else if ( rv == TD_END_Method ){
			char msg[256];
			sprintf(msg,"Exported %d rows",count);
			testMessage->write(STATUS_MESSAGE,
"End of Data Reached");
			testMessage->write(STATUS_MESSAGE,msg);
			
		}else if (rv == TD_Success ){
			//Write Row
			putRow(data,dataLen);
			count++;
			mRowsSentCount++;

		}else if (rv == TD_SYNC_Barrier || rv == 
TD_SYNC_TELINFO ){
			//Write Row
			//putRow(data,dataLen);
			barrier_count++;
		}

		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,
"Acquisition complete");
	}

	return rv;
}

TD_StatusCode TestExport::Application()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = APPLICATION_METHOD;

	return rv;
}














