/**********************************************************
***************
*
* Copyright 2005-2008, 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
**************************************************************
***********/

#ifndef __MVS__                     //DR144824
  #include "TestLoad.h"             //DR144824
#else                               //DR144824
  #include "PTCLODTD.h"             //DR144824
#endif                              //DR144824

TestLoad::TestLoad(ThreadTestParameters* params, int instance, 
Synchronizer* sync, long start, long end): TestBase(params,
instance,sync,start,end)
{}

TestLoad::~TestLoad()
{}

void TestLoad::printTestOptions()
{
	cout << "Multi-Threaded Load Test" << endl;              
   /* DR97368 */
}

TD_StatusCode TestLoad::run()
{
	TD_StatusCode rv = TD_Success;

	createConnection();

	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Acquisition
		**********************************************/

		rv = Acquisition();

		if ( rv < TD_Error )
		{
			/********************************
**************
			* Application
			*************************************
*********/


			rv = Application();

		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Load Successful");
	}

	testMessage->write(EXIT_MESSAGE,"Load Thread Exited");

	return rv;
}

TD_StatusCode TestLoad::runRestart()
{ 
	TD_StatusCode rv = TD_Success;

	if( mParams->mCreateNewConnection ){
		createConnection();
		mConnection->AddAttribute(TD_RESTARTMODE,1);
	}

	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Restart
		**********************************************/

		rv = Restart();

		if ( rv < TD_Error )
		{
			/**********************************
************
			* Acquisition
			************************************
**********/

			/* DR99279 BEGIN */

			if(!mEndAcquisitionDone)
			{
				rv = Acquisition();
			}

			/* DR99279 END */

			if ( rv < TD_Error )
			{
				/****************************
******************
				* Application
				********************************
**************/

				rv = Application();

			}
		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,
"Load Successful");
	}

	return rv;
}

TD_StatusCode TestLoad::Acquisition()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = ACQUISITION_METHOD;
	char* msg = (char*)malloc(256);

	/**********************************************
	* Acquisition
	**********************************************/

	int readStatus = 0;
	char* rowBuffer = (char*)malloc(mRowSize);
	int lineLen = 0;
	int rowCount = 0;
	int nextCheckPoint = mCheckPointInterval;
	unsigned short lenRow;
	int checkpoint_count = 0;
	int rowNumber = 0;
	char* dataBuffer = NULL;
	TD_Length dataBufferLen = 0;
	int bufferRowCount = 0;
	int bufferCount = 0;

	//Load Rows
	bool doneLoading = false;

	while (!doneLoading){

		if( mBufferModeEnabled )
		{
			readStatus = getBuffer(&dataBuffer,
&dataBufferLen,&bufferRowCount);
		}else{
			readStatus = getRow(rowBuffer);
		}

		switch ( readStatus ){
			
			case READ_SUCCESS:

				if( mBufferModeEnabled )
				{
					
if( !limitReached(bufferCount) ){
						
rv = mConnection->PutBuffer(dataBuffer,dataBufferLen,1);

						
sprintf(msg,"Sent Buffer With Length: %d",dataBufferLen);
						
testMessage->write(STATUS_MESSAGE,msg);
						
sprintf(msg,"Sent buffer %d: %d rows sent",
bufferCount,bufferRowCount);
						
testMessage->write(STATUS_MESSAGE,msg);
						
mRowsSentCount += bufferRowCount;
						
mBuffersSentCount++;

						
if ( rv == TD_END_Method || rv == TD_SYNC_Barrier ||
 rv == TD_SYNC_TELINFO )
						{
							
sprintf(msg,"PutBuffer Returned Invalid Return Code: %d",rv);
							
testMessage->write(STATUS_MESSAGE,msg);
							
//doneLoading = true;
							
rv = TD_Error;
						}
					}else{
						
#ifndef __MVS__
sprintf(msg,"Skipped buffer %d: %d rows not sent : Limit Reached",
bufferCount,bufferRowCount);
#else
sprintf(msg,"Skipped buffer %d: %d rows not sent : Limit Reached",\
bufferCount,bufferRowCount);
#endif //__MVS__
						
testMessage->write(STATUS_MESSAGE,msg);
					}

					bufferCount++;
					rowCount += bufferRowCount;

					if( failReached(bufferCount) )
					{
						rv = TD_Error;
						mHaveFailed = 1;
					}

				}else{
#ifndef __MVS__
					
lenRow = *((unsigned short *)rowBuffer);
#else
					lenRow = 101;
#endif
					
memcpy(&rowNumber, rowBuffer+3, sizeof(int));   /* DR124305 */

					if( !limitReached(rowCount) ){
						
rv = mConnection->PutRow( rowBuffer+2, lenRow);
						
sprintf(msg,"Sent row %d: %d",rowCount,rowNumber);
						
testMessage->write(STATUS_MESSAGE,msg);
						mRowsSentCount++;
					}else{
						
sprintf(msg,"Skipped row %d: %d Limit Reached",rowCount,rowNumber);
						
testMessage->write(STATUS_MESSAGE,msg);
					}

					rowCount++; 
//always increment so we do checkpoint on time
					
					if( failReached(rowCount) )
					{
						rv = TD_Error;
						mHaveFailed = 1;
					}
				}
				break;

			case READ_DONE:
				doneLoading = true;
				break;

			default:
				doneLoading = true;
				rv = TD_Error;
		}

		/**********************************************
		* CheckPoint
		**********************************************/

		if( rv >= TD_Error){
			doneLoading = true;

		}else if( ( rowCount == nextCheckPoint &&
 !mBufferModeEnabled ) || 
			( bufferCount == nextCheckPoint &&
 mBufferModeEnabled) ){
	
			rv = Checkpoint();

			if( rv >= TD_Error ){
				doneLoading = true;
			}else{
				nextCheckPoint += mCheckPointInterval;
			}

		}
		
	}

	free(rowBuffer);

	if( rv >= TD_Error )
	{
		rv = processStatusCode(rv);
	}else{
		rv = TD_Success;
		char msgBuffer[256];
		sprintf(msgBuffer,"Sent %d Rows",rowCount);
		mCurrentStatus = rv;
		mCurrentMethod = END_ACQUISITION_METHOD;
		testMessage->write(STATUS_MESSAGE,msgBuffer);

		/**********************************************
		* End Acquisition
		**********************************************/
		while ( rv != TD_END_Method && rv < TD_Error )
		{
			rv = mConnection->EndAcquisition();
		
			rv = processStatusCode(rv);
		}

		if ( rv < TD_Error){
			mCurrentStatus = rv;
			mEndAcquisitionDone = true;   
                      /* DR99279 */
			
testMessage->write(STATUS_MESSAGE,"Acquisition complete");
		}

	}

	free(msg);
	return rv;
}

TD_StatusCode TestLoad::Application()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = APPLICATION_METHOD;

	/**********************************************
	* Application
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{
		rv = mConnection->ApplyRows();

		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE
,"Application complete");
	}

	return rv;
}





