/******************************************************
*******************
*
* Copyright 2005-2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************
************/

#ifndef __MVS__                   //DR144824
#include "TestSim.h"              //DR144824
#else                             //DR144824
#include "PTCSIMTD.h"             //DR144824
#endif                            //DR144824


TestSim::TestSim(ThreadTestParameters* params, int instance,
Synchronizer* sync, long start, long end): TestBase(params,
instance,sync,start,end) {}

TestSim::~TestSim()
{}


void TestSim::printTestOptions()
{
	cout << "Multi-Threaded Simulation Test" << endl;    
                   /* DR97368 */
}

TD_StatusCode TestSim::run()
{
	TD_StatusCode rv = TD_Success;

	//mConnection->AddAttribute(TD_SYSTEM_OPERATOR,TD_LOAD);
	//mConnection->AddArrayAttribute(TD_TRACE_LEVEL,2,
	//TD_OPER,TD_OFF,NULL);

	switch( mInstanceID ){
		
		case 1:
			mWait = 10000000;
			break;
		case 2:
			mWait = 0;
			break;
		default:
			break;

	}

	/**********************************************
	* Initiate
	**********************************************/

	mPhase = 0;
	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Acquisition
		**********************************************/
		mPhase = 0;
		rv = Acquisition();

		switch( mInstanceID ){
		
			case 1:
				mWait = 0;
				break;
			case 2:
				mWait = 1000000;
				break;
			default:
				mWait = 0;
				break;

		}

		if ( rv < TD_Error )
		{
			/*************************
*********************
			* Application
			****************************
******************/
			mPhase = 0;
			rv = Application();

		}
	}

	/**********************************************
	* Terminate
	**********************************************/
	mPhase = 0;

   	switch( mInstanceID ){
                        
            	case 1:
                   	mWait = 0;
                    	break;
            	case 2:
                    	mWait = 1000000;
                     	break;
 		default:
     			mWait = 0;
     			break;
 
   	}

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Simulation Ended");
	}

	testMessage->write(EXIT_MESSAGE,"Thread Exiting");

	return rv;
}

TD_StatusCode TestSim::runRestart()
{ 
	TD_StatusCode rv = TD_Success;

	/**********************************************
	* Initiate
	**********************************************/

	mPhase = 0;
	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Restart
		**********************************************/

		mPhase = 0;
		rv = Restart();

		if ( rv < TD_Error )
		{
			/*******************************
***************
			* Acquisition
			********************************
**************/

			mPhase = 0;
			rv = Acquisition();

			if ( rv < TD_Error )
			{
				/***********************
***********************
				* Application
				************************
**********************/

				mPhase = 0;
				rv = Application();

			}
		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	mPhase = 0;
	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Load Successful");
	}

	return rv;
}

TD_StatusCode TestSim::Initiate()
{
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = INITIATE_METHOD;

	while ( rv != TD_END_Method && rv < TD_Error )
	{
		for(int i = 0; i < 2 ;){
			testMessage->write(STATUS_MESSAGE,
"Message Check");
			i++;
		}

		switch ( mPhase ){

			case 0:
				rv = TD_Error;
				//rv = TD_SYNC_Barrier;
				break;

			case 1:
				if ( mInstanceID == 1 )
					rv = TD_SYNC_TELINFO;
				else
					rv = TD_SYNC_Barrier;
				break;

			default:
				mCurrentStatus = rv; 
				testMessage->write(STATUS_MESSAGE,
"Driver Initiated");
				rv = TD_END_Method;
				break;
		}

		mPhase++;

		rv = processStatusCode(rv);

	}

	return rv;
}

TD_StatusCode TestSim::Acquisition()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = ACQUISITION_METHOD;

	/**********************************************
	* Acquisition
	**********************************************/

	int readStatus = 0;
	char* rowBuffer = (char*)malloc(mRowSize);
	int lineLen = 0;
	int rowCount = 0;
	int nextCheckPoint = mCheckPointInterval;
	unsigned short lenRow;

	//Load Rows
	bool doneLoading = false;

	while (!doneLoading){

		//readStatus = getRow(rowBuffer);

		if ( rowCount == 1 ){
			readStatus = READ_DONE;
		}else{
			readStatus = READ_SUCCESS;
		}

		switch ( readStatus ){
			
			case READ_SUCCESS:
				lenRow =
 *((unsigned short *)rowBuffer); 

				//rv =
 mConnection->PutRow( rowBuffer+2, lenRow);
				rv = TD_Success;
				
//if( mInstanceID == 2 && rowCount == 34 ){
				//		rv = TD_Error;
				//}

				rowCount++;
				break;

			case READ_DONE:
				doneLoading = true;
				break;

			default:
				doneLoading = true;
				rv = TD_Error;
		}

		/**********************************************
		* CheckPoint
		**********************************************/

		if( rv >= TD_Error){
			doneLoading = true;

		}else if( rowCount == nextCheckPoint ){
	
			rv = Checkpoint();

			if( rv >= TD_Error ){
				doneLoading = true;
			}else{
				nextCheckPoint += mCheckPointInterval;
			}

		}
	}

	free(rowBuffer);

	if( rv >= TD_Error )
	{
		rv = processStatusCode(rv);
	}else{
		char msgBuffer[256];
		sprintf(msgBuffer,"Sent %d Rows",rowCount);
		mCurrentStatus = rv;
		mCurrentMethod = END_ACQUISITION_METHOD;
		testMessage->write(STATUS_MESSAGE,msgBuffer);

		rv = TD_Success;
		mPhase = 0;
		/**********************************************
		* End Acquisition
		**********************************************/
		while ( rv != TD_END_Method && rv < TD_Error )
		{
			for(int i = 0; i < mWait;){
				i++;
			}

			switch ( mPhase ){

				case 0:
					rv = TD_SYNC_Barrier;
					break;

				case 1:
					if ( mInstanceID == 1 )
						rv = TD_SYNC_TELINFO;
					else
						rv = TD_END_Method;
					break;

				default:
					rv = TD_END_Method;
					break;
			}

			mPhase++;

			rv = processStatusCode(rv);
		}

		if ( rv < TD_Error){
			mCurrentStatus = rv;
			
testMessage->write(STATUS_MESSAGE,"Acquisition complete");
		}

	}

	return rv;
}

TD_StatusCode TestSim::Application()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = APPLICATION_METHOD;

	/**********************************************
	* Application
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{
		for(int i = 0; i < mWait;){
			i++;
		}

		switch ( mPhase ){

			case 0:
				rv = TD_SYNC_Barrier;
				break;

			case 1:
				if ( mInstanceID == 1 )
					rv = TD_SYNC_TELINFO;
				else
					rv = TD_SYNC_Barrier;
				break;

			default:
				rv = TD_END_Method;
				break;
		}

		mPhase++;
	
		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE
,"Application complete");
	}

	return rv;
}

TD_StatusCode TestSim::Terminate()
{
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = TERMINATE_METHOD;

	/**********************************************
	* Terminate
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{
		for(int i = 0; i < mWait;){
			i++;
		}

		switch ( mPhase ){

			case 0:
				rv = TD_SYNC_Barrier;
				break;

			case 1:
				if ( mInstanceID == 1 )
					rv = TD_SYNC_Barrier;
				else
					rv = TD_END_Method;
				break;

			default:
				rv = TD_END_Method;
				break;
		}

		mPhase++;
	
		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		
testMessage->write(STATUS_MESSAGE,"Driver Terminated");

		if( mRunStatus == RESTART ){
			mRunStatus = NORMAL;
			rv = runRestart();
		}

	}

	return rv;
}

TD_StatusCode TestSim::Checkpoint()
{
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = CHECKPOINT_METHOD;
	int tPhase = 0;

	mCurrentStatus = rv;
	testMessage->write(STATUS_MESSAGE,"Checkpoint Begin");

	/**********************************************
	* Application
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{
		for(int i = 0; i < mWait;){
			i++;
		}

		switch ( tPhase ){

			case 0:
				rv = TD_SYNC_Barrier;
				break;

			case 1:
				rv = TD_SYNC_TELINFO;
				break;

			case 2:
				rv = TD_SYNC_TELINFO;
				break;

			case 3:
				rv = TD_SYNC_Barrier;
				break;

			default:
				rv = TD_END_Method;
				break;
		}

		tPhase++;
	
		rv = processStatusCode(rv);
	}

	if( rv == TD_END_Method ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE
,"Checkpoint complete");

		mLastCheckpointPosition = ftell(mSource);
	}

	return rv;

}

TD_StatusCode TestSim::Restart()
{
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = RESTART_METHOD;

	/**********************************************
	* Application
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{

		for(int i = 0; i < mWait;){
			i++;
		}

		switch ( mPhase ){

			case 0:
				rv = TD_SYNC_Barrier;
				break;

			case 1:
				rv = TD_SYNC_TELINFO;
				break;

			default:
				rv = TD_END_Method;
				break;
		}

		mPhase++;
	
		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Restart complete");

		if( mSource != NULL ){
			fseek(mSource,mLastCheckpointPosition,
SEEK_SET);
		}
	}

	return rv;
}









