/******************************************************
*******************
*
* Copyright 2005-2008, 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
************************************************************
*************/


#ifndef __MVS__
#include "TestStream.h"
#else
#include "PTCSTRTD.h"                //DR144824
#endif//__MVS__

TestStream::TestStream(ThreadTestParameters* params, int instance,
 Synchronizer* sync, long start, long end): TestBase(params,
instance,sync,start,end)
{}

TestStream::~TestStream()
{}

void TestStream::printTestOptions()
{
	cout << "Multi-Threaded Stream Test" << endl;              
 /* DR97368 */
}

TD_StatusCode TestStream::run()
{
	TD_StatusCode rv = TD_Success;
	createConnection();
	mConnection->AddAttribute(TD_ROBUST,"No");
	rv = mConnection->UseDMLGroups(&mIndexes[0],1);


	/**********************************************
	* Initiate
	**********************************************/

	testMessage->write(STATUS_MESSAGE,"Entering Initiate");

	rv = Initiate();

	testMessage->write(STATUS_MESSAGE,"Leaving Initiate");

	if ( rv < TD_Error )
	{
		/**********************************************
		* Acquisition
		**********************************************/

		rv = Acquisition();
	}

	/**********************************************
	* Terminate
	**********************************************/
	testMessage->write(STATUS_MESSAGE,"Entering Terminate");

	rv = Terminate();

	testMessage->write(STATUS_MESSAGE,"Leaving Terminate");

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Load Successful");
	}

	testMessage->write(EXIT_MESSAGE,"Stream Thread Exitied");

	return rv;
}

TD_StatusCode TestStream::runRestart()
{ 
	TD_StatusCode rv = TD_Success;

	if( mParams->mCreateNewConnection ){
		createConnection();
		mConnection->AddAttribute(TD_RESTARTMODE,1);
		rv = mConnection->UseDMLGroups(&mIndexes[0],1);
	}



	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Restart
		**********************************************/

		rv = Restart();

		if ( rv < TD_Error )
		{
			/*******************************
***************
			* Acquisition
			********************************
**************/

			rv = Acquisition();

			if ( rv < TD_Error )
			{
				/************************
**********************
				* Application
				*************************
*********************/

				//rv = Application();

			}
		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,"Load Successful");
	}

	return rv;
}

TD_StatusCode TestStream::Acquisition()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = ACQUISITION_METHOD;
	char* msg = (char*)malloc(256);

	/**********************************************
	* Acquisition
	**********************************************/

	int readStatus = 0;
	char* rowBuffer = (char*)malloc(mRowSize);
	int lineLen = 0;
	int rowCount = 0;
	int nextCheckPoint = mCheckPointInterval;
	unsigned short lenRow;
	int checkpoint_count = 0;
	int rowNumber = 0;
	char* dataBuffer = NULL;
	TD_Length dataBufferLen = 0;
	int bufferRowCount = 0;
	int bufferCount = 0;

	//Load Rows
	bool doneLoading = false;

	while (!doneLoading){

		if( mBufferModeEnabled )
		{
			
readStatus = getBuffer(&dataBuffer,&dataBufferLen,&bufferRowCount);
		}else{
			readStatus = getRow(rowBuffer);
		}

		switch ( readStatus ){
			
			case READ_SUCCESS:
				if( mBufferModeEnabled )
				{
					
if( !limitReached(bufferCount) ){
						
rv = mConnection->PutBuffer(dataBuffer,dataBufferLen,1);

						
sprintf(msg,"Sent Buffer With Length: %d",dataBufferLen);
						
testMessage->write(STATUS_MESSAGE,msg);
						
sprintf(msg,"Sent buffer %d: %d rows sent",
bufferCount,bufferRowCount);
						
testMessage->write(STATUS_MESSAGE,msg);
						
mRowsSentCount += bufferRowCount;
						
mBuffersSentCount++;

						
if ( rv == TD_END_Method || rv == TD_SYNC_Barrier 
|| rv == TD_SYNC_TELINFO )
						{
							
sprintf(msg,"PutBuffer Returned Invalid Return Code: %d",rv);
							
testMessage->write(STATUS_MESSAGE,msg);
							
//doneLoading = true;
							
rv = TD_Error;
						}
					}
else{
						
sprintf(msg,"Skipped buffer %d: %d rows not sent\
 : Limit Reached",bufferCount,bufferRowCount);
						
testMessage->write(STATUS_MESSAGE,msg);
					
}

					bufferCount++;
					rowCount += bufferRowCount;

					if( failReached(bufferCount) )
					{
						rv = TD_Error;
						mHaveFailed = 1;
					}

				}
else{
#ifndef __MVS__
					
lenRow = *((unsigned short *)rowBuffer);
#else
					
lenRow = 101;
#endif
					
memcpy(&rowNumber, rowBuffer+3, sizeof(int));   /* DR124305 */

					
if( !limitReached(rowCount) ){
						
rv = mConnection->PutRow( rowBuffer+2, lenRow);
						
sprintf(msg,"Sent row %d: %d",rowCount,rowNumber);
						
testMessage->write(STATUS_MESSAGE,msg);
						
mRowsSentCount++;
					}else{
						
sprintf(msg,"Skipped row %d: %d Limit Reached",rowCount,rowNumber);
						
testMessage->write(STATUS_MESSAGE,msg);
					}

					
rowCount++; //always increment so we do checkpoint on time
					
					
if( failReached(rowCount) )
					{
						rv = TD_Error;
						mHaveFailed = 1;
					}
				}
				break;

			case READ_DONE:
				doneLoading = true;
				break;

			default:
				doneLoading = true;
				rv = TD_Error;
		}

		/**********************************************
		* CheckPoint
		**********************************************/

		if( rv >= TD_Error){
			doneLoading = true;

		}else if( ( rowCount == nextCheckPoint &&
 !mBufferModeEnabled ) || 
			( bufferCount == nextCheckPoint &&
 mBufferModeEnabled) ){
	
			rv = Checkpoint();

			if( rv >= TD_Error ){
				doneLoading = true;
			}else{
				nextCheckPoint += mCheckPointInterval;
			}

		}
		
	}

	free(rowBuffer);

	if( rv >= TD_Error )
	{
		rv = processStatusCode(rv);
	}else{
		rv = TD_Success;
		char msgBuffer[256];
		sprintf(msgBuffer,"Sent %d Rows",rowCount);
		mCurrentStatus = rv;
		mCurrentMethod = END_ACQUISITION_METHOD;
		testMessage->write(STATUS_MESSAGE,msgBuffer);

		/**********************************************
		* End Acquisition
		**********************************************/
		while ( rv != TD_END_Method && rv < TD_Error )
		{
			rv = mConnection->EndAcquisition();
		
			rv = processStatusCode(rv);
		}

		if ( rv < TD_Error){
			mCurrentStatus = rv;
			testMessage->write(STATUS_MESSAGE,
"Acquisition complete");
		}

	}

	free(msg);
	return rv;
}

TD_StatusCode TestStream::Application()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = APPLICATION_METHOD;

	/**********************************************
	* Application
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{
		rv = mConnection->ApplyRows();
	
		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE
,"Application complete");
	}

	return rv;
}












