/***********************************************************
**************
*
* Copyright 2005-2008, 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************
************/


#ifndef __MVS__
#include "TestUpdate.h"
#else
#include "PTCUPDTD.h"                //DR144824
#endif//__MVS__

TestUpdate::TestUpdate(ThreadTestParameters* params, int instance,
 Synchronizer* sync, long start, long end): TestBase(params,instance,
sync,start,end)
{}

TestUpdate::~TestUpdate()
{}

void TestUpdate::printTestOptions()
{
	cout << "Multi-Threaded Update Test" << endl;              
 /* DR97368 */
}

TD_StatusCode TestUpdate::run()
{
	TD_StatusCode rv = TD_Success;
	createConnection();

	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Acquisition
		**********************************************/

		rv = Acquisition();

		if ( rv < TD_Error )
		{
			/**************************************
********
			* Application
			**************************************
********/

			rv = Application();

		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		
testMessage->write(STATUS_MESSAGE,"Update Successful");
	}

	testMessage->write(EXIT_MESSAGE,"Update Thread Exited");

	return rv;
}

TD_StatusCode TestUpdate::runRestart()
{ 
	TD_StatusCode rv = TD_Success;

	if( mParams->mCreateNewConnection ){
		createConnection();
		mConnection->AddAttribute(TD_RESTARTMODE,1);
	}

	/**********************************************
	* Initiate
	**********************************************/

	rv = Initiate();

	if ( rv < TD_Error )
	{
		/**********************************************
		* Restart
		**********************************************/

		rv = Restart();

		if ( rv < TD_Error )
		{
			/***********************************
***********
			* Acquisition

			**************************************
********/

			/* DR99279 BEGIN */

			if(!mEndAcquisitionDone)
			{
				rv = Acquisition();
			}

			/* DR99279 END */

			if ( rv < TD_Error )
			{
				/******************************
****************
				* Application
				********************************
**************/

				rv = Application();

			}
		}
	}

	/**********************************************
	* Terminate
	**********************************************/

	rv = Terminate();

	if ( rv < TD_Error )
	{
		mCurrentStatus = rv;
		
testMessage->write(STATUS_MESSAGE,"Update Successful");
	}

	return rv;
}

TD_StatusCode TestUpdate::Acquisition()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = ACQUISITION_METHOD;
	char* msg = (char*)malloc(256);
	int checkpoint_count = 0;
	int rowNumber = 0;
	char* dataBuffer = NULL;
	TD_Length dataBufferLen = 0;
	int bufferRowCount = 0;
	int bufferCount = 0;

	/**********************************************
	* Acquisition
	**********************************************/

	int readStatus = 0;
	char* rowBuffer = (char*)malloc(mRowSize);
	int lineLen = 0;
	int rowCount = 0;
	int nextCheckPoint = mCheckPointInterval;
	int nextIndex = 1;
	unsigned short lenRow;

	rv = mConnection->UseDMLGroups(&mIndexes[0],1);

	//Load Rows
	bool doneLoading = false;

	while (!doneLoading){

		if( mBufferModeEnabled )
		{
			readStatus =
 getBuffer(&dataBuffer,&dataBufferLen,&bufferRowCount);
		}else{
			readStatus =
 getRow(rowBuffer);
		}

		switch ( readStatus ){
			
			case READ_SUCCESS:

				if( mBufferModeEnabled )
				{
					
if( !limitReached(bufferCount) ){
						
rv = mConnection->PutBuffer(dataBuffer,dataBufferLen,1);

						
sprintf(msg,"Sent Buffer With Length: %d",dataBufferLen);
						
testMessage->write(STATUS_MESSAGE,msg);
						
sprintf(msg,"Sent buffer %d: %d rows sent",bufferCount,
bufferRowCount);
						
testMessage->write(STATUS_MESSAGE,msg);
						
mRowsSentCount += bufferRowCount;
						
mBuffersSentCount++;

						
if ( rv == TD_END_Method || rv == TD_SYNC_Barrier 
|| rv == TD_SYNC_TELINFO )
						{
							
sprintf(msg,"PutBuffer Returned Invalid Return Code: %d",rv);
							
testMessage->write(STATUS_MESSAGE,msg);
							
//doneLoading = true;
							
rv = TD_Error;
						}
					}else{
						
sprintf(msg,"Skipped buffer %d: %d rows not sent :\
 Limit Reached",bufferCount,bufferRowCount);
						
testMessage->write(STATUS_MESSAGE,msg);
					}

					bufferCount++;
					rowCount += bufferRowCount;

					if( failReached(bufferCount) )
					{
						rv = TD_Error;
						mHaveFailed = 1;
					}

				}else{
#ifndef __MVS__
					
lenRow = *((unsigned short *)rowBuffer);
#else
					lenRow = 101;
#endif
					
memcpy(&rowNumber, rowBuffer+3, sizeof(int));   /* DR124305 */

					
if( !limitReached(rowCount) ){
						
rv = mConnection->PutRow( rowBuffer+2, lenRow);
						
sprintf(msg,"Sent row %d: %d",rowCount,rowNumber);
						
testMessage->write(STATUS_MESSAGE,msg);
						
mRowsSentCount++;
					}else{
						
sprintf(msg,"Skipped row %d: %d Limit Reached",rowCount,rowNumber);
						
testMessage->write(STATUS_MESSAGE,msg);
					}

					
rowCount++; //always increment so we do checkpoint on time
					
					if( failReached(rowCount) )
					{
						rv = TD_Error;
						mHaveFailed = 1;
					}
				}
				break;

			case READ_DONE:
				doneLoading = true;
				break;

			default:
				doneLoading = true;
				rv = TD_Error;
		}

		/**********************************************
		* CheckPoint
		**********************************************/

		if( rv >= TD_Error){
			doneLoading = true;

		}else if( ( rowCount == nextCheckPoint &&
 !mBufferModeEnabled ) || 
			( bufferCount == nextCheckPoint &&
 mBufferModeEnabled) ){
	
			rv = Checkpoint();

			if( rv >= TD_Error ){
				doneLoading = true;
			}else{
				nextCheckPoint += mCheckPointInterval;
			}

		}

		/**********************************************
		* Switch Target Tables
		*	- Switch target table after every putrow
		**********************************************/
		
rv = mConnection->UseDMLGroups(&mIndexes[nextIndex],1);
		nextIndex++;
		if( nextIndex > 1 )
			nextIndex = 0;
	}

	free(rowBuffer);

	if( rv >= TD_Error )
	{
		rv = processStatusCode(rv);
	}else{
		rv = TD_Success;
		char msgBuffer[256];
		sprintf(msgBuffer,"Sent %d Rows",rowCount);
		mCurrentStatus = rv;
		mCurrentMethod = END_ACQUISITION_METHOD;
		testMessage->write(STATUS_MESSAGE,msgBuffer);

		/**********************************************
		* End Acquisition
		**********************************************/
		while ( rv != TD_END_Method && rv < TD_Error )
		{
			rv = mConnection->EndAcquisition();

			rv = processStatusCode(rv);
		}

		if ( rv < TD_Error){
			mCurrentStatus = rv;
			mEndAcquisitionDone = true;        
                 /* DR99279 */
			testMessage->write(STATUS_MESSAGE
,"Acquisition complete");
		}

	}

	free(msg);
	return rv;
}

TD_StatusCode TestUpdate::Application()
{ 
	TD_StatusCode rv = TD_Success;
	mCurrentMethod = APPLICATION_METHOD;

	/**********************************************
	* Application
	**********************************************/
	while ( rv != TD_END_Method && rv < TD_Error )
	{
		rv = mConnection->ApplyRows();

		rv = processStatusCode(rv);
	}

	if( rv < TD_Error ){
		mCurrentStatus = rv;
		testMessage->write(STATUS_MESSAGE,
"Application complete");
	}

	return rv;
}





