/*************************************************************************
*
* Copyright 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* History Information:
*
* Revision     Date      DR      DID       Comments
* ---------------------------------------------------------------------
* 13.00.00.01  06182010  142720  mv121803  Dynamically load TELAPI
*
*************************************************************************/

#ifndef TESTBASE_H
#define TESTBASE_H

#ifndef __MVS__
#include "common.h"
#include "ThreadTestParameters.h"
#include "PerfTestMessage.h"
#include "Synchronizer.h"
#else
#include "COMMOND.H"                    //DR144824
#include "PTCTHTPD.h"                   //DR144824
#include "PTCPERTD.h"                   //DR144824
#include "PTCTSYND.h"                   //DR144824
#endif //__MVS__
 
class TestBase {

	public:
		
		TestBase(ThreadTestParameters*,int,Synchronizer*, long, long);
		~TestBase();

	public:

		// Wait Functions
		TD_StatusCode	waitForManager();

		// Test Message Functions
		//HANDLE	getMessageHandle();
		int		getMessageType() { return testMessage->getMessageType();}
		string	getMessage() { return testMessage->read();}
		bool	hasMessage() { return testMessage->hasMessage();}
		void	nextMessage() { testMessage->skipRead(); }

		// TELINFO functions
		TD_StatusCode	getTELINFO(char** telinfo, TD_Length* length);
		TD_StatusCode	setTELINFO(char* telinfo, TD_Length length);

		// Row Related Functions
				int		getRow(char*);
		virtual int		putRow(char*,TD_Length);
				int		getBuffer(char**,TD_Length*,int*);
		
		// Error Handling Functions
		void	signalError(TD_StatusCode status);
		void	setErrorInfo(int eCode);
		void	setErrorCode(int eCode);
		void	setErrorType(int eType);
		void	setErrorMsg(const char* eMsg);

		int			getErrorCode(){ return mErrorCode; };
		int			getErrorType(){ return mErrorType; };
		const char*	getErrorMsg(){ return mErrorMsg; };

		//Status functions
		void			setRunStatus(int status);
		int				getRunStatus();
		int				getCurrentMethod();
		TD_StatusCode	getCurrentStatus();

		//Instance functions
		void	setInstanceID(int id);
		int		getInstanceID();

		//Function to process status code
		TD_StatusCode	processStatusCode(TD_StatusCode status);

		// Virtual Functions
		virtual void printTestOptions();
		virtual TD_StatusCode run();
		virtual TD_StatusCode runRestart();

		virtual TD_StatusCode Initiate();
		virtual TD_StatusCode Acquisition();
		virtual TD_StatusCode Application();
		virtual TD_StatusCode Terminate();
		virtual TD_StatusCode Checkpoint();
		virtual TD_StatusCode Restart();

		//exta stuff
		void createConnection();
		int failReached(int rowCount);
		int limitReached(int rowCount);

	protected:

		ConnectionX*          mConnection;         //DR142720
		int	              mRowsSentCount;
		int	              mCheckpointsTaken;
		ThreadTestParameters* mParams;

		// Synchronization variables
		Synchronizer* 		mSync;
		int			mActiveHandle;

		// Status Variables
		PerfTestMessage*	testMessage;
		TD_StatusCode		mCurrentStatus;
		int			mRunStatus;
		int    		        mCurrentMethod;
		int			mCurrentMessageType;

		// Instance Variables
		int	    mInstanceID;

		// End Acquisition flag needed for restart purposes
		bool        mEndAcquisitionDone;                  /* DR99279 */

		// DML Group Indexes
		TD_Index	mIndexes[MAX_INDEXES];

		// Checkpoint Data
		char*		mCheckPointData;
		TD_Length	mCheckPointLength;
		int		mCheckPointInterval;

		// Source File Variables
		FILE*		mSource;
		long		mStart;
		long		mEnd;
		long		mLastCheckpointPosition;
		size_t		mRowSize;
		int		mSkipRow;

		// Error Variables
		int		mErrorCode;
		int		mErrorType;
		char*		mErrorMsg;

		// Debug Phase File Variables
		FILE*           mPhaseFile;
		int		mHaveFailed;
		TD_OperatorType	mOperType;

		//Buffer Mode Variables
		BufferLayout mBufferLayout;
		int		mBufferModeEnabled;
		int		mBufferRowCountLimit;
		char*		mBuffer;
		TD_Length       mBufferLen;
		char*           mLeftOverRow;
		TD_Length       mLeftOverRowLen;
		int		mBuffersSentCount;
		


};

#endif
