/* 
* Copyright 2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
 *
 * This document, and the information contained herein, are the exclusive
 * property of Teradata. In no case shall this document or its contents be
 * reproduced or copied by any means, in whole or in part, or disseminated
 * outside of the company, without prior written permission of a company
 * officer.
 *
 * History Information:
 *
 * Revision     Date      DR      DID       Comments
 * ---------------------------------------------------------------------
 * 15.00.00.00  12262013 TPT20211 kb186006  Copyright year change
 * 14.00.00.01  10302010  144824  mv121803  Support MVS
 * 13.00.00.01  06182010  142720  mv121803  Dynamically load TELAPI
 *
 */

#ifndef COMMON_H
#define COMMON_H

/**@#-*/
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
/**@#+*/

#ifdef __MVS__
#define _OPEN_THREADS
#include <sys/types.h>
#define _XOPEN_SOURCE_EXTENDED 1
#endif //__ MVS__

#include <pthread.h>
//#include <windows.h>
#include <stdio.h>
#include <time.h>
#include <sys/timeb.h>  // for timing functions
//#include <sql.h>        // ODBC include files
//#include <sqlext.h>

/* DR99279 BEGIN */

#if defined (__linux__)
#include <string>
#else
#include <string.h>
#endif

/* DR99279 END */

#include <vector>
#include <list>
//#include <string>
#include <iostream>
#include <fstream>

#ifndef __MVS__                                 //DR144824
  #include "connection.h"
  #include "connectionX.h"                      //DR142720
#else                                           //DR144824
  #include "CONNECTI.H"                         //DR144824
  #include "CONNECTX.H"                         //DR144824
#endif                                          //DR144824

//#include "TelapiException.h"

/**@#-*/
using namespace std;
using namespace teradata::client::API;
/**@#+*/

#define FALSE 0
#define TRUE 1

#define MAX_SQL_ERR_BUF 512
#define MAX_MSG_BUF 1024*64
#define MAX_SQL_CONNECT_STR_BUF 1024*4
#define MAX_SQL_COLUMN_TYPE_BUF MAX_MSG_BUF
#define MAX_SQL_COLUMN_NAME_BUF 128
#define MAX_SQL_INSERT_STMT_BUF MAX_MSG_BUF
#define MAX_PERF_TEST_THREADS 2028
#define MAX_DATE_STR_LEN 10
#define MAX_GENERATED_DATA_COUNT 16

// Thread Message Types
#define STATUS_MESSAGE 0
#define SYNC_BARRIER 1
#define SYNC_TELINFO 2
#define ERROR_MESSAGE 3
#define EXIT_MESSAGE 4
#define ID_MESSAGE 5

// GetRow Return Types
#define READ_SUCCESS 0
#define READ_DONE	 1
#define READ_ERROR	 -1

// Test Types
#define	NO_TEST	0
#define LOAD_TEST 1
#define EXPORT_TEST 2
#define UPDATE_TEST 3
#define	STREAM_TEST 4
#define SIM_TEST 5

// Run Status
#define NORMAL 0
#define ABORT 1
#define RESTART 2
#define WAIT_AGAIN 3

// DML Index Variables
#define MAX_INDEXES 30

// Methods
#define	PRE_INITIATE_METHOD 0
#define INITIATE_METHOD 1
#define ACQUISITION_METHOD 2
#define END_ACQUISITION_METHOD 3
#define APPLICATION_METHOD 4
#define TERMINATE_METHOD 5
#define CHECKPOINT_METHOD 6
#define RESTART_METHOD 7

#ifdef WIN32
#define TIMECALC(d, s, m) ( d.m=*(_int64*)&s.m/1000000.00000 )
#else
#define TIMECALC(d, s, m) ( d.m=(double)s.m.tv_sec + (double)s.m.tv_usec/1000000.00000 )
#endif

typedef struct {
   int MaxMsgSize;
   int RowHeaderSize;
   int RowLengthSize;
   int BufTrailerSize;
} BufferLayout;

#ifndef __MVS__
/**
 * Defines the starting values for standard clock time, CPU user time, and CPU kernel time.
 */
struct mytimes_prev {
    struct timeval stck;
    struct timeval cpuu;
    struct timeval cpuk;
};

/**
 * Defines the measured values of standard clock time, CPU user time, and CPU kernel time.
 */
struct mytimes_tot {
    struct timeval stck;
    struct timeval cpuu;
    struct timeval cpuk;
};

/**
 * Defines an accumulation of standard clock time, CPU user time, CPU kernel time, and CPU user & kernel time combined.
 */
struct mytimes_totd {
    double stck;
    double cpuu;
    double cpuk;
    double cput;
};

/**
 * Measures current standard clock and CPU times, then adds them to any previous measurement as the running total.
 * @param struct mytimes_prev * -- any previous measurement
 * @param struct mytimes_tot * -- the running total of taken measurements
 */
extern void cputime(struct mytimes_prev *, struct mytimes_tot *);
#endif //__MVS__

#endif // COMMON_H
