/*************************************************************************
*
* Copyright 2005-2009, 2014 by Teradata Corporation. 
* All Rights Reserved.
* TERADATA CONFIDENTIAL TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/

/*******************************************************************************
   This test prints the outputs of all the events available to TPTAPI, before 
   and after each phase of the application for each operator/driver.

   1. operator: LOAD
      Phases  : Initiate, PutRow, EndAcquisition, ApplyRows, Terminate
      Events  : TD_Evt_ConnectStatus,
                TD_Evt_DBSError,
                TD_Evt_CLIError,

                TD_Evt_BufferLayout,
                TD_Evt_RowCounts,
                TD_Evt_BlockCount,
                TD_Evt_ApplyCount,
                TD_Evt_ExportCount,
                TD_Evt_ErrorTable1,
                TD_Evt_ErrorTable2,
                TD_Evt_RowsCheckpointed,
                TD_Evt_BufferModeSupport,
                TD_Evt_MacroNames,
                TD_Evt_PackFactor,

                TD_Evt_CPUTime,
                TD_Evt_LastError,
                TD_Evt_ExitCode,
                TD_Evt_RunStats,
                TD_Evt_UserCommand,
                TD_Evt_Version 

   2. operator: EXPORT
      Phases  : Initiate, GetRow, Terminate
      Events  : TD_Evt_ConnectStatus,
                TD_Evt_DBSError,
                TD_Evt_CLIError,

                TD_Evt_BufferLayout,
                TD_Evt_RowCounts,
                TD_Evt_BlockCount,
                TD_Evt_ApplyCount,
                TD_Evt_ExportCount,
                TD_Evt_ErrorTable1,
                TD_Evt_ErrorTable2,
                TD_Evt_RowsCheckpointed,
                TD_Evt_BufferModeSupport,
                TD_Evt_MacroNames,
                TD_Evt_PackFactor,

                TD_Evt_CPUTime,
                TD_Evt_LastError,
                TD_Evt_ExitCode,
                TD_Evt_RunStats,
                TD_Evt_UserCommand,
                TD_Evt_Version 

   3. operator: STREAM
      Phases  : Initiate, PutRow, EndAcquisition, Terminate
      Events  : TD_Evt_ConnectStatus,
                TD_Evt_DBSError,
                TD_Evt_CLIError,

                TD_Evt_BufferLayout,
                TD_Evt_RowCounts,
                TD_Evt_BlockCount,
                TD_Evt_ApplyCount,
                TD_Evt_ExportCount,
                TD_Evt_ErrorTable1,
                TD_Evt_ErrorTable2,
                TD_Evt_RowsCheckpointed,
                TD_Evt_BufferModeSupport,
                TD_Evt_MacroNames,
                TD_Evt_PackFactor,

                TD_Evt_CPUTime,
                TD_Evt_LastError,
                TD_Evt_ExitCode,
                TD_Evt_RunStats,
                TD_Evt_UserCommand,
                TD_Evt_Version

   4. operator: UPDATE
      Phases  : Initiate, PutRow, EndAcquisition, ApplyRows, Terminate
      Events  : TD_Evt_ConnectStatus,
                TD_Evt_DBSError,
                TD_Evt_CLIError,

                TD_Evt_BufferLayout,
                TD_Evt_RowCounts,
                TD_Evt_BlockCount,
                TD_Evt_ApplyCount,
                TD_Evt_ExportCount,
                TD_Evt_ErrorTable1,
                TD_Evt_ErrorTable2,
                TD_Evt_RowsCheckpointed,
                TD_Evt_BufferModeSupport,
                TD_Evt_MacroNames,
                TD_Evt_PackFactor,

                TD_Evt_CPUTime,
                TD_Evt_LastError,
                TD_Evt_ExitCode,
                TD_Evt_RunStats,
                TD_Evt_UserCommand,
                TD_Evt_Version

********************************************************************************/



#ifndef __MVS__

  #include "eventLog.h"
  #include "OptionsManager.h"
  #include <stdio.h>
  #include "string.h"

#else

  #include "GenericD.h"
  #include "OptionsM.h"
  #include <stdio.h>
  #include "string.h"

#endif //__MVS__

using namespace teradata::client::API;

eventLog::eventLog(OptionsManager * om)
{
	inputStream = NULL;
	outputStream = NULL;
	inputFileName = NULL;
	outputFileName = NULL;
    m_om = om;
}

eventLog::~eventLog(void)
{
	if( inputFileName != NULL ){
		free(inputFileName);
	}

	if( outputFileName != NULL ){
		free(outputFileName);
	}

}

#ifdef __MVS__
int eventLog::getBufferSize() {
     int rc;
     char filename[100];
     fldata_t fileinfo;

  //Open File if not already open
  if( inputStream == NULL )
    {
      inputStream = fopen(inputFileName, "rb, type=record");
      
      if( inputStream == NULL )
	{
	  cout << "Error opening file " << inputFileName << endl;
	  return -1;
	}
      
      rc = fldata(inputStream,filename,&fileinfo);
      if (rc != 0)
	printf("flfldata failed\n");
      else
	printf("max rec length is %d\n",fileinfo.__maxreclen);
      
    }
  return (int)fileinfo.__maxreclen;
}
#endif //__MVS__

/*************************************************
 Demonstrating TPTAPI events with LOAD operator
 using Checkpoint.
*************************************************/

void eventLog::testLoad()
{

   int returnValue = 0;
   char *errorMsg = NULL;
   TD_ErrorType errorType;

   char *user_name     = strdup(m_om -> getUserName().c_str());
   char *user_password = strdup(m_om -> getUserPassword().c_str());
   char *target_table  = strdup(m_om -> getTargetTable().c_str());
   char *tdp_id        = strdup(m_om -> getTdpId().c_str());
   char *log_table     = strdup(m_om -> getLogTable().c_str());

   char *error_table1  = strdup(m_om -> getErrorTable1().c_str());
   char *error_table2  = strdup(m_om -> getErrorTable2().c_str());

   char *dml_statment = (char *)malloc(1024);

   char *cp_data = NULL;
   TD_Length cp_data_len = 0;

   sprintf(dml_statment, "INSERT INTO %s( :Associate_Id, :Associate_Name, :Salary, :DOJ, \
:Designation, :Loan_Amount, :Martial_Status, :No_Of_Dependents);", target_table);

   cout << "\n*** Load Driver Example ***" << endl; 
   /* DR97368 */
   teradata::client::API::Connection *conn = new Connection();

   /**********************************************
   * Set Operator Type and Trace/Log Levels
   **********************************************/
   conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
   //conn -> AddAttribute(TD_TRACE_OUTPUT, "load.txt");
   //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_OFF, NULL);

   /**********************************************
   * Add Attributes
   **********************************************/
   conn -> AddAttribute(TD_USER_NAME, user_name);
   conn -> AddAttribute(TD_USER_PASSWORD, user_password);
   conn -> AddAttribute(TD_TARGET_TABLE, target_table);
   conn -> AddAttribute(TD_TDP_ID, tdp_id);
   conn -> AddAttribute(TD_LOG_TABLE, log_table);

   conn -> AddAttribute(TD_ERROR_TABLE_1, error_table1);
   conn -> AddAttribute(TD_ERROR_TABLE_2, error_table2);

   /**********************************************
   * Add Schema
   **********************************************/
   Schema *schema = new Schema((char*)"input");  //TPT-17718
   schema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
   schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
   schema -> AddColumn((char*)"Salary", TD_FLOAT, 8, 8, 1);  //TPT-17718
   schema -> AddColumn((char*)"DOJ", TD_DATE, 4 , 3, 1);  //TPT-17718
   schema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718

#ifndef __MVS__
   schema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
   schema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif //__MVS__

   schema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
   schema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1, 3, 1);  //TPT-17718

   conn -> AddSchema(schema);

   /**********************************************
   * Add DMLGroups
   **********************************************/
   TD_Index dmlGroupIndex = 0;

   DMLGroup *dmlGr = new DMLGroup();
   dmlGr -> AddStatement(dml_statment);
   conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

   /********************************************** 
   *  DR143810 : Begin
   *  Before Initiate
   **********************************************/
   printInBlock("Entering Initiate");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_RowCounts);
   showEvent(conn, TD_Evt_ApplyCount);
   showEvent(conn, TD_Evt_ErrorTable1);
   showEvent(conn, TD_Evt_ErrorTable2);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");

   /**********************************************
   * Initiate
   **********************************************/
   returnValue = conn -> Initiate();

   cout << "\nDriver Initiated with status " << returnValue << endl;

   /**********************************************
   *  DR143810 : Begin
   *  Before PutRow
   **********************************************/
   printInBlock("Entering PutRow");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_RowCounts);
   showEvent(conn, TD_Evt_ApplyCount);
   showEvent(conn, TD_Evt_ErrorTable1);
   showEvent(conn, TD_Evt_ErrorTable2);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");

   if ( returnValue < TD_Error )
   {
     /**********************************************
     * Acquisition
     **********************************************/

#ifndef __MVS__
     const int recLen = 78;
     char rowBuffer[recLen]; /* must include the EOL byte */
#else
     int maxRecLen = getBufferSize();
     char *rowBuffer = new char[maxRecLen];
#endif		
     int loadStatus = 0;
     TD_Length bytes;
     int count = 0;

     while( loadStatus != -1 )
     {
#ifndef __MVS__
       loadStatus = GetRowData(rowBuffer, recLen);
#else
       loadStatus = GetRowData(rowBuffer, maxRecLen);
#endif

       if( loadStatus != -1 )
       {
         /* pick up the first 2 bytes as the row length of the indicator mode record */
#ifndef __MVS__
         bytes = *((unsigned short *)rowBuffer);
         returnValue = conn -> PutRow( rowBuffer + 2, bytes );
#else
         returnValue = conn -> PutRow( rowBuffer, loadStatus );
#endif
         if ( returnValue >= TD_Error )
         {
           cout << "\nPutRow failed on row " << count+1 << " with status " << returnValue << endl;
           loadStatus = GetRowData(NULL, 0); //Abort load
         }
         else
         {
           count++;
           if ( count == 500 )
           {

             returnValue = conn -> Checkpoint(&cp_data, &cp_data_len);

             /**********************************************
             *  DR143810 : Begin
             *  After 500 rows
             **********************************************/
             printInBlock("After 500 rows ");
             showEvent(conn, TD_Evt_Version);
             showEvent(conn, TD_Evt_ConnectStatus);
             showEvent(conn, TD_Evt_DBSError);
             showEvent(conn, TD_Evt_CLIError);
             showEvent(conn, TD_Evt_BufferLayout);
             showEvent(conn, TD_Evt_RowCounts);
             showEvent(conn, TD_Evt_ApplyCount);
             showEvent(conn, TD_Evt_ErrorTable1);
             showEvent(conn, TD_Evt_ErrorTable2);
             showEvent(conn, TD_Evt_RowsCheckpointed);
             showEvent(conn, TD_Evt_ExitCode);
             showEvent(conn, TD_Evt_CPUTime);
             for(int i = 0; i < 70; i++)
               printf("*");
             printf("\n");

           }
         }

       }

     }

     /**********************************************
     *  DR143810 : Begin
     *  Before EndAcquisition
     **********************************************/
     printInBlock("Entering EndAcquisition");
     showEvent(conn, TD_Evt_Version);
     showEvent(conn, TD_Evt_ConnectStatus);
     showEvent(conn, TD_Evt_DBSError);
     showEvent(conn, TD_Evt_CLIError);
     showEvent(conn, TD_Evt_BufferLayout);
     showEvent(conn, TD_Evt_RowCounts);
     showEvent(conn, TD_Evt_ApplyCount);
     showEvent(conn, TD_Evt_ErrorTable1);
     showEvent(conn, TD_Evt_ErrorTable2);
     showEvent(conn, TD_Evt_RowsCheckpointed);
     showEvent(conn, TD_Evt_ExitCode);
     showEvent(conn, TD_Evt_CPUTime);
     for(int i = 0; i < 70; i++)
       printf("*");
     printf("\n");


     if ( returnValue < TD_Error )
     {
       cout << "\nSent " << count << " rows" << endl;

       /**********************************************
       * End Acquisition
       **********************************************/
       returnValue = conn -> EndAcquisition();
			
       cout << "\nAcquisition completed with status " << returnValue << endl;

       if ( returnValue < TD_Error )
       {

         /**********************************************
         * Application
         **********************************************/
         /**********************************************
         *  DR143810 : Begin
         *  Before ApplyRows
         **********************************************/
         printInBlock("Entering ApplyRows");
         showEvent(conn, TD_Evt_Version);
         showEvent(conn, TD_Evt_ConnectStatus);
         showEvent(conn, TD_Evt_DBSError);
         showEvent(conn, TD_Evt_CLIError);
         showEvent(conn, TD_Evt_BufferLayout);
         showEvent(conn, TD_Evt_RowCounts);
         showEvent(conn, TD_Evt_ApplyCount);
         showEvent(conn, TD_Evt_ErrorTable1);
         showEvent(conn, TD_Evt_ErrorTable2);
         showEvent(conn, TD_Evt_RowsCheckpointed);
         showEvent(conn, TD_Evt_ExitCode);
         showEvent(conn, TD_Evt_CPUTime);
         for(int i = 0; i < 70; i++)
           printf("*");
         printf("\n");

         returnValue = conn -> ApplyRows();
				
         cout << "\nRows Applied with status " << returnValue << endl;

         if ( returnValue < TD_Error )
         {
           cout << "\nLoad completed successfully" << endl;
         }
         else
         {
           //Get Error Information
           cout << "\nError occured during Application" << endl;
					
           conn -> GetErrorInfo(&errorMsg, &errorType);
           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
           }
           else
           {
             cout << "No Error Info Available" << endl;
           }
         }
       }
       else
       {
         //Get Error Information
         cout << "\nError occured during EndAcquisition" << endl;
         conn -> GetErrorInfo(&errorMsg, &errorType);
         if ( errorMsg != NULL )
         {
           cout << errorMsg << endl;
           cout << "Type: " << errorType << endl;
         }
         else
         {
           cout << "No Error Info Available" << endl;
         }
       }
     }
     else
     {
       //Get Error Information
			
       cout << "\nError occured during Acquisition" << endl;
       conn -> GetErrorInfo(&errorMsg, &errorType);
       if ( errorMsg != NULL )
       {
         cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
       }
       else
       {
         cout << "No Error Info Available" << endl;
       }
     }

   }
   else
   {
     //Get Error Information
     cout << "\nError occured during Initiate" << endl;
     conn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "No Error Info Available" << endl;
     }
   }

   /**********************************************
   *  DR143810 : Begin
   *  After ApplyRows 
   **********************************************/
   printInBlock("After ApplyRows");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_RowCounts);
   showEvent(conn, TD_Evt_ApplyCount);
   showEvent(conn, TD_Evt_ErrorTable1);
   showEvent(conn, TD_Evt_ErrorTable2);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");


   /**********************************************
   * Terminate
   **********************************************/
   returnValue = conn -> Terminate();

   cout << "\nDriver Terminated with status " << returnValue << endl;

   if ( returnValue >= TD_Error )
   {
     //Get Error Information
     cout << "\nError occured during Terminate" << endl;
     conn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "\nNo Error Info Available" << endl;
     }
   }

   /**********************************************
   * Clean Up
   **********************************************/
   cout << "Deleting objects" << endl;
   delete dmlGr;
   delete schema;
   delete conn;

#ifdef __MVS__
   // if (rowBuffer != NULL)
      delete [] rowBuffer;
#endif

   free(user_name);
   free(user_password);
   free(target_table);
   free(tdp_id);
   free(log_table);
   free(dml_statment);
   free(error_table1);
   free(error_table2);

   cout << "*** Load Complete ***" << endl;                   
   /* DR97368 */
}

/***************************************************
 * Method for testing Export Driver
***************************************************/

void eventLog::testExport()
{

   int returnValue = 0;
   char *errorMsg  = NULL;
   TD_ErrorType errorType;

   char *user_name     = strdup(m_om -> getUserName().c_str());
   char *user_password = strdup(m_om -> getUserPassword().c_str());
   char *target_table  = strdup(m_om -> getTargetTable().c_str());
   char *tdp_id        = strdup(m_om -> getTdpId().c_str());

   char *sel_statment  = (char *)malloc(1024);
   sprintf(sel_statment, "sel Associate_Name from %s;", target_table);

   cout << "\n*** Export Driver Example ***" << endl;           

   teradata::client::API::Connection *conn = new Connection();

   /**********************************************
   * Set Operator Type and Trace/Log Levels
   **********************************************/
   conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);
   //conn -> AddAttribute(TD_TRACE_OUTPUT, "export.txt");
   //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);

   /**********************************************
   * Add Attributes
   **********************************************/
   conn -> AddAttribute(TD_USER_NAME, user_name);
   conn -> AddAttribute(TD_USER_PASSWORD, user_password);
   conn -> AddAttribute(TD_SELECT_STMT, sel_statment);
   conn -> AddAttribute(TD_TDP_ID, tdp_id);

   conn -> AddAttribute(TD_TARGET_TABLE, target_table);

   /**********************************************
   * Add Schema - Using Dynamic Schema
   **********************************************/
   //Schema *schema = new Schema("output");
   //schema -> AddColumn("Associate_Name", TD_CHAR, 25);

   //conn -> AddSchema(schema);

   /**********************************************
   *  DR143810 : Begin
   *  Entering Initiate
   **********************************************/
   printInBlock("Entering Initiate");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_BlockCount);
   showEvent(conn, TD_Evt_ExportCount);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*"); 
   printf("\n"); 

   /**********************************************
   * Initiate
   **********************************************/
   returnValue = conn -> Initiate();

   cout << "\nDriver Initiated with status " << returnValue << endl;

   if ( returnValue < TD_Error )
   {
     /**********************************************
     *  DR143810 : Begin
     *  Entering Acquisition
     **********************************************/
     printInBlock("Entering Acquisition");
     showEvent(conn, TD_Evt_Version);
     showEvent(conn, TD_Evt_ConnectStatus);
     showEvent(conn, TD_Evt_DBSError);
     showEvent(conn, TD_Evt_CLIError);
     showEvent(conn, TD_Evt_BufferLayout);
     showEvent(conn, TD_Evt_BlockCount);
     showEvent(conn, TD_Evt_ExportCount);
     showEvent(conn, TD_Evt_RowsCheckpointed);
     showEvent(conn, TD_Evt_ExitCode);
     showEvent(conn, TD_Evt_CPUTime);

     for(int i = 0; i < 70; i++)
       printf("*");
     printf("\n");

     /**********************************************
     * Acquisition
     **********************************************/
     int count = 0;
     char *data;
     TD_Length dataLen = 0;
     int exportStatus = 0;

     while( exportStatus != -1)
     {
       returnValue = conn -> GetRow(&data, &dataLen);

       if ( returnValue >= TD_Error )
       {
         cout << "\nGetRow failed on row " << count+1 << " with status " << returnValue << endl;
         exportStatus = WriteRowData(NULL, 0);
         //Abort write
       }
       else 
       if ( returnValue == TD_END_Method )
       {
         cout << "\nEnd of Data Reached" << endl;
         exportStatus = WriteRowData(NULL, 0);
         //end write
         count++;
       }
       else
       {
         exportStatus = WriteRowData(data, dataLen);
         count++;
         if(count == 1)
         {
           /**********************************************
           *  DR143810 : Begin
           *  After First Call to GetRow 
           **********************************************/
           printInBlock("After First Call to GetRow");
           showEvent(conn, TD_Evt_Version);
           showEvent(conn, TD_Evt_ConnectStatus);
           showEvent(conn, TD_Evt_DBSError);
           showEvent(conn, TD_Evt_CLIError);
           showEvent(conn, TD_Evt_BufferLayout);
           showEvent(conn, TD_Evt_BlockCount);
           showEvent(conn, TD_Evt_ExportCount);
           showEvent(conn, TD_Evt_RowsCheckpointed);
           showEvent(conn, TD_Evt_ExitCode);
           showEvent(conn, TD_Evt_CPUTime);

           for(int i = 0; i < 70; i++)
             printf("*");
           printf("\n");

         }
       }
     }
     
     cout << "Export completed with status " << returnValue << endl;

     if ( returnValue < TD_Error )
     {
       cout << "Export completed successfully" << endl;

       /**********************************************
       *  DR143810 : Begin
       *  After Export
       **********************************************/
       printInBlock("After Export");
       showEvent(conn, TD_Evt_Version);
       showEvent(conn, TD_Evt_ConnectStatus);
       showEvent(conn, TD_Evt_DBSError);
       showEvent(conn, TD_Evt_CLIError);
       showEvent(conn, TD_Evt_BufferLayout);
       showEvent(conn, TD_Evt_BlockCount);
       showEvent(conn, TD_Evt_ExportCount);
       showEvent(conn, TD_Evt_RowsCheckpointed);
       showEvent(conn, TD_Evt_ExitCode);
       showEvent(conn, TD_Evt_CPUTime);

       for(int i = 0; i < 70; i++)
         printf("*");
       printf("\n");
     }
     else
     {
       //Get Error Information
       cout << "Error occured during Export" << endl;
       conn -> GetErrorInfo(&errorMsg, &errorType);
       if ( errorMsg != NULL )
       {
         cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
       }
       else
       {
         cout << "No Error Info Available" << endl;
       }
     }
   }
   else
   {
     //Get Error Information
     cout << "Error occured during Initiate" << endl;
     conn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "No Error Info Available" << endl;
     }
   }

   /**********************************************
   * Terminate
   **********************************************/
   returnValue = conn -> Terminate();
   cout << "\nDriver Terminated with status " << returnValue << endl;

   if ( returnValue >= TD_Error )
   {
     //Get Error Information
     cout << "Error occured during Terminate" << endl;
     conn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "No Error Info Available" << endl;
     }
   }

   /**********************************************
   * Clean Up
   **********************************************/
   cout << "Deleting Objects" << endl;

   //delete schema;
   delete conn;

   free(user_name);
   free(user_password);
   free(target_table);
   free(tdp_id);
   free(sel_statment);

   cout << "*** Export Complete ***" << endl;            
}

/*************************************************
 Demonstrating TPTAPI events with Update operator 
 without checkpointing the data
*************************************************/
void eventLog::testUpdate()
{

   int returnValue = 0;
   char *errorMsg = NULL;
   TD_ErrorType errorType;

   char *user_name     = strdup(m_om -> getUserName().c_str());
   char *user_password = strdup(m_om -> getUserPassword().c_str());
   char *target_table  = strdup(m_om -> getTargetTable().c_str());
   char *tdp_id        = strdup(m_om -> getTdpId().c_str());
   char *log_table     = strdup(m_om -> getLogTable().c_str());

   char *error_table1  = strdup(m_om -> getErrorTable1().c_str());
   char *error_table2  = strdup(m_om -> getErrorTable2().c_str());

   char *dml_statment = (char *)malloc(1024);

   char *cp_data = NULL;
   TD_Length cp_data_len = 0;

   sprintf(dml_statment, "INSERT INTO %s( :Associate_Id, :Associate_Name, :Salary, :DOJ, \
:Designation, :Loan_Amount, :Martial_Status, :No_Of_Dependents);", target_table);

   cout << "\n*** Update Driver Example ***" << endl; 
   /* DR97368 */
   teradata::client::API::Connection *conn = new Connection();

   /**********************************************
   * Set Operator Type and Trace/Log Levels
   **********************************************/
   conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_UPDATE);
   //conn -> AddAttribute(TD_TRACE_OUTPUT, "load.txt");
   //conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_OFF, NULL);

   /**********************************************
   * Add Attributes
   **********************************************/
   conn -> AddAttribute(TD_USER_NAME, user_name);
   conn -> AddAttribute(TD_USER_PASSWORD, user_password);
   conn -> AddAttribute(TD_TARGET_TABLE, target_table);
   conn -> AddAttribute(TD_TDP_ID, tdp_id);
   conn -> AddAttribute(TD_LOG_TABLE, log_table);

   conn -> AddAttribute(TD_ERROR_TABLE_1, error_table1);
   conn -> AddAttribute(TD_ERROR_TABLE_2, error_table2);

   /**********************************************
   * Add Schema
   **********************************************/
   Schema *schema = new Schema((char*)"input");  //TPT-17718
   schema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
   schema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
   schema -> AddColumn((char*)"Salary", TD_FLOAT, 8, 8, 1);  //TPT-17718
   schema -> AddColumn((char*)"DOJ", TD_DATE, 4 , 3, 1);  //TPT-17718
   schema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718

#ifndef __MVS__
   schema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
   schema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif //__MVS__

   schema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
   schema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1, 3, 1);  //TPT-17718

   conn -> AddSchema(schema);

   /**********************************************
   * Add DMLGroups
   **********************************************/
   TD_Index dmlGroupIndex = 0;

   DMLGroup *dmlGr = new DMLGroup();
   dmlGr -> AddStatement(dml_statment);
   conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

   /********************************************** 
   *  DR143810 : Begin
   *  Before Initiate
   **********************************************/
   printInBlock("Entering Initiate");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_RowCounts);
   showEvent(conn, TD_Evt_ApplyCount);
   showEvent(conn, TD_Evt_ErrorTable1);
   showEvent(conn, TD_Evt_ErrorTable2);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");

   /**********************************************
   * Initiate
   **********************************************/
   returnValue = conn -> Initiate();

   cout << "\nDriver Initiated with status " << returnValue << endl;

   /**********************************************
   *  DR143810 : Begin
   *  Before PutRow
   **********************************************/
   printInBlock("Entering PutRow");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_RowCounts);
   showEvent(conn, TD_Evt_ApplyCount);
   showEvent(conn, TD_Evt_ErrorTable1);
   showEvent(conn, TD_Evt_ErrorTable2);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");

   if ( returnValue < TD_Error )
   {
     /**********************************************
     * Acquisition
     **********************************************/

#ifndef __MVS__
     const int recLen = 78;
     char rowBuffer[recLen]; /* must include the EOL byte */
#else
     int maxRecLen = getBufferSize();
     char *rowBuffer = new char[maxRecLen];
#endif		
     int loadStatus = 0;
     TD_Length bytes;
     int count = 0;

     while( loadStatus != -1 )
     {
#ifndef __MVS__
       loadStatus = GetRowData(rowBuffer, recLen);
#else
       loadStatus = GetRowData(rowBuffer, maxRecLen);
#endif

       if( loadStatus != -1 )
       {
         /* pick up the first 2 bytes as the row length of the indicator mode record */
#ifndef __MVS__
         bytes = *((unsigned short *)rowBuffer);
         returnValue = conn -> PutRow( rowBuffer + 2, bytes );
#else
         returnValue = conn -> PutRow( rowBuffer, loadStatus );
#endif
         if ( returnValue >= TD_Error )
         {
           cout << "\nPutRow failed on row " << count+1 << " with status " << returnValue << endl;
           loadStatus = GetRowData(NULL, 0); //Abort load
         }
         else
         {
           count++;
           if ( count == 500 )
           {

             /**********************************************
             *  DR143810 : Begin
             *  After 500 rows
             **********************************************/
             printInBlock("After 500 rows ");
             showEvent(conn, TD_Evt_Version);
             showEvent(conn, TD_Evt_ConnectStatus);
             showEvent(conn, TD_Evt_DBSError);
             showEvent(conn, TD_Evt_CLIError);
             showEvent(conn, TD_Evt_BufferLayout);
             showEvent(conn, TD_Evt_RowCounts);
             showEvent(conn, TD_Evt_ApplyCount);
             showEvent(conn, TD_Evt_ErrorTable1);
             showEvent(conn, TD_Evt_ErrorTable2);
             showEvent(conn, TD_Evt_RowsCheckpointed);
             showEvent(conn, TD_Evt_ExitCode);
             showEvent(conn, TD_Evt_CPUTime);
             for(int i = 0; i < 70; i++)
               printf("*");
             printf("\n");

           }
         }

       }

     }

     /**********************************************
     *  DR143810 : Begin
     *  Before EndAcquisition
     **********************************************/
     printInBlock("Entering EndAcquisition");
     showEvent(conn, TD_Evt_Version);
     showEvent(conn, TD_Evt_ConnectStatus);
     showEvent(conn, TD_Evt_DBSError);
     showEvent(conn, TD_Evt_CLIError);
     showEvent(conn, TD_Evt_BufferLayout);
     showEvent(conn, TD_Evt_RowCounts);
     showEvent(conn, TD_Evt_ApplyCount);
     showEvent(conn, TD_Evt_ErrorTable1);
     showEvent(conn, TD_Evt_ErrorTable2);
     showEvent(conn, TD_Evt_RowsCheckpointed);
     showEvent(conn, TD_Evt_ExitCode);
     showEvent(conn, TD_Evt_CPUTime);
     for(int i = 0; i < 70; i++)
       printf("*");
     printf("\n");


     if ( returnValue < TD_Error )
     {
       cout << "\nSent " << count << " rows" << endl;

       /**********************************************
       * End Acquisition
       **********************************************/
       returnValue = conn -> EndAcquisition();
			
       cout << "\nAcquisition completed with status " << returnValue << endl;

       if ( returnValue < TD_Error )
       {

         /**********************************************
         * Application
         **********************************************/
         /**********************************************
         *  DR143810 : Begin
         *  Before ApplyRows
         **********************************************/
         printInBlock("Entering ApplyRows");
         showEvent(conn, TD_Evt_Version);
         showEvent(conn, TD_Evt_ConnectStatus);
         showEvent(conn, TD_Evt_DBSError);
         showEvent(conn, TD_Evt_CLIError);
         showEvent(conn, TD_Evt_BufferLayout);
         showEvent(conn, TD_Evt_RowCounts);
         showEvent(conn, TD_Evt_ApplyCount);
         showEvent(conn, TD_Evt_ErrorTable1);
         showEvent(conn, TD_Evt_ErrorTable2);
         showEvent(conn, TD_Evt_RowsCheckpointed);
         showEvent(conn, TD_Evt_ExitCode);
         showEvent(conn, TD_Evt_CPUTime);
         for(int i = 0; i < 70; i++)
           printf("*");
         printf("\n");

         returnValue = conn -> ApplyRows();
				
         cout << "\nRows Applied with status " << returnValue << endl;

         if ( returnValue < TD_Error )
         {
           cout << "\nUpdate completed successfully" << endl;
         }
         else
         {
           //Get Error Information
           cout << "\nError occured during Application" << endl;
					
           conn -> GetErrorInfo(&errorMsg, &errorType);
           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
           }
           else
           {
             cout << "No Error Info Available" << endl;
           }
         }
       }
       else
       {
         //Get Error Information
         cout << "\nError occured during EndAcquisition" << endl;
         conn -> GetErrorInfo(&errorMsg, &errorType);
         if ( errorMsg != NULL )
         {
           cout << errorMsg << endl;
           cout << "Type: " << errorType << endl;
         }
         else
         {
           cout << "No Error Info Available" << endl;
         }
       }
     }
     else
     {
       //Get Error Information
			
       cout << "\nError occured during Acquisition" << endl;
       conn -> GetErrorInfo(&errorMsg, &errorType);
       if ( errorMsg != NULL )
       {
         cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
       }
       else
       {
         cout << "No Error Info Available" << endl;
       }
     }

   }
   else
   {
     //Get Error Information
     cout << "\nError occured during Initiate" << endl;
     conn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "No Error Info Available" << endl;
     }
   }

   /**********************************************
   *  DR143810 : Begin
   *  After ApplyRows 
   **********************************************/
   printInBlock("After ApplyRows");
   showEvent(conn, TD_Evt_Version);
   showEvent(conn, TD_Evt_ConnectStatus);
   showEvent(conn, TD_Evt_DBSError);
   showEvent(conn, TD_Evt_CLIError);
   showEvent(conn, TD_Evt_BufferLayout);
   showEvent(conn, TD_Evt_RowCounts);
   showEvent(conn, TD_Evt_ApplyCount);
   showEvent(conn, TD_Evt_ErrorTable1);
   showEvent(conn, TD_Evt_ErrorTable2);
   showEvent(conn, TD_Evt_RowsCheckpointed);
   showEvent(conn, TD_Evt_ExitCode);
   showEvent(conn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");


   /**********************************************
   * Terminate
   **********************************************/
   returnValue = conn -> Terminate();

   cout << "\nDriver Terminated with status " << returnValue << endl;

   if ( returnValue >= TD_Error )
   {
     //Get Error Information
     cout << "\nError occured during Terminate" << endl;
     conn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "\nNo Error Info Available" << endl;
     }
   }

   /**********************************************
   * Clean Up
   **********************************************/
   cout << "Deleting objects" << endl;
   delete dmlGr;
   delete schema;
   delete conn;

#ifdef __MVS__
   // if (rowBuffer != NULL)
      delete [] rowBuffer;
#endif

   free(user_name);
   free(user_password);
   free(target_table);
   free(tdp_id);
   free(log_table);
   free(dml_statment);
   free(error_table1);
   free(error_table2);

   cout << "*** Update Complete ***" << endl;                   
   /* DR97368 */
}

/*************************************************
 Demonstrating TPTAPI events with Stream operator
 in buffer mode
*************************************************/
void eventLog::testExportToStream()
{
   int strStatus = 0;
   int expStatus = 0;
   int queryStatus = 0;
   char *errorMsg = NULL;

   TD_ErrorType errorType;
   BufferLayout layout; /* BufferLayout struct defined in GetBuffer.h */
   RowCounts strCount;  /* RowCounts struct defined in GetBuffer.h */
   RunStats strStats; /* RunStats struct defined in GetBuffer.h */
   char *dataptr;
   TD_Length datalen;
   char *msg = (char *)malloc(256);

   /**********************************************
   * Retrieve Options From Option File
   **********************************************/
   int numSessions = m_om -> getNumSessions();

   char *user_name           = strdup(m_om -> getUserName().c_str());
   char *user_password       = strdup(m_om -> getUserPassword().c_str());
   char *target_table        = strdup(m_om -> getTargetTable().c_str());
   char *target_table_export = strdup(m_om -> getTargetTableExport().c_str());

   char *tdp_id        = strdup(m_om -> getTdpId().c_str());
   char *log_table     = strdup(m_om -> getLogTable().c_str());
   char *error_table   = strdup(m_om -> getErrorTable().c_str());

   char *ins_statement = (char *)malloc(1024);
   char *sel_statement = (char *)malloc(1024);

   sprintf(ins_statement, "INSERT INTO %s( :Associate_Id, :Associate_Name, :Salary, :DOJ, \
:Designation, :Loan_Amount, :Martial_Status, :No_Of_Dependents);", target_table);

   sprintf(sel_statement, "SELECT * FROM %s order by 1;", target_table_export);

   cout << "*** Export to Stream Driver Example ***" << endl;
   cout << "Select Statement: " << sel_statement << endl;
   cout << "Insert Statement: " << ins_statement << endl;
   cout << "Number of Sessions: " << numSessions << endl;
   cout << endl;

   teradata::client::API::Connection *strConn = new Connection(); /* Stream */
   teradata::client::API::Connection *expConn = new Connection(); /* Export */

   /**********************************************
   * Set Operator Type and Trace/Log Levels
   **********************************************/
   strConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);
   expConn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);

   //strConn -> AddAttribute(TD_TRACE_OUTPUT,(char*)"stream.txt");
   //strConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_OFF, NULL);
   //expConn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"export.txt");
   //expConn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_OFF, NULL);

   /**********************************************
   * Add Attributes
   **********************************************/
   strConn -> AddAttribute(TD_USER_NAME, user_name);
   strConn -> AddAttribute(TD_USER_PASSWORD, user_password);
   strConn -> AddAttribute(TD_TARGET_TABLE, target_table);

   strConn -> AddAttribute(TD_TDP_ID, tdp_id);
   strConn -> AddAttribute(TD_LOG_TABLE, log_table);
   strConn -> AddAttribute(TD_ERROR_TABLE, error_table);

   strConn -> AddAttribute(TD_BUFFER_MODE, (char*)"Yes");  //TPT-17718
   strConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
   strConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);

   expConn -> AddAttribute(TD_USER_NAME, user_name);
   expConn -> AddAttribute(TD_USER_PASSWORD, user_password);
   expConn -> AddAttribute(TD_SELECT_STMT, sel_statement);
   expConn -> AddAttribute(TD_TDP_ID, tdp_id);
   expConn -> AddAttribute(TD_MAX_SESSIONS, numSessions);
   expConn -> AddAttribute(TD_MIN_SESSIONS, numSessions);
   expConn -> AddAttribute(TD_BUFFER_MODE, (char*)"Yes");  //TPT-17718

   /**********************************************
   * Add Schema
   **********************************************/
   Schema *strSchema = new Schema((char*)"input");  //TPT-17718
   strSchema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
   strSchema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
   strSchema -> AddColumn((char*)"Salary", TD_FLOAT, 8, 8, 1);  //TPT-17718
   strSchema -> AddColumn((char*)"DOJ", TD_DATE, 4 , 3, 1);  //TPT-17718
   strSchema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718

#ifndef __MVS__
   strSchema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
   strSchema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif //__MVS__

   strSchema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
   strSchema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1, 3, 1);  //TPT-17718

   strConn -> AddSchema(strSchema);

   Schema *expSchema = new Schema((char*)"output");  //TPT-17718

   expSchema -> AddColumn((char*)"Associate_Id", TD_INTEGER, 4, 1, 1);  //TPT-17718
   expSchema -> AddColumn((char*)"Associate_Name", TD_CHAR, 25);  //TPT-17718
   expSchema -> AddColumn((char*)"Salary", TD_FLOAT, 8, 8, 1);  //TPT-17718
   expSchema -> AddColumn((char*)"DOJ", TD_DATE, 4 , 3, 1);  //TPT-17718
   expSchema -> AddColumn((char*)"Designation", TD_VARCHAR, 25);  //TPT-17718

#ifndef __MVS__
   expSchema -> AddColumn((char*)"Loan_Amount", TD_DECIMAL, 4, 5, 2);  //TPT-17718
#else
   expSchema -> AddColumn("Loan_Amount", TD_DECIMAL, 3, 5, 2);
#endif //__MVS__

   expSchema -> AddColumn((char*)"Martial_Status", TD_CHAR, 1);  //TPT-17718
   expSchema -> AddColumn((char*)"No_Of_Dependents", TD_BYTEINT, 1, 3, 1);  //TPT-17718

   expConn -> AddSchema(expSchema);

   /**********************************************
   * Add DMLGroups
   **********************************************/
   TD_Index dmlGroupIndex = 0;

   DMLGroup *dmlGr = new DMLGroup();
   dmlGr -> AddStatement(ins_statement);
   strConn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

   /**********************************************
   *  DR143810 : Begin
   *  Entering Initiate
   **********************************************/
   printInBlock("Entering Initiate");
   showEvent(strConn, TD_Evt_Version);
   showEvent(strConn, TD_Evt_ConnectStatus);
   showEvent(strConn, TD_Evt_DBSError);
   showEvent(strConn, TD_Evt_CLIError);
   showEvent(strConn, TD_Evt_BufferLayout);
   showEvent(strConn, TD_Evt_RowCounts);
   showEvent(strConn, TD_Evt_ApplyCount);
   showEvent(strConn, TD_Evt_ErrorTable1);
   showEvent(strConn, TD_Evt_MacroNames);
   showEvent(strConn, TD_Evt_PackFactor);
   showEvent(strConn, TD_Evt_RunStats);
   showEvent(strConn, TD_Evt_RowsCheckpointed);
   showEvent(strConn, TD_Evt_ExitCode);
   showEvent(strConn, TD_Evt_CPUTime);
   for(int i = 0; i < 70; i++)
     printf("*");
   printf("\n");

   /**********************************************
   * Initiate Stream Driver
   **********************************************/
   cout << endl << "*** Stream Initiate ***" << endl;
   strStatus = strConn -> Initiate();

   printStatus("\nStream Driver Initiated with status ", strStatus);

   if ( strStatus < TD_Error )
   {
     /**********************************************
     *  DR143810 : Begin
     *  Entering GetBuffer(Acquisition)
     **********************************************/
     printInBlock("Entering GetBuffer");
     showEvent(strConn, TD_Evt_Version);
     showEvent(strConn, TD_Evt_ConnectStatus);
     showEvent(strConn, TD_Evt_DBSError); 
     showEvent(strConn, TD_Evt_BufferLayout);
     showEvent(strConn, TD_Evt_RowCounts);
     showEvent(strConn, TD_Evt_ApplyCount);
     showEvent(strConn, TD_Evt_ErrorTable1);
     showEvent(strConn, TD_Evt_MacroNames);
     showEvent(strConn, TD_Evt_PackFactor);
     showEvent(strConn, TD_Evt_RunStats);
     showEvent(strConn, TD_Evt_RowsCheckpointed);
     showEvent(strConn, TD_Evt_ExitCode);
     showEvent(strConn, TD_Evt_CPUTime);
     for(int i = 0; i < 70; i++)
       printf("*");
     printf("\n");

     /**********************************************
     * Query Buffer Layout
     **********************************************/
     queryStatus = strConn -> GetEvent(TD_Evt_BufferLayout, &dataptr, &datalen);

     if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
     {
       layout = *((BufferLayout *)dataptr);
     }

     /* Give Buffer Layout to Export Driver */
     expConn -> AddAttribute(TD_BUFFER_MAX_SIZE, layout.MaxMsgSize);
     expConn -> AddAttribute(TD_BUFFER_LENGTH_SIZE, layout.RowLengthSize);
     expConn -> AddAttribute(TD_BUFFER_HEADER_SIZE, layout.RowHeaderSize);
     expConn -> AddAttribute(TD_BUFFER_TRAILER_SIZE, layout.BufTrailerSize);

     /**********************************************
     * Initiate Export Driver
     **********************************************/
     cout << endl << "*** Export Initiate ***" << endl;
     expStatus = expConn -> Initiate();
     printStatus("\nExport Driver Initiated with status ", expStatus);

     if ( expStatus < TD_Error )
     {

       /**********************************************
       * Acquisition
       **********************************************/
       cout << endl << "*** Acquisition ***" << endl;
       char *dataBuffer;
       TD_Length dataLen = 0;
       int count = 0;
       int transferDone = 0;

       while(!transferDone)
       {
         /* Get a buffer from Export */
         expStatus = expConn -> GetBuffer(&dataBuffer, &dataLen);

         if ( expStatus >= TD_Error )
         {
           sprintf(msg, "GetBuffer failed on buffer %d with status ", count+1);
           printStatus(msg,expStatus);
           transferDone = 1;

         }
         else if ( expStatus == TD_END_Method )
         {
           cout << "End of Data Reached" << endl;
           transferDone = 1;
         }
         else
         {
           /* Pass buffer to Stream */
           strStatus = strConn -> PutBuffer(dataBuffer, dataLen, 1);

           if ( strStatus != TD_Success )
           {
             sprintf(msg, "PutBuffer failed on buffer %d with status ", count+1);
             printStatus(msg, strStatus);
             transferDone = 1;
           }
           else
           {
             count++;
             if( count == 100 )
             {
               /**********************************************
               *  DR143810 : Begin
               *  After 100 rows 
               **********************************************/
               printInBlock("After 100 rows");
               showEvent(strConn, TD_Evt_Version);
               showEvent(strConn, TD_Evt_ConnectStatus);
               showEvent(strConn, TD_Evt_DBSError);
               showEvent(strConn, TD_Evt_BufferLayout);
               showEvent(strConn, TD_Evt_RowCounts);
               showEvent(strConn, TD_Evt_ApplyCount);
               showEvent(strConn, TD_Evt_ErrorTable1);
               showEvent(strConn, TD_Evt_MacroNames);
               showEvent(strConn, TD_Evt_PackFactor);
               showEvent(strConn, TD_Evt_RunStats);
               showEvent(strConn, TD_Evt_RowsCheckpointed);
               showEvent(strConn, TD_Evt_ExitCode);
               showEvent(strConn, TD_Evt_CPUTime);
               for(int i = 0; i < 70; i++)
                 printf("*");
               printf("\n");
          
             }

           }
         }
       }

       if ( strStatus < TD_Error && expStatus < TD_Error )
       {
         /**********************************************
         * End Acquisition
         **********************************************/

         strStatus = strConn -> EndAcquisition();
         printStatus("Acquisition completed with status ", strStatus);

         if ( strStatus < TD_Error )
         {
           /**********************************************
           *  DR143810 : Begin
           *  After Acquisition
           **********************************************/
           printInBlock("After Acquisition");
           showEvent(strConn, TD_Evt_Version);
           showEvent(strConn, TD_Evt_ConnectStatus);
           showEvent(strConn, TD_Evt_DBSError);
           showEvent(strConn, TD_Evt_BufferLayout);
           showEvent(strConn, TD_Evt_RowCounts);
           showEvent(strConn, TD_Evt_ApplyCount);
           showEvent(strConn, TD_Evt_ErrorTable1);
           showEvent(strConn, TD_Evt_MacroNames);
           showEvent(strConn, TD_Evt_PackFactor);
           showEvent(strConn, TD_Evt_RunStats);
           showEvent(strConn, TD_Evt_RowsCheckpointed);
           showEvent(strConn, TD_Evt_ExitCode);
           showEvent(strConn, TD_Evt_CPUTime);
           for(int i = 0; i < 70; i++)
             printf("*");
           printf("\n");


           /**********************************************
           * Application - No Application For Stream
           **********************************************/

           if ( strStatus < TD_Error )
           {
             cout << "\nTransfer completed successfully" << endl;
             cout << "\nTransferred " << count << " buffers" << endl;

           }
           else
           {
             //Get Error Information
             cout << "Error occured during Application" << endl;
             strConn -> GetErrorInfo(&errorMsg, &errorType);
             if ( errorMsg != NULL )
             {
               cout << errorMsg << endl;
               cout << "Type: " << errorType << endl;
             }
             else
             {
               cout << "No Error Info Available" << endl;
             }
           }
         }
         else
         {
           //Get Error Information
           cout << "Error occured during EndAcquisition" << endl;
           strConn -> GetErrorInfo(&errorMsg, &errorType);
           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
           }
           else
           {
             cout << "No Error Info Available" << endl;
           }
         }
       }
       else
       {
         //Get Error Information
         if ( strStatus >= TD_Error )
         {
           cout << "\nStream Error occured during Acquisition" << endl;
           strConn -> GetErrorInfo(&errorMsg, &errorType);
           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
           }
           else
           {
             cout << "No Error Info Available" << endl;
           }
         }
         if ( expStatus >= TD_Error )
         {
           cout << "\nExport Error occured during Acquisition" << endl;
           expConn -> GetErrorInfo(&errorMsg, &errorType);
           if ( errorMsg != NULL )
           {
             cout << errorMsg << endl;
             cout << "Type: " << errorType << endl;
           }
           else
           {
             cout << "No Error Info Available" << endl;
           }
         }
       }
     }
     else 
     {
       //Get Error Information
       cout << "\nError occured during Export Initiate" << endl;
       expConn -> GetErrorInfo(&errorMsg, &errorType);
       if ( errorMsg != NULL )
       {
         cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
       }
       else
       {
         cout << "No Error Info Available" << endl;
       }
     }
   }
   else
   {
     //Get Error Information
     cout << "\nError occured during Stream Initiate" << endl;
     strConn -> GetErrorInfo(&errorMsg, &errorType);
     if ( errorMsg != NULL )
     {
       cout << errorMsg << endl;
       cout << "Type: " << errorType << endl;
     }
     else
     {
       cout << "No Error Info Available" << endl;
     }
   }

   /**********************************************
   * Terminate Drivers
   **********************************************/
   cout << endl << "*** Terminate ***" << endl;
   strStatus = strConn -> Terminate();
   expStatus = expConn -> Terminate();

   printStatus("Stream Driver Terminated with status ", strStatus);
   printStatus("Export Driver Terminated with status ", expStatus);

   if ( strStatus >= TD_Error || expStatus >= TD_Error )
   {
     //Get Error Information
     if ( strStatus >= TD_Error )
     {
       cout << "\nError occured during Stream Terminate" << endl;
       strConn -> GetErrorInfo(&errorMsg, &errorType);
       if ( errorMsg != NULL )
       {
         cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
       } 
       else
       {
         cout << "No Error Info Available" << endl;
       }
     }

     if ( expStatus >= TD_Error )
     {
       cout << "Error occured during Export Terminate" << endl;
       expConn -> GetErrorInfo(&errorMsg, &errorType);
       if ( errorMsg != NULL )
       {
         cout << errorMsg << endl;
         cout << "Type: " << errorType << endl;
       }
       else
       {
         cout << "No Error Info Available" << endl;
       }
     }
   }

   /**********************************************
   * Clean Up
   **********************************************/

   cout << "\nDeleting objects" << endl;
   delete dmlGr;
   delete expSchema;
   delete strSchema;
   delete strConn;
   delete expConn;

   free(user_name);
   free(user_password);
   free(target_table);
   free(target_table_export);
   free(tdp_id);

   free(log_table);
   free(error_table);
   free(ins_statement);
   free(sel_statement);
   free(msg);

   cout << endl << "\n*** Export To Stream Complete ***" << endl;
}


/*************************** Utility Functions **
**************************************/

/***************************************
* Function sets input file name
***************************************/
void eventLog::setInput(char* fname){

	if( inputFileName != NULL )
	{
		delete inputFileName;
	}

	inputFileName = strdup(fname);
}


/***************************************
* Function sets output file name
***************************************/
void eventLog::setOutput(char* fname){
	if( outputFileName != NULL )
	{
		delete outputFileName;
	}

	outputFileName = strdup(fname);
}


/***************************************
* Function reads in a row of data
***************************************/
int eventLog::GetRowData(char* rowBuffer, size_t rowLength){

     unsigned short rlength;
       
	//Close File if End Signal Received
	if ( rowLength <= 0 )
	{
		if ( inputStream != NULL )
		{
			fclose(inputStream);
		}

		return -1;
	}

	//Open File if not already open
	if( inputStream == NULL )
	{
#ifndef __MVS__
		inputStream = fopen(inputFileName, "rb");
#else 
		inputStream = fopen(inputFileName, "rb, type=record");
#endif //__MVS__

		if( inputStream == NULL )
		{
			cout << "Error opening file " <<
 inputFileName << endl;
			return -1;
		}
	}

	//Read  the lenght of each record..read first two bytes
#ifndef __MVS__
	size_t recLen =fread(rowBuffer, 1, (size_t)2, inputStream);
#else
	size_t recLen =fread(rowBuffer, 1, rowLength, inputStream);
#endif //__MVS__

#ifndef __MVS__
	if ( recLen < 1 ){
#else
	if ( recLen < 2 ){
#endif //__MVS__
		if ( feof(inputStream) )  {
			cout << "End of file reached" << endl;
			if( inputStream != NULL ){
				fclose(inputStream);
			}
			return -1;
		}
		else{
			/* return some read error here and exit */
			cout << "read error" << endl;
			if( inputStream != NULL ){
				fclose(inputStream);
			}
			return -1;
		}
	}
     //read row of data with size rlength

#ifndef __MVS__
        rlength = *((unsigned short *)rowBuffer);
        size_t recoLen = fread(rowBuffer+2,1,(size_t)rlength+1,
inputStream);

	return 0;
#else
	return recLen;
#endif //__MVS__

}


/*****************************************************
* Function writes out a row of data in indicator mode
******************************************************/
int eventLog::WriteRowData(char* data, TD_Length dataLen){

	if( data != NULL )
	{
		//Open File if not already open
		if( outputStream == NULL )
		{
#ifndef __MVS__
			outputStream = fopen(outputFileName, "wb");
#else
			outputStream = fopen(outputFileName, "wb,\
 type=record");
#endif //__MVS__

			if( outputStream == NULL )
			{
				cout << "Error opening file " <<
 outputFileName << endl;
				return -1;
			}
		}

		unsigned long i = 0;
		unsigned short shortdataLen = (unsigned short)dataLen;
		char* rowBuffer = (char*)malloc(dataLen+20);

#ifndef __MVS__
		memcpy(rowBuffer, &shortdataLen, 2);

		for(i = 0; i < dataLen; i++)
		{
			rowBuffer[i+2] = data[i];
		}
		rowBuffer[i+2] = 0x0a;

		//fprintf (stream, "%s\n",rowBuffer+1); 
		// account for indicator byte

		fwrite(rowBuffer, dataLen+3, 1, outputStream);

#else
		for(i = 0; i < dataLen; i++)
		{
			rowBuffer[i] = data[i];
		}

		//fprintf (stream, "%s\n",rowBuffer+1);
 // account for indicator byte

		fwrite(rowBuffer, 1, dataLen, outputStream);
#endif //__MVS__

		free(rowBuffer);

	}else{
		if( outputStream != NULL )
		{
			fclose(outputStream);
		}
		return -1;
	}

	return 0;

}


/* DR143810 : BEGIN */
void eventLog::showEvent(Connection *conn, TD_EventType eventType)
{
   char *dataptr;
   TD_Length datalen;
   int ret;

   int val;
   char error[256];
   for(int i = 0; i < 256; i++)
     error[i] = 0;
  
   ret = conn -> GetEvent(eventType, &dataptr, &datalen);

   switch(eventType)
   {
     case TD_Evt_Version:  
              cout << "\n EVENT : TD_Evt_Version";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   TPTAPI Version   : " << *((char **)dataptr);
                cout << "\n   Operator Version : " << *((char **)(dataptr + sizeof(char **)));
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;
    
     case TD_Evt_ConnectStatus:  
              cout << "\n EVENT : TD_Evt_ConnectStatus";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Sessions requested : " << *((int *)dataptr);
                cout << "\n   Sessions connected : " << *((int *)(dataptr + sizeof(int)));
                cout << "\n   CLI Error          : " << *((int *)(dataptr + 2 * sizeof(int)));
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_DBSError:
              cout << "\n EVENT : TD_Evt_DBSError";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                val = *((int *)dataptr);
               
                cout << "\n   DBS Error  : " << val;
                if(val != 0 )
                {
                  memcpy(error, ( dataptr + sizeof(int) ), 255);
                  cout << "\n   Error desc : " << error;
                }
                else
                {
                  cout << "";
                }
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_CLIError:
              cout << "\n EVENT : TD_Evt_CLIError";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                val = *((int *)dataptr);

                cout << "\n   CLI Error  : " << val; 
                if(val != 0 )
                {
                  memcpy(error, ( dataptr + sizeof(int) ), 255);
                  cout << "\n   Error desc : " << error ;
                }
                else
                {
                  cout << "";
                }
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_BufferLayout:
              cout << "\n EVENT : TD_Evt_BufferLayout";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Maximum buffer size : " << *( (int *)(dataptr +  0 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Row header size     : " << *( (int *)(dataptr +  1 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Row length size     : " << *( (int *)(dataptr +  2 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Buffer trailer size : " << *( (int *)(dataptr +  3 * sizeof(int) ) ); /* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_RowCounts:
              cout << "\n EVENT : TD_Evt_RowCounts";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Rows received : " << *( (int *)(dataptr +  0 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Rows sent     : " << *( (int *)(dataptr +  1 * sizeof(int) ) ); /* TPT-23416 */

                int type = conn -> GetType();
                if( type == TD_LOAD || type == TD_UPDATE ) 
                {
                  cout << "\n   Rows applied  : " << *( (int *)(dataptr +  2 * sizeof(int) ) ); /* TPT-23416 */
                }
                else if(type == TD_STREAM) 
                {
                  cout << "\n   Rows that caused DBS error  : " << *( (int *)(dataptr +  2 * sizeof(int) ) ); /* TPT-23416 */
                }
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_RunStats:
              cout << "\n EVENT : TD_Evt_RunStats";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   SQL statements sent        : " << *( (int *)(dataptr +  0 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   DBS requests sent          : " << *( (int *)(dataptr +  1 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Rows received              : " << *( (int *)(dataptr +  2 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Rows sent                  : " << *( (int *)(dataptr +  3 * sizeof(int) ) ); /* TPT-23416 */
                cout << "\n   Rows that cuased DBS error : " << *( (int *)(dataptr +  4 * sizeof(int) ) ); /* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_MacroNames:
              cout << "\n EVENT : TD_Evt_MacroNames";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                int len = *( (int *)(dataptr +  0 * sizeof(int) ) );/* TPT-23416 */ 
                char name[255];
                memcpy(name, ( dataptr + sizeof(int) ), len);
     
                cout << "\n   Length of the macro : " << len;
                cout << "\n   Macro name          : " << name;
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_PackFactor:
              cout << "\n EVENT : TD_Evt_PackFactor";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Pack factor : " << *( (int *)(dataptr +  0 * sizeof(int) ) );/* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_BlockCount:
              cout << "\n EVENT : TD_Evt_BlockCount";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Total blocks from all select statements : " << *( (int *)(dataptr + 0 * sizeof(int) ) );/* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_ExportCount:
              cout << "\n EVENT : TD_Evt_ExportCount";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Rows exported : " << *( (int *)(dataptr + 0 * sizeof(int) ) );/* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_ApplyCount:
              cout << "\n EVENT : TD_Evt_ApplyCount";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   No. of rows inserted to DBS : " << *( (int *)(dataptr + 0 * sizeof(int) ) ); /* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_ErrorTable1:
              cout << "\n EVENT : TD_Evt_ErrorTable1";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   No. of rows in error table-1: " << *( (int *)(dataptr + 0 * sizeof(int) ) ); /* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_ErrorTable2:
              cout << "\n EVENT : TD_Evt_ErrorTable2";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   No. of rows in error table-2: " << *( (int *)(dataptr + 0 * sizeof(int) ) ); /* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;


     case TD_Evt_RowsCheckpointed:
              cout << "\n EVENT : TD_Evt_RowsCheckpointed";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   No. of rows checkpointed : " << *( (int *)(dataptr + 0 * sizeof(int) ) ); /* TPT-23416 */
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_ExitCode:
              cout << "\n EVENT : TD_Evt_ExitCode";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                cout << "\n   Driver exit code : " << *( (short *)dataptr );
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     case TD_Evt_CPUTime:
              cout << "\n EVENT : TD_Evt_CPUTime";
              if ( ret < TD_Error && ret != TD_Unavailable )
              {
                double tm;
                memcpy((char *)&tm, dataptr, sizeof(double));
                cout << "\n   CPU time : " << tm ;
              }
              else
              {
                printf("\n   Event not available. ");
              }
              break;

     default           : 
              cout << " \n   Event not found. \n\n";
              break;
   }
   cout <<"\n";

}

void eventLog::printInBlock(const char *str)    //TPT-17718
{
  int printChars = 70;
  int len = 0;
  int i = 0;
  int printFrom = 0;

  printf("\n");

  for(i = 0; i < printChars; i++)
    printf("*");

  printf("\n");

  len = strlen(str);
  printFrom = (printChars / 2) - (len / 2);
  
  for(i = 0; i < printChars; i++)
  {
    if( (i == 0) || (i == (printChars - 1)) )
      printf("*");
    else if( i == printFrom ) 
    {
      printf("%s", str);
      i = printFrom + len;
    }
    else
      printf(" ");
  }

  printf("\n");

  for(i = 0; i < printChars; i++)
    printf("*");

  printf("\n");

}

/* DR143810 : END */

/***********************************************
* Function prints string version of status code
************************************************/
void eventLog::printStatus(const char *msg, int code)  //TPT-17718
{
  cout << msg;

  switch(code)
  {
    case TD_Success:
                     cout << "TD_Success";
                     break;
    case TD_SYNC_Barrier:
                     cout << "TD_SYNC_Barrier";
                     break;
    case TD_SYNC_TELINFO:
                     cout << "TD_SYNC_TELINFO";
                     break;
    case TD_END_Method:
                     cout << "TD_END_Method";
                     break;
    case TD_Unavailable:
                     cout << "TD_Unavailable";
                     break;
    case TD_Error:
                     cout << "Error Code: TD_Error";
                     break;
    default:
                     cout << "Error Code: " << code;
                     break;
  }

  cout << endl;
}


/*****************************************************
* Function writes out a buffer of data in indicator mode
******************************************************/
int eventLog::WriteBufferData(const char *data, TD_Length dataLen)  //TPT-17718
{

  if( data != NULL )
  {
    //Open File if not already open
    if( outputStream == NULL )
    {
      outputStream = fopen(outputFileName, "wb");

      if( outputStream == NULL )
      {
        cout << "Error opening file " << outputFileName << endl;
        return -1;
      }
    }

    fwrite(data, dataLen, 1, outputStream);

  }
  else
  {
    if( outputStream != NULL )
    {
      fclose(outputStream);
    }
    return -1;
  }

  return 0;

}
