/*************************************************************************
*
* Copyright 2005-2010, 2012-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/

#include <stdio.h>

#ifndef __MVS__
#include "connection.h"
#else
#include "connecti.h"
#endif //__MVS__

//#include "exception.h"

using namespace teradata::client::API;

#if defined( __MVS__) || defined(WIN32)
#else 
#define _OSUNIX_
#endif

#if defined(__MVS__)
#define OS_MVS
#elif defined(WIN32) || defined(WIN64)
#define OS_WINDOWS
#elif defined(__sun)
#define OS_SOLARIS
#elif defined(_AIX)
#define OS_AIX
#elif defined(HPUX)
#define OS_HPUX
#elif defined(__linux__)
#define OS_LINUX
#elif defined(__HIGHC__) || defined(_ATT4)
#define OS_MPRAS
#elif defined(__APPLE__)    // TPT-11370
#define OS_OSX              // TPT-11370
#else
#error Building on a unknown platform
#endif

#if defined(OS_SOLARIS) || defined(OS_AIX) || defined(OS_HPUX) || defined(OS_LINUX) || defined(OS_MPRAS) || defined(OS_OSX) // TPT-11370
#define OS_UNIX
#endif

class OptionsManager;

typedef struct {
   int MaxMsgSize;
   int RowHeaderSize;
   int RowLengthSize;
   int BufTrailerSize;
} BufferLayout;

typedef struct {
   int RowsReceived;
   int RowsSent;
   int RowsApplied;
} RowCounts;

typedef struct {
   int SQLStmtsSent;
   int RequestsSent;
   int RowsReceived;
   int RowsSent;
   int DBSErrorRows;
} RunStats;

class eventLog
{
public:
	eventLog(OptionsManager * om);
	~eventLog(void);

	//Basic Tests
	void testLoad();
	void testExport();
	void testUpdate();
        void testExportToStream();

        // DR143810
        void showEvent(Connection *, TD_EventType);
    
	//Utility Functions
	void setInput(char*);
	void setOutput(char*);
	int GetRowData(char* rowBuffer, size_t rowLength);
	int WriteRowData(char* rowBuffer, TD_Length dataLen);
#ifdef __MVS__
	int getBufferSize();
#endif //__MVS__
     
        void printStatus(const char*,int);   //TPT-17718
        int WriteBufferData(const char* rowBuffer, TD_Length dataLen);  //TPT-17718

private:
        void printInBlock(const char *);   //TPT-17718

	FILE* inputStream;
	FILE* outputStream;
	char* inputFileName;
	char* outputFileName;
    OptionsManager * m_om;

};









