/*************************************************************************
*
* Copyright 2006-2009, 2012-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/

#include <stdio.h>
#include "connection.h"
//#include "exception.h"

#if defined( __MVS__) || defined(WIN32)
#else 
#define _OSUNIX_
#endif

#if defined(__MVS__)
#define OS_MVS
#elif defined(WIN32) || defined(WIN64)
#define OS_WINDOWS
#elif defined(__sun)
#define OS_SOLARIS
#elif defined(_AIX)
#define OS_AIX
#elif defined(HPUX)
#define OS_HPUX
#elif defined(__linux__)
#define OS_LINUX
#elif defined(__HIGHC__) || defined(_ATT4)
#define OS_MPRAS
#elif defined(__APPLE__)    // TPT-11370
#define OS_OSX              // TPT-11370
#else
#error Building on a unknown platform
#endif

#if defined(OS_SOLARIS) || defined(OS_AIX) || defined(OS_HPUX) || defined(OS_LINUX) || defined(OS_MPRAS) || defined(OS_OSX) // TPT-11370
#define OS_UNIX
#endif

typedef struct {
   int MaxMsgSize;
   int RowHeaderSize;
   int RowLengthSize;
   int BufTrailerSize;
} BufferLayout;

typedef struct {
   int RowsReceived;
   int RowsSent;
   int RowsApplied;
} RowCounts;

typedef struct {
   int SQLStmtsSent;
   int RequestsSent;
   int RowsReceived;
   int RowsSent;
   int DBSErrorRows;
} RunStats;

class OptionsManager;

class GenericDriver
{
public:
	GenericDriver(OptionsManager * om);
	~GenericDriver(void);

	//Basic Tests
	void testExportToLoad();
	void testExportToFile();
	void testExportToUpdate();
	void testExportToStream();
	void testExportToPutRow();

	//Utility Functions
	void printStatus(const char*,int);    //TPT-17718
	void setInput(char*);
	void setOutput(char*);
	int GetRowData(char* rowBuffer, size_t rowLength);
	int WriteRowData(char* rowBuffer, TD_Length dataLen);
	int WriteBufferData(char* rowBuffer, TD_Length dataLen);

private:

	FILE* inputStream;
	FILE* outputStream;
	char* inputFileName;
	char* outputFileName;
    OptionsManager * m_om;

};
