/*************************************************************************
*
* Title: DMLGroup Class Definition
*
*
* Copyright 2005-2009, 2014-2019 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
*
* Each DMLGroup has one or more DML Statements along with a set of DML Options. 
* The user is provided with a set of constants for the DML Options.
* These DML Options are processed by the DMLGroup class into a five bit string. The default
* setting for DML Options is 30. See AddDMLOption function for more. The DMLGroup
* name is set internally, not by the user.
*
* Purpose: 
*
* Store DML statements and process DML options.
*
* History Information
*
* Revision    Date     DCR    DID      Comments
* ----------- -------- ------ -------- ---------------------------------------
* 17.20.00.03 02152019 TPT-27753 MDV      Add attribute TD_TREATDBSRESTARTASFATAL
* 16.20.00.02 03062018 TPT-31781 SD186076 Add new attribute TD_ROLENAME
* 16.20.00.01 02052018 TPT-31769 SD186076 Add new attribute TD_TRANSFORM_GROUP
* 16.20.00.00 11092017 TPT-30258 DM186065 Add new attribute TD_TIME_ZONE_SESS_INFO
* 16.00.00.03 11072016 TPT-28942 vm185012  Add new attribute TD_JOBMAXROWSIZE
* 16.00.00.02 07072016 TPT-18584 pg255017 TPT API support for 1MB perm row feature
* 16.00.00.01 12312015 TPT-24264 pg255017 TPT API support for LDI feature.
* 16.00.00.00 07232015 TPT-24265 by255002 TPT API Support for DBS Unicode
*                                          Pass Through Feature
* 15.10.00.05 03022015 TPT-23015 vm185012 New event for QueryBand bytes
* 15.10.00.04 02272015 TPT-23356 by255002 NOSPOOL success notification via TPTAPI
* 15.10.00.03 12032014 TPT-21736 by255002 Report node connected/session through a TPTAPI event
* 15.10.00.02 12032014 TPT-21735 by255002 Report throughput per session as an event in TPTAPI
* 15.10.00.01 11192014 TPT-23389 by255002 Add new event type 'TD_BlockCount64' for TPTAPI
* 15.00.00.02 20032014 TPT-20873 by255002 TPTAPI : Export, Load, and Update operators
*                                         to support FastFail feature 
* 15.00.00.01 12262013 TPT-20211 kb186006 Copyright year change
* 15.00.00.00 08162013 TPT-18172 kb186006 Support statistical events in TPTAPI for Application
*                                         and Acquisition phase
* 14.10.00.02 03132012 TPT3525 by255002 RFC: Enhance TPTAPI to get the actual column
*                                       type (no translation of TIMESTAMP to CHAR) to
*                                       from Export Driver
* 14.10.00.01 02292012 TPT-10264 vm185012 Support TD_Evt_ApplyCount64.
* 14.10.00.00 03052012 TPT-388 by255002 MacroCharSet Support for TPTAPI
* 14.00.00.06 11082011 1951   vm185012 Support for 64 bit row counts
* 14.00.00.05 10212011 3504   by255002 support RATE and PERIODICITY
* 14.00.00.04 06092011 151071 vm185012 Attribute to disable TMSM
* 14.00.00.03 03012011 148301 vm185012 CLI220 hang issue
* 14.00.00.02 12092010 145813 jk186005 TELAPI_EXPORT has been added to export the functions
* 14.00.00.01 12082010 145669 vm185012 Add new event 'TD_Evt_OperVersion'
* 13.01.00.02 05122010 139831 vm185012 Add new event 'TD_Evt_MacroNames'
* 13.01.00.01 10122009 135707 TSL      Add TD_LOGSQL attribute
* 13.00.01.00 03182009 130830 vm185012 Re-Add ncr namespace
* 13.00.00.04 06132008 123343 jg185041 Add TD_DROPLOGTABLE and
*                                      TD_DROPWORKTABLE attributes
* 13.00.00.03 01232008 116135 ec185008 Update namespace
* 13.00.00.02 01172008 116137 jg185041 Add TD_APPENDERRORTABLE and
*                                      TD_DROPERRORTABLE attributes
* 13.00.00.01 01162008 117623 jg185041 Add RowsCheckpointed event
* 13.00.00.00 01042008 116134 kb185011 Add EXTRA ROWS DML Option
* 09.00.00.04 01122007 108496 jg185041 Add TD_IGNORE_MAX_DECIMAL_DIGITS attribute
* 09.00.00.03 11172006  89727 jg185041 Add network UTF16 support
* 09.00.00.02 11062006 105030 ls185040 Add Array Support
* 09.00.00.01 10192006 108140 kb185011 Add Query Band Support
* 09.00.00.00 08072006 105027 kb185011 Add GetBuffer Support
* 08.02.00.04 08282006 106760 ls185040 Change TD_Evt_Version number to 101
* 08.02.00.03 08242006 106760 ls185040 Add TD_Evt_Version to TD_EventType
* 08.02.00.02 04072006 98234  jg185041 Add Large Decimal/BIGINT Support
* 08.02.00.01 04072006 101112 kb185011 Add ReplicationOverride Support
* 08.02.00.00 03292006 102610 ls185040 Port to MP-RAS
* 01.00.00.13 11012005  97368 jg185041 Add current copyright notice
* 01.00.00.12 10052005  96274 kb185011 Remove use of generic typedefs
* 01.00.00.11 09152005  98513 lc185033 Added block loading
* 01.00.00.10 09072005  98509 ls185040 Added TD_ERROR_TABLE attribute for Stream operator
* 01.00.00.09 09012005  89728 jg185041 Update Event method support
* 01.00.00.08 08232005  96274 jg185041 Clean up shipped header files
* 01.00.00.07 07262005  89709 sp185045 Update for linux porting
* 01.00.00.06 06082005  96168 sl185022 Add typedefs for user integer data types
* 01.00.00.05 04122005  89681 jg185041 Add Serialize and Use options for Stream operator
* 01.00.00.04 03302005  94012 jg185041 Add TELAPI_EXPORT macro to enable dll linking
* 01.00.00.03 03282005  89756 kb185011 Remove old Exception Support
* 01.00.00.03 03282005  89756 kb185011 Replace All printf/cout with TELAPIDBG Macro
* 01.00.00.02 03282005  89698 kb185011 Add Class Copy Constructor
* 01.00.00.02 03282005  89698 kb185011 Add Include TelapiConstants.h
* 01.00.00.01 02212005  89680 kb185011 Add DML Option Support
* 01.00.00.01 02212005  89680 kb185011 Modify DMLGroup Interface
* 01.00.00.00 02012005  89676 dr185013 Initial Creation
*
*************************************************************************/

#ifndef DMLGROUP_H
#define DMLGROUP_H

/* DR96274 BEGIN */

//#include "types.h"
//#include "TelapiConstants.h" /* 89698 */
//#include "platform.h"                                           /* DR94012 */
//#include <vector>

/* DR89709  --START */
#if defined(__linux__) /*  DR 89709  START*/
 #include <string>  
#else  

#include <string.h>  
#endif /* DR 89709 END */
/* DR89709 ---END  */

/* DR102610 BEGIN */
#if defined(__HIGHC__) || defined(_ATT4)
#else
using namespace std;
#endif
/* DR102610 END */

// TelapiConstants.h

// *********************************************
// * Attribute Names for use with Attribute Class
// ********************************************** //

// TPT-1951 : BEGIN /* DR89728 BEGIN */

typedef enum {
   TD_Evt_BufferLayout = 1,                    // 1
   TD_Evt_ConnectStatus,                       // 2
   TD_Evt_RowCounts,                           // 3
   TD_Evt_BlockCount,                          // 4
   TD_Evt_ApplyCount,                          // 5
   TD_Evt_ExportCount,                         // 6
   TD_Evt_ErrorTable1,                         // 7
   TD_Evt_ErrorTable2,                         // 8
   TD_Evt_CLIError,                            // 9
   TD_Evt_DBSError,                            // 10
   TD_Evt_CPUTime,                             // 11
   TD_Evt_LastError,                           // 12
   TD_Evt_ExitCode,                            // 13
   TD_Evt_RunStats,                            // 14
   TD_Evt_UserCommand,                         // 15
   TD_Evt_RowsCheckpointed,                    // 16      // DR117623
   TD_Evt_BufferModeSupport,                   // 17
   TD_Evt_MacroNames,                          // 18      // DR139831
   TD_Evt_PackFactor,                          // 19      // DR119698
   TD_Evt_OperVersion,                         // 20      // DR145669

   TD_Evt_RowCounts64,                         // 21      // TPT-1951
   TD_Evt_RowsCheckpointed64,                  // 22      // TPT-1951
   TD_Evt_ExportCount64,                       // 23      // TPT-1951
   TD_Evt_RunStats64,                          // 24      // TPT-1951
   TD_Evt_ErrorTable1_64,                      // 25      // TPT-1951
   TD_Evt_ApplyCount64,                        // 26      // TPT-10264
   TD_Evt_BlockCount64,                        /* 27         TPT-23389 */
   TD_Evt_AcquisitionPhaseStats,               // 28      // TPT-18172
   TD_Evt_ApplicationPhaseStat,                // 29      // TPT-18172
   TD_Evt_LoadPhaseStats,                      // 30      // TPT-18172
   TD_Evt_ExportPhaseStats,                    // 31      // TPT-18172
   TD_Evt_SessionNodeId,                       /* 32         TPT-21735 */
   TD_Evt_SessionThrougput,                    /* 33         TPT-21736 */
   TD_Evt_ExportNoSpoolMode,                   /* 34         TPT-23356 */
   TD_Evt_QueryBandByteCount,                  /* 35         TPT-23015 */

   TD_Evt_RowsDiscarded=37,                    /* 37         TPT-18584 */
   TD_Evt_BufferLayout2,                       /* 38         TPT-18584 */

   TD_Evt_Version = 101
} TD_EventType;

// TPT-1951 : END /* DR89728 END */

typedef enum TD_Attribute {
	TD_SYSTEM_OPERATOR = 0,
	TD_USER_NAME,
	TD_USER_PASSWORD,
	TD_TDP_ID,
	TD_LOG_TABLE,
	TD_TARGET_TABLE,
	TD_ERROR_TABLE_1,
	TD_ERROR_TABLE_2,
	TD_ERROR_TABLE,                                          /* DR98509 */
	TD_BUFFERS,                                              /* DR97875 */
	TD_BUFFER_SIZE,
	TD_BUFFER_MODE,                                          /* DR98513 */
	TD_BUFFER_MAX_SIZE,                                      /* DR105027 */
	TD_BUFFER_HEADER_SIZE,                                   /* DR105027 */
	TD_BUFFER_LENGTH_SIZE,                                   /* DR105027 */
	TD_BUFFER_TRAILER_SIZE,                                  /* DR105027 */
	TD_ERROR_LIMIT,
	TD_MAX_SESSIONS,
	TD_MIN_SESSIONS,
	TD_RESTARTMODE,                                          /* DR96033 */
	TD_INSTANCE_NUM,                                         /* DR96033 */
	TD_MAX_INSTANCES,                                        /* DR96033 */
	TD_CHARSET,                                              /* DR96033 */
	TD_TENACITY_HOURS,
	TD_TENACITY_SLEEP,
	TD_ACCOUNT_ID,
	TD_DATE_FORM,
	TD_NOTIFY_EXIT,
	TD_NOTIFY_EXIT_IS_DLL,
	TD_NOTIFY_LEVEL,
	TD_NOTIFY_METHOD,
	TD_NOTIFY_STRING,	
	TD_PRIVATE_LOG_NAME,
	TD_TRACE_LEVEL,
	TD_TRACE_OUTPUT, /* DR89756 */
	TD_WORKINGDATABASE,
	TD_PAUSE_ACQ,
	TD_AMP_CHECK,
	TD_DELETE_TASK,
	TD_WORK_TABLE,
	TD_PACK,
	TD_PACKMAXIMUM,
	TD_MACRODATABASE,
	TD_ROBUST,
    TD_SELECT_STMT,
	TD_BLOCK_SIZE,
	TD_DATA_ENCRYPTION,                                         /* DR96622 */
	TD_LOGON_MECH,                                              /* DR96622 */
	TD_LOGON_MECH_DATA,                                         /* DR96622 */
	TD_WILDCARDINSERT,                                          /* DR96622 */
	TD_REPLICATION_OVERRIDE,                                    /* DR101112 */
	TD_MAX_DECIMAL_DIGITS,                                      /* DR98234 */
	TD_IGNORE_MAX_DECIMAL_DIGITS,                               /* DR108496 */
	TD_QUERY_BAND_SESS_INFO,                                    /* DR108140 */
	TD_ARRAYSUPPORT,                                            /* DR105030 */
	TD_MSG_ENCODING,                                            /* DR89727 */
    TD_APPENDERRORTABLE,                                        /* DR116137 */
    TD_DROPERRORTABLE,                                          /* DR116137 */
    TD_DROPLOGTABLE,                                            /* DR123343 */
    TD_DROPWORKTABLE,                                           /* DR123343 */
	TD_DROPMACRO,                                               /* DR122793 */
	TD_OUTLIMIT,                                                /* DR124975 */
	TD_SPOOLMODE,                                               /* DR124978 */
    TD_LOGSQL,                                                  /* DR135707 */
    TD_AUTORESTART,                                             /* DR148301 */
    TD_TMSMSUPPORT,                                             /* DR151071 */
    TD_RATE,                                                    /*TPT-3504*/
    TD_PERIODICITY,                                             /*TPT-3504*/
    TD_MACROCHARSET,                                             //TPT-388
    TD_RETN_ACT_DATATYPE,                                       /*TPT-3525*/
    TD_TASMFASTFAIL=72,                                         //TPT-20873
    TD_UNICODEPASSTHROUGH,                                       /* TPT-24265 */
    TD_LDILOADJOB,                                            /* TPT-24264 */
    TD_LDILOADGROUP,                                          /* TPT-24264 */
    TD_PAUSELDI,                                               /* TPT-24264 */
    TD_LDIMACROTABLENAME,                                       /* TPT-24264 */
    TD_DISCARDLARGEROW,                                         // TPT-18584
    TD_JOBMAXROWSIZE,                                           // TPT-28942
    TD_TIME_ZONE_SESS_INFO,                                     // TPT-30258
    TD_TRANSFORM_GROUP,                                         // TPT-31769
    TD_ROLENAME,                                                // TPT-31781
    TD_TREATDBSRESTARTASFATAL                                   //TPT-27753
}TD_Attribute;

// ***************************************
// * DMLOptions for use with DMLGroup Class
// *************************************** //

typedef enum DMLOption {
	MARK_DUPLICATE_ROWS = 0,		//0
	MARK_DUPLICATE_INSERT_ROWS,		//1
	MARK_DUPLICATE_UPDATE_ROWS,		//2
	MARK_MISSING_ROWS,				//3
	MARK_MISSING_UPDATE_ROWS,		//4
	MARK_MISSING_DELETE_ROWS,		//5
	IGNORE_DUPLICATE_ROWS,			//6
	IGNORE_DUPLICATE_INSERT_ROWS,	//7
	IGNORE_DUPLICATE_UPDATE_ROWS,	//8
	IGNORE_MISSING_ROWS,			//9
	IGNORE_MISSING_UPDATE_ROWS,		//10
	IGNORE_MISSING_DELETE_ROWS,		//11
	INSERT_FOR_MISSING_UPDATE_ROWS,	//12
	MARK_EXTRA_ROWS,                //13 - DR116134
	MARK_EXTRA_UPDATE_ROWS,         //14 - DR116134
	MARK_EXTRA_DELETE_ROWS,         //15 - DR116134
	IGNORE_EXTRA_ROWS,              //16 - DR116134
	IGNORE_EXTRA_UPDATE_ROWS,       //17 - DR116134
	IGNORE_EXTRA_DELETE_ROWS        //18 - DR116134
}DMLOption;

namespace ncr {         /* DR130830 */ 
namespace teradata {
namespace client {
namespace API {

/* DR96274 END */

class DMLGroup
{
	//General Class Functions
	public:
		TELAPI_EXPORT DMLGroup();                               /* DR94012 */
		TELAPI_EXPORT DMLGroup(TD_Encoding encoding);           /* DR89727 */
		TELAPI_EXPORT ~DMLGroup();                              /* DR94012 */
		TELAPI_EXPORT DMLGroup(const DMLGroup& g); /* DR89698 */ 

	//Functions available to User
	public:
		TELAPI_EXPORT void	AddStatement(char* statement);      /* DR94012 */
		TELAPI_EXPORT void	AddDMLOption(DMLOption option);     /* DR94012 */
		TELAPI_EXPORT void	AddSerializeOn(TD_Count argcount, ...);  /* DR89681 */ /* DR96168 */
		TELAPI_EXPORT void	AddUseList(TD_Count argcount, ...);      /* DR89681 */ /* DR96168 */
		TELAPI_EXPORT void  AddArraySupport(char *arraysupport); /* DR105030 */

    //Functions used Internally
	public:
		TELAPI_EXPORT void	SetLabel(char* label);
		TELAPI_EXPORT char*	GetStatement(TD_Index index);				    /* DR96168 */
		TELAPI_EXPORT char*	GetSerializeColumn(TD_Index index);             /* DR96168 */ /* DR89681 */
		TELAPI_EXPORT char*	GetUseColumn(TD_Index index);                   /* DR96168 */ /* DR89681 */
		TELAPI_EXPORT TD_Count	GetStatementCount();						/* DR96168 */
		TELAPI_EXPORT TD_Count	GetSerializeCount();                        /* DR96168 */ /* DR89681 */
		TELAPI_EXPORT TD_Count	GetUseCount();                              /* DR96168 */ /* DR89681 */
		TELAPI_EXPORT TD_Count	GetOptions();								/* DR96168 */
		TELAPI_EXPORT vector<char*> GetDmlStatements();
		TELAPI_EXPORT vector<char*> GetSerializeList();                       /* DR89681 */
		TELAPI_EXPORT vector<char*> GetUseList();                             /* DR89681 */
		TELAPI_EXPORT char *        GetArraySupport();                        /* DR105030 */
		TELAPI_EXPORT TD_Encoding     GetEncoding();                          /* DR89727 */
		TELAPI_EXPORT void	SetName(string name);
		TELAPI_EXPORT string  GetName();

	private:
		void	dmlOptionsRule( char* key, char *key1, char *key2, char* key3 );

	private:
		char* mLabel;
		vector<char*> mStatementList;                      /* DR102610 */
		vector<char*> mSerializeList;                      /* DR89681 */ /* DR102610 */
		vector<char*> mUseList;                            /* DR89681 */ /* DR102610 */
		char         *mArraySupport;                       /* DR105030 */
		int mOptions;
		string mName;
		TD_Encoding   mEncoding;                           /* DR89727 */
		void         *mDMLGroupVars;                       /* DR89727 */

	//Variables for DML Options
	private:
		int _flag[2][4]; /* DR116134 */  /* DR130830 */ 
		int _upsert;
		int _specified;
		int _defaults;

};

/* DR96274 BEGIN */

} // namespace API
} // namespace client
} // namespace teradata
} // namespace ncr
/* DR96274 END */
namespace teradata = ncr::teradata;    /* DR130830 */ 
#endif

