/*************************************************************************
*
* Title: TEL API DMLGroupX Class Definition
*
*
* Copyright 2010-2011, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
*
* This class enables access to the DMLGroup class when the TPTAPI 
* library is loaded dynamically during the runtime.
* The functionality of this class methods will be identical to the those
* of the DMLGroup class.
*
* Purpose: 
*
* Store DML statements and process DML options.
*
* History Information
*
* Revision    Date     DCR    DID      Comments
* ----------- -------- ------ -------- ---------------------------------------
* 15.00.00.00 12262013 TPT-20211 kb186006  Copyright year change
* 14.00.00.00 12092010 145813 jk186005 TELAPI_EXPORT has been added to export the functions
* 13.10.00.01 05122010 142720 mv121803 Initial created
*
*************************************************************************/

#ifndef DMLGROUPX_H
#define DMLGROUPX_H

#include "DMLGroup.h"

namespace ncr {      
namespace teradata {
namespace client {
namespace API {

class DMLGroupX : public DMLGroup
{
    //General Class Functions
    public:
	TELAPI_EXPORT DMLGroupX();
	TELAPI_EXPORT DMLGroupX(TD_Encoding encoding);
	TELAPI_EXPORT virtual ~DMLGroupX();   

    //Functions available to User
    public:
	TELAPI_EXPORT virtual void  AddStatement(char* statement);
	TELAPI_EXPORT virtual void  AddDMLOption(DMLOption option);
	TELAPI_EXPORT virtual void  AddSerializeOn(TD_Count argcount, ...); 
	TELAPI_EXPORT virtual void  AddUseList(TD_Count argcount, ...); 
	TELAPI_EXPORT virtual void  AddArraySupport(char *arraysupport);

    //Functions used Internally
    public:
	TELAPI_EXPORT void	SetLabel(char* label);
	TELAPI_EXPORT char*	GetStatement(TD_Index index);
	TELAPI_EXPORT char*	GetSerializeColumn(TD_Index index);
	TELAPI_EXPORT char*	GetUseColumn(TD_Index index);
	TELAPI_EXPORT TD_Count       GetStatementCount();
	TELAPI_EXPORT TD_Count       GetSerializeCount();
	TELAPI_EXPORT TD_Count       GetUseCount();     
	TELAPI_EXPORT TD_Count       GetOptions(); 
	TELAPI_EXPORT vector<char*>  GetDmlStatements();
	TELAPI_EXPORT vector<char*>  GetSerializeList();
	TELAPI_EXPORT vector<char*>  GetUseList();     
	TELAPI_EXPORT char *         GetArraySupport();

	TELAPI_EXPORT TD_Encoding     GetEncoding(); 
        TELAPI_EXPORT void*           GetDMLGroupVars();  //DR148806

	TELAPI_EXPORT void	SetName(string name);
	TELAPI_EXPORT string  GetName();

};

//
// Class Factory "C" functions 
//
typedef DMLGroupX *   (*pfn_newDMLGroupX)(); 
typedef DMLGroupX *   (*pfn_newDMLGroupXEncoding)(TD_Encoding); 
typedef DMLGroupX *   (*pfn_copyDMLGroupX)(DMLGroupX *, DMLGroupX *); 
typedef void          (*pfn_delDMLGroupX)(DMLGroupX *); 


} // namespace API
} // namespace client
} // namespace teradata
} // namespace ncr 

namespace teradata = ncr::teradata;
#endif

