/*************************************************************************
*
* Title: TEL API ConnectionX Class Definition
*
*
* Copyright 2010-2012, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Description:
*
* This class enables access to the ConnectionX class when the TPTAPI library
* is loaded dynamically during the runtime.
* The functionality of this class methods will be identical to the those
* of the ConnectionX class.
*
* Purpose:
*
* Store table schema information.
*
* History Information
*
* Revision    Date     DCR    DID      Comments
* ----------- -------- ------ -------- ---------------------------------------
* 15.00.00.00 12262013 TPT-20211 kb186006  Copyright year change
* 14.10.00.01 05172012 TPT-11431 vm185012  TPTAPI to add support for gcc 4.3 Linux compiler
* 14.00.00.02 04152011 148806 mv121803 Dynamic Schema doesn't work when TELAPI
*                                      library is loaded dynamically
* 14.00.00.01 10292010 144824 mv121803 Support MVS
* 13.10.00.01 05212010 142720 mv121803 Initial Creation
*
*************************************************************************/

#ifndef CONNECTIONX_H
#define CONNECTIONX_H

#ifndef __MVS__                                              //DR144824
  #include "connection.h"
  #include "schemaX.h"
  #include "DMLGroupX.h"

#else                                                       //DR144824

  #include "CONNECTI.H"                                     //DR144824
  #include "SCHEMAX.H"                                      //DR144824
  #include "DMLGRPX.H"                                      //DR144824

#endif                                                      //DR144824

namespace ncr {   
namespace teradata {
namespace client {
namespace API {   

class ConnectionX : public Connection
{
    public:
	TELAPI_EXPORT ConnectionX();
	TELAPI_EXPORT ConnectionX(TD_Encoding encoding);
	TELAPI_EXPORT virtual ~ConnectionX();   

       /**
        * Initiate connection to DB and the protocol
	* @return TD_StatusCode
	*/
	TELAPI_EXPORT virtual TD_StatusCode  Initiate();

       /**
	* send data row to the DB
	* @param data
	* @param length of data
	* @return TD_StatusCode
	*/
        /* DR96168 Dr96160 */
	TELAPI_EXPORT virtual TD_StatusCode  PutRow(char*  data, 
                                                    TD_Length  length);

       /**
	* send buffer to the DB
	* @param buffer pointer
	* @param length of buffer
	* @return TD_StatusCode
	*/
	TELAPI_EXPORT virtual TD_StatusCode  PutBuffer(char*  data, 
                                                       TD_Length  length, 
                                                       TD_Boolean indicator);

       /**
	* get data row from the DB
	* @param data address
	* @param address of length of data
	* @return TD_StatusCode
	*/
        /* DR96168 DR96160 DR100926 */
	TELAPI_EXPORT virtual TD_StatusCode GetRow(char** data, 
                                                   TD_Length* length);  

       /**
        * get data buffer from the DB
        * @param data address
        * @param address of length of buffer
        * @return TD_StatusCode
        */
        /* DR105027 */
        TELAPI_EXPORT virtual TD_StatusCode GetBuffer(char** data, 
                                                      TD_Length* length);

       /**
	* Terminate connection to DB and clean loaded libraries from memory
	* @return TD_StatusCode
	*/
	TELAPI_EXPORT virtual TD_StatusCode  Terminate(); /* DR96160 */

       /**
	* save checkpoint record
	* @param checkpoint record address
	* @param address of length of checkpoint record
	* @return TD_StatusCode
	*/
	TELAPI_EXPORT virtual TD_StatusCode  Checkpoint(char **data, 
                                                     TD_Length* len);

       /**
	* restart operation using checkpoint record
	* @param checkpoint record
	* @param length of checkpoint record
	* @return TD_StatusCode
	*/
	TELAPI_EXPORT virtual TD_StatusCode  Restart(char *data, 
                                                   TD_Length len);
	TELAPI_EXPORT virtual TD_StatusCode  GetEvent(TD_EventType eventType,
		                                 char** eventData,
                                                 TD_Length* eventDataLen,
						 TD_Index eventIndex = 0); 

	/**
	 * Allows users to pass values to operator
	 * @param eventType 
	 * @param eventData
	 * @param eventDataLen 
	 * @param eventIndex
	 * @return TD_StatusCode
	 */
	TELAPI_EXPORT virtual TD_StatusCode  PutEvent(TD_EventType eventType,
		                                 char* eventData,
						 TD_Length eventDataLen,
                                                 TD_Index eventIndex = 0);

       /**
	* end of acquisition phase
	* @param data
	* @param length of data
	* @return StatusCode
	*/
	TELAPI_EXPORT virtual TD_StatusCode  EndAcquisition();
		  
       /**
	* signal start of application phase
	* @param data
	* @param length of data
	* @return StatusCode
	*/
        TELAPI_EXPORT virtual TD_StatusCode  ApplyRows(); 

	TELAPI_EXPORT virtual void GetErrorInfo(char** errorMsg, 
                                             TD_ErrorType* errorType); 

	TELAPI_EXPORT virtual void  AddAttribute(TD_Attribute, 
                                             const char* value);   // TPT-11431
	TELAPI_EXPORT virtual void  AddAttribute(TD_Attribute, 
                                            TD_IntValue value);
	TELAPI_EXPORT virtual void  AddArrayAttribute(TD_Attribute,
						TD_Count count,
						char* value, ... ); 
	TELAPI_EXPORT virtual void  AddArrayAttribute(TD_Attribute,
						  TD_Count count,       
                                                  TD_IntValue value, ... );
	TELAPI_EXPORT virtual void  AddSchema(SchemaX*);

	TELAPI_EXPORT virtual TD_StatusCode   GetSchema(SchemaX** schema); 
	TELAPI_EXPORT virtual TD_StatusCode   AddDMLGroup(DMLGroupX*,TD_Index*);

	TELAPI_EXPORT virtual int    GetType();
	TELAPI_EXPORT virtual int    GetCheckPointInterval();

	TELAPI_EXPORT virtual TD_StatusCode  UseDMLGroups(TD_Index*, TD_Count);

	TELAPI_EXPORT virtual TD_StatusCode PutTELINFO(char* TELINFO,
				                       TD_Length length);
        TELAPI_EXPORT virtual TD_StatusCode GetTELINFO(char** TELINFO,
			             		       TD_Length* length);

        TELAPI_EXPORT virtual void  AddSchema(SchemaX **schema,      //DR148806
                                              char *colInfo,         //DR148806
                                              TD_Length numCols);    //DR148806

        TELAPI_EXPORT virtual TD_StatusCode  GetSchema(char **cols,  //DR148806
                                                 TD_Count *numCols,  //DR148806
                                                 TD_Length *length); //DR148806

        // The following methods are commented out because they 
        // has not implemented in the Connection class
        // TELAPI_EXPORT  char*  Serialize();
        // TELAPI_EXPORT  void   Deserialize(char* xml);
        // TELAPI_EXPORT Attribute*  GetAttribute(char* name);

   private:                                                         //DR148806
        vector<SchemaX*>  mSchemaXList;                             //DR148806

};

// Class Factory "C" functions which inistantiates the ConnectionX class
typedef ConnectionX* (*pfn_newConnectionX)();    
typedef ConnectionX* (*pfn_newConnectionXEncoding)(TD_Encoding);    
typedef ConnectionX* (*pfn_copyConnectionX)(ConnectionX *, ConnectionX *);    
typedef void         (*pfn_delConnectionX)(ConnectionX *); 

} // namespace API
} // namespace client
} // namespace teradata
} // namespace ncr                    // DR130830
namespace teradata = ncr::teradata;   // DR130830
#endif

