/*************************************************************************
*
* Copyright 2005-2008, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
*************************************************************************/


#ifndef __MVS__
#include "synchronize.h"
#else
#include "synchron.h"
#endif//__MVS__

synchronize::synchronize()
{
  barriercode[0] = SYNC_BARRIER1;
  barriercode[1] = SYNC_BARRIER2;
  barriercode[2] = SYNC_BARRIER3;
  barriernum = 0;
  initialized = false;
  telinfosync = false;
  aborted = false;

  // DR119676 : Begin
  schemasync = false;
  // DR119676 : End
}

synchronize::~synchronize()
{
  terminate();
}

int synchronize::set_numslaves(int num)
{
  if(num == -1)
    {
      master = false;
    }
  else
    {
      master = true;
    }
  numslaves = num;
  return 0;
}

int synchronize::set_masterip(string ip)
{
  masterip = ip;
  return 0;
}

int synchronize::initialize()
{
  if(initialized == false)
    {
      if(master)
	return m_initialize();
      else
	return s_initialize();
    }
  return 0;
}

int synchronize::getid()
{
  if(master)
    return -1;
  else
    return myid;
}

int synchronize::m_initialize()
{
  //create socket
#if !defined(UNIX) && !defined(__MVS__)
  wVersionRequested = MAKEWORD( 2, 2 );
  err = WSAStartup( wVersionRequested, &wsaData );
#endif
 //TPT-10838 BEGIN
         /*******************************************
          *For Compatibility of IPv4 and IPv6: BEGIN*
          *******************************************/

         int rc;
         struct addrinfo hints, *res = NULL;
         const char* server = masterip.c_str();
         char servport[] = "41234";

         memset(&hints, 0x00, sizeof(hints));
         hints.ai_flags    = AI_NUMERICHOST;
         hints.ai_family   = AF_UNSPEC;
         hints.ai_socktype = SOCK_STREAM;

         rc = getaddrinfo(server, servport, &hints, &res);
         if (rc != 0)
         {
                 cout << "Host not found" << endl;
                 return SYNC_ERROR;
         }

         sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
         if (sock < 0)
         {
                 cout << "Socket creation failed" << endl;
                 return SYNC_ERROR;
         }

         if(bind(sock, res->ai_addr, res->ai_addrlen) < 0)
         {
                 cout << "Socket bind error" << endl;
                printf("\n The Bind error:  %s\n", strerror(errno));
                 return SYNC_ERROR;
         }

         /*******************************************
          *For Compatibility of IPv4 and IPv6: END  *
          *******************************************/
 //TPT-10838 END
  listen(sock, numslaves);
  //listen for slaves
#if defined(UNIX) || defined(__MVS__)
  int clientsocket;
#else
  SOCKET clientsocket;
#endif
  struct sockaddr_in6 clientaddr;  //IPv6 //TPT-10838
  int clilen = sizeof(struct sockaddr_in6);//IPv6   //TPT-10838
  cout << " Waiting for " << numslaves << " slaves\n";    
  /* DR99279 */
  for(int i = 0; i < numslaves; i++)
    {
#if defined(UNIX) || defined(__MVS__)
#ifdef HPUX
      clientsocket = accept(sock,
			    (struct sockaddr *) &clientaddr,
			    &clilen);
#elif defined(__HIGHC__)||defined(_ATT4)
      clientsocket = accept(sock, (struct sockaddr *) &clientaddr,
 &clilen);
#else
      clientsocket = accept(sock,
			    (struct sockaddr *) &clientaddr,
			    (socklen_t *) &clilen);
#endif
#else
      clientsocket = accept(sock,
			    (struct sockaddr *) &clientaddr,
			    &clilen);
#endif
      cout << " Accepted client " << i << "\n";         
    /* DR99279 */
      slavesockets.push_back(clientsocket);
      slaveaddress.push_back(clientaddr);
      slavestatus.push_back(SLAVE_RUNNING);
      //send id to slave
      char tmp = i;
      send(slavesockets[i], &tmp, sizeof(char), 0);
    }
  initialized = true;
  return 0;
}

int synchronize::s_initialize()
{
  //connect to master
#if !defined(UNIX) && !defined(__MVS__)
  wVersionRequested = MAKEWORD( 2, 2 );
  err = WSAStartup( wVersionRequested, &wsaData );
#endif
 //TPT-10838 BEGIN
         /*******************************************
          *For Compatibility of IPv4 and IPv6: BEGIN*
          *******************************************/

         int rc;
         struct addrinfo hints, *res = NULL;
         const char* server = masterip.c_str();
         char servport[] = "41234";

         memset(&hints, 0x00, sizeof(hints));
         hints.ai_flags    = AI_NUMERICHOST;
         hints.ai_family   = AF_UNSPEC;
         hints.ai_socktype = SOCK_STREAM;

         rc = getaddrinfo(server, servport, &hints, &res);
         if (rc != 0)
         {
                 cout << "Host not found" << endl;
                 return SYNC_ERROR;
         }

         sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
         if (sock < 0)
         {
                 cout << "Socket creation failed" << endl;
                 return SYNC_ERROR;
         }

         cout << "Connecting to " << masterip << "\n";
         if(connect(sock, res->ai_addr, res->ai_addrlen) != 0)
         {
                 cout << "Connection failed " << errno << endl;
                 return -1;
         }

         /*******************************************
          *For Compatibility of IPv4 and IPv6: END  *
          *******************************************/
 //TPT-10838 END
  cout << " Connection accepted\n";
  //get id
  char tmp;
  recv(sock, &tmp, sizeof(char), 0);
  myid = (int) tmp; 
  initialized = true;
  return 0;
}

// DR119676 : Begin

int synchronize::syncschema(Connection *conn)
{
  if(master)
    return m_syncschema(conn);
  else
    return s_barrier(conn);

}

int synchronize::m_syncschema(Connection *conn)
{
  int returnval;
  schemasync = true;       //switch to have barrier send telinfo
  // command code
  returnval = m_barrier();  //wait for slaves
  if(returnval != 0)
    return SYNC_ERROR;
  
  //get schema
  TD_Length schema_length;
  char *cols;
  TD_Count numCols;

  int temp;
  returnval = conn -> GetSchema(&cols, &numCols, &schema_length);
  temp = htonl(schema_length);

  //send schema length
  broadcast(reinterpret_cast<char *> (&temp), sizeof(int));
  cout << " Sending SCHEMA LENGTH of size " << schema_length << "\n";

  //send number of columns
  temp = htonl(numCols);
  broadcast(reinterpret_cast<char *> (&temp), sizeof(int));
  cout << " Sending Number of Columns in SCHEMA-" << numCols << "\n";

  //send the schema(with all columns info.)
  broadcast(cols, schema_length);
  returnval = m_barrier();

  if(returnval != 0)
    return SYNC_ERROR;
  cout << " SCHEMA sync finished\n";
  return 0;
}

int synchronize::s_syncschema(Connection *conn)
{
  int returnval;
 
  int temp;
  char *schemabuffer;
  TD_Length schema_length;
  TD_Count numCols;
  Schema* tschema = NULL;
  // recieving the size of the schema
  recv(sock, (char *)&temp, sizeof(int), 0);
  schema_length = ntohl(temp);
  cout << " Receiving SCHEMA of size " << schema_length << "\n";

  // recieving the number of columns in the schema
  recv(sock, (char *)&temp, sizeof(int), 0);
  numCols = ntohl(temp);
  cout << " Receiving Number of Columns in SCHEMA-" << numCols << "\n";

  // recieving the schema 
  schemabuffer = new char[schema_length];
  recv(sock, schemabuffer, schema_length, 0);
  
  // Adding the schema to Connection object
  conn -> AddSchema(&tschema, schemabuffer, numCols);

  delete [] schemabuffer;

  returnval = s_barrier(conn);
  if(returnval != 0)
    return SYNC_ERROR;

  cout << " SCHEMA sync finished\n";                     
  // DR99279 
  return 0;
}

// DR119676 : End

int synchronize::synctelinfo(Connection *conn)
{
  if(master)
    return m_synctelinfo(conn);
  else
    return s_barrier(conn);
}

int synchronize::m_synctelinfo(Connection *conn)
{
  int returnval;
  telinfosync = true;       //switch to have barrier send telinfo
  // command code
  returnval = m_barrier();  //wait for slaves
  if(returnval != 0)
    return SYNC_ERROR;
  //get telinfo
  TD_Length telinfo_length;
  char *telinfo;
  int temp;
  returnval = conn->GetTELINFO(&telinfo, &telinfo_length);
  temp = htonl(telinfo_length);
  //send telinfo
  broadcast(reinterpret_cast<char*> (&temp), sizeof(int));
  cout << " Sending TELINFO of size " << telinfo_length << "\n";
  broadcast(telinfo, telinfo_length);
  returnval = m_barrier();
  if(returnval != 0)
    return SYNC_ERROR;
  cout << " TELINFO sync finished\n";
  return 0;
}

int synchronize::s_synctelinfo(Connection *conn)
{
  int returnval;
  //  returnval = s_barrier();
  //  if(returnval != 0)
  //  return SYNC_ERROR;
  //get telinfo
  int temp;
  char *telinfobuffer;
  TD_Length telinfo_length;
  recv(sock, (char*)&temp, sizeof(int), 0);
  telinfo_length = ntohl(temp);
  cout << " Receiving TELINFO of size " << telinfo_length << "\n";
  telinfobuffer = new char[telinfo_length];
  //TPT-17996 Begin
  for ( int total = 0; total < telinfo_length ; )
  {              
    int num = recv(sock, telinfobuffer + total, telinfo_length - total, 0);   
    total = total + num;                                                     
    //printf("\n  Received bytes = %d and bytes to be received = %d\n", num, \
telinfo_length - total);
    //if ( total == telinfo_length )                                         
    //printf("\n Total Bytes received before PutTELINFO call:%d\n\n", total);
  }                                                                         
  //TPT-17996 End
  conn->PutTELINFO(telinfobuffer, telinfo_length);
  delete [] telinfobuffer;
  returnval = s_barrier(conn);
  if(returnval != 0)
    return SYNC_ERROR;
  cout << " TELINFO sync finished\n";                     
  /* DR99279 */
  return 0;
}

int synchronize::switchbarrier()
  /*
    switch to the next barrier to indicate that it is now
    on to the next phase of the test.  Master will sync
    all slaves so that they start a new phase at the same time
  */
{
  barriernum = (barriernum+1)%3;
  cout << " Switching to barrier " << (int)barriercode[barriernum] 
<< "\n";
  return 0;
}

int synchronize::barrier(Connection *conn)
{
  if(master)
    return m_barrier();
  else
    return s_barrier(conn);
}

/*
int synchronize::barrier(Connection *conn, Connection *connL, Schema *schema)
{
  if(master)
    return m_barrier();
  else
    return s_barrier(conn, connL, schema);
}
*/

int synchronize::m_barrier()
{
  //wait for each of the slaves
  char recvcode;
  int bytes;
  for(int i = 0; i < numslaves; i++)
    {
      recvcode = 0;
      if(slavestatus[i] == SLAVE_RUNNING)
	//if slave is currently running wait for slave
        {
          bytes = recv(slavesockets[i], &recvcode, sizeof(char), 0);
          if(bytes == 0)  //did not recieve any data from slave
            //slave has most likely aborted
            {
              slavestatus[i] = SYNC_ABORT;
              cout << "Error synchronizing with slave " << i 
<< " aborting\n";
              //abort the rest of the slaves
              abort();
              return SYNC_ERROR;
            }
          if(recvcode >= SYNC_BARRIER1 && recvcode <= SYNC_BARRIER3)
            {
              if(recvcode == barriernum) //master and slave is synced
                {
                  slavestatus[i] = recvcode;
                }
              else if(recvcode == (3+barriernum-1)%3)
 //slave is behind master
                {
                  cout << "Synchronizing slave " << i << "\n";
                  recvcode = SYNC_CONT;
                  send(slavesockets[i], &recvcode, sizeof(char), 0);
                  i--;  //sync this slave again
                }
              else //master is behind slave
                {
                  slavestatus[i] = recvcode;
                  return 0;
                }
            }
          else  //error synchronizing assume slave has aborted
            {
              slavestatus[i] = SYNC_ABORT; //mark as aborted
              cout << "Error synchronizing with slave " << i 
<< " aborting\n";
              abort();
              return SYNC_ERROR;
            }
        }
      if(slavestatus[i] == (barriernum+1)%3) //slave ahead of master
        {
          cout << " The master instance fell behind\n";    
 /* DR99279 */   
          return 0;
        }
    }
	if(schemasync == true)           // DR119676
		recvcode = SYNC_SCHEMA;      // DR119676
	else                             // DR119676
      if(telinfosync == true)
        recvcode = SYNC_TELINFO;
      else
        recvcode = SYNC_CONT;

  schemasync = false;                // DR119676
  telinfosync = false;
  for(int j = 0; j < numslaves; j++)
    {
      slavestatus[j] = SLAVE_RUNNING;
      send(slavesockets[j], &recvcode, sizeof(char), 0); 
    }
  return 0;
}

int synchronize::broadcast(char *data, int len)
{
  for(int i =0; i < numslaves; i++)
    {
      send(slavesockets[i], data, len, 0);
    }
  return 0;
}

int synchronize::sendmasterdata(char *data, int len)
{
  int temp = htonl(len);
#if defined(__HIGHC__) || defined(_ATT4)
  send(sock, (char*)&temp, sizeof(int), 0);
#else
  send(sock, (const char*)&temp, sizeof(int), 0);
#endif
  send(sock, data, len, 0);
  return 0;
}

char **synchronize::getslavedata()
{
  char **sdata;
  sdata = new char* [numslaves+1];
  for(int i = 0; i < numslaves; i++)
    {
      int tmp;
      //std::cout << "recving from slave " << i << "\n";
      recv(slavesockets[i], (char*)&tmp, sizeof(int), 0);
      int len = ntohl(tmp);
      sdata[i] = new char[len];
      recv(slavesockets[i], sdata[i], len, 0);
    }
  sdata[numslaves] = NULL;
  return sdata;
}

int synchronize::s_barrier(Connection *conn)
{
  char recvcode;
  send(sock, barriercode+barriernum, sizeof(char), 0);
 //signal master
  recv(sock, &recvcode, sizeof(char), 0); //wait for signal
  //from master
  //recvcode is the action master wants slave to take
  if(recvcode == SYNC_CONT) //continue processing
  {
    return 0;
  }
  else 
  {
    if(recvcode == SYNC_TELINFO) //wait for telinfo and update
    {
      return s_synctelinfo(conn);
    }
    else 
    {
      if (recvcode == SYNC_SCHEMA)
      {
        return s_syncschema(conn);
      }
      else  //else master has errored
      {
        cout << " Master aborted\n";                          
        abort();
        return SYNC_ERROR;
      }
    }
  }
}


/*
int synchronize::s_barrier(Connection *conn, Connection *connL, Schema *schema)
{
  char recvcode;
  send(sock, barriercode+barriernum, sizeof(char), 0);
 //signal master
  recv(sock, &recvcode, sizeof(char), 0); //wait for signal
  //from master
  //recvcode is the action master wants slave to take
  if(recvcode == SYNC_CONT) //continue processing
    {
      return 0;
    }

  else if(recvcode == SYNC_SCHEMA)
  {
	  return s_syncschema(conn, schema);
  }

  // DR119676 : End

  else if(recvcode == SYNC_TELINFO) //wait for telinfo and update
    {
      return s_synctelinfo(conn);
    }
  else  //else master has errored
    {
      cout << " Master aborted\n";                          
      abort();
      return SYNC_ERROR;
    }
}
*/

int synchronize::abort()
{
  if(!aborted)  //each node can only abort once
    {
      aborted = true;
      if(!master)
	{
	  //send abort signal to master
	  char code = SYNC_ABORT;
	  send(sock, &code, sizeof(char), 0);
	}
      else
	{
	  char code = SYNC_ABORT;
	  //wait for signals from slaves
	  for(int i = 0; i < slavesockets.size(); i++)
	    {
	      if(slavestatus[i] != SYNC_ABORT)  
//if slave is not aborted
		//if slave has not aborted abort the
		{
		  cout << "Signaling slave " << i << " to abort\n";
		  send(slavesockets[i], &code, sizeof(char), 0);
		}
	    }
	}
    }
  return 0;
}

int synchronize::terminate()
{
  if(master)
    {
      for(int i = 0; i < numslaves && (unsigned int)i <
 slavesockets.size(); i++)
        {
#if defined(UNIX) || defined(__MVS__)
          close(slavesockets[i]);
#else
          closesocket(slavesockets[i]);
#endif
	}
    }
#if defined(UNIX) || defined(__MVS__)
  close(sock);
#else
  closesocket(sock);
#endif
  return 0;
}








