/*************************************************************************
*
* Copyright 2005-2011, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
* Multi_node sample
* Load operator sends 100 rows from inputFile to the Database.
* Update operator sends 100 rows from inputFile to the Database.
* Stream operator sends 100 rows from inputFile to the Database.
* Export operator sends 100 rows from Database to outputFile.
* Here some of the rows transfered by "master" and some of the
* rows transfered by "slave".
*
*************************************************************************/

#ifndef __MVS__
#include "multitest.h"
#include "OptionsManager.h"
#include "stdio.h"
#include "stdlib.h"
#include "time.h"
#else
#include "PTCMULTI.h"
#include "PTCOPTMA.h"
#endif//__MVS__

multitest::multitest(OptionsManager * om): m_om(om)
{
}

multitest::~multitest()
{
}

int multitest::set_masterip(string ip)
{
  masterip = ip;
  return mysync.set_masterip(ip);
}

int multitest::set_numslaves(int num)
{
  numslaves = num;
  return 0;
}

int multitest::set_master(bool flag)
{
  master = flag;
  if(flag == true)
    return mysync.set_numslaves(numslaves);
  else
    return mysync.set_numslaves(-1);
}

int multitest::set_inputfile(string name)
{
  inputfile = name;
  return 0;
}

int multitest::set_outputfile(string name)
{
  outputfile = name;
  return 0;
}

int multitest::runload(int sessions)
{
  time_t now;
  time(&now);
  syncerr = 0;

  if(mysync.initialize() != 0)
  {
    cout << "ERROR communicating to slave or master\n";
    return -1;
  }

  conn = new Connection;

  //set attributes
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            TEST TO RUN = RUNLOAD            " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *error_table_1 = strdup(m_om -> getErrorTable1().c_str());
  char *error_table_2 = strdup(m_om -> getErrorTable2().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());

  char *dml_statment  = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);", target_table);
 
  cout << "\nTest Description      :Multiple sample" << endl;
  cout << "Opertype              :LOAD" << endl;
  cout << "Date                  :" << ctime(&now);
  cout << "TDP ID                :" << tdp_id << endl;
  cout << "TDP username          :" << user_name << endl;
  cout << "TDP password          :<not displayable>" << endl;
  cout << "Target Table          :" << target_table << endl;
  cout << "Input file            :" << inputfile << endl;
  cout << "DML Statement         :" << dml_statment << endl;

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_LOAD);
  
  /*
  if(master)
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"master.txt");  //TPT-17718
  }
  else
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"slave.txt");  //TPT-17718
  }
  conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, TD_GENERAL, NULL);
  */
  
  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  
  if(master) //if master set session
  {
    /* DR99279 */
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }
  
  conn -> AddAttribute(TD_MAX_INSTANCES, numslaves+1);
  conn -> AddAttribute(TD_INSTANCE_NUM, mysync.getid()+2);

  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddAttribute(TD_ERROR_TABLE_1, error_table_1);
  conn -> AddAttribute(TD_ERROR_TABLE_2, error_table_2);

  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);
  //delete schema;

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;
  DMLGroup* dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statment);
  conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate Connection
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              LOAD INITIATE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_initiate() != 0)                             
  {
    cout << "ERROR in Connection Initiate\n";        
    /* DR99279 */
    delete conn;
    return -1;
  }
  cout << "Load operator initiated Successfully" << endl;
  
  //PutRow
  if(mysync.barrier(conn) != 0)
  {
    aborttest();
    return -1;
  }
  
  /**********************************************
  * Query Version Number
  **********************************************/
  char *dataptr;
  TD_Length datalen;
  int queryStatus = 0;

  queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              VERSION INFO                  " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "TPTAPI Version  : ";
    cout << *((char**)dataptr) << endl;
    cout << "Operator Version: ";
    cout << *((char**)(dataptr + sizeof(char**))) << endl;
  }
  else
  {
    printf("TD_Evt_Version event failed with status = %d", queryStatus);
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            ACQUISITION  PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_putrow() != 0)                              
  {
    cout << "ERROR in Connection PutRow\n";         
    /* DR99279 */
    aborttest();
    return -1;
  }

  if(api_endacquisition() != 0)                            
  {
    cout << "ERROR in Connection EndAcquisition\n";    
    /* DR99279 */
    aborttest();
    return -1;
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "           APPLICATION  PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_applyrows() != 0)                                 
  {
    cout << "ERROR in Connection ApplyRows\n";          
    /* DR99279 */
    aborttest();
    return -1;
  }
  
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "             TERMINATION  PHASE              " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_terminate() != 0)                                   
  {
    cout << "ERROR in Connection Terminate\n";          
    /* DR99279 */
    aborttest();
    return -1;
  }
  
  delete conn;
  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(log_table);
  free(error_table_1);
  free(error_table_2);
  free(dml_statment);
  return 0;
}

int multitest::runexport(int sessions)
{
  time_t now;
  time(&now);

  if(mysync.initialize() != 0)
  {
    cout << "ERROR communicating to slave or master\n";
    return -1;
  }

  conn = new Connection;
  //set attributes
  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());

  char *sel_statment  = (char*)malloc(1024);
  sprintf(sel_statment, "sel col1, col2(CHAR(96)) from %s;", target_table);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "          TEST TO RUN = RUNEXPORT            " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  cout << "\nTest Description      :Multiple sample" << endl;
  cout << "Opertype              :EXPORT" << endl;
  cout << "Date                  :" << ctime(&now);
  cout << "TDP ID                :" << tdp_id << endl;
  cout << "TDP username          :" << user_name << endl;
  cout << "TDP password          :<not displayable>" << endl;
  cout << "Target Table          :" << target_table << endl;
  cout << "Output file           :" << outputfile << endl;
  cout << "DML statement         :" << sel_statment << endl;

  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_EXPORT);

  /*
  if(master)  //trace output for only master
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT,(char*)"m_export.txt");  //TPT-17718
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
    TD_GENERAL, NULL);
  }
  */

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_SELECT_STMT, sel_statment);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);

  conn -> AddAttribute(TD_INSTANCE_NUM, mysync.getid()+2);
  conn -> AddAttribute(TD_MAX_INSTANCES, numslaves+1);

  if(master) //if master set session
  {
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }

  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"output");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);

  /**********************************************
  * Initiate Connection
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "               INITIATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_initiate() != 0)                                    
  /* DR97368 */
  {
    cout << "ERROR in Connection Initiate\n";          
    /* DR99279 */
    delete conn;
    return -1;
  }
  
  /**********************************************
  * Query Version Number
  **********************************************/
  char *dataptr;
  TD_Length datalen;
  int queryStatus = 0;

  queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              VERSION INFO                  " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "TPTAPI Version  : ";
    cout << *((char**)dataptr) << endl;
    cout << "Operator Version: ";
    cout << *((char**)(dataptr + sizeof(char**))) << endl;
  }
  else
  {
    printf("TD_Evt_Version event failed with status = %d", queryStatus);
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            ACQUISITION  PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_getrow() != 0)                                       
  /* DR97368 */
  {
    cout << "ERROR in Connection GetRow\n";             
    /* DR99279 */
    aborttest();
    return -1;
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "             TERMINATION  PHASE              " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_terminate() != 0)                                    
  /* DR97368 */
  {
    cout << "ERROR in Connection Terminate\n";          
    /* DR99279 */
    aborttest();
    return -1;
  }

  delete conn;
  return 0;
}

int multitest::runupdate(int sessions)
{
  time_t now;
  time(&now);

  if(mysync.initialize() != 0)
  {
    cout << "ERROR communicating to slave or master\n";
    return -1;
  }

  conn = new Connection;
  //set attributes
  char * tdp_id         = strdup(m_om -> getTdpId().c_str());
  char * user_name      = strdup(m_om -> getUserName().c_str());
  char * user_password  = strdup(m_om -> getUserPassword().c_str());

  char * log_table      = strdup(m_om -> getLogTable().c_str());
  char * target_tableA  = strdup(m_om -> getTargetTable(0).c_str());
  char * target_tableB  = strdup(m_om -> getTargetTable(1).c_str());
  char * work_tableA    = strdup(m_om -> getWorkTable(0).c_str());
  char * work_tableB    = strdup(m_om -> getWorkTable(1).c_str());
  char * error_table_1A = strdup(m_om -> getErrorTable1(0).c_str());
  char * error_table_1B = strdup(m_om -> getErrorTable1(1).c_str());
  char * error_table_2A = strdup(m_om -> getErrorTable2(0).c_str());
  char * error_table_2B = strdup(m_om -> getErrorTable2(1).c_str());

  char * dml_statmentA  = (char*)malloc(1024);
  sprintf(dml_statmentA, "INSERT INTO %s( :col1, :col2);", target_tableA);
  char * dml_statmentB  = (char*)malloc(1024);
  sprintf(dml_statmentB, "INSERT INTO %s( :col1, :col2);", target_tableB);
  
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "          TEST TO RUN = RUNUPDATE            " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "\nTest Description      :Multiple sample" << endl;
  cout << "Opertype              :UPDATE" << endl;
  cout << "Date                  :" << ctime(&now);
  cout << "TDP ID                :" << tdp_id << endl;
  cout << "TDP username          :" << user_name << endl;
  cout << "TDP password          :<not displayable>" << endl;
  cout << "Target TableA         :" << target_tableA << endl;
  cout << "Target TableB         :" << target_tableB << endl;
  cout << "Input file            :" << inputfile << endl;
  cout << "DML statmentA         :" << dml_statmentA << endl;
  cout << "DML statmentB         :" << dml_statmentB << endl;

  
  /**********************************************
  * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_UPDATE);

  /*
  if(master)  //trace output for only master
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"updatetrace.txt");//TPT-17718
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
    TD_GENERAL, NULL);
  }
  */

  /**********************************************
  * attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD,user_password);

  conn -> AddAttribute(TD_MAX_INSTANCES, numslaves+1);
  conn -> AddAttribute(TD_INSTANCE_NUM, mysync.getid()+2);
 
  if(master) //if master set session
  {
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }
  
  conn -> AddAttribute(TD_LOG_TABLE, log_table);
  conn -> AddArrayAttribute(TD_WORK_TABLE  , 2, work_tableA, \
  work_tableB, NULL);
  conn -> AddArrayAttribute(TD_TARGET_TABLE, 2, target_tableA,\
  target_tableB, NULL); 
  conn -> AddArrayAttribute(TD_ERROR_TABLE_1, 2, error_table_1A,\
  error_table_1B, NULL);
  conn -> AddArrayAttribute(TD_ERROR_TABLE_2, 2, error_table_2A,\
  error_table_2B, NULL);
  
  /**********************************************
  * DML Groups
  **********************************************/

  Schema *schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);
  
  TD_Index dmlGroup1, dmlGroup2;
  DMLGroup *dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statmentA);
  dmlGr -> AddDMLOption(MARK_DUPLICATE_ROWS);
  conn -> AddDMLGroup(dmlGr, &dmlGroup1);
  //cout << "dml group1 add status: " << conn -> AddDMLGroup(dmlGr,
  //&dmlGroup1) << "\n";
  
  DMLGroup *dmlGr2 = new DMLGroup();
  dmlGr2 -> AddStatement(dml_statmentB);
  dmlGr2 -> AddDMLOption(IGNORE_DUPLICATE_ROWS);
  conn -> AddDMLGroup(dmlGr2, &dmlGroup2);
  //cout << "dml group2 add status: " << conn -> AddDMLGroup(dmlGr2,
  //&dmlGroup2) << "\n";

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "               INITIATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_initiate() != 0)                                      
  /* DR97368 */
  {
    cout << "ERROR in Connection Initiate\n";           
    /* DR99279 */
    delete conn;
    return -1;
  }

  //PutRow
  //set the dml group based on id
  
  if((mysync.getid()+2)%2 == 0)
  {
    cout << "Using DML Group 1\n";
    conn -> UseDMLGroups(&dmlGroup1, 1);
  }
  else
  {
    cout << "Using DML Group 2\n";
    conn -> UseDMLGroups(&dmlGroup2, 1);
  }

  if(mysync.barrier(conn) != 0)
  {
    aborttest();
    return -1;
  }
  
  /**********************************************
  * Query Version Number
  **********************************************/
  char *dataptr;
  TD_Length datalen;
  int queryStatus = 0;

  queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              VERSION INFO                  " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "TPTAPI Version  : ";
    cout << *((char**)dataptr) << endl;
    cout << "Operator Version: ";
    cout << *((char**)(dataptr + sizeof(char**))) << endl;
  }
  else
  {
    printf("TD_Evt_Version event failed with status = %d", queryStatus);
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            ACQUISITION  PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_putrow() != 0)                                       
  /* DR97368 */
  {
    cout << "ERROR in Connection PutRow\n";            
    /* DR99279 */
    aborttest();
    return -1;
  }

  //End Acquisition

  if(api_endacquisition() != 0)                                
  /* DR97368 */
  {
    cout << "ERROR in Connection EndAcquisition\n";      
    /* DR99279 */
    aborttest();
    return -1;
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "           APPLICATION  PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_applyrows() != 0)                                     
  /* DR97368 */
  {
    cout << "ERROR in Connection ApplyRows\n";          
    /* DR99279 */
    aborttest();
    return -1;
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "             TERMINATION  PHASE              " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_terminate() != 0)                                  
  /* DR97368 */
  {
    cout << "ERROR in Connection Terminate\n";          
    /* DR99279 */
    aborttest();
    return -1;
  }

  delete conn;
  free(user_name);
  free(user_password);
  free(tdp_id);
  free(log_table);
  free(target_tableA);
  free(target_tableB);
  free(work_tableA);
  free(work_tableB);
  free(error_table_1A);
  free(error_table_1B);
  free(error_table_2A);
  free(error_table_2B);
  free(dml_statmentA);
  free(dml_statmentB);
  return 0;  
}

int multitest::runstream(int sessions)
{
  time_t now;
  time(&now);

  if(mysync.initialize() != 0)
  {
    cout << "ERROR communicating to slave or master\n";
    return -1;
  }

  conn = new Connection;
  //set attributes
  char *tdp_id        = strdup(m_om -> getTdpId().c_str());
  char *user_name     = strdup(m_om -> getUserName().c_str());
  char *user_password = strdup(m_om -> getUserPassword().c_str());
  char *target_table  = strdup(m_om -> getTargetTable().c_str());
  char *log_table     = strdup(m_om -> getLogTable().c_str());

  char *dml_statment  = (char*)malloc(1024);
  sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);", target_table);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            TEST TO RUN = RUNLOAD            " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "\nTest Description      :Multiple sample" << endl;
  cout << "Opertype              :STREAM" << endl;
  cout << "Date                  :" << ctime(&now);
  cout << "TDP ID                :" << tdp_id << endl;
  cout << "TDP username          :" << user_name << endl;
  cout << "TDP password          :<not displayable>" << endl;
  cout << "Target Table          :" << target_table << endl;
  cout << "Input file            :" << inputfile << endl;
  cout << "DML Statement         :" << dml_statment << endl;  

  /**********************************************
   * Set Operator Type and Trace/Log Levels
  **********************************************/
  conn -> AddAttribute(TD_SYSTEM_OPERATOR, TD_STREAM);

  /*
  if(master)
  {
    conn -> AddAttribute(TD_TRACE_OUTPUT, (char*)"streamlog.txt");  //TPT-17718
    conn -> AddArrayAttribute(TD_TRACE_LEVEL, 2, TD_OPER_ALL, \
    TD_GENERAL, NULL);
  }
  */

  /**********************************************
  * Add Attributes
  **********************************************/
  conn -> AddAttribute(TD_TDP_ID, tdp_id);
  conn -> AddAttribute(TD_USER_NAME, user_name);
  conn -> AddAttribute(TD_USER_PASSWORD, user_password);
  conn -> AddAttribute(TD_TARGET_TABLE, target_table);
  conn -> AddAttribute(TD_LOG_TABLE, log_table);

  if(master) //if master set session
  {
    conn -> AddAttribute(TD_MAX_SESSIONS, sessions);
    conn -> AddAttribute(TD_MIN_SESSIONS, sessions);
  }
 
  conn -> AddAttribute(TD_MAX_INSTANCES, numslaves+1);
  conn -> AddAttribute(TD_INSTANCE_NUM, mysync.getid()+2);
  
  //conn -> AddAttribute(TD_ERROR_TABLE_1, "tdload_e1");
  //conn -> AddAttribute(TD_ERROR_TABLE_2, "tdload_e2");

  /**********************************************
  * Add Schema
  **********************************************/
  Schema * schema = new Schema((char*)"input");  //TPT-17718
  schema -> AddColumn((char*)"col1", TD_INTEGER, 4);  //TPT-17718
  schema -> AddColumn((char*)"col2", TD_CHAR, 96);  //TPT-17718
  conn -> AddSchema(schema);
  //delete schema;

  /**********************************************
  * Add DMLGroups
  **********************************************/
  TD_Index dmlGroupIndex = 0;
  DMLGroup* dmlGr = new DMLGroup();
  dmlGr -> AddStatement(dml_statment);
  dmlGr -> AddDMLOption(MARK_DUPLICATE_ROWS);
  conn -> AddDMLGroup(dmlGr, &dmlGroupIndex);

  /**********************************************
  * Initiate Connection
  **********************************************/
  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "               INITIATE PHASE                " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_initiate() != 0)                                      
  /* DR97368 */
  {
    cout << "ERROR in Connection Initiate\n";           
    /* DR99279 */
    aborttest();
    return -1;
  }

  //PutRow

  if(mysync.barrier(conn) != 0)
  {
    aborttest();
    return -1;
  }
  
  /**********************************************
  * Query Version Number
  **********************************************/
  char *dataptr;
  TD_Length datalen;
  int queryStatus = 0;

  queryStatus = conn -> GetEvent(TD_Evt_Version, &dataptr, &datalen);

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "              VERSION INFO                  " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if ( queryStatus < TD_Error && queryStatus != TD_Unavailable )
  {
    cout << "TPTAPI Version  : ";
    cout << *((char**)dataptr) << endl;
    cout << "Operator Version: ";
    cout << *((char**)(dataptr + sizeof(char**))) << endl;
  }
  else
  {
    printf("TD_Evt_Version event failed with status = %d", queryStatus);
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "            ACQUISITION  PHASE               " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_putrow() != 0)                                       
  /* DR97368 */
  {
    cout << "ERROR in Connection PutRow\n";             
    /* DR99279 */
    aborttest();
    return -1;
  }

  //End Acquisition

  if(api_endacquisition() != 0)                               
  /* DR97368 */
  {
    cout << "ERROR in Connection EndAcquisition\n";     
    /* DR99279 */
    aborttest();
    return -1;
  }

  cout << "\n%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;
  cout << "             TERMINATION  PHASE              " << endl;
  cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << endl;

  if(api_terminate() != 0)                                     
  /* DR97368 */
  {
    cout << "ERROR in Connection Terminate\n";          
    /* DR99279 */
    aborttest();
    return -1;
  }

  delete conn;
  free(user_name);
  free(user_password);
  free(target_table);
  free(tdp_id);
  free(log_table);
  free(dml_statment);
  return 0;
}

int multitest::process_returnvalue(int val)
{
  switch(val)
  {
    case TD_SYNC_Barrier:
      cout << "Received TD_SYNC_Barrier\n";                
      /* DR99279 */
      return mysync.barrier(conn);
      break;
    case TD_SYNC_TELINFO:
      cout << "Received TD_SYNC_TELINFO\n";               
      /* DR99279 */
      return mysync.synctelinfo(conn);
      break;
    case TD_END_Method:
      return mysync.switchbarrier();
      cout << "Received TD_END_Method\n";                  
      /* DR99279 */
      break;
    default:
      cout << "ERROR processing return code:" << val << "\n";

    char* errorMsg = NULL;
    TD_ErrorType errorType;

    conn -> GetErrorInfo(&errorMsg, &errorType);
    if (errorMsg != NULL) 
    {
      cout << errorMsg << endl;
      cout << "Type: " << errorType << endl;
    }
    else
    {
      cout << "No Error Info Available" << endl;
    }
    break;
  }
  return 0;
}

int multitest::api_initiate()                                  
/* DR97368 */
{
  int reval = TD_END_Method + 1;
  while(reval != TD_END_Method)
  {
    if(mysync.barrier(conn) != 0)
    {
      return -1;
    }
       
    reval = conn -> Initiate();
    if(reval >= TD_Error)
    {
      cout << "Connection Initiate returned error code: " << reval << "\n";
      return -1;
    }
	
    if(process_returnvalue(reval) != 0)
    {
      return -1;
    }
  }
	
  cout << "Connection Initiate finished\n";            
  /* DR99279 */
  return 0;
}

int multitest::api_putrow()                                    
/* DR97368 */
{
  //cout << "sending row\n";
  TD_StatusCode reval = TD_Success;
  rowsent = bytessent = 0;
  char *rowbuffer;
  char namebuffer[255];
  unsigned short int rowlen;

  if(master)
  {
    sprintf(namebuffer, "%s", inputfile.c_str());
  }
  else
  {
#if defined(__MVS__)
    sprintf(namebuffer, "%s%i", inputfile.c_str(),mysync.getid()+1);
    //on MVS, add number to input dataset name 
    //'XX.XX.XX(AB)'1 to 'XX.XX.XX.(AB1)'
    int nbLen = strlen(namebuffer);
    if (namebuffer[nbLen-3] == ')') 
    {
      char lastChar = namebuffer[nbLen-1];
      namebuffer[nbLen-1] = namebuffer[nbLen-2];
      namebuffer[nbLen-2] = namebuffer[nbLen-3];
      namebuffer[nbLen-3]=lastChar;
    }
#else
    sprintf(namebuffer, "%s.%i", inputfile.c_str(),mysync.getid()+1); 
#endif
  }
  if(file.openinput(namebuffer) != 0)
  {
    cout << "ERROR opening file\n";
    return -1;
  }
  while(1)
  {
    if((rowsent % 100000 == 0) && (rowsent > 0))
    {
      cout << "Sending " << rowsent << " rows\n";
    }
    if(file.getrow(&rowbuffer, rowlen) == 0)
    //if(rowsent < 2500000)
    {
      reval = conn->PutRow(rowbuffer, rowlen);
      if(reval == TD_Success)
      {
        rowsent++;
      }
      else if(reval >= TD_Error)
      { 
        cout << "Connection PutRow errored on iteration " << rowsent << "\n";
        return -1;
      }
      else
      {
        if(process_returnvalue(reval) != 0)
        return -1;
      } 
    }
    else
    {
      break;
    }
  }
  cout << "Sent " << rowsent << " rows\n";
  file.closeinput();
  return 0;
}

int multitest::api_getrow()                                   
/* DR97368 */
{
  //cout << "getting row\n";
  TD_StatusCode reval = TD_Success;
  rowrecv = 0;
  bytesrecv = 0;
  char *data;
  TD_Length datalen;
  char namebuff[255];

  if(master)
  {
      sprintf(namebuff, "%s", outputfile.c_str());
  }
  else
  {
#if defined(__MVS__)
    sprintf(namebuff, "%s%i", outputfile.c_str(),mysync.getid()+1); 
#else
    sprintf(namebuff, "%s.%i", outputfile.c_str(),mysync.getid()+1);
#endif
  }

  if(file.openoutput(namebuff) != 0)
  {
    cout << "ERROR opening file\n";
    return -1;
  }
  while(reval != TD_END_Method)
  {
    //get row
    //process return value if not success
    reval = conn -> GetRow(&data, &datalen);
    if(reval == TD_Success)
    {
      //write the row
      if(rowrecv % 100000 == 0)
      {
        if(rowrecv != 0)
        {
          rowrecv++;
          cout << "Exported " << rowrecv << " rows\n";
        }
        else
        {
          cout << "Exported first row\n";
          rowrecv++;
        }
      }
      else
      rowrecv++;
      file.writerow(data, (unsigned short int)datalen);
    }
    else if(reval >= TD_Error)
    return -1;
    else
    {
      if(process_returnvalue(reval) != 0)
      return -1;
    }
  }
  cout << "Exported " << rowrecv << " rows\n";            
  /* DR99279 */
  file.closeoutput();
  return 0;
}

int multitest::api_endacquisition()                           
/* DR97368 */
{
  //cout << "ending acquisition\n";                      
  /* DR99279 */
  TD_StatusCode reval = TD_Success;
  while(reval != TD_END_Method)
  {
    if(mysync.barrier(conn) != 0)
      return -1;
      reval = conn -> EndAcquisition();
      if(reval >= TD_Error)
      {
        return -1;
      }
      if(process_returnvalue(reval) != 0)
	return -1;
  }
  cout << "Connection EndAcquisition complete\n";         
  /* DR99279 */
  return 0;
}


int multitest::api_applyrows()                               
   /* DR97368 */
{
  //cout << "apply rows\n";
  TD_StatusCode reval = TD_Success;
  while(reval != TD_END_Method)
  {
      if(mysync.barrier(conn) != 0)
	return -1;
      reval = conn -> ApplyRows();
      if(reval >= TD_Error)
      {
	 return -1;
      }
      if(process_returnvalue(reval)  != 0)
      return -1;
  }
  cout << "Connection ApplyRows complete\n";              
  /* DR99279 */
  return 0;
}


int multitest::api_terminate()                                
/* DR97368 */
{
  //cout << "terminating connection object\n";
  TD_StatusCode reval = TD_Success;
  while(reval != TD_END_Method)
  {
      if(mysync.barrier(conn) != 0)
	return -1;
      reval = conn -> Terminate();
      if(reval >= TD_Error)
      {
        return -1;
      }
      if(process_returnvalue(reval) != 0)
	return -1;
    }
  cout << "Connection terminated\n";
  return 0;
}


void multitest::aborttest()
{
  char *errorMsg = NULL;
  TD_ErrorType errorType;

  cout << "Test failed\n";
  mysync.abort();
  cout << "Terminating with return code " << conn->Terminate() <<  "\n";

  conn -> GetErrorInfo(&errorMsg,&errorType);
  if( errorMsg != NULL )
  {
    cout << errorMsg << endl;
    cout << "Type: " << errorType << endl;
  }
  else
  { 
    cout << "No Error Info Available" << endl;
  }
  //delete conn;
}










