/*******************************************************
******************
*
* Copyright 2005-2011, 2013-2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
****************************************************************
*********/


// TelapiThreadTest.cpp : Defines the entry point for the
// console application.
//

//#include "stdafx.h"
#ifndef __MVS__
#include "common.h"
#include "ThreadTest.h"
#include "OptionsManager.h"
#else
#include "common.h"
#include "PTCTHDTE.h"
#include "PTCOPTMA.h"
#endif

#define _XOPEN_SOURCE_EXTENDED 1
#include <string.h>

// Platform definitions

#if defined(WIN32) || defined(WIN64)
#define OS_WINDOWS
#elif defined(__sun)
#define OS_SOLARIS
#elif defined(__linux__)
#define OS_LINUX
#elif defined(HPUX)
#define OS_HPUX
#elif defined(_AIX)
#define OS_AIX
#endif

int main( int argc,char * argv[] )
{
    OptionsManager * om = new OptionsManager();
    if (argc != 2) {
        std::cout << "Usage:\n";
        std::cout << "TelapiThreadTest <config file>\n";
        return -1;
    }
    else 
    {
        om->readOptionsFromFile(argv[1]);
    }

	ThreadTestParameters* params = NULL;
	ThreadTest* t = NULL;
	Schema* schema = NULL;
	DMLGroup* dmlGr = NULL;
	DMLGroup* dmlGr2 = NULL;
	char input;
	char * user_name = strdup(om->getUserName().c_str());
	char * user_password = strdup(om->getUserPassword().c_str());
	char * target_table = strdup(om->getTargetTable().c_str());
	char * target_tableA = strdup(om->getTargetTable(0).c_str());
	char * target_tableB = strdup(om->getTargetTable(1).c_str());
	char * tdp_id = strdup(om->getTdpId().c_str());
	char * file_name = strdup(om->getInputFileName().c_str());
	char * outfile_name = strdup(om->getOutputFileName().c_str());
	char * dml_statment = (char*)malloc(1024);
	char * dml_statmentA = (char*)malloc(1024);
	char * dml_statmentB = (char*)malloc(1024);

	int testToRun = om->getOperType();


	switch ( testToRun ){

		case LOAD_TEST:
			/***********************************
***********
			* Uses Load to load 100 rows
			*******************************
***************/
			params = new ThreadTestParameters(TD_LOAD,4);
			params->SetLogon(user_name,user_password,
tdp_id);
			params->SetFile(file_name);
			params->SetTable(1,target_table);
			/* SetCheckpoint - num of rows/buffers
 to checkpoint at : restart tries */
			params->SetCheckpoint(10,1);
			params->SetSessions(4,4);
			/* SetBufferMode - 1 - on, 0 - off :
 then num of rows per buffer */
			//params->SetBufferMode(1,10); 

			schema = new Schema((char*)"input");  //TPT-17718
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_CHAR,96);  //TPT-17718
			params->mSchemaList.push_back(schema);

            sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);"
, om->getTargetTable().c_str());
			dmlGr = new DMLGroup();
			dmlGr->AddStatement(dml_statment);
			params->mDMLGroupList.push_back(dmlGr);

			t = new ThreadTest(params);
			t->runLoad();
			break;

		case EXPORT_TEST:
			/*************************************
*********
			* Uses Export to export from target table
			***************************************
*******/
			params = new ThreadTestParameters(TD_EXPORT,
4);
			params->SetLogon(user_name,user_password,
tdp_id);
			params->SetFile(outfile_name);
			params->SetTable(1,target_table);
			params->SetSessions(4,4);

            sprintf(dml_statment, "sel * from %s;",
 om->getTargetTable().c_str());
			params->AddAttribute(TD_SELECT_STMT,1,dml_statment);

			schema = new Schema((char*)"input");  //TPT-17718
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_VARCHAR,32000);  //TPT-17718
			params->mSchemaList.push_back(schema);

			t = new ThreadTest(params);
			t->runExport();
			break;

		case UPDATE_TEST:
			/*************************************
*********
			* Uses Update to load 50 rows into one table
			* and then 50 rows into another table
			**************************************
********/
			params =
 new ThreadTestParameters(TD_UPDATE,4);
			params->SetLogon(user_name,
user_password,tdp_id);
			params->SetFile(file_name);
			params->SetTable(2,
target_tableA,target_tableB);
			/* SetCheckpoint - num of rows/buffers to
 checkpoint at : restart tries */
			params->SetCheckpoint(10,1);
			params->SetSessions(4,4);
			/* SetBufferMode - 1 - on, 0 - off :
 then num of rows per buffer */
			//params->SetBufferMode(1,10); 

			schema = new Schema((char*)"input");  //TPT-17718
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_CHAR,96);  //TPT-17718
			params->mSchemaList.push_back(schema);

            sprintf(dml_statmentA, "INSERT INTO %s( :col1, :col2);",
 om->getTargetTable(0).c_str());
            sprintf(dml_statmentB, "INSERT INTO %s( :col1, :col2);",
 om->getTargetTable(1).c_str());

			dmlGr = new DMLGroup();
			dmlGr->AddStatement(dml_statmentA);
			params->mDMLGroupList.push_back(dmlGr);

			dmlGr2 = new DMLGroup();
			dmlGr2->AddStatement(dml_statmentB);
			params->mDMLGroupList.push_back(dmlGr2);

			t = new ThreadTest(params);
			t->runUpdate();
			break;

		case STREAM_TEST:
			/***********************************
***********
			* Uses Stream to load 100 rows
			**************************************
********/
			params = new ThreadTestParameters(TD_STREAM,
4);
			params->SetLogon(user_name,user_password,
tdp_id);
			params->SetFile(file_name);
			params->SetTable(1,target_table);
			/* SetCheckpoint - num of rows/buffers to 
checkpoint at : restart tries */
			params->SetCheckpoint(10,1);
			params->SetSessions(4,4);
			/* SetBufferMode - 1 - on, 0 - off : 
then num of rows per buffer */
			//params->SetBufferMode(1,10); 

			schema = new Schema((char*)"input");  //TPT-17718
			schema->AddColumn((char*)"col1",TD_INTEGER,4);  //TPT-17718
			schema->AddColumn((char*)"col2",TD_CHAR,96);  //TPT-17718
			params->mSchemaList.push_back(schema);

            sprintf(dml_statment, "INSERT INTO %s( :col1, :col2);",
 om->getTargetTable().c_str());
			dmlGr = new DMLGroup();
			dmlGr->AddStatement(dml_statment);
			params->mDMLGroupList.push_back(dmlGr);

			t = new ThreadTest(params);
			t->runStream();
			break;

		
          /* DR146403 - Remove the SIM_TEST case statement
            to remove the TD_NO_OPERATOR references */

		default:
			cout << "No Test Specified" << endl;

	}


	if ( params != NULL )
		delete params;

	if ( t != NULL )
		delete t;

    free(user_name);
    free(user_password);
    free(target_table);
    free(target_tableA);
    free(target_tableB);
    free(tdp_id);
    free(file_name);
    free(outfile_name);
    free(dml_statment);
    free(dml_statmentA);
    free(dml_statmentB);

	//system("PAUSE");

	cout << "*** Application Complete ***" << endl;

	pthread_exit(NULL);

	return 0;
}
