/******************************************************
*******************
*
* Copyright 2005-2010, 2014 by Teradata Corporation.
* All Rights Reserved.
* TERADATA CONFIDENTIAL AND TRADE SECRET
*
* This copyrighted material is the Confidential, Unpublished
* Property of the Teradata Corporation.  This copyright notice and
* any other copyright notices included in machine readable
* copies must be reproduced on all authorized copies.
*
**********************************************************
***************/




#ifndef __MVS__
#include "ThreadTest.h"
#include "TestLoad.h"
#include "TestUpdate.h"
#include "TestStream.h"
#include "TestExport.h"
#include "TestSim.h"
#else
#include "PTCTHDTE.h"
#include "PTCLOADT.h"
#include "PTCUPDAT.h"
#include "PTCSTRMT.h"
#include "PTCEXPRT.h"
#include "PTCSIMT.h"
#endif

ThreadTest::ThreadTest(ThreadTestParameters* p)
{
  mParams = p;
  mActiveHandle = 0;
  mRunStatus = NORMAL;

  mSync = new Synchronizer();

  errorLog = NULL;
}

ThreadTest::~ThreadTest()
{

  if( !tBases.empty() )
    {
      vector<TestBase*>::iterator iter = tBases.begin();

      for( ; iter != tBases.end(); iter ++ )
	{
	  TestBase* tb = *iter;
	  delete tb;
	}

    }

  if ( mSync != NULL ){
    delete mSync;
  }

  if ( errorLog != NULL ){
    fclose(errorLog);
  }
}

/**
 * Gets the entry point for the load test.
 * Refer to the header file for a more detailed method description.
 */
//extern "C" void *ThreadTestLoad(void* lpParameter) {
extern "C" void *ThreadTestLoad(void* lpParameter) {
  TestLoad &tLoad = *reinterpret_cast<TestLoad *>(lpParameter);
  //printf( "obj location: %i\n",
  // reinterpret_cast<int>(lpParameter));   /* DR100926 */
  printf("obj location: %x\n", lpParameter);    
                         /* DR100926 */
  tLoad.run();
  pthread_exit(NULL);
  return 0;
}

/**
 * Gets the entry point for the update test.
 * Refer to the header file for a more detailed method description.
 */
extern "C" void *ThreadTestUpdate(void* lpParameter) {
  TestUpdate &tUpdate = *reinterpret_cast<TestUpdate *>(lpParameter);
  tUpdate.run();
  pthread_exit(NULL);
  return 0;
}

/**
 * Gets the entry point for the stream test.
 * Refer to the header file for a more detailed method description.
 */
extern "C" void *ThreadTestStream(void* lpParameter) {
  TestStream &tStream = *reinterpret_cast<TestStream *>(lpParameter);
  tStream.run();
  pthread_exit(NULL);
  return 0;
}

/**
 * Gets the entry point for the stream test.
 * Refer to the header file for a more detailed method description.
 */
extern "C" void *ThreadTestExport(void* lpParameter) {
  TestExport &tExport = *reinterpret_cast<TestExport *>(lpParameter);
  tExport.run();
  pthread_exit(NULL);
  return 0;
}

/**
 * Gets the entry point for the stream test.
 * Refer to the header file for a more detailed method description.
 */
extern "C" void *ThreadTestSim(void* lpParameter) {
  TestSim &tSim = *reinterpret_cast<TestSim *>(lpParameter);
  tSim.run();
  pthread_exit(NULL);
  return 0;
}

/**
 * Function handles messages from threads
 *	Manager waits until a thread signals and then handles message
 *	Continues until all threads have exited
 */
int ThreadTest::serviceThreads()
{
  int remainingThreads = mParams->mNumThreads;
  int syncWaitingThreads = 0;
  int masterBarrierType = -1;
  int slaveBarrierType = -1;
  int currentBarrierType = -1;

  while (remainingThreads > 0) {
    //cout << "[Manager] Waiting for threads" << endl;     
          /* DR99279 */
    int ret = waitForThreads();

    if (ret == -1) {
      printf("Error: Failed to wait on thread, rc = %d\n", ret);
      return -1;
    }

    if (ret < mParams->mNumThreads) { 

      /**********************************************
       * Get test object associated with thread that
       * signaled and process the message
       **********************************************/
      TestBase* myTest = tBases[ret];
      int instanceID = myTest->getInstanceID();
      int	currentMethod = myTest->getCurrentMethod();
      int threadID = ret+1;

      while( myTest->hasMessage() )
	{
	  int msgType = myTest->getMessageType();

	  /**********************************************
	   * Process Message
	   **********************************************/
	  switch(msgType){

	  case SYNC_BARRIER:
	    /***************************************************
	     * Thread has reached a Barrier synchronization point
	     *	1. Increment count of threads waiting
	     *	2. Print string message
	     *	3. If all threads have signaled:
	     *		- reset counter
	     *		- check if status is abort
	     *		- awaken threads
	     ***************************************************/
	    currentBarrierType = myTest->getCurrentStatus();
	    syncWaitingThreads++;

	    printf("  [Thread %d] ", instanceID);
	    cout << myTest->getMessage() << endl;

	    if ( myTest->hasMessage() ){
	      printf("  [Thread %d] ", instanceID);
	      cout << myTest->getMessage() << endl;
	    }

	    if ( instanceID == 1 && masterBarrierType == -1){
	      masterBarrierType = currentBarrierType;
	      if ( mParams->mNumThreads == 1)
		slaveBarrierType = currentBarrierType;
	    }else if ( slaveBarrierType == -1 ){
	      slaveBarrierType = currentBarrierType;
	    }

	    if( syncWaitingThreads == remainingThreads ){
	      cout << "[Manager] Barrier Reached" << endl;

	      //Take necessary action
	      processBarrier(masterBarrierType,slaveBarrierType,
currentMethod);
								
	      //Reset barrier variables
	      syncWaitingThreads = 0;
	      masterBarrierType = -1;
	      slaveBarrierType = -1;

	      awakenThreads();
	    }
	    break;

	  case SYNC_TELINFO:
	    /***************************************************
	     * Thread has reached a TELINFO synchronization point
	     *	1. Increment count of threads waiting
	     *	2. Print string message
	     *	3. If all threads have signaled:
	     *		- reset counter
	     *		- check if status is abort
	     *		- else sync TELINFO and awaken threads
	     ***************************************************/
	    currentBarrierType = myTest->getCurrentStatus();
	    syncWaitingThreads++;

	    printf("  [Thread %d] ", instanceID);
	    cout << myTest->getMessage() << endl;

	    if ( myTest->hasMessage() ){
	      printf("  [Thread %d] ", instanceID);
	      cout << myTest->getMessage() << endl;
	    }

	    if ( instanceID == 1 && masterBarrierType == -1){
	      masterBarrierType = currentBarrierType;
	      if ( mParams->mNumThreads == 1)
		slaveBarrierType = currentBarrierType;
	    }else if ( slaveBarrierType == -1 ){
	      slaveBarrierType = currentBarrierType;
	    }

	    if( syncWaitingThreads == remainingThreads ){
	      cout << "[Manager] TELINFO Barrier Reached" << endl;
							
	      //Take necessary action
	      processBarrier(masterBarrierType,slaveBarrierType,
currentMethod);
                                                        
	      //Reset barrier variables
	      syncWaitingThreads = 0;  
	      masterBarrierType = -1;  
	      slaveBarrierType = -1;

	      awakenThreads();
	    }
	    break;

	  case STATUS_MESSAGE:
	    /***************************************************
	     * Thread wants to print a message to console
	     *	1. print message
	     ***************************************************/
	    printf("  [Thread %d] ", instanceID);
	    cout << myTest->getMessage() << endl;
	    break;

	  case EXIT_MESSAGE:
	    /***************************************************
	     * A thread has exited
	     *	1. check for any unread messages and print them
	     *	2. reduce number of remaining threads
	     ***************************************************/
	    printf("[Manager] Instance %d Terminated\n",instanceID);

	    while( myTest->hasMessage() )
	      {
		int msgType = myTest->getMessageType();

		switch(msgType){

		case SYNC_BARRIER:
		  myTest->nextMessage();
		  printf("  [Thread %d] ", instanceID);
		  cout << "Unread Msg: Synching barrier " << endl;
		  break;

		case SYNC_TELINFO:
		  myTest->nextMessage();
		  printf("  [Thread %d] ", instanceID);
		  cout << "Unread Msg: Synching TELINFO " << endl;
		  break;

		case STATUS_MESSAGE:
		  printf("  [Thread %d] ", instanceID);
		  cout << "Unread Msg:" << myTest->getMessage()
 << endl;
		  break;

		case EXIT_MESSAGE:
		  printf("  [Thread %d] ", instanceID);
		  cout << "Unread Msg:" << myTest->getMessage()
 << endl;
		  break;
		default:
		  printf("  [Thread %d] ", instanceID);
		  cout << "Unread Msg:" << myTest->getMessage()
 << endl;
		  cout << "Error Code: " << myTest->getErrorCode()
 << endl;
		  cout << "Error Msg: " << myTest->getErrorMsg()
 << endl;
		}

	      }

	    // Clean Up Thread Resources
	    remainingThreads--; 
// Adjust the count of remaining threads
	    
printf("[Manager] %d threads remaining\n", remainingThreads);
	    break;

	  default:
	    /***************************************************
	     * An error has occured
	     *	1. Change run status to ABORT or RESTART
	     *	2. Increment count of threads waiting ( error 
             * thread will wait )
	     *	3. Print error string message
	     *	4. If all threads have signaled:
	     *		- reset counter
	     *		- broadcast abort message to all threads
	     *		- awaken threads
	     ***************************************************/

	    //Only attempt to restart mRestartAttempts times
	    if( mParams->mRestartAttempts > 0 ){
	      mRunStatus = RESTART;
							
	      //Terminate throws errors with further
	      //error info about last error
	      //so we don't need to count it against the
	      //restart attempts
	      if( currentMethod != TERMINATE_METHOD ){
		mParams->mRestartAttempts--;
	      }
	    }else {
	      mRunStatus = ABORT;
	    }

	    syncWaitingThreads++; 
//Thread that had error must wait after messaging
						
	    // Print and Log Error Messages			
			
						
	    string threadErrorMsg = myTest->getMessage();
         
	    printf("  [Thread %d] %s\n", instanceID, 
threadErrorMsg.c_str());
	    printf("  [Thread %d] Error Code: %d\n",
 instanceID, myTest->getErrorCode());
	    printf("  [Thread %d] Error Msg : %s\n",
 instanceID, myTest->getErrorMsg());

	    if ( errorLog == NULL ){
	      errorLog = fopen("errorLog.txt","w");
	    }

	    if ( errorLog == NULL ){
	      cout << "Error opening error log file" << endl;
	    }else{

	      fprintf(errorLog,"  [Thread %d] %s\n",
 instanceID, threadErrorMsg.c_str());
	      fprintf(errorLog,"  [Thread %d] Error Code: %d\n",
 instanceID, myTest->getErrorCode());
	      fprintf(errorLog,"  [Thread %d] Error Msg : %s\n",
 instanceID, myTest->getErrorMsg());
	    }

	    //cout << myTest->getMessage() << endl;
	    //printf("Error Code: %d\n",myTest->getErrorCode());
	    //printf("Error Msg: %s\n", myTest->getErrorMsg());

	    masterBarrierType = TD_Error;
	    slaveBarrierType = TD_Error;

	    if( syncWaitingThreads == remainingThreads ){
	      cout << "[Manager] Error Barrier Reached" << endl;

	      //Take necessary action
	      processBarrier(masterBarrierType,slaveBarrierType
,currentMethod);
                                                        
	      //Reset barrier variables
	      syncWaitingThreads = 0;
	      masterBarrierType = -1;
	      slaveBarrierType = -1;

	      awakenThreads();
	    }
	  }
	}

    } else {    
      printf("[Manager] %d is an invalid thread reference number\n",
ret);
      return -1;
    }

  }

  //void* threadReturn;

  //for( int i = 0; i < mParams->mNumThreads; i++ ){
  //	pthread_join(hthread[i],&threadReturn);
  //	printf("[Manager] Thread %d has terminated\n");
  //}

  return 0;
}

/**
 * Function sets up threads for Load Test
 */
int ThreadTest::runLoad()
{
  /***************************************************
   * Divide up load between threads
   *	1. Use File Length and Thread count to determine
   *		how many rows each thread should load
   ***************************************************/

  //Get Length of Load File
  ifstream source;
  long length = 0;

  source.open (mParams->mSourceFileName, ios::binary );
  if(!source.good()){
    cout << "Error: Could not open data file" << endl;
    return -1;
  }
#ifndef __MVS__
  source.seekg (0, ios::end);
  length = source.tellg();
#else
  while (!source.eof()) {
    source.get();
    ++length;
  }
#endif //__MVS__
  source.close();

  //Determine Load Length for each thread
  long loadLen = length/mParams->mNumThreads;

  /***************************************************************
   * Prepare TestLoad class instances
   *	Each thread will have a TestLoad class instance
   *	Each TestLoad class encapsulates a Connection object
   *          // DR97368 //
   *		1. Create Testload instance
   *			- initiate with test parameters, 
   * manager event handles,
   *				and load file range
   *			- check status in case problem initiating
   *		2. Set instance number in TestLoad instance
   *		3. Add TestLoad instance to vector
   ****************************************************************/
  tBases.reserve(mParams->mNumThreads);

  long pos = 0;

  for (int i = 0; i < mParams->mNumThreads; i++) {
    TestLoad* newTL = new TestLoad(mParams,i+1,mSync,pos,pos+loadLen);

    if(newTL->getRunStatus() != NORMAL ){
      cout << "Error creating TestLoad instance " << i << endl;
      return -1;
    }

    newTL->setInstanceID(i+1);
    tBases.push_back(newTL);
    pos += loadLen;
  }

  //((TestLoad*)(tBases.at(0)))->printTestOptions();  
// Only print the test options for the first instance of sel,
  // the rest are the same

  ((TestLoad*)(tBases[0]))->printTestOptions(); 
//added by sudha for linux testing
  /***************************************************
   * Create Threads
   *	1. Spawn thread
   ***************************************************/
  hthread.reserve(mParams->mNumThreads);

  for (int i = 0; i < mParams->mNumThreads; i++) {

    cout << "[Manager] Creating Thread " << i << "/" 
<< mParams->mNumThreads << endl;
    int rc = pthread_create(&(hthread[i]), NULL,
 ThreadTestLoad, (void *)tBases[i]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }


  }

  /***************************************************
   * Service Threads
   *	After threads setup, manager waits for messages
   ***************************************************/
  int rv = serviceThreads();

  // Get Load Stats

  return 0;
}

/**
 * Function sets up threads for Update Test
 */
int ThreadTest::runUpdate()
{
  /***************************************************
   * Divide up load between threads
   *	1. Use File Length and Thread count to determine
   *		how many rows each thread should load
   ***************************************************/

  //Get Length of Load File
  ifstream source;
  long length = 0;

  source.open (mParams->mSourceFileName, ios::binary );
  if(!source.good()){
    cout << "Error: Could not open data file" << endl;
    return -1;
  }
#ifndef __MVS__
  source.seekg (0, ios::end);
  length = source.tellg();
#else
  while (!source.eof()) {
    source.get();
    ++length;
  }
#endif //__MVS__
  source.close();

  //Determine Load Length for each thread
  long loadLen = length/mParams->mNumThreads;

  /***************************************************************
   * Prepare TestUpdate class instances
   *	Each thread will have a TestUpdate class instance
   *	Each TestLoad class encapsulates a Connection object     
   *         // DR97368 //
   *		1. Create TestUpdate instance
   *			- initiate with test parameters,
   *  manager event handles,
   *				and load file range
   *			- check status in case problem initiating
   *		2. Set instance number in TestUpdate instance
   *		3. Add TestUpdate instance to vector
   ****************************************************************/
  tBases.reserve(mParams->mNumThreads);

  long pos = 0;

  for (int i = 0; i < mParams->mNumThreads; i++) {
    TestUpdate* newTL = new TestUpdate(mParams,i+1,
mSync,pos,pos+loadLen);

    if(newTL->getRunStatus() != NORMAL ){
      cout << "Error creating TestUpdate instance " << i << endl;
      return -1;
    }

    newTL->setInstanceID(i+1);
    tBases.push_back(newTL);
    pos += loadLen;
  }

  //((TestUpdate*)(tBases.at(0)))->printTestOptions();  
// Only print the test options for the first instance of sel,
  // the rest are the same

  ((TestUpdate*)(tBases[0]))->printTestOptions(); //added by
  // sudha for linux tsting
  /***************************************************
   * Create Threads
   *	1. Spawn thread
   ***************************************************/
  hthread.reserve(mParams->mNumThreads);

  for (int i = 0; i < mParams->mNumThreads; i++) {
    cout << "[Manager] Creating Thread" << endl;
    int rc = pthread_create(&(hthread[i]), NULL, 
ThreadTestUpdate, (void *)tBases[i]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }
  }

  /***************************************************
   * Service Threads
   *	After threads setup, manager waits for messages
   ***************************************************/
  int rv = serviceThreads();

  // Get Update Stats

  return 0;
}

/**
 * Function sets up threads for Stream Test
 */
int ThreadTest::runStream()
{
  /***************************************************
   * Divide up load between threads
   *	1. Use File Length and Thread count to determine
   *		how many rows each thread should load
   ***************************************************/

  //Get Length of Stream File
  ifstream source;
  long length = 0;

  source.open (mParams->mSourceFileName, ios::binary );
  if(!source.good()){
    cout << "Error: Could not open data file" << endl;
    return -1;
  }
#ifndef __MVS__
  source.seekg (0, ios::end);
  length = source.tellg();
#else
  while (!source.eof()) {
    source.get();
    ++length;
  }
#endif //__MVS__
  source.close();

  //Determine Load Length for each thread
  long loadLen = length/mParams->mNumThreads;

  /***************************************************************
   * Prepare TestStream class instances
   *	Each thread will have a TestStream class instance
   *	Each TestStream class encapsulates a Connection object 
   *           // DR97368 //
   *		1. Create TestStream instance
   *			- initiate with test parameters, 
   * manager event handles,
   *				and load file range
   *		2. Set instance number in TestStream instance
   *		3. Add TestStream instance to vector
   ****************************************************************/

  // Prepare TestLoad Objects

  tBases.reserve(mParams->mNumThreads);

  long pos = 0;

  for (int i = 0; i < mParams->mNumThreads; i++) {
    TestStream* newTL = new TestStream(mParams,i+1,mSync,pos,
 pos+loadLen);

    if(newTL->getRunStatus() != NORMAL ){
      cout << "Error creating TestSim instance " << i << endl;
      return -1;
    }

    newTL->setInstanceID(i+1);
    tBases.push_back(newTL);
    pos += loadLen;
  }

  //((TestStream*)(tBases.at(0)))->printTestOptions();  
// Only print the test options for the first instance of sel, 
  //the rest are the same
  ((TestStream*)(tBases[0]))->printTestOptions(); 
//added by sudha

  /***************************************************
   * Create Threads
   *	1. Spawn thread
   ***************************************************/

  hthread.reserve(mParams->mNumThreads);

  for (int i = 0; i < mParams->mNumThreads; i++) {
    cout << "[Manager] Creating Thread" << endl;
    int rc = pthread_create(&(hthread[i]), NULL,
 ThreadTestStream, (void *)tBases[i]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }
  }

  /***************************************************
   * Service Threads
   *	After threads setup, manager waits for messages
   ***************************************************/
  int rv = serviceThreads();

  // Get Stream Stats

  return 0;
}


/**
 * Function sets up threads for Export Test
 */
int ThreadTest::runExport()
{
  /***************************************************************
   * Prepare Export File
   ****************************************************************/

#ifndef __MVS__
  FILE* outFile = fopen(mParams->mSourceFileName,"wb");
#else
  FILE* outFile = fopen(mParams->mSourceFileName,"wb,type=record,\
recfm=fb,lrecl=101");
#endif //__MVS__

  if ( outFile == NULL ){
    //Error Could Not Open File
    cout << "Error opening output file " <<
 mParams->mSourceFileName << endl;
    return -1;
  }

  /***************************************************************
   * Prepare TestExport class instances
   *	Each thread will have a TestExport class instance
   *	Each TestExport class encapsulates a Connection object  
   *       // DR97368 //
   *		1. Create TestExport instance
   *			- initiate with test parameters,
   *  manager event handles
   *			- check status in case problem initiating
   *		2. Set instance number in TestExport instance
   *		3. Add TestExport instance to vector
   ****************************************************************/
  tBases.reserve(mParams->mNumThreads);

  for (int i = 0; i < mParams->mNumThreads; i++) {
    TestExport* newTL = new TestExport(mParams,i+1,mSync,0,0,outFile);

    if(newTL->getRunStatus() != NORMAL ){
      cout << "Error creating TestExport instance " << i << endl;
      return -1;
    }

    newTL->setInstanceID(i+1);
    tBases.push_back(newTL);
  }

  //((TestLoad*)(tBases.at(0)))->printTestOptions();  
// Only print the test options for the first instance 
  //of sel, the rest are the same
  ((TestLoad*)(tBases[0]))->printTestOptions();//added by sudha

  /***************************************************
   * Create Threads
   *	1. Spawn thread - pass pointer to TestExport instance
   ***************************************************/
  hthread.reserve(mParams->mNumThreads);

  for (int i = 0; i < mParams->mNumThreads; i++) {

    cout << "[Manager] Creating Thread" << endl;
    int rc = pthread_create(&(hthread[i]), NULL,
 ThreadTestExport, (void *)tBases[i]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }

  }

  /***************************************************
   * Service Threads
   *	After threads setup, manager waits for messages
   ***************************************************/
  int rv = serviceThreads();

  // Get Export Stats


  /***************************************************************
   * Clean Up
   ****************************************************************/

  if ( outFile != NULL ){
    fclose(outFile);
  }

  return 0;
}


/**
 * Function sets up threads for Simulation Test
 */
int ThreadTest::runSim()
{
  //Get Length of Stream File
  ifstream source;
  long length = 0;

  source.open (mParams->mSourceFileName, ios::binary );
  if(!source.good()){
    cout << "Error: Could not open data file" << endl;
    return -1;
  }
  source.seekg (0, ios::end);
  length = source.tellg();
  source.close();

  //Determine Load Length for each thread
  long loadLen = length/mParams->mNumThreads;

  // Prepare TestLoad Objects

  tBases.reserve(mParams->mNumThreads);

  long pos = 0;

  for (int i = 0; i < mParams->mNumThreads; i++) {
    TestSim* newTL = new TestSim(mParams,i+1,mSync,pos,pos+loadLen);

    if(newTL->getRunStatus() != NORMAL ){
      cout << "Error creating TestSim instance " << i << endl;
      return -1;
    }

    newTL->setInstanceID(i+1);
    tBases.push_back(newTL);
    pos += loadLen;
  }

  //((TestSim*)(tBases.at(0)))->printTestOptions();  
// Only print the test options for the first instance of
  // sel, the rest are the same
  ((TestSim*)(tBases[0]))->printTestOptions();
//added by sudha

  // Create Threads

  hthread.reserve(mParams->mNumThreads);

  for (int i = 0; i < mParams->mNumThreads; i++) {
    cout << "[Manager] Creating Thread" << endl;
    int rc = pthread_create(&(hthread[i]), NULL,
 ThreadTestSim, (void *)tBases[i]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }
  }

  int rv = serviceThreads();

  // Get Sim Stats

  return 0;
}


/**
 * Wait to be signaled by threads
 * Return reference number of thread
 * which signals
 */
int ThreadTest::waitForThreads()
{
  return mSync->waitForThreads();
}

/**
 * Function processes the current barrier
 */
void ThreadTest::processBarrier(int mBarrier, int sBarrier, 
int cMethod)
{
  broadcast(NORMAL);

  //If slaves have reached END_Method but master has not then
  //we need to tell slaves to wait again
  if ( sBarrier == TD_END_Method && mBarrier != TD_END_Method ){
    cout << "[Manager] Signaling slaves to wait again" << endl;
    broadcast(WAIT_AGAIN);
  }

  switch( mRunStatus )
    {
    case RESTART:
      if( cMethod != TERMINATE_METHOD ){
	cout << "[Manager] Signaling Restart" << endl;
	broadcast(mRunStatus);
	mRunStatus = NORMAL;
      }
      break;
    case ABORT:
      if( cMethod != TERMINATE_METHOD ){
	cout << "[Manager] Signaling Abort" << endl;
	broadcast(mRunStatus);
	mRunStatus = NORMAL;
      }
      break;
    case WAIT_AGAIN:
    case NORMAL:
    default:
      if ( mBarrier == TD_SYNC_TELINFO ){
	cout << "[Manager] Synchronizing TELINFO" << endl;
	syncTELINFO();
	cout << "[Manager] TELINFO Synchronized" << endl;
      }
      break;
    }

}

/**
 * Get TELINFO from master thread
 * Pass copy to slave threads
 */
int ThreadTest::syncTELINFO()
{
  char* telinfo;
  TD_Length length;
  int rv = 0;

  //Get TELINFO from master thread
  rv = tBases[0]->getTELINFO(&telinfo,&length);

  if ( rv != 0 ){
    return -1;
  }

  for(int i = 1; i < mParams->mNumThreads; i++ ){
    char* telcopy = (char*)malloc(length);
		
    memcpy(telcopy,telinfo,length);

    rv = tBases[i]->setTELINFO(telcopy,length);

    free(telcopy);

    if( rv != 0 ){
      break;
    }
  }

  return rv;
}


/**
 * Signals Threads to awaken
 */
int ThreadTest::awakenThreads()
{
  int rv = 0;

  if( mActiveHandle == 0 ){
    mActiveHandle = 1;

    mSync->awakenThreads(0);

  }else{
    mActiveHandle = 0;

    mSync->awakenThreads(1);

  }

  return rv;
}

/**
 * Broadcast run status to threads
 */
int ThreadTest::broadcast(int status)
{
  for(int i = 0; i < mParams->mNumThreads; i++ ){
    tBases[i]->setRunStatus(status);
  }

  return 0;
}
















