/* 
 * Copyright 2005-2010, 2014 by Teradata Corporation.
 * All Rights Reserved.
 * TERADATA CONFIDENTIAL AND TRADE SECRET
 *
 * This document, and the information contained herein, are the exclusive
 * property of Teradata. In no case shall this document or its contents be
 * reproduced or copied by any means, in whole or in part, or disseminated
 * outside of the company, without prior written permission of a company
 * officer.
 *
 */

#ifndef PERF_TEST_MESSAGE_H
#define PERF_TEST_MESSAGE_H

//#include "Mutex.h"
#include "common.h"

#ifndef __MVS__
#include "Synchronizer.h"
#else
#include "PTCTSYNC.h"
#endif

/**
 * Defines a thread-safe performance test message.
 */
class PerfTestMessage {
private:
    /**
     * Instance ID
     */
	int mInstance;

   /**
     * Synchronizer Object
     */
	Synchronizer* mSync;


    /**
     * Stores a handle to a message event.
     */
    //HANDLE hmsg;
	void* hmsg;

    /**
     * Stores a lockable resource.
     */
    //Mutex *mutx;
	pthread_mutex_t mutx;

    /**
     * Defines a type of an array (vector) of strings of performance test messages.
     */
    typedef vector<string> vecstr_t;

    /**
     * Stores an array (vector) of strings of performance test messages.
     */

    /* DR99279 BEGIN */

    //vecstr_t msgs;
    #define MAX_MESSAGE_SIZE 10000
    string msgs[MAX_MESSAGE_SIZE];

	/**
     * Stores an array (vector) of integers of performance test message types.
     */
    //vector<int> msgTypes;
    int msgTypes[MAX_MESSAGE_SIZE];

    /* DR99279 END */

    /**
     * Stores the index to where the next message shall be written.
     */
    vecstr_t::size_type nextWrite;

    /**
     * Stores the index into to where the next message shall be read.
     */
	vecstr_t::size_type nextRead;
public:
    /**
     * Initializes the handle to the message event.
     */
    PerfTestMessage(int,Synchronizer*);

    /**
     * Closes the handle to the message event.
     */
    virtual ~PerfTestMessage();

    /**
     * Gets the handle to performance test messages, which is used by the main thread to print test output.
     * @return The handle to performance test messages.
     */
    const void* getMessageHandle() const {return hmsg;}

    /**
     * Writes a message into the array of strings of performance test messages.
	 * @param msgType -- type of message to be written
     * @param msg -- the message to be written
     */
    void write(int msgType, const string &msg);

    /**
     * Reads a message from the array of stings of performance test messages.
     */
    string read();

	/**
     * Returns the type of the next message
     */
    int getMessageType();

	/**
     * Moves to next message for read
     */
    void skipRead();

    /**
     * Checks whether there is a message ready to be read.
     * @return TRUE if a message is ready to be read, otherwise FALSE.
     */
    //bool hasMessage() {return (nextRead < nextWrite);}
    bool hasMessage() { return (nextRead != nextWrite);}
};

#endif // PERF_TEST_MESSAGE_H
